/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/


// ClassType_H.template
#ifndef _775MP7JRSDOMINANTCOLORTYPE_H
#define _775MP7JRSDOMINANTCOLORTYPE_H

#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOM.hpp>

#include "Mp7JrsDefines.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsDominantColorTypeFactory.h"
#include "Mp7JrsNode.h"
#include "Mp7JrsConvert.h"


// no includefile for extension defined 
// file Mp7JrsDominantColorType_ExtInclude.h


#include "Mp7JrsDType.h"
#include "Mp7JrsMpeg7BaseType.h"
#include "Mp7JrsVisualDType.h"
class IMp7JrsColorQuantizationType;
class IMp7JrsColorSpaceType;
class IMp7JrsDominantColorType;
class IMp7JrsDominantColorType_Value_CollectionType;
class IMp7Jrsunsigned5;
class IMp7JrsVisualDType;
typedef Mp7JrsPtr< IMp7JrsColorQuantizationType > Mp7JrsColorQuantizationPtr;
typedef Mp7JrsPtr< IMp7JrsColorSpaceType > Mp7JrsColorSpacePtr;
typedef Mp7JrsPtr< IMp7JrsDominantColorType_Value_CollectionType > Mp7JrsDominantColorType_Value_CollectionPtr;
typedef Mp7JrsPtr< IMp7JrsDominantColorType> Mp7JrsDominantColorPtr;
typedef Mp7JrsPtr< IMp7Jrsunsigned5 > Mp7Jrsunsigned5Ptr;
typedef Mp7JrsPtr< IMp7JrsVisualDType > Mp7JrsVisualDPtr;

/** 
 * Generated interface IMp7JrsDominantColorType for class Mp7JrsDominantColorType<br>
 * Located at: mpeg7-2004.xsd, line 353<br>
 * Classified: Class<br>
 * Derived from: VisualDType (Class)<br>
 */
class MP7JRS_EXPORT IMp7JrsDominantColorType 
 :
        public IMp7JrsVisualDType
{
public:
    // TODO: make these protected?
    IMp7JrsDominantColorType();
    virtual ~IMp7JrsDominantColorType();
        /** @name Sequence

         */
        //@{
    /**
     * Get ColorSpace element.
     * @return Mp7JrsColorSpacePtr
     */
    virtual Mp7JrsColorSpacePtr GetColorSpace() const = 0;

    virtual bool IsValidColorSpace() const = 0;

    /**
     * Set ColorSpace element.
     * @param item const Mp7JrsColorSpacePtr &
     */
    virtual void SetColorSpace(const Mp7JrsColorSpacePtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate ColorSpace element.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidateColorSpace() = 0;

    /**
     * Get ColorQuantization element.
     * @return Mp7JrsColorQuantizationPtr
     */
    virtual Mp7JrsColorQuantizationPtr GetColorQuantization() const = 0;

    virtual bool IsValidColorQuantization() const = 0;

    /**
     * Set ColorQuantization element.
     * @param item const Mp7JrsColorQuantizationPtr &
     */
    virtual void SetColorQuantization(const Mp7JrsColorQuantizationPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate ColorQuantization element.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidateColorQuantization() = 0;

    /**
     * Get SpatialCoherency element.
     * @return Mp7Jrsunsigned5Ptr
     */
    virtual Mp7Jrsunsigned5Ptr GetSpatialCoherency() const = 0;

    /**
     * Set SpatialCoherency element.
     * @param item const Mp7Jrsunsigned5Ptr &
     */
    // Mandatory			
    virtual void SetSpatialCoherency(const Mp7Jrsunsigned5Ptr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

    /**
     * Get Value element.
     * @return Mp7JrsDominantColorType_Value_CollectionPtr
     */
    virtual Mp7JrsDominantColorType_Value_CollectionPtr GetValue() const = 0;

    /**
     * Set Value element.
     * @param item const Mp7JrsDominantColorType_Value_CollectionPtr &
     */
    // Mandatory collection
    virtual void SetValue(const Mp7JrsDominantColorType_Value_CollectionPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

        //@}
    /**
     * Gets or creates Mp7JrsNodePtr child elements of DominantColorType.
     * Currently this type contains 4 child elements of type "Mp7JrsNodePtr".

     * <ul>
     * <li>ColorSpace</li>
     * <li>ColorQuantization</li>
     * <li>SpatialCoherency</li>
     * <li>urn:mpeg:mpeg7:schema:2004:Value</li>
     * </ul>
     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     * \par<code>A</code> 
     * - Select all child elements named <code>A</code>.
     * - In case of no element, create a new one.
     * - For collections: all subsequent XPath expressions are applied on all collection items (e.g. <code>A/B</code> query or create, childnodes <code>B</code> of all <code>A</code>s). 
     * \par<code>A[i]</code>
     * - Select the <code>i</code>th child element of all <code>A</code>.
     * - Please note, that XPath indexes start at 1.
     * - If there is no such <code>i</code>th element, try to create a new one at index <code>i</code>.
     * - Single elements require <code>i</code> to be 1 otherwise an exception is thrown.
     * - For collections, <code>i</code> must not exceed the number of existing items +1 (appending is allowed, filling gaps not).
     * - If the index is too high, a <code>MissingElementException</code> is thrown, and you can extract the last valid index. (Example: If a collection contains 2 elements and you are requesting the 4th element).
     * 
     * \par <code>A[\@xsi:type="T"]</code>
     *     - Select all child elements named <code>A</code> of type <code>T</code>.
     *     - Currently only typenames without namespace are recognised.
     *     - If there is no such element, create and append a new one.
     * \par <code>A[i][\@xsi:type="T"]</code>
     *     - Select the <code>i</code>th child element named <code>A</code>, which must be of type <code>T</code>.
     *     - If there is no such element at index <code>i</code>, a new one is created and inserted (appended).
     * \par <code>A[\@xsi:type="T"][i]</code>
     *     - Out of all child elements named <code>A</code> of type <code>T</code>, take the <code>i</code>th one.
     *     - If there is no such element at index <code>i</code>, create and append one.
     * 
     * \par <code>A[last()]</code>
     *     - Select the last child element named <code>A</code>.
     *     - In case of no element, create a new one.
     * \par <code>A[last()][\@xsi:type="T"]</code>
     *     - Select the last child element named <code>A</code>, which must be of type <code>T</code>.
     *     - For collections: in case of no such type, create and append a new one.
     *     - For single elements: if an element with different type exists, return the empty list.
     * 
     * \par <code>A[\@xsi:type="T"][last()]</code>
     *     - Out of all child elements named <code>A</code> of type <code>T</code>, take the last one.
     *     - For collections: in case of no such type, create and append a new one.
     *     - For single elements: if an element with different type exists, return the empty list.
     * 
     * \par <code>A[position() \> last()]</code>
     *     - Force creation of a child element named <code>A</code> with its default type.
     *     - For collections: create and append the default element.
     *     - For single elements: if any element exists, an exception is thrown.
     *
     * \par <code>A[position() \> last()][\@xsi:type="T"]</code>
     *     - Force creation of a child element named <code>A</code> of type <code>T</code>.
     *     - For collections: create and append the element.
     *     - For single elements: if any element exists, an exception is thrown.
     *
     * \par <code>A[\@xsi:type="T"][position() \> last()]</code>
     *     - Force creation of a child element named <code>A</code> of type <code>T</code>.
     *     - Behaves the same way as the previous expression.
     * 
     * 
     * Supported XPath expressions for GetFromXPath<br>
     * The same expressions like GetOrCreateFromXPath plus
     * \par <code>A[position() \< i] </code>
     *     - Select all child elements named <code>A</code> at indexes lower than <code>i</code>.
     *     - <code>i</code> must be greater than -1, otherwise an exception is thrown.
     * \par <code>A[position() \> i] </code>
     *     - Select all child elements named <code>A</code> at indexes greater than <code>i</code>.
     *     - <code>i</code> must be greater than -1, otherwise an exception is thrown.
     * \par <code>A[position() \> i | position() \< j] </code>
     *     - Select all child elements named <code>A</code> at indexes greater than <code>i</code> and lower than <code>j</code>.
     *     - <code>i</code> and <code>j</code> must be greater than -1, otherwise an exception is thrown.
     *     - If <code>i</code> is greater than <code>j</code> then an empty list is returned.
     *
     * These three expressions can be combined with the <code>\@xsi:type</code> attribute to get a more restricted query.
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL) = 0;

    
// no includefile for extension defined 
// file Mp7JrsDominantColorType_ExtMethodDef.h


// no includefile for extension defined 
// file Mp7JrsDominantColorType_ExtPropDef.h

};




/*
 * Generated class Mp7JrsDominantColorType<br>
 * Located at: mpeg7-2004.xsd, line 353<br>
 * Classified: Class<br>
 * Derived from: VisualDType (Class)<br>
 * Defined in mpeg7-2004.xsd, line 353.<br>
 */
class MP7JRS_EXPORT Mp7JrsDominantColorType :
        public IMp7JrsDominantColorType
{
    friend class Mp7JrsDominantColorTypeFactory; // constructs objects

public:
    /*
     * Retrieves the base class of Mp7JrsDominantColorType
     
     * @return Mp7JrsPtr< Mp7JrsVisualDType >
     */
    virtual Mp7JrsPtr< Mp7JrsVisualDType > GetBase() const;
        /* @name Sequence

         */
        //@{
    /*
     * Get ColorSpace element.
     * @return Mp7JrsColorSpacePtr
     */
    virtual Mp7JrsColorSpacePtr GetColorSpace() const;

    virtual bool IsValidColorSpace() const;

    /*
     * Set ColorSpace element.
     * @param item const Mp7JrsColorSpacePtr &
     */
    virtual void SetColorSpace(const Mp7JrsColorSpacePtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate ColorSpace element.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidateColorSpace();

    /*
     * Get ColorQuantization element.
     * @return Mp7JrsColorQuantizationPtr
     */
    virtual Mp7JrsColorQuantizationPtr GetColorQuantization() const;

    virtual bool IsValidColorQuantization() const;

    /*
     * Set ColorQuantization element.
     * @param item const Mp7JrsColorQuantizationPtr &
     */
    virtual void SetColorQuantization(const Mp7JrsColorQuantizationPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate ColorQuantization element.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidateColorQuantization();

    /*
     * Get SpatialCoherency element.
     * @return Mp7Jrsunsigned5Ptr
     */
    virtual Mp7Jrsunsigned5Ptr GetSpatialCoherency() const;

    /*
     * Set SpatialCoherency element.
     * @param item const Mp7Jrsunsigned5Ptr &
     */
    // Mandatory			
    virtual void SetSpatialCoherency(const Mp7Jrsunsigned5Ptr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

    /*
     * Get Value element.
     * @return Mp7JrsDominantColorType_Value_CollectionPtr
     */
    virtual Mp7JrsDominantColorType_Value_CollectionPtr GetValue() const;

    /*
     * Set Value element.
     * @param item const Mp7JrsDominantColorType_Value_CollectionPtr &
     */
    // Mandatory collection
    virtual void SetValue(const Mp7JrsDominantColorType_Value_CollectionPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

        //@}
    /*
     * Gets or creates Mp7JrsNodePtr child elements of DominantColorType.
     * Currently this type contains 4 child elements of type "Mp7JrsNodePtr".

     * <ul>
     * <li>ColorSpace</li>
     * <li>ColorQuantization</li>
     * <li>SpatialCoherency</li>
     * <li>urn:mpeg:mpeg7:schema:2004:Value</li>
     * </ul>
     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     * \par<code>A</code> 
     * - Select all child elements named <code>A</code>.
     * - In case of no element, create a new one.
     * - For collections: all subsequent XPath expressions are applied on all collection items (e.g. <code>A/B</code> query or create, childnodes <code>B</code> of all <code>A</code>s). 
     * \par<code>A[i]</code>
     * - Select the <code>i</code>th child element of all <code>A</code>.
     * - Please note, that XPath indexes start at 1.
     * - If there is no such <code>i</code>th element, try to create a new one at index <code>i</code>.
     * - Single elements require <code>i</code> to be 1 otherwise an exception is thrown.
     * - For collections, <code>i</code> must not exceed the number of existing items +1 (appending is allowed, filling gaps not).
     * - If the index is too high, a <code>MissingElementException</code> is thrown, and you can extract the last valid index. (Example: If a collection contains 2 elements and you are requesting the 4th element).
     * 
     * \par <code>A[\@xsi:type="T"]</code>
     *     - Select all child elements named <code>A</code> of type <code>T</code>.
     *     - Currently only typenames without namespace are recognised.
     *     - If there is no such element, create and append a new one.
     * \par <code>A[i][\@xsi:type="T"]</code>
     *     - Select the <code>i</code>th child element named <code>A</code>, which must be of type <code>T</code>.
     *     - If there is no such element at index <code>i</code>, a new one is created and inserted (appended).
     * \par <code>A[\@xsi:type="T"][i]</code>
     *     - Out of all child elements named <code>A</code> of type <code>T</code>, take the <code>i</code>th one.
     *     - If there is no such element at index <code>i</code>, create and append one.
     * 
     * \par <code>A[last()]</code>
     *     - Select the last child element named <code>A</code>.
     *     - In case of no element, create a new one.
     * \par <code>A[last()][\@xsi:type="T"]</code>
     *     - Select the last child element named <code>A</code>, which must be of type <code>T</code>.
     *     - For collections: in case of no such type, create and append a new one.
     *     - For single elements: if an element with different type exists, return the empty list.
     * 
     * \par <code>A[\@xsi:type="T"][last()]</code>
     *     - Out of all child elements named <code>A</code> of type <code>T</code>, take the last one.
     *     - For collections: in case of no such type, create and append a new one.
     *     - For single elements: if an element with different type exists, return the empty list.
     * 
     * \par <code>A[position() \> last()]</code>
     *     - Force creation of a child element named <code>A</code> with its default type.
     *     - For collections: create and append the default element.
     *     - For single elements: if any element exists, an exception is thrown.
     *
     * \par <code>A[position() \> last()][\@xsi:type="T"]</code>
     *     - Force creation of a child element named <code>A</code> of type <code>T</code>.
     *     - For collections: create and append the element.
     *     - For single elements: if any element exists, an exception is thrown.
     *
     * \par <code>A[\@xsi:type="T"][position() \> last()]</code>
     *     - Force creation of a child element named <code>A</code> of type <code>T</code>.
     *     - Behaves the same way as the previous expression.
     * 
     * 
     * Supported XPath expressions for GetFromXPath<br>
     * The same expressions like GetOrCreateFromXPath plus
     * \par <code>A[position() \< i] </code>
     *     - Select all child elements named <code>A</code> at indexes lower than <code>i</code>.
     *     - <code>i</code> must be greater than -1, otherwise an exception is thrown.
     * \par <code>A[position() \> i] </code>
     *     - Select all child elements named <code>A</code> at indexes greater than <code>i</code>.
     *     - <code>i</code> must be greater than -1, otherwise an exception is thrown.
     * \par <code>A[position() \> i | position() \< j] </code>
     *     - Select all child elements named <code>A</code> at indexes greater than <code>i</code> and lower than <code>j</code>.
     *     - <code>i</code> and <code>j</code> must be greater than -1, otherwise an exception is thrown.
     *     - If <code>i</code> is greater than <code>j</code> then an empty list is returned.
     *
     * These three expressions can be combined with the <code>\@xsi:type</code> attribute to get a more restricted query.
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL);

    

    /* Get the class name.
     * @return The name of this class.
     */
    virtual XMLCh * ToText() const;

    /* Parse this type.
     * @param txt buffer to parse from.
     * @return true if parsing was successful, false otherwise.
     */
    virtual bool Parse(const XMLCh * const txt);

    /* Parse the content from a string.  Note, this is purely for ComplexTypes with SimpleContent and SimpleTypes.
     * @param txt buffer to parse from.
     * @return true if parsing was successful, false otherwise.
     */
	virtual bool ContentFromString(const XMLCh * const txt);

    /* Convert the content to string.  Note, this is purely for ComplexTypes with SimpleContent and SimpleTypes. All other types may return NULL or the class name.
     * @return the allocated XMLCh * if conversion was successful. The caller must free memory using Mp7JrsUtil.deallocate().
     */
	virtual XMLCh * ContentToString() const;


    /* Serialialize this object to a DOM tree.
     * @param doc DOM document.
     * @param parent Parent node in DOM tree.
     */
    virtual void Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem = NULL);

    /* Deserialize this object from a DOM tree.
     * @param doc DOM document.
     * @param parent Parent element in DOM tree.
     * @param current Current element in DOM tree.
     */
    virtual bool Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current);

    /* Creates and inserts a Childnode of the specified Type
     * @param type The tag type of the child to be created (tag name).
     * @param xsitype The xsi type of the child to be created if available.
     * @return the created child with the correct dynamic type for the specified type.
     */
    virtual Mp7JrsNodePtr CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype);
         
// no includefile for extension defined 
// file Mp7JrsDominantColorType_ExtMyMethodDef.h


public:

    virtual ~Mp7JrsDominantColorType();

protected:
    Mp7JrsDominantColorType();
    virtual void DeepCopy(const Mp7JrsNodePtr &original);

    virtual Mp7JrsNodeEnum GetAllChildElements() const;

    virtual Mp7JrsNodePtr GetBaseRoot();
    virtual const Mp7JrsNodePtr GetBaseRoot() const;

private:

    // Base Class
    Mp7JrsPtr< Mp7JrsVisualDType > m_Base;

    Mp7JrsColorSpacePtr m_ColorSpace;
    bool m_ColorSpace_Exist; // For optional elements 
    Mp7JrsColorQuantizationPtr m_ColorQuantization;
    bool m_ColorQuantization_Exist; // For optional elements 
    Mp7Jrsunsigned5Ptr m_SpatialCoherency;
    Mp7JrsDominantColorType_Value_CollectionPtr m_Value;

// no includefile for extension defined 
// file Mp7JrsDominantColorType_ExtMyPropDef.h


public:
    virtual void Init(); // Called by inheritees to recursively update m_myself
protected:
    void Cleanup();
};

#endif // _775MP7JRSDOMINANTCOLORTYPE_H

