/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/


// ClassType_H.template
#ifndef _863MP7JRSEXTENTTYPE_H
#define _863MP7JRSEXTENTTYPE_H

#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOM.hpp>

#include "Mp7JrsDefines.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsExtentTypeFactory.h"
#include "Mp7JrsNode.h"
#include "Mp7JrsConvert.h"


// no includefile for extension defined 
// file Mp7JrsExtentType_ExtInclude.h


class IMp7JrsExtentType;
typedef Mp7JrsPtr< IMp7JrsExtentType> Mp7JrsExtentPtr;

/** 
 * Generated interface IMp7JrsExtentType for class Mp7JrsExtentType<br>
 * Located at: mpeg7-2004.xsd, line 6991<br>
 * Classified: Class<br>
 */
class MP7JRS_EXPORT IMp7JrsExtentType 
 :
        public Mp7JrsNode
{
public:
    // TODO: make these protected?
    IMp7JrsExtentType();
    virtual ~IMp7JrsExtentType();
    /**
     * Get measurementType attribute.
     * @return XMLCh *
     */
    virtual XMLCh * GetmeasurementType() const = 0;

    /**
     * Set measurementType attribute.
     * @param item XMLCh *
     */
    // Mandatory
    virtual void SetmeasurementType(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

    /**
     * Get unit attribute.
     * @return XMLCh *
     */
    virtual XMLCh * Getunit() const = 0;

    /**
     * Set unit attribute.
     * @param item XMLCh *
     */
    // Mandatory
    virtual void Setunit(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

    /**
     * Get value attribute.
     * @return XMLCh *
     */
    virtual XMLCh * Getvalue() const = 0;

    /**
     * Set value attribute.
     * @param item XMLCh *
     */
    // Mandatory
    virtual void Setvalue(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

    /**
     * Gets or creates Mp7JrsNodePtr child elements of ExtentType.
     * Currently just supports rudimentary XPath expressions as ExtentType contains no child elements of type "Mp7JrsNodePtr" at all.

     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL) = 0;

    
// no includefile for extension defined 
// file Mp7JrsExtentType_ExtMethodDef.h


// no includefile for extension defined 
// file Mp7JrsExtentType_ExtPropDef.h

};




/*
 * Generated class Mp7JrsExtentType<br>
 * Located at: mpeg7-2004.xsd, line 6991<br>
 * Classified: Class<br>
 * Defined in mpeg7-2004.xsd, line 6991.<br>
 */
class MP7JRS_EXPORT Mp7JrsExtentType :
        public IMp7JrsExtentType
{
    friend class Mp7JrsExtentTypeFactory; // constructs objects

public:
    /*
     * Get measurementType attribute.
     * @return XMLCh *
     */
    virtual XMLCh * GetmeasurementType() const;

    /*
     * Set measurementType attribute.
     * @param item XMLCh *
     */
    // Mandatory
    virtual void SetmeasurementType(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

    /*
     * Get unit attribute.
     * @return XMLCh *
     */
    virtual XMLCh * Getunit() const;

    /*
     * Set unit attribute.
     * @param item XMLCh *
     */
    // Mandatory
    virtual void Setunit(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

    /*
     * Get value attribute.
     * @return XMLCh *
     */
    virtual XMLCh * Getvalue() const;

    /*
     * Set value attribute.
     * @param item XMLCh *
     */
    // Mandatory
    virtual void Setvalue(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

    /*
     * Gets or creates Mp7JrsNodePtr child elements of ExtentType.
     * Currently just supports rudimentary XPath expressions as ExtentType contains no child elements of type "Mp7JrsNodePtr" at all.

     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL);

    

    /* Get the class name.
     * @return The name of this class.
     */
    virtual XMLCh * ToText() const;

    /* Parse this type.
     * @param txt buffer to parse from.
     * @return true if parsing was successful, false otherwise.
     */
    virtual bool Parse(const XMLCh * const txt);

    /* Parse the content from a string.  Note, this is purely for ComplexTypes with SimpleContent and SimpleTypes.
     * @param txt buffer to parse from.
     * @return true if parsing was successful, false otherwise.
     */
	virtual bool ContentFromString(const XMLCh * const txt);

    /* Convert the content to string.  Note, this is purely for ComplexTypes with SimpleContent and SimpleTypes. All other types may return NULL or the class name.
     * @return the allocated XMLCh * if conversion was successful. The caller must free memory using Mp7JrsUtil.deallocate().
     */
	virtual XMLCh * ContentToString() const;


    /* Serialialize this object to a DOM tree.
     * @param doc DOM document.
     * @param parent Parent node in DOM tree.
     */
    virtual void Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem = NULL);

    /* Deserialize this object from a DOM tree.
     * @param doc DOM document.
     * @param parent Parent element in DOM tree.
     * @param current Current element in DOM tree.
     */
    virtual bool Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current);

    /* Creates and inserts a Childnode of the specified Type
     * @param type The tag type of the child to be created (tag name).
     * @param xsitype The xsi type of the child to be created if available.
     * @return the created child with the correct dynamic type for the specified type.
     */
    virtual Mp7JrsNodePtr CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype);
         
// no includefile for extension defined 
// file Mp7JrsExtentType_ExtMyMethodDef.h


public:

    virtual ~Mp7JrsExtentType();

protected:
    Mp7JrsExtentType();
    virtual void DeepCopy(const Mp7JrsNodePtr &original);

    virtual Mp7JrsNodeEnum GetAllChildElements() const;

    virtual Mp7JrsNodePtr GetBaseRoot();
    virtual const Mp7JrsNodePtr GetBaseRoot() const;

private:
    XMLCh * m_measurementType;
    XMLCh * m_unit;
    XMLCh * m_value;



// no includefile for extension defined 
// file Mp7JrsExtentType_ExtMyPropDef.h


public:
    virtual void Init(); // Called by inheritees to recursively update m_myself
protected:
    void Cleanup();
};

#endif // _863MP7JRSEXTENTTYPE_H

