/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/


// ClassType_H.template
#ifndef _893MP7JRSFILTERINGTYPE_H
#define _893MP7JRSFILTERINGTYPE_H

#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOM.hpp>

#include "Mp7JrsDefines.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsFilteringTypeFactory.h"
#include "Mp7JrsNode.h"
#include "Mp7JrsConvert.h"


// no includefile for extension defined 
// file Mp7JrsFilteringType_ExtInclude.h


class IMp7JrsFilteringType;
class IMp7JrsFilterType;
class IMp7JrspaddingType;
class IMp7JrsSignalPlaneSampleType;
typedef Mp7JrsPtr< IMp7JrsFilteringType> Mp7JrsFilteringPtr;
typedef Mp7JrsPtr< IMp7JrsFilterType > Mp7JrsFilterPtr;
typedef Mp7JrsPtr< IMp7JrspaddingType > Mp7JrspaddingPtr;
typedef Mp7JrsPtr< IMp7JrsSignalPlaneSampleType > Mp7JrsSignalPlaneSamplePtr;

/** 
 * Generated interface IMp7JrsFilteringType for class Mp7JrsFilteringType<br>
 * Located at: mpeg7-2004.xsd, line 7375<br>
 * Classified: Class<br>
 */
class MP7JRS_EXPORT IMp7JrsFilteringType 
 :
        public Mp7JrsNode
{
public:
    // TODO: make these protected?
    IMp7JrsFilteringType();
    virtual ~IMp7JrsFilteringType();
    /**
     * Get xPad attribute.
     * @return Mp7JrspaddingPtr
     */
    virtual Mp7JrspaddingPtr GetxPad() const = 0;

    /**
     * Get xPad attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExistxPad() const = 0;

    /**
     * Set xPad attribute.
     * @param item const Mp7JrspaddingPtr &
     */
    virtual void SetxPad(const Mp7JrspaddingPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate xPad attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidatexPad() = 0;

    /**
     * Get yPad attribute.
     * @return Mp7JrspaddingPtr
     */
    virtual Mp7JrspaddingPtr GetyPad() const = 0;

    /**
     * Get yPad attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExistyPad() const = 0;

    /**
     * Set yPad attribute.
     * @param item const Mp7JrspaddingPtr &
     */
    virtual void SetyPad(const Mp7JrspaddingPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate yPad attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidateyPad() = 0;

    /**
     * Get zPad attribute.
     * @return Mp7JrspaddingPtr
     */
    virtual Mp7JrspaddingPtr GetzPad() const = 0;

    /**
     * Get zPad attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExistzPad() const = 0;

    /**
     * Set zPad attribute.
     * @param item const Mp7JrspaddingPtr &
     */
    virtual void SetzPad(const Mp7JrspaddingPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate zPad attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidatezPad() = 0;

    /**
     * Get tPad attribute.
     * @return Mp7JrspaddingPtr
     */
    virtual Mp7JrspaddingPtr GettPad() const = 0;

    /**
     * Get tPad attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExisttPad() const = 0;

    /**
     * Set tPad attribute.
     * @param item const Mp7JrspaddingPtr &
     */
    virtual void SettPad(const Mp7JrspaddingPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate tPad attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidatetPad() = 0;

        /** @name Sequence

         */
        //@{
    /**
     * Get Filter element.
     * @return Mp7JrsPtr< Mp7JrsNode >
     * Possible types (within this library):
     * <ul>
     * <li>IMp7JrsFilter1DPtr</li>
     * <li>IMp7JrsFilter2DPtr</li>
     * <li>IMp7JrsFilterSeparablePtr</li>
     * </ul>
     */
    virtual Mp7JrsPtr< Mp7JrsNode > GetFilter() const = 0;

    /**
     * Set Filter element.
     * @param item const Mp7JrsPtr< Mp7JrsNode > &
     * Valid types for item:
     * <ul>
     * <li>IMp7JrsFilter1DPtr</li>
     * <li>IMp7JrsFilter2DPtr</li>
     * <li>IMp7JrsFilterSeparablePtr</li>
     * </ul>
     */
    // Mandatory abstract element, possible types are
    // Mp7JrsFilter1DPtr
    // Mp7JrsFilter2DPtr
    // Mp7JrsFilterSeparablePtr
    virtual void SetFilter(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

    /**
     * Get PadSize element.
     * @return Mp7JrsSignalPlaneSamplePtr
     */
    virtual Mp7JrsSignalPlaneSamplePtr GetPadSize() const = 0;

    virtual bool IsValidPadSize() const = 0;

    /**
     * Set PadSize element.
     * @param item const Mp7JrsSignalPlaneSamplePtr &
     */
    virtual void SetPadSize(const Mp7JrsSignalPlaneSamplePtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate PadSize element.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidatePadSize() = 0;

    /**
     * Get Shift element.
     * @return Mp7JrsSignalPlaneSamplePtr
     */
    virtual Mp7JrsSignalPlaneSamplePtr GetShift() const = 0;

    virtual bool IsValidShift() const = 0;

    /**
     * Set Shift element.
     * @param item const Mp7JrsSignalPlaneSamplePtr &
     */
    virtual void SetShift(const Mp7JrsSignalPlaneSamplePtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate Shift element.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidateShift() = 0;

    /**
     * Get CropStart element.
     * @return Mp7JrsSignalPlaneSamplePtr
     */
    virtual Mp7JrsSignalPlaneSamplePtr GetCropStart() const = 0;

    virtual bool IsValidCropStart() const = 0;

    /**
     * Set CropStart element.
     * @param item const Mp7JrsSignalPlaneSamplePtr &
     */
    virtual void SetCropStart(const Mp7JrsSignalPlaneSamplePtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate CropStart element.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidateCropStart() = 0;

    /**
     * Get CropEnd element.
     * @return Mp7JrsSignalPlaneSamplePtr
     */
    virtual Mp7JrsSignalPlaneSamplePtr GetCropEnd() const = 0;

    virtual bool IsValidCropEnd() const = 0;

    /**
     * Set CropEnd element.
     * @param item const Mp7JrsSignalPlaneSamplePtr &
     */
    virtual void SetCropEnd(const Mp7JrsSignalPlaneSamplePtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate CropEnd element.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidateCropEnd() = 0;

        //@}
    /**
     * Gets or creates Mp7JrsNodePtr child elements of FilteringType.
     * Currently this type contains 5 child elements of type "Mp7JrsNodePtr".

     * <ul>
     * <li>Filter</li>
     * <li>PadSize</li>
     * <li>Shift</li>
     * <li>CropStart</li>
     * <li>CropEnd</li>
     * </ul>
     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     * \par<code>A</code> 
     * - Select all child elements named <code>A</code>.
     * - In case of no element, create a new one.
     * - For collections: all subsequent XPath expressions are applied on all collection items (e.g. <code>A/B</code> query or create, childnodes <code>B</code> of all <code>A</code>s). 
     * \par<code>A[i]</code>
     * - Select the <code>i</code>th child element of all <code>A</code>.
     * - Please note, that XPath indexes start at 1.
     * - If there is no such <code>i</code>th element, try to create a new one at index <code>i</code>.
     * - Single elements require <code>i</code> to be 1 otherwise an exception is thrown.
     * - For collections, <code>i</code> must not exceed the number of existing items +1 (appending is allowed, filling gaps not).
     * - If the index is too high, a <code>MissingElementException</code> is thrown, and you can extract the last valid index. (Example: If a collection contains 2 elements and you are requesting the 4th element).
     * 
     * \par <code>A[\@xsi:type="T"]</code>
     *     - Select all child elements named <code>A</code> of type <code>T</code>.
     *     - Currently only typenames without namespace are recognised.
     *     - If there is no such element, create and append a new one.
     * \par <code>A[i][\@xsi:type="T"]</code>
     *     - Select the <code>i</code>th child element named <code>A</code>, which must be of type <code>T</code>.
     *     - If there is no such element at index <code>i</code>, a new one is created and inserted (appended).
     * \par <code>A[\@xsi:type="T"][i]</code>
     *     - Out of all child elements named <code>A</code> of type <code>T</code>, take the <code>i</code>th one.
     *     - If there is no such element at index <code>i</code>, create and append one.
     * 
     * \par <code>A[last()]</code>
     *     - Select the last child element named <code>A</code>.
     *     - In case of no element, create a new one.
     * \par <code>A[last()][\@xsi:type="T"]</code>
     *     - Select the last child element named <code>A</code>, which must be of type <code>T</code>.
     *     - For collections: in case of no such type, create and append a new one.
     *     - For single elements: if an element with different type exists, return the empty list.
     * 
     * \par <code>A[\@xsi:type="T"][last()]</code>
     *     - Out of all child elements named <code>A</code> of type <code>T</code>, take the last one.
     *     - For collections: in case of no such type, create and append a new one.
     *     - For single elements: if an element with different type exists, return the empty list.
     * 
     * \par <code>A[position() \> last()]</code>
     *     - Force creation of a child element named <code>A</code> with its default type.
     *     - For collections: create and append the default element.
     *     - For single elements: if any element exists, an exception is thrown.
     *
     * \par <code>A[position() \> last()][\@xsi:type="T"]</code>
     *     - Force creation of a child element named <code>A</code> of type <code>T</code>.
     *     - For collections: create and append the element.
     *     - For single elements: if any element exists, an exception is thrown.
     *
     * \par <code>A[\@xsi:type="T"][position() \> last()]</code>
     *     - Force creation of a child element named <code>A</code> of type <code>T</code>.
     *     - Behaves the same way as the previous expression.
     * 
     * 
     * Supported XPath expressions for GetFromXPath<br>
     * The same expressions like GetOrCreateFromXPath plus
     * \par <code>A[position() \< i] </code>
     *     - Select all child elements named <code>A</code> at indexes lower than <code>i</code>.
     *     - <code>i</code> must be greater than -1, otherwise an exception is thrown.
     * \par <code>A[position() \> i] </code>
     *     - Select all child elements named <code>A</code> at indexes greater than <code>i</code>.
     *     - <code>i</code> must be greater than -1, otherwise an exception is thrown.
     * \par <code>A[position() \> i | position() \< j] </code>
     *     - Select all child elements named <code>A</code> at indexes greater than <code>i</code> and lower than <code>j</code>.
     *     - <code>i</code> and <code>j</code> must be greater than -1, otherwise an exception is thrown.
     *     - If <code>i</code> is greater than <code>j</code> then an empty list is returned.
     *
     * These three expressions can be combined with the <code>\@xsi:type</code> attribute to get a more restricted query.
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL) = 0;

    
// no includefile for extension defined 
// file Mp7JrsFilteringType_ExtMethodDef.h


// no includefile for extension defined 
// file Mp7JrsFilteringType_ExtPropDef.h

};




/*
 * Generated class Mp7JrsFilteringType<br>
 * Located at: mpeg7-2004.xsd, line 7375<br>
 * Classified: Class<br>
 * Defined in mpeg7-2004.xsd, line 7375.<br>
 */
class MP7JRS_EXPORT Mp7JrsFilteringType :
        public IMp7JrsFilteringType
{
    friend class Mp7JrsFilteringTypeFactory; // constructs objects

public:
    /*
     * Get xPad attribute.
     * @return Mp7JrspaddingPtr
     */
    virtual Mp7JrspaddingPtr GetxPad() const;

    /*
     * Get xPad attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExistxPad() const;

    /*
     * Set xPad attribute.
     * @param item const Mp7JrspaddingPtr &
     */
    virtual void SetxPad(const Mp7JrspaddingPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate xPad attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidatexPad();

    /*
     * Get yPad attribute.
     * @return Mp7JrspaddingPtr
     */
    virtual Mp7JrspaddingPtr GetyPad() const;

    /*
     * Get yPad attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExistyPad() const;

    /*
     * Set yPad attribute.
     * @param item const Mp7JrspaddingPtr &
     */
    virtual void SetyPad(const Mp7JrspaddingPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate yPad attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidateyPad();

    /*
     * Get zPad attribute.
     * @return Mp7JrspaddingPtr
     */
    virtual Mp7JrspaddingPtr GetzPad() const;

    /*
     * Get zPad attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExistzPad() const;

    /*
     * Set zPad attribute.
     * @param item const Mp7JrspaddingPtr &
     */
    virtual void SetzPad(const Mp7JrspaddingPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate zPad attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidatezPad();

    /*
     * Get tPad attribute.
     * @return Mp7JrspaddingPtr
     */
    virtual Mp7JrspaddingPtr GettPad() const;

    /*
     * Get tPad attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExisttPad() const;

    /*
     * Set tPad attribute.
     * @param item const Mp7JrspaddingPtr &
     */
    virtual void SettPad(const Mp7JrspaddingPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate tPad attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidatetPad();

        /* @name Sequence

         */
        //@{
    /*
     * Get Filter element.
     * @return Mp7JrsPtr< Mp7JrsNode >
     * Possible types (within this library):
     * <ul>
     * <li>IMp7JrsFilter1DPtr</li>
     * <li>IMp7JrsFilter2DPtr</li>
     * <li>IMp7JrsFilterSeparablePtr</li>
     * </ul>
     */
    virtual Mp7JrsPtr< Mp7JrsNode > GetFilter() const;

    /*
     * Set Filter element.
     * @param item const Mp7JrsPtr< Mp7JrsNode > &
     * Valid types for item:
     * <ul>
     * <li>IMp7JrsFilter1DPtr</li>
     * <li>IMp7JrsFilter2DPtr</li>
     * <li>IMp7JrsFilterSeparablePtr</li>
     * </ul>
     */
    // Mandatory abstract element, possible types are
    // Mp7JrsFilter1DPtr
    // Mp7JrsFilter2DPtr
    // Mp7JrsFilterSeparablePtr
    virtual void SetFilter(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

    /*
     * Get PadSize element.
     * @return Mp7JrsSignalPlaneSamplePtr
     */
    virtual Mp7JrsSignalPlaneSamplePtr GetPadSize() const;

    virtual bool IsValidPadSize() const;

    /*
     * Set PadSize element.
     * @param item const Mp7JrsSignalPlaneSamplePtr &
     */
    virtual void SetPadSize(const Mp7JrsSignalPlaneSamplePtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate PadSize element.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidatePadSize();

    /*
     * Get Shift element.
     * @return Mp7JrsSignalPlaneSamplePtr
     */
    virtual Mp7JrsSignalPlaneSamplePtr GetShift() const;

    virtual bool IsValidShift() const;

    /*
     * Set Shift element.
     * @param item const Mp7JrsSignalPlaneSamplePtr &
     */
    virtual void SetShift(const Mp7JrsSignalPlaneSamplePtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate Shift element.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidateShift();

    /*
     * Get CropStart element.
     * @return Mp7JrsSignalPlaneSamplePtr
     */
    virtual Mp7JrsSignalPlaneSamplePtr GetCropStart() const;

    virtual bool IsValidCropStart() const;

    /*
     * Set CropStart element.
     * @param item const Mp7JrsSignalPlaneSamplePtr &
     */
    virtual void SetCropStart(const Mp7JrsSignalPlaneSamplePtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate CropStart element.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidateCropStart();

    /*
     * Get CropEnd element.
     * @return Mp7JrsSignalPlaneSamplePtr
     */
    virtual Mp7JrsSignalPlaneSamplePtr GetCropEnd() const;

    virtual bool IsValidCropEnd() const;

    /*
     * Set CropEnd element.
     * @param item const Mp7JrsSignalPlaneSamplePtr &
     */
    virtual void SetCropEnd(const Mp7JrsSignalPlaneSamplePtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate CropEnd element.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidateCropEnd();

        //@}
    /*
     * Gets or creates Mp7JrsNodePtr child elements of FilteringType.
     * Currently this type contains 5 child elements of type "Mp7JrsNodePtr".

     * <ul>
     * <li>Filter</li>
     * <li>PadSize</li>
     * <li>Shift</li>
     * <li>CropStart</li>
     * <li>CropEnd</li>
     * </ul>
     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     * \par<code>A</code> 
     * - Select all child elements named <code>A</code>.
     * - In case of no element, create a new one.
     * - For collections: all subsequent XPath expressions are applied on all collection items (e.g. <code>A/B</code> query or create, childnodes <code>B</code> of all <code>A</code>s). 
     * \par<code>A[i]</code>
     * - Select the <code>i</code>th child element of all <code>A</code>.
     * - Please note, that XPath indexes start at 1.
     * - If there is no such <code>i</code>th element, try to create a new one at index <code>i</code>.
     * - Single elements require <code>i</code> to be 1 otherwise an exception is thrown.
     * - For collections, <code>i</code> must not exceed the number of existing items +1 (appending is allowed, filling gaps not).
     * - If the index is too high, a <code>MissingElementException</code> is thrown, and you can extract the last valid index. (Example: If a collection contains 2 elements and you are requesting the 4th element).
     * 
     * \par <code>A[\@xsi:type="T"]</code>
     *     - Select all child elements named <code>A</code> of type <code>T</code>.
     *     - Currently only typenames without namespace are recognised.
     *     - If there is no such element, create and append a new one.
     * \par <code>A[i][\@xsi:type="T"]</code>
     *     - Select the <code>i</code>th child element named <code>A</code>, which must be of type <code>T</code>.
     *     - If there is no such element at index <code>i</code>, a new one is created and inserted (appended).
     * \par <code>A[\@xsi:type="T"][i]</code>
     *     - Out of all child elements named <code>A</code> of type <code>T</code>, take the <code>i</code>th one.
     *     - If there is no such element at index <code>i</code>, create and append one.
     * 
     * \par <code>A[last()]</code>
     *     - Select the last child element named <code>A</code>.
     *     - In case of no element, create a new one.
     * \par <code>A[last()][\@xsi:type="T"]</code>
     *     - Select the last child element named <code>A</code>, which must be of type <code>T</code>.
     *     - For collections: in case of no such type, create and append a new one.
     *     - For single elements: if an element with different type exists, return the empty list.
     * 
     * \par <code>A[\@xsi:type="T"][last()]</code>
     *     - Out of all child elements named <code>A</code> of type <code>T</code>, take the last one.
     *     - For collections: in case of no such type, create and append a new one.
     *     - For single elements: if an element with different type exists, return the empty list.
     * 
     * \par <code>A[position() \> last()]</code>
     *     - Force creation of a child element named <code>A</code> with its default type.
     *     - For collections: create and append the default element.
     *     - For single elements: if any element exists, an exception is thrown.
     *
     * \par <code>A[position() \> last()][\@xsi:type="T"]</code>
     *     - Force creation of a child element named <code>A</code> of type <code>T</code>.
     *     - For collections: create and append the element.
     *     - For single elements: if any element exists, an exception is thrown.
     *
     * \par <code>A[\@xsi:type="T"][position() \> last()]</code>
     *     - Force creation of a child element named <code>A</code> of type <code>T</code>.
     *     - Behaves the same way as the previous expression.
     * 
     * 
     * Supported XPath expressions for GetFromXPath<br>
     * The same expressions like GetOrCreateFromXPath plus
     * \par <code>A[position() \< i] </code>
     *     - Select all child elements named <code>A</code> at indexes lower than <code>i</code>.
     *     - <code>i</code> must be greater than -1, otherwise an exception is thrown.
     * \par <code>A[position() \> i] </code>
     *     - Select all child elements named <code>A</code> at indexes greater than <code>i</code>.
     *     - <code>i</code> must be greater than -1, otherwise an exception is thrown.
     * \par <code>A[position() \> i | position() \< j] </code>
     *     - Select all child elements named <code>A</code> at indexes greater than <code>i</code> and lower than <code>j</code>.
     *     - <code>i</code> and <code>j</code> must be greater than -1, otherwise an exception is thrown.
     *     - If <code>i</code> is greater than <code>j</code> then an empty list is returned.
     *
     * These three expressions can be combined with the <code>\@xsi:type</code> attribute to get a more restricted query.
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL);

    

    /* Get the class name.
     * @return The name of this class.
     */
    virtual XMLCh * ToText() const;

    /* Parse this type.
     * @param txt buffer to parse from.
     * @return true if parsing was successful, false otherwise.
     */
    virtual bool Parse(const XMLCh * const txt);

    /* Parse the content from a string.  Note, this is purely for ComplexTypes with SimpleContent and SimpleTypes.
     * @param txt buffer to parse from.
     * @return true if parsing was successful, false otherwise.
     */
	virtual bool ContentFromString(const XMLCh * const txt);

    /* Convert the content to string.  Note, this is purely for ComplexTypes with SimpleContent and SimpleTypes. All other types may return NULL or the class name.
     * @return the allocated XMLCh * if conversion was successful. The caller must free memory using Mp7JrsUtil.deallocate().
     */
	virtual XMLCh * ContentToString() const;


    /* Serialialize this object to a DOM tree.
     * @param doc DOM document.
     * @param parent Parent node in DOM tree.
     */
    virtual void Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem = NULL);

    /* Deserialize this object from a DOM tree.
     * @param doc DOM document.
     * @param parent Parent element in DOM tree.
     * @param current Current element in DOM tree.
     */
    virtual bool Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current);

    /* Creates and inserts a Childnode of the specified Type
     * @param type The tag type of the child to be created (tag name).
     * @param xsitype The xsi type of the child to be created if available.
     * @return the created child with the correct dynamic type for the specified type.
     */
    virtual Mp7JrsNodePtr CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype);
         
// no includefile for extension defined 
// file Mp7JrsFilteringType_ExtMyMethodDef.h


public:

    virtual ~Mp7JrsFilteringType();

protected:
    Mp7JrsFilteringType();
    virtual void DeepCopy(const Mp7JrsNodePtr &original);

    virtual Mp7JrsNodeEnum GetAllChildElements() const;

    virtual Mp7JrsNodePtr GetBaseRoot();
    virtual const Mp7JrsNodePtr GetBaseRoot() const;

private:
    Mp7JrspaddingPtr m_xPad;
    bool m_xPad_Exist;
    Mp7JrspaddingPtr m_xPad_Default;
    Mp7JrspaddingPtr m_yPad;
    bool m_yPad_Exist;
    Mp7JrspaddingPtr m_yPad_Default;
    Mp7JrspaddingPtr m_zPad;
    bool m_zPad_Exist;
    Mp7JrspaddingPtr m_zPad_Default;
    Mp7JrspaddingPtr m_tPad;
    bool m_tPad_Exist;
    Mp7JrspaddingPtr m_tPad_Default;


    Mp7JrsPtr< Mp7JrsNode > m_Filter;
    Mp7JrsSignalPlaneSamplePtr m_PadSize;
    bool m_PadSize_Exist; // For optional elements 
    Mp7JrsSignalPlaneSamplePtr m_Shift;
    bool m_Shift_Exist; // For optional elements 
    Mp7JrsSignalPlaneSamplePtr m_CropStart;
    bool m_CropStart_Exist; // For optional elements 
    Mp7JrsSignalPlaneSamplePtr m_CropEnd;
    bool m_CropEnd_Exist; // For optional elements 

// no includefile for extension defined 
// file Mp7JrsFilteringType_ExtMyPropDef.h


public:
    virtual void Init(); // Called by inheritees to recursively update m_myself
protected:
    void Cleanup();
};

#endif // _893MP7JRSFILTERINGTYPE_H

