/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/


// ClassType_H.template
#ifndef _1769MP7JRSREGIONLOCATORTYPE_BOX_LOCALTYPE_H
#define _1769MP7JRSREGIONLOCATORTYPE_BOX_LOCALTYPE_H

#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOM.hpp>

#include "Mp7JrsDefines.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsRegionLocatorType_Box_LocalTypeFactory.h"
#include "Mp7JrsNode.h"
#include "Mp7JrsConvert.h"


// no includefile for extension defined 
// file Mp7JrsRegionLocatorType_Box_LocalType_ExtInclude.h


#include "Mp7JrsBoxListType.h"
#include "Mp7JrsIntegerMatrixType.h"
#include "Mp7JrsintegerVector.h"
class IMp7JrsBoxListType;
class IMp7Jrsdim_LocalType;
class IMp7JrsRegionLocatorType_Box_LocalType;
typedef Mp7JrsPtr< IMp7JrsBoxListType > Mp7JrsBoxListPtr;
typedef Mp7JrsPtr< IMp7Jrsdim_LocalType > Mp7Jrsdim_LocalPtr;
typedef Mp7JrsPtr< IMp7JrsRegionLocatorType_Box_LocalType> Mp7JrsRegionLocatorType_Box_LocalPtr;

/** 
 * Generated interface IMp7JrsRegionLocatorType_Box_LocalType for class Mp7JrsRegionLocatorType_Box_LocalType<br>
 * Located at: mpeg7-2004.xsd, line 1172<br>
 * Classified: Class<br>
 * Derived from: BoxListType (Class)<br>
 */
class MP7JRS_EXPORT IMp7JrsRegionLocatorType_Box_LocalType 
 :
        public IMp7JrsBoxListType
{
public:
    // TODO: make these protected?
    IMp7JrsRegionLocatorType_Box_LocalType();
    virtual ~IMp7JrsRegionLocatorType_Box_LocalType();
    /**
     * Retrieves the base collection of Mp7JrsIntegerMatrixType
     * (Inherited from Mp7JrsIntegerMatrixType)
     
     * @return Mp7JrsPtr< Mp7JrsintegerVector >
     */
    virtual Mp7JrsPtr< Mp7JrsintegerVector > GetBaseCollection() const = 0;


    /**
     * Initializes the collection.
     * (Inherited from Mp7JrsintegerVector)
     * @param maxElems Number of elements for which memory is initially allocated.
     */
    virtual void Initialize(unsigned int maxElems) = 0;


    /**
     * Adds a new element to the collection.
     * (Inherited from Mp7JrsintegerVector)
     * @param toAdd Element to be added. Default element type: integer
     */
    virtual void addElement(const int &toAdd, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Sets the element at the given position in the collection.
     * (Inherited from Mp7JrsintegerVector)
     * @param toSet New value of the element. Default element type: integer
     * @param setAt Position of the element which shall be set.
     */
    virtual void setElementAt(const int &toSet, unsigned int setAt, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Inserts the element at the given position in the collection.
     * (Inherited from Mp7JrsintegerVector)
     * @param toInsert Element to be inserted. Default element type: integer
     * @param insertAt Position where the element will be inserted.
     */
    virtual void insertElementAt(const int &toInsert, unsigned int insertAt, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Removes all elements from the collection.
     * (Inherited from Mp7JrsintegerVector)
     */
    virtual void removeAllElements(Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Removes the specified element from the collection.
     * (Inherited from Mp7JrsintegerVector)
     * @param removeAt Position of the element to be removed. 
     */
    virtual void removeElementAt(unsigned int removeAt, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;



    /**
     * Check whether the collection contains the given element.
     * (Inherited from Mp7JrsintegerVector)
     * @param toCheck Element to be checked for. Default element type: integer
     * @return True if the element is in the collection, false otherwise.
     */
    virtual bool containsElement(const int &toCheck) = 0;


    /**
     * Get the current capacity of the collection.
     * (Inherited from Mp7JrsintegerVector)
     * @return The number of elements for which memory has been allocated.
     */
    virtual unsigned int curCapacity() const = 0;


    /**
     * Gets a constant pointer to the element at the given position.
     * (Inherited from Mp7JrsintegerVector)
     * @param Position of the element to be retrieved.
     * @return A pointer to the given element or a null pointer if there is no element at the specified position. Default element type: integer.
     */
    virtual const int elementAt(unsigned int getAt) const = 0;


    /**
     * Gets a pointer to the element at the given position.
     * (Inherited from Mp7JrsintegerVector)
     * @param Position of the element to be retrieved.
     * @return A pointer to the given element or a null pointer if there is no element at the specified position. Default element type: integer.
     */
    virtual int elementAt(unsigned int getAt) = 0;


    /**
     * Get the size of the collection.
     * (Inherited from Mp7JrsintegerVector)
     * @return The current number of elements in the collection.
     */
    virtual unsigned int size() const = 0;


    /**
     * Allocate additional memory for the speciifed number of elements.
     * (Inherited from Mp7JrsintegerVector)
     * @param length Number of elements to allocate memory for.
     */
    virtual void ensureExtraCapacity(unsigned int length) = 0;


    // JRS: addition to Xerces collection
    /**
     * Get the position of the specified element. 
     * (Inherited from Mp7JrsintegerVector)
     * @param toCheck Element to be found in the collection. Default element type: integer
     * @return The position of the specified element or -1 if the element has not been found.
     */
    virtual int elementIndexOf(const int &toCheck) const = 0;


    /**
     * Get unlocatedRegion attribute.
     * @return bool
     */
    virtual bool GetunlocatedRegion() const = 0;

    /**
     * Get unlocatedRegion attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExistunlocatedRegion() const = 0;

    /**
     * Set unlocatedRegion attribute.
     * @param item bool
     */
    virtual void SetunlocatedRegion(bool item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate unlocatedRegion attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidateunlocatedRegion() = 0;

    /**
     * Get dim attribute.
<br>
     * (Inherited from Mp7JrsIntegerMatrixType)
     * @return Mp7Jrsdim_LocalPtr
     */
    virtual Mp7Jrsdim_LocalPtr Getdim() const = 0;

    /**
     * Set dim attribute.
<br>
     * (Inherited from Mp7JrsIntegerMatrixType)
     * @param item const Mp7Jrsdim_LocalPtr &
     */
    // Mandatory
    virtual void Setdim(const Mp7Jrsdim_LocalPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

    /**
     * Gets or creates Mp7JrsNodePtr child elements of RegionLocatorType_Box_LocalType.
     * Currently just supports rudimentary XPath expressions as RegionLocatorType_Box_LocalType contains no child elements of type "Mp7JrsNodePtr" at all.

     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL) = 0;

    
// no includefile for extension defined 
// file Mp7JrsRegionLocatorType_Box_LocalType_ExtMethodDef.h


// no includefile for extension defined 
// file Mp7JrsRegionLocatorType_Box_LocalType_ExtPropDef.h

};




/*
 * Generated class Mp7JrsRegionLocatorType_Box_LocalType<br>
 * Located at: mpeg7-2004.xsd, line 1172<br>
 * Classified: Class<br>
 * Derived from: BoxListType (Class)<br>
 * Defined in mpeg7-2004.xsd, line 1172.<br>
 */
class MP7JRS_EXPORT Mp7JrsRegionLocatorType_Box_LocalType :
        public IMp7JrsRegionLocatorType_Box_LocalType
{
    friend class Mp7JrsRegionLocatorType_Box_LocalTypeFactory; // constructs objects

public:
    /*
     * Retrieves the base class of Mp7JrsRegionLocatorType_Box_LocalType
     
     * @return Mp7JrsPtr< Mp7JrsBoxListType >
     */
    virtual Mp7JrsPtr< Mp7JrsBoxListType > GetBase() const;
    /*
     * Retrieves the base collection of Mp7JrsIntegerMatrixType
     * (Inherited from Mp7JrsIntegerMatrixType)
     
     * @return Mp7JrsPtr< Mp7JrsintegerVector >
     */
    virtual Mp7JrsPtr< Mp7JrsintegerVector > GetBaseCollection() const;


    /*
     * Initializes the collection.
     * (Inherited from Mp7JrsintegerVector)
     * @param maxElems Number of elements for which memory is initially allocated.
     */
    virtual void Initialize(unsigned int maxElems);


    /*
     * Adds a new element to the collection.
     * (Inherited from Mp7JrsintegerVector)
     * @param toAdd Element to be added. Default element type: integer
     */
    virtual void addElement(const int &toAdd, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Sets the element at the given position in the collection.
     * (Inherited from Mp7JrsintegerVector)
     * @param toSet New value of the element. Default element type: integer
     * @param setAt Position of the element which shall be set.
     */
    virtual void setElementAt(const int &toSet, unsigned int setAt, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Inserts the element at the given position in the collection.
     * (Inherited from Mp7JrsintegerVector)
     * @param toInsert Element to be inserted. Default element type: integer
     * @param insertAt Position where the element will be inserted.
     */
    virtual void insertElementAt(const int &toInsert, unsigned int insertAt, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Removes all elements from the collection.
     * (Inherited from Mp7JrsintegerVector)
     */
    virtual void removeAllElements(Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Removes the specified element from the collection.
     * (Inherited from Mp7JrsintegerVector)
     * @param removeAt Position of the element to be removed. 
     */
    virtual void removeElementAt(unsigned int removeAt, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);



    /*
     * Check whether the collection contains the given element.
     * (Inherited from Mp7JrsintegerVector)
     * @param toCheck Element to be checked for. Default element type: integer
     * @return True if the element is in the collection, false otherwise.
     */
    virtual bool containsElement(const int &toCheck);


    /*
     * Get the current capacity of the collection.
     * (Inherited from Mp7JrsintegerVector)
     * @return The number of elements for which memory has been allocated.
     */
    virtual unsigned int curCapacity() const;


    /*
     * Gets a constant pointer to the element at the given position.
     * (Inherited from Mp7JrsintegerVector)
     * @param Position of the element to be retrieved.
     * @return A pointer to the given element or a null pointer if there is no element at the specified position. Default element type: integer.
     */
    virtual const int elementAt(unsigned int getAt) const;


    /*
     * Gets a pointer to the element at the given position.
     * (Inherited from Mp7JrsintegerVector)
     * @param Position of the element to be retrieved.
     * @return A pointer to the given element or a null pointer if there is no element at the specified position. Default element type: integer.
     */
    virtual int elementAt(unsigned int getAt);


    /*
     * Get the size of the collection.
     * (Inherited from Mp7JrsintegerVector)
     * @return The current number of elements in the collection.
     */
    virtual unsigned int size() const;


    /*
     * Allocate additional memory for the speciifed number of elements.
     * (Inherited from Mp7JrsintegerVector)
     * @param length Number of elements to allocate memory for.
     */
    virtual void ensureExtraCapacity(unsigned int length);


    // JRS: addition to Xerces collection
    /*
     * Get the position of the specified element. 
     * (Inherited from Mp7JrsintegerVector)
     * @param toCheck Element to be found in the collection. Default element type: integer
     * @return The position of the specified element or -1 if the element has not been found.
     */
    virtual int elementIndexOf(const int &toCheck) const;


    /*
     * Get unlocatedRegion attribute.
     * @return bool
     */
    virtual bool GetunlocatedRegion() const;

    /*
     * Get unlocatedRegion attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExistunlocatedRegion() const;

    /*
     * Set unlocatedRegion attribute.
     * @param item bool
     */
    virtual void SetunlocatedRegion(bool item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate unlocatedRegion attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidateunlocatedRegion();

    /*
     * Get dim attribute.
<br>
     * (Inherited from Mp7JrsIntegerMatrixType)
     * @return Mp7Jrsdim_LocalPtr
     */
    virtual Mp7Jrsdim_LocalPtr Getdim() const;

    /*
     * Set dim attribute.
<br>
     * (Inherited from Mp7JrsIntegerMatrixType)
     * @param item const Mp7Jrsdim_LocalPtr &
     */
    // Mandatory
    virtual void Setdim(const Mp7Jrsdim_LocalPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

    /*
     * Gets or creates Mp7JrsNodePtr child elements of RegionLocatorType_Box_LocalType.
     * Currently just supports rudimentary XPath expressions as RegionLocatorType_Box_LocalType contains no child elements of type "Mp7JrsNodePtr" at all.

     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL);

    

    /* Get the class name.
     * @return The name of this class.
     */
    virtual XMLCh * ToText() const;

    /* Parse this type.
     * @param txt buffer to parse from.
     * @return true if parsing was successful, false otherwise.
     */
    virtual bool Parse(const XMLCh * const txt);

    /* Parse the content from a string.  Note, this is purely for ComplexTypes with SimpleContent and SimpleTypes.
     * @param txt buffer to parse from.
     * @return true if parsing was successful, false otherwise.
     */
	virtual bool ContentFromString(const XMLCh * const txt);

    /* Convert the content to string.  Note, this is purely for ComplexTypes with SimpleContent and SimpleTypes. All other types may return NULL or the class name.
     * @return the allocated XMLCh * if conversion was successful. The caller must free memory using Mp7JrsUtil.deallocate().
     */
	virtual XMLCh * ContentToString() const;


    /* Serialialize this object to a DOM tree.
     * @param doc DOM document.
     * @param parent Parent node in DOM tree.
     */
    virtual void Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem = NULL);

    /* Deserialize this object from a DOM tree.
     * @param doc DOM document.
     * @param parent Parent element in DOM tree.
     * @param current Current element in DOM tree.
     */
    virtual bool Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current);

    /* Creates and inserts a Childnode of the specified Type
     * @param type The tag type of the child to be created (tag name).
     * @param xsitype The xsi type of the child to be created if available.
     * @return the created child with the correct dynamic type for the specified type.
     */
    virtual Mp7JrsNodePtr CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype);
         
// no includefile for extension defined 
// file Mp7JrsRegionLocatorType_Box_LocalType_ExtMyMethodDef.h


public:

    virtual ~Mp7JrsRegionLocatorType_Box_LocalType();

protected:
    Mp7JrsRegionLocatorType_Box_LocalType();
    virtual void DeepCopy(const Mp7JrsNodePtr &original);

    virtual Mp7JrsNodeEnum GetAllChildElements() const;

    virtual Mp7JrsNodePtr GetBaseRoot();
    virtual const Mp7JrsNodePtr GetBaseRoot() const;

private:
    bool m_unlocatedRegion;
    bool m_unlocatedRegion_Exist;
    bool m_unlocatedRegion_Default;

    // Base Class
    Mp7JrsPtr< Mp7JrsBoxListType > m_Base;


// no includefile for extension defined 
// file Mp7JrsRegionLocatorType_Box_LocalType_ExtMyPropDef.h


public:
    virtual void Init(); // Called by inheritees to recursively update m_myself
protected:
    void Cleanup();
};

#endif // _1769MP7JRSREGIONLOCATORTYPE_BOX_LOCALTYPE_H

