/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/


// ClassType_H.template
#ifndef _2187MP7JRSSPOKENCONTENTLINKTYPE_H
#define _2187MP7JRSSPOKENCONTENTLINKTYPE_H

#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOM.hpp>

#include "Mp7JrsDefines.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsSpokenContentLinkTypeFactory.h"
#include "Mp7JrsNode.h"
#include "Mp7JrsConvert.h"


// no includefile for extension defined 
// file Mp7JrsSpokenContentLinkType_ExtInclude.h


class IMp7JrsnonNegativeReal;
class IMp7JrsSpokenContentLinkType;
class IMp7Jrsunsigned16;
class IMp7JrszeroToOneType;
typedef Mp7JrsPtr< IMp7JrsnonNegativeReal > Mp7JrsnonNegativeRealPtr;
typedef Mp7JrsPtr< IMp7JrsSpokenContentLinkType> Mp7JrsSpokenContentLinkPtr;
typedef Mp7JrsPtr< IMp7Jrsunsigned16 > Mp7Jrsunsigned16Ptr;
typedef Mp7JrsPtr< IMp7JrszeroToOneType > Mp7JrszeroToOnePtr;

/** 
 * Generated interface IMp7JrsSpokenContentLinkType for class Mp7JrsSpokenContentLinkType<br>
 * Located at: mpeg7-2004.xsd, line 2183<br>
 * Classified: Class<br>
 */
class MP7JRS_EXPORT IMp7JrsSpokenContentLinkType 
 :
        public Mp7JrsNode
{
public:
    // TODO: make these protected?
    IMp7JrsSpokenContentLinkType();
    virtual ~IMp7JrsSpokenContentLinkType();
    /**
     * Get probability attribute.
     * @return Mp7JrszeroToOnePtr
     */
    virtual Mp7JrszeroToOnePtr Getprobability() const = 0;

    /**
     * Get probability attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool Existprobability() const = 0;

    /**
     * Set probability attribute.
     * @param item const Mp7JrszeroToOnePtr &
     */
    virtual void Setprobability(const Mp7JrszeroToOnePtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate probability attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void Invalidateprobability() = 0;

    /**
     * Get acousticScore attribute.
     * @return Mp7JrsnonNegativeRealPtr
     */
    virtual Mp7JrsnonNegativeRealPtr GetacousticScore() const = 0;

    /**
     * Get acousticScore attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExistacousticScore() const = 0;

    /**
     * Set acousticScore attribute.
     * @param item const Mp7JrsnonNegativeRealPtr &
     */
    virtual void SetacousticScore(const Mp7JrsnonNegativeRealPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate acousticScore attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidateacousticScore() = 0;

    /**
     * Get nodeOffset attribute.
     * @return Mp7Jrsunsigned16Ptr
     */
    virtual Mp7Jrsunsigned16Ptr GetnodeOffset() const = 0;

    /**
     * Get nodeOffset attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExistnodeOffset() const = 0;

    /**
     * Set nodeOffset attribute.
     * @param item const Mp7Jrsunsigned16Ptr &
     */
    virtual void SetnodeOffset(const Mp7Jrsunsigned16Ptr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate nodeOffset attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidatenodeOffset() = 0;

    /**
     * Gets or creates Mp7JrsNodePtr child elements of SpokenContentLinkType.
     * Currently just supports rudimentary XPath expressions as SpokenContentLinkType contains no child elements of type "Mp7JrsNodePtr" at all.

     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL) = 0;

    
// no includefile for extension defined 
// file Mp7JrsSpokenContentLinkType_ExtMethodDef.h


// no includefile for extension defined 
// file Mp7JrsSpokenContentLinkType_ExtPropDef.h

};




/*
 * Generated class Mp7JrsSpokenContentLinkType<br>
 * Located at: mpeg7-2004.xsd, line 2183<br>
 * Classified: Class<br>
 * Defined in mpeg7-2004.xsd, line 2183.<br>
 */
class MP7JRS_EXPORT Mp7JrsSpokenContentLinkType :
        public IMp7JrsSpokenContentLinkType
{
    friend class Mp7JrsSpokenContentLinkTypeFactory; // constructs objects

public:
    /*
     * Get probability attribute.
     * @return Mp7JrszeroToOnePtr
     */
    virtual Mp7JrszeroToOnePtr Getprobability() const;

    /*
     * Get probability attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool Existprobability() const;

    /*
     * Set probability attribute.
     * @param item const Mp7JrszeroToOnePtr &
     */
    virtual void Setprobability(const Mp7JrszeroToOnePtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate probability attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void Invalidateprobability();

    /*
     * Get acousticScore attribute.
     * @return Mp7JrsnonNegativeRealPtr
     */
    virtual Mp7JrsnonNegativeRealPtr GetacousticScore() const;

    /*
     * Get acousticScore attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExistacousticScore() const;

    /*
     * Set acousticScore attribute.
     * @param item const Mp7JrsnonNegativeRealPtr &
     */
    virtual void SetacousticScore(const Mp7JrsnonNegativeRealPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate acousticScore attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidateacousticScore();

    /*
     * Get nodeOffset attribute.
     * @return Mp7Jrsunsigned16Ptr
     */
    virtual Mp7Jrsunsigned16Ptr GetnodeOffset() const;

    /*
     * Get nodeOffset attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExistnodeOffset() const;

    /*
     * Set nodeOffset attribute.
     * @param item const Mp7Jrsunsigned16Ptr &
     */
    virtual void SetnodeOffset(const Mp7Jrsunsigned16Ptr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate nodeOffset attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidatenodeOffset();

    /*
     * Gets or creates Mp7JrsNodePtr child elements of SpokenContentLinkType.
     * Currently just supports rudimentary XPath expressions as SpokenContentLinkType contains no child elements of type "Mp7JrsNodePtr" at all.

     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL);

    

    /* Get the class name.
     * @return The name of this class.
     */
    virtual XMLCh * ToText() const;

    /* Parse this type.
     * @param txt buffer to parse from.
     * @return true if parsing was successful, false otherwise.
     */
    virtual bool Parse(const XMLCh * const txt);

    /* Parse the content from a string.  Note, this is purely for ComplexTypes with SimpleContent and SimpleTypes.
     * @param txt buffer to parse from.
     * @return true if parsing was successful, false otherwise.
     */
	virtual bool ContentFromString(const XMLCh * const txt);

    /* Convert the content to string.  Note, this is purely for ComplexTypes with SimpleContent and SimpleTypes. All other types may return NULL or the class name.
     * @return the allocated XMLCh * if conversion was successful. The caller must free memory using Mp7JrsUtil.deallocate().
     */
	virtual XMLCh * ContentToString() const;


    /* Serialialize this object to a DOM tree.
     * @param doc DOM document.
     * @param parent Parent node in DOM tree.
     */
    virtual void Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem = NULL);

    /* Deserialize this object from a DOM tree.
     * @param doc DOM document.
     * @param parent Parent element in DOM tree.
     * @param current Current element in DOM tree.
     */
    virtual bool Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current);

    /* Creates and inserts a Childnode of the specified Type
     * @param type The tag type of the child to be created (tag name).
     * @param xsitype The xsi type of the child to be created if available.
     * @return the created child with the correct dynamic type for the specified type.
     */
    virtual Mp7JrsNodePtr CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype);
         
// no includefile for extension defined 
// file Mp7JrsSpokenContentLinkType_ExtMyMethodDef.h


public:

    virtual ~Mp7JrsSpokenContentLinkType();

protected:
    Mp7JrsSpokenContentLinkType();
    virtual void DeepCopy(const Mp7JrsNodePtr &original);

    virtual Mp7JrsNodeEnum GetAllChildElements() const;

    virtual Mp7JrsNodePtr GetBaseRoot();
    virtual const Mp7JrsNodePtr GetBaseRoot() const;

private:
    Mp7JrszeroToOnePtr m_probability;
    bool m_probability_Exist;
    Mp7JrszeroToOnePtr m_probability_Default;
    Mp7JrsnonNegativeRealPtr m_acousticScore;
    bool m_acousticScore_Exist;
    Mp7Jrsunsigned16Ptr m_nodeOffset;
    bool m_nodeOffset_Exist;
    Mp7Jrsunsigned16Ptr m_nodeOffset_Default;



// no includefile for extension defined 
// file Mp7JrsSpokenContentLinkType_ExtMyPropDef.h


public:
    virtual void Init(); // Called by inheritees to recursively update m_myself
protected:
    void Cleanup();
};

#endif // _2187MP7JRSSPOKENCONTENTLINKTYPE_H

