/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/


// ClassType_H.template
#ifndef _2229MP7JRSSTREAMSECTIONTYPE_H
#define _2229MP7JRSSTREAMSECTIONTYPE_H

#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOM.hpp>

#include "Mp7JrsDefines.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsStreamSectionTypeFactory.h"
#include "Mp7JrsNode.h"
#include "Mp7JrsConvert.h"


// no includefile for extension defined 
// file Mp7JrsStreamSectionType_ExtInclude.h


#include "Mp7JrsunitType.h"
class IMp7JrsStreamSectionType;
typedef Mp7JrsPtr< IMp7JrsStreamSectionType> Mp7JrsStreamSectionPtr;

/** 
 * Generated interface IMp7JrsStreamSectionType for class Mp7JrsStreamSectionType<br>
 * Located at: mpeg7-2004.xsd, line 9158<br>
 * Classified: Class<br>
 */
class MP7JRS_EXPORT IMp7JrsStreamSectionType 
 :
        public Mp7JrsNode
{
public:
    // TODO: make these protected?
    IMp7JrsStreamSectionType();
    virtual ~IMp7JrsStreamSectionType();
    /**
     * Get unit attribute.
     * @return Mp7JrsunitType::Enumeration
     */
    virtual Mp7JrsunitType::Enumeration Getunit() const = 0;

    /**
     * Get unit attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool Existunit() const = 0;

    /**
     * Set unit attribute.
     * @param item Mp7JrsunitType::Enumeration
     */
    virtual void Setunit(Mp7JrsunitType::Enumeration item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate unit attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void Invalidateunit() = 0;

    /**
     * Get start attribute.
     * @return unsigned
     */
    virtual unsigned Getstart() const = 0;

    /**
     * Get start attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool Existstart() const = 0;

    /**
     * Set start attribute.
     * @param item unsigned
     */
    virtual void Setstart(unsigned item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate start attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void Invalidatestart() = 0;

    /**
     * Get length attribute.
     * @return unsigned
     */
    virtual unsigned Getlength() const = 0;

    /**
     * Get length attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool Existlength() const = 0;

    /**
     * Set length attribute.
     * @param item unsigned
     */
    virtual void Setlength(unsigned item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate length attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void Invalidatelength() = 0;

    /**
     * Gets or creates Mp7JrsNodePtr child elements of StreamSectionType.
     * Currently just supports rudimentary XPath expressions as StreamSectionType contains no child elements of type "Mp7JrsNodePtr" at all.

     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL) = 0;

    
// no includefile for extension defined 
// file Mp7JrsStreamSectionType_ExtMethodDef.h


// no includefile for extension defined 
// file Mp7JrsStreamSectionType_ExtPropDef.h

};




/*
 * Generated class Mp7JrsStreamSectionType<br>
 * Located at: mpeg7-2004.xsd, line 9158<br>
 * Classified: Class<br>
 * Defined in mpeg7-2004.xsd, line 9158.<br>
 */
class MP7JRS_EXPORT Mp7JrsStreamSectionType :
        public IMp7JrsStreamSectionType
{
    friend class Mp7JrsStreamSectionTypeFactory; // constructs objects

public:
    /*
     * Get unit attribute.
     * @return Mp7JrsunitType::Enumeration
     */
    virtual Mp7JrsunitType::Enumeration Getunit() const;

    /*
     * Get unit attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool Existunit() const;

    /*
     * Set unit attribute.
     * @param item Mp7JrsunitType::Enumeration
     */
    virtual void Setunit(Mp7JrsunitType::Enumeration item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate unit attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void Invalidateunit();

    /*
     * Get start attribute.
     * @return unsigned
     */
    virtual unsigned Getstart() const;

    /*
     * Get start attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool Existstart() const;

    /*
     * Set start attribute.
     * @param item unsigned
     */
    virtual void Setstart(unsigned item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate start attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void Invalidatestart();

    /*
     * Get length attribute.
     * @return unsigned
     */
    virtual unsigned Getlength() const;

    /*
     * Get length attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool Existlength() const;

    /*
     * Set length attribute.
     * @param item unsigned
     */
    virtual void Setlength(unsigned item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate length attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void Invalidatelength();

    /*
     * Gets or creates Mp7JrsNodePtr child elements of StreamSectionType.
     * Currently just supports rudimentary XPath expressions as StreamSectionType contains no child elements of type "Mp7JrsNodePtr" at all.

     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL);

    

    /* Get the class name.
     * @return The name of this class.
     */
    virtual XMLCh * ToText() const;

    /* Parse this type.
     * @param txt buffer to parse from.
     * @return true if parsing was successful, false otherwise.
     */
    virtual bool Parse(const XMLCh * const txt);

    /* Parse the content from a string.  Note, this is purely for ComplexTypes with SimpleContent and SimpleTypes.
     * @param txt buffer to parse from.
     * @return true if parsing was successful, false otherwise.
     */
	virtual bool ContentFromString(const XMLCh * const txt);

    /* Convert the content to string.  Note, this is purely for ComplexTypes with SimpleContent and SimpleTypes. All other types may return NULL or the class name.
     * @return the allocated XMLCh * if conversion was successful. The caller must free memory using Mp7JrsUtil.deallocate().
     */
	virtual XMLCh * ContentToString() const;


    /* Serialialize this object to a DOM tree.
     * @param doc DOM document.
     * @param parent Parent node in DOM tree.
     */
    virtual void Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem = NULL);

    /* Deserialize this object from a DOM tree.
     * @param doc DOM document.
     * @param parent Parent element in DOM tree.
     * @param current Current element in DOM tree.
     */
    virtual bool Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current);

    /* Creates and inserts a Childnode of the specified Type
     * @param type The tag type of the child to be created (tag name).
     * @param xsitype The xsi type of the child to be created if available.
     * @return the created child with the correct dynamic type for the specified type.
     */
    virtual Mp7JrsNodePtr CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype);
         
// no includefile for extension defined 
// file Mp7JrsStreamSectionType_ExtMyMethodDef.h


public:

    virtual ~Mp7JrsStreamSectionType();

protected:
    Mp7JrsStreamSectionType();
    virtual void DeepCopy(const Mp7JrsNodePtr &original);

    virtual Mp7JrsNodeEnum GetAllChildElements() const;

    virtual Mp7JrsNodePtr GetBaseRoot();
    virtual const Mp7JrsNodePtr GetBaseRoot() const;

private:
    Mp7JrsunitType::Enumeration m_unit;
    bool m_unit_Exist;
    unsigned m_start;
    bool m_start_Exist;
    unsigned m_length;
    bool m_length_Exist;



// no includefile for extension defined 
// file Mp7JrsStreamSectionType_ExtMyPropDef.h


public:
    virtual void Init(); // Called by inheritees to recursively update m_myself
protected:
    void Cleanup();
};

#endif // _2229MP7JRSSTREAMSECTIONTYPE_H

