/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/


// ClassType_H.template
#ifndef _2455MP7JRSTEXTUALTYPE_H
#define _2455MP7JRSTEXTUALTYPE_H

#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOM.hpp>

#include "Mp7JrsDefines.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsTextualTypeFactory.h"
#include "Mp7JrsNode.h"
#include "Mp7JrsConvert.h"


// no includefile for extension defined 
// file Mp7JrsTextualType_ExtInclude.h


#include "Mp7JrsphoneticAlphabetType.h"
#include "Mp7JrsTextualBaseType.h"
class IMp7JrsTextualBaseType;
class IMp7JrsTextualBaseType_phoneticTranscription_CollectionType;
class IMp7JrsTextualType;
typedef Mp7JrsPtr< IMp7JrsTextualBaseType > Mp7JrsTextualBasePtr;
typedef Mp7JrsPtr< IMp7JrsTextualBaseType_phoneticTranscription_CollectionType > Mp7JrsTextualBaseType_phoneticTranscription_CollectionPtr;
typedef Mp7JrsPtr< IMp7JrsTextualType> Mp7JrsTextualPtr;

/** 
 * Generated interface IMp7JrsTextualType for class Mp7JrsTextualType<br>
 * Located at: mpeg7-2004.xsd, line 3720<br>
 * Classified: Class<br>
 * Derived from: TextualBaseType (Class)<br>
 */
class MP7JRS_EXPORT IMp7JrsTextualType 
 :
        public IMp7JrsTextualBaseType
{
public:
    // TODO: make these protected?
    IMp7JrsTextualType();
    virtual ~IMp7JrsTextualType();
    /**
     * Sets the content of Mp7JrsTextualBaseType<br>
     * (Inherited from Mp7JrsTextualBaseType)
     * @param item XMLCh *
     */
    virtual void SetContent(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Retrieves the content of Mp7JrsTextualBaseType 
<br>
     * (Inherited from Mp7JrsTextualBaseType)
     * @return XMLCh *
     */
    virtual XMLCh * GetContent() const = 0;

    /**
     * Get lang attribute.
<br>
     * (Inherited from Mp7JrsTextualBaseType)
     * @return XMLCh *
     */
    virtual XMLCh * Getlang() const = 0;

    /**
     * Get lang attribute validity information.
     * (Inherited from Mp7JrsTextualBaseType)
     * @return true if attribute value is valid, false if not.
     */
    virtual bool Existlang() const = 0;

    /**
     * Set lang attribute.
<br>
     * (Inherited from Mp7JrsTextualBaseType)
     * @param item XMLCh *
     */
    virtual void Setlang(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate lang attribute.
     * (Inherited from Mp7JrsTextualBaseType)
     
     * (Validate it again by setting a value)
     */
    virtual void Invalidatelang() = 0;

    /**
     * Get phoneticTranscription attribute.
<br>
     * (Inherited from Mp7JrsTextualBaseType)
     * @return Mp7JrsTextualBaseType_phoneticTranscription_CollectionPtr
     */
    virtual Mp7JrsTextualBaseType_phoneticTranscription_CollectionPtr GetphoneticTranscription() const = 0;

    /**
     * Get phoneticTranscription attribute validity information.
     * (Inherited from Mp7JrsTextualBaseType)
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExistphoneticTranscription() const = 0;

    /**
     * Set phoneticTranscription attribute.
<br>
     * (Inherited from Mp7JrsTextualBaseType)
     * @param item const Mp7JrsTextualBaseType_phoneticTranscription_CollectionPtr &
     */
    virtual void SetphoneticTranscription(const Mp7JrsTextualBaseType_phoneticTranscription_CollectionPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate phoneticTranscription attribute.
     * (Inherited from Mp7JrsTextualBaseType)
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidatephoneticTranscription() = 0;

    /**
     * Get phoneticAlphabet attribute.
<br>
     * (Inherited from Mp7JrsTextualBaseType)
     * @return Mp7JrsphoneticAlphabetType::Enumeration
     */
    virtual Mp7JrsphoneticAlphabetType::Enumeration GetphoneticAlphabet() const = 0;

    /**
     * Get phoneticAlphabet attribute validity information.
     * (Inherited from Mp7JrsTextualBaseType)
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExistphoneticAlphabet() const = 0;

    /**
     * Set phoneticAlphabet attribute.
<br>
     * (Inherited from Mp7JrsTextualBaseType)
     * @param item Mp7JrsphoneticAlphabetType::Enumeration
     */
    virtual void SetphoneticAlphabet(Mp7JrsphoneticAlphabetType::Enumeration item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate phoneticAlphabet attribute.
     * (Inherited from Mp7JrsTextualBaseType)
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidatephoneticAlphabet() = 0;

    /**
     * Gets or creates Mp7JrsNodePtr child elements of TextualType.
     * Currently just supports rudimentary XPath expressions as TextualType contains no child elements of type "Mp7JrsNodePtr" at all.

     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL) = 0;

    
// no includefile for extension defined 
// file Mp7JrsTextualType_ExtMethodDef.h


// no includefile for extension defined 
// file Mp7JrsTextualType_ExtPropDef.h

};




/*
 * Generated class Mp7JrsTextualType<br>
 * Located at: mpeg7-2004.xsd, line 3720<br>
 * Classified: Class<br>
 * Derived from: TextualBaseType (Class)<br>
 * Defined in mpeg7-2004.xsd, line 3720.<br>
 */
class MP7JRS_EXPORT Mp7JrsTextualType :
        public IMp7JrsTextualType
{
    friend class Mp7JrsTextualTypeFactory; // constructs objects

public:
    /*
     * Retrieves the base class of Mp7JrsTextualType
     
     * @return Mp7JrsPtr< Mp7JrsTextualBaseType >
     */
    virtual Mp7JrsPtr< Mp7JrsTextualBaseType > GetBase() const;
    /*
     * Sets the content of Mp7JrsTextualBaseType<br>
     * (Inherited from Mp7JrsTextualBaseType)
     * @param item XMLCh *
     */
    virtual void SetContent(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Retrieves the content of Mp7JrsTextualBaseType 
<br>
     * (Inherited from Mp7JrsTextualBaseType)
     * @return XMLCh *
     */
    virtual XMLCh * GetContent() const;

    /*
     * Get lang attribute.
<br>
     * (Inherited from Mp7JrsTextualBaseType)
     * @return XMLCh *
     */
    virtual XMLCh * Getlang() const;

    /*
     * Get lang attribute validity information.
     * (Inherited from Mp7JrsTextualBaseType)
     * @return true if attribute value is valid, false if not.
     */
    virtual bool Existlang() const;

    /*
     * Set lang attribute.
<br>
     * (Inherited from Mp7JrsTextualBaseType)
     * @param item XMLCh *
     */
    virtual void Setlang(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate lang attribute.
     * (Inherited from Mp7JrsTextualBaseType)
     
     * (Validate it again by setting a value)
     */
    virtual void Invalidatelang();

    /*
     * Get phoneticTranscription attribute.
<br>
     * (Inherited from Mp7JrsTextualBaseType)
     * @return Mp7JrsTextualBaseType_phoneticTranscription_CollectionPtr
     */
    virtual Mp7JrsTextualBaseType_phoneticTranscription_CollectionPtr GetphoneticTranscription() const;

    /*
     * Get phoneticTranscription attribute validity information.
     * (Inherited from Mp7JrsTextualBaseType)
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExistphoneticTranscription() const;

    /*
     * Set phoneticTranscription attribute.
<br>
     * (Inherited from Mp7JrsTextualBaseType)
     * @param item const Mp7JrsTextualBaseType_phoneticTranscription_CollectionPtr &
     */
    virtual void SetphoneticTranscription(const Mp7JrsTextualBaseType_phoneticTranscription_CollectionPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate phoneticTranscription attribute.
     * (Inherited from Mp7JrsTextualBaseType)
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidatephoneticTranscription();

    /*
     * Get phoneticAlphabet attribute.
<br>
     * (Inherited from Mp7JrsTextualBaseType)
     * @return Mp7JrsphoneticAlphabetType::Enumeration
     */
    virtual Mp7JrsphoneticAlphabetType::Enumeration GetphoneticAlphabet() const;

    /*
     * Get phoneticAlphabet attribute validity information.
     * (Inherited from Mp7JrsTextualBaseType)
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExistphoneticAlphabet() const;

    /*
     * Set phoneticAlphabet attribute.
<br>
     * (Inherited from Mp7JrsTextualBaseType)
     * @param item Mp7JrsphoneticAlphabetType::Enumeration
     */
    virtual void SetphoneticAlphabet(Mp7JrsphoneticAlphabetType::Enumeration item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate phoneticAlphabet attribute.
     * (Inherited from Mp7JrsTextualBaseType)
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidatephoneticAlphabet();

    /*
     * Gets or creates Mp7JrsNodePtr child elements of TextualType.
     * Currently just supports rudimentary XPath expressions as TextualType contains no child elements of type "Mp7JrsNodePtr" at all.

     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL);

    

    /* Get the class name.
     * @return The name of this class.
     */
    virtual XMLCh * ToText() const;

    /* Parse this type.
     * @param txt buffer to parse from.
     * @return true if parsing was successful, false otherwise.
     */
    virtual bool Parse(const XMLCh * const txt);

    /* Parse the content from a string.  Note, this is purely for ComplexTypes with SimpleContent and SimpleTypes.
     * @param txt buffer to parse from.
     * @return true if parsing was successful, false otherwise.
     */
	virtual bool ContentFromString(const XMLCh * const txt);

    /* Convert the content to string.  Note, this is purely for ComplexTypes with SimpleContent and SimpleTypes. All other types may return NULL or the class name.
     * @return the allocated XMLCh * if conversion was successful. The caller must free memory using Mp7JrsUtil.deallocate().
     */
	virtual XMLCh * ContentToString() const;


    /* Serialialize this object to a DOM tree.
     * @param doc DOM document.
     * @param parent Parent node in DOM tree.
     */
    virtual void Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem = NULL);

    /* Deserialize this object from a DOM tree.
     * @param doc DOM document.
     * @param parent Parent element in DOM tree.
     * @param current Current element in DOM tree.
     */
    virtual bool Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current);

    /* Creates and inserts a Childnode of the specified Type
     * @param type The tag type of the child to be created (tag name).
     * @param xsitype The xsi type of the child to be created if available.
     * @return the created child with the correct dynamic type for the specified type.
     */
    virtual Mp7JrsNodePtr CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype);
         
// no includefile for extension defined 
// file Mp7JrsTextualType_ExtMyMethodDef.h


public:

    virtual ~Mp7JrsTextualType();

protected:
    Mp7JrsTextualType();
    virtual void DeepCopy(const Mp7JrsNodePtr &original);

    virtual Mp7JrsNodeEnum GetAllChildElements() const;

    virtual Mp7JrsNodePtr GetBaseRoot();
    virtual const Mp7JrsNodePtr GetBaseRoot() const;

private:

    // Base Class
    Mp7JrsPtr< Mp7JrsTextualBaseType > m_Base;


// no includefile for extension defined 
// file Mp7JrsTextualType_ExtMyPropDef.h


public:
    virtual void Init(); // Called by inheritees to recursively update m_myself
protected:
    void Cleanup();
};

#endif // _2455MP7JRSTEXTUALTYPE_H

