/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/


// ClassType_H.template
#ifndef _2457MP7JRSTEXTUREBROWSINGTYPE_H
#define _2457MP7JRSTEXTUREBROWSINGTYPE_H

#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOM.hpp>

#include "Mp7JrsDefines.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsTextureBrowsingTypeFactory.h"
#include "Mp7JrsNode.h"
#include "Mp7JrsConvert.h"


// no includefile for extension defined 
// file Mp7JrsTextureBrowsingType_ExtInclude.h


#include "Mp7JrsDType.h"
#include "Mp7JrsMpeg7BaseType.h"
#include "Mp7JrsTextureBrowsingType_Regularity_LocalType.h"
#include "Mp7JrsVisualDType.h"
class IMp7JrsTextureBrowsingType;
class IMp7JrsTextureBrowsingType_CollectionType;
class IMp7JrsVisualDType;
typedef Mp7JrsPtr< IMp7JrsTextureBrowsingType_CollectionType > Mp7JrsTextureBrowsingType_CollectionPtr;
typedef Mp7JrsPtr< IMp7JrsTextureBrowsingType> Mp7JrsTextureBrowsingPtr;
typedef Mp7JrsPtr< IMp7JrsVisualDType > Mp7JrsVisualDPtr;

/** 
 * Generated interface IMp7JrsTextureBrowsingType for class Mp7JrsTextureBrowsingType<br>
 * Located at: mpeg7-2004.xsd, line 741<br>
 * Classified: Class<br>
 * Derived from: VisualDType (Class)<br>
 */
class MP7JRS_EXPORT IMp7JrsTextureBrowsingType 
 :
        public IMp7JrsVisualDType
{
public:
    // TODO: make these protected?
    IMp7JrsTextureBrowsingType();
    virtual ~IMp7JrsTextureBrowsingType();
        /** @name Sequence

         */
        //@{
    /**
     * Get Regularity element.
     * @return Mp7JrsTextureBrowsingType_Regularity_LocalType::Enumeration
     */
    virtual Mp7JrsTextureBrowsingType_Regularity_LocalType::Enumeration GetRegularity() const = 0;

    /**
     * Set Regularity element.
     * @param item Mp7JrsTextureBrowsingType_Regularity_LocalType::Enumeration
     */
    // Mandatory			
    virtual void SetRegularity(Mp7JrsTextureBrowsingType_Regularity_LocalType::Enumeration item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

    /**
     * Get TextureBrowsingType_LocalType element.
     * @return Mp7JrsTextureBrowsingType_CollectionPtr
     */
    virtual Mp7JrsTextureBrowsingType_CollectionPtr GetTextureBrowsingType_LocalType() const = 0;

    /**
     * Set TextureBrowsingType_LocalType element.
     * @param item const Mp7JrsTextureBrowsingType_CollectionPtr &
     */
    // Mandatory collection
    virtual void SetTextureBrowsingType_LocalType(const Mp7JrsTextureBrowsingType_CollectionPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

        //@}
    /**
     * Gets or creates Mp7JrsNodePtr child elements of TextureBrowsingType.
     * Currently just supports rudimentary XPath expressions as TextureBrowsingType contains no child elements of type "Mp7JrsNodePtr" at all.

     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL) = 0;

    
// no includefile for extension defined 
// file Mp7JrsTextureBrowsingType_ExtMethodDef.h


// no includefile for extension defined 
// file Mp7JrsTextureBrowsingType_ExtPropDef.h

};




/*
 * Generated class Mp7JrsTextureBrowsingType<br>
 * Located at: mpeg7-2004.xsd, line 741<br>
 * Classified: Class<br>
 * Derived from: VisualDType (Class)<br>
 * Defined in mpeg7-2004.xsd, line 741.<br>
 */
class MP7JRS_EXPORT Mp7JrsTextureBrowsingType :
        public IMp7JrsTextureBrowsingType
{
    friend class Mp7JrsTextureBrowsingTypeFactory; // constructs objects

public:
    /*
     * Retrieves the base class of Mp7JrsTextureBrowsingType
     
     * @return Mp7JrsPtr< Mp7JrsVisualDType >
     */
    virtual Mp7JrsPtr< Mp7JrsVisualDType > GetBase() const;
        /* @name Sequence

         */
        //@{
    /*
     * Get Regularity element.
     * @return Mp7JrsTextureBrowsingType_Regularity_LocalType::Enumeration
     */
    virtual Mp7JrsTextureBrowsingType_Regularity_LocalType::Enumeration GetRegularity() const;

    /*
     * Set Regularity element.
     * @param item Mp7JrsTextureBrowsingType_Regularity_LocalType::Enumeration
     */
    // Mandatory			
    virtual void SetRegularity(Mp7JrsTextureBrowsingType_Regularity_LocalType::Enumeration item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

    /*
     * Get TextureBrowsingType_LocalType element.
     * @return Mp7JrsTextureBrowsingType_CollectionPtr
     */
    virtual Mp7JrsTextureBrowsingType_CollectionPtr GetTextureBrowsingType_LocalType() const;

    /*
     * Set TextureBrowsingType_LocalType element.
     * @param item const Mp7JrsTextureBrowsingType_CollectionPtr &
     */
    // Mandatory collection
    virtual void SetTextureBrowsingType_LocalType(const Mp7JrsTextureBrowsingType_CollectionPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

        //@}
    /*
     * Gets or creates Mp7JrsNodePtr child elements of TextureBrowsingType.
     * Currently just supports rudimentary XPath expressions as TextureBrowsingType contains no child elements of type "Mp7JrsNodePtr" at all.

     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL);

    

    /* Get the class name.
     * @return The name of this class.
     */
    virtual XMLCh * ToText() const;

    /* Parse this type.
     * @param txt buffer to parse from.
     * @return true if parsing was successful, false otherwise.
     */
    virtual bool Parse(const XMLCh * const txt);

    /* Parse the content from a string.  Note, this is purely for ComplexTypes with SimpleContent and SimpleTypes.
     * @param txt buffer to parse from.
     * @return true if parsing was successful, false otherwise.
     */
	virtual bool ContentFromString(const XMLCh * const txt);

    /* Convert the content to string.  Note, this is purely for ComplexTypes with SimpleContent and SimpleTypes. All other types may return NULL or the class name.
     * @return the allocated XMLCh * if conversion was successful. The caller must free memory using Mp7JrsUtil.deallocate().
     */
	virtual XMLCh * ContentToString() const;


    /* Serialialize this object to a DOM tree.
     * @param doc DOM document.
     * @param parent Parent node in DOM tree.
     */
    virtual void Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem = NULL);

    /* Deserialize this object from a DOM tree.
     * @param doc DOM document.
     * @param parent Parent element in DOM tree.
     * @param current Current element in DOM tree.
     */
    virtual bool Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current);

    /* Creates and inserts a Childnode of the specified Type
     * @param type The tag type of the child to be created (tag name).
     * @param xsitype The xsi type of the child to be created if available.
     * @return the created child with the correct dynamic type for the specified type.
     */
    virtual Mp7JrsNodePtr CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype);
         
// no includefile for extension defined 
// file Mp7JrsTextureBrowsingType_ExtMyMethodDef.h


public:

    virtual ~Mp7JrsTextureBrowsingType();

protected:
    Mp7JrsTextureBrowsingType();
    virtual void DeepCopy(const Mp7JrsNodePtr &original);

    virtual Mp7JrsNodeEnum GetAllChildElements() const;

    virtual Mp7JrsNodePtr GetBaseRoot();
    virtual const Mp7JrsNodePtr GetBaseRoot() const;

private:

    // Base Class
    Mp7JrsPtr< Mp7JrsVisualDType > m_Base;

    Mp7JrsTextureBrowsingType_Regularity_LocalType::Enumeration m_Regularity;
    Mp7JrsTextureBrowsingType_CollectionPtr m_TextureBrowsingType_LocalType;

// no includefile for extension defined 
// file Mp7JrsTextureBrowsingType_ExtMyPropDef.h


public:
    virtual void Init(); // Called by inheritees to recursively update m_myself
protected:
    void Cleanup();
};

#endif // _2457MP7JRSTEXTUREBROWSINGTYPE_H

