/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/


// ClassType_H.template
#ifndef _2597MP7JRSUSERDESCRIPTIONTYPE_H
#define _2597MP7JRSUSERDESCRIPTIONTYPE_H

#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOM.hpp>

#include "Mp7JrsDefines.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsUserDescriptionTypeFactory.h"
#include "Mp7JrsNode.h"
#include "Mp7JrsConvert.h"


// no includefile for extension defined 
// file Mp7JrsUserDescriptionType_ExtInclude.h


#include "Mp7JrsCompleteDescriptionType.h"
#include "Mp7JrsContentManagementType.h"
class IMp7JrsAgentType;
class IMp7JrsCompleteDescriptionType_OrderingKey_CollectionType;
class IMp7JrsCompleteDescriptionType_Relationships_CollectionType;
class IMp7JrsContentManagementType;
class IMp7JrsDescriptionMetadataType;
class IMp7JrsUserDescriptionType;
class IMp7JrsUserDescriptionType_UsageHistory_CollectionType;
class IMp7JrsUserDescriptionType_UserPreferences_CollectionType;
typedef Mp7JrsPtr< IMp7JrsAgentType > Mp7JrsAgentPtr;
typedef Mp7JrsPtr< IMp7JrsCompleteDescriptionType_OrderingKey_CollectionType > Mp7JrsCompleteDescriptionType_OrderingKey_CollectionPtr;
typedef Mp7JrsPtr< IMp7JrsCompleteDescriptionType_Relationships_CollectionType > Mp7JrsCompleteDescriptionType_Relationships_CollectionPtr;
typedef Mp7JrsPtr< IMp7JrsContentManagementType > Mp7JrsContentManagementPtr;
typedef Mp7JrsPtr< IMp7JrsDescriptionMetadataType > Mp7JrsDescriptionMetadataPtr;
typedef Mp7JrsPtr< IMp7JrsUserDescriptionType_UsageHistory_CollectionType > Mp7JrsUserDescriptionType_UsageHistory_CollectionPtr;
typedef Mp7JrsPtr< IMp7JrsUserDescriptionType_UserPreferences_CollectionType > Mp7JrsUserDescriptionType_UserPreferences_CollectionPtr;
typedef Mp7JrsPtr< IMp7JrsUserDescriptionType> Mp7JrsUserDescriptionPtr;

/** 
 * Generated interface IMp7JrsUserDescriptionType for class Mp7JrsUserDescriptionType<br>
 * Located at: mpeg7-2004.xsd, line 2786<br>
 * Classified: Class<br>
 * Derived from: ContentManagementType (Class)<br>
 */
class MP7JRS_EXPORT IMp7JrsUserDescriptionType 
 :
        public IMp7JrsContentManagementType
{
public:
    // TODO: make these protected?
    IMp7JrsUserDescriptionType();
    virtual ~IMp7JrsUserDescriptionType();
        /** @name Sequence

         */
        //@{
    /**
     * Get User element.
     * @return Mp7JrsPtr< Mp7JrsNode >
     * Possible types (within this library):
     * <ul>
     * <li>IMp7JrsOrganizationPtr</li>
     * <li>IMp7JrsPersonGroupPtr</li>
     * <li>IMp7JrsPersonPtr</li>
     * </ul>
     */
    virtual Mp7JrsPtr< Mp7JrsNode > GetUser() const = 0;

    virtual bool IsValidUser() const = 0;

    /**
     * Set User element.
     * @param item const Mp7JrsPtr< Mp7JrsNode > &
     * Valid types for item:
     * <ul>
     * <li>IMp7JrsOrganizationPtr</li>
     * <li>IMp7JrsPersonGroupPtr</li>
     * <li>IMp7JrsPersonPtr</li>
     * </ul>
     */
    virtual void SetUser(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate User element.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidateUser() = 0;

    /**
     * Get UserPreferences element.
     * @return Mp7JrsUserDescriptionType_UserPreferences_CollectionPtr
     */
    virtual Mp7JrsUserDescriptionType_UserPreferences_CollectionPtr GetUserPreferences() const = 0;

    /**
     * Set UserPreferences element.
     * @param item const Mp7JrsUserDescriptionType_UserPreferences_CollectionPtr &
     */
    // Mandatory collection
    virtual void SetUserPreferences(const Mp7JrsUserDescriptionType_UserPreferences_CollectionPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

    /**
     * Get UsageHistory element.
     * @return Mp7JrsUserDescriptionType_UsageHistory_CollectionPtr
     */
    virtual Mp7JrsUserDescriptionType_UsageHistory_CollectionPtr GetUsageHistory() const = 0;

    /**
     * Set UsageHistory element.
     * @param item const Mp7JrsUserDescriptionType_UsageHistory_CollectionPtr &
     */
    // Mandatory collection
    virtual void SetUsageHistory(const Mp7JrsUserDescriptionType_UsageHistory_CollectionPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

        //@}
        /** @name Sequence

         */
        //@{
    /**
     * Get DescriptionMetadata element.
<br>
     * (Inherited from Mp7JrsCompleteDescriptionType)
     * @return Mp7JrsDescriptionMetadataPtr
     */
    virtual Mp7JrsDescriptionMetadataPtr GetDescriptionMetadata() const = 0;

    virtual bool IsValidDescriptionMetadata() const = 0;

    /**
     * Set DescriptionMetadata element.
<br>
     * (Inherited from Mp7JrsCompleteDescriptionType)
     * @param item const Mp7JrsDescriptionMetadataPtr &
     */
    virtual void SetDescriptionMetadata(const Mp7JrsDescriptionMetadataPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate DescriptionMetadata element.
     * (Inherited from Mp7JrsCompleteDescriptionType)
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidateDescriptionMetadata() = 0;

    /**
     * Get Relationships element.
<br>
     * (Inherited from Mp7JrsCompleteDescriptionType)
     * @return Mp7JrsCompleteDescriptionType_Relationships_CollectionPtr
     */
    virtual Mp7JrsCompleteDescriptionType_Relationships_CollectionPtr GetRelationships() const = 0;

    /**
     * Set Relationships element.
<br>
     * (Inherited from Mp7JrsCompleteDescriptionType)
     * @param item const Mp7JrsCompleteDescriptionType_Relationships_CollectionPtr &
     */
    // Mandatory collection
    virtual void SetRelationships(const Mp7JrsCompleteDescriptionType_Relationships_CollectionPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

    /**
     * Get OrderingKey element.
<br>
     * (Inherited from Mp7JrsCompleteDescriptionType)
     * @return Mp7JrsCompleteDescriptionType_OrderingKey_CollectionPtr
     */
    virtual Mp7JrsCompleteDescriptionType_OrderingKey_CollectionPtr GetOrderingKey() const = 0;

    /**
     * Set OrderingKey element.
<br>
     * (Inherited from Mp7JrsCompleteDescriptionType)
     * @param item const Mp7JrsCompleteDescriptionType_OrderingKey_CollectionPtr &
     */
    // Mandatory collection
    virtual void SetOrderingKey(const Mp7JrsCompleteDescriptionType_OrderingKey_CollectionPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

        //@}
    /**
     * Gets or creates Mp7JrsNodePtr child elements of UserDescriptionType.
     * Currently this type contains 6 child elements of type "Mp7JrsNodePtr".

     * <ul>
     * <li>DescriptionMetadata</li>
     * <li>urn:mpeg:mpeg7:schema:2004:Relationships</li>
     * <li>urn:mpeg:mpeg7:schema:2004:OrderingKey</li>
     * <li>User</li>
     * <li>urn:mpeg:mpeg7:schema:2004:UserPreferences</li>
     * <li>urn:mpeg:mpeg7:schema:2004:UsageHistory</li>
     * </ul>
     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     * \par<code>A</code> 
     * - Select all child elements named <code>A</code>.
     * - In case of no element, create a new one.
     * - For collections: all subsequent XPath expressions are applied on all collection items (e.g. <code>A/B</code> query or create, childnodes <code>B</code> of all <code>A</code>s). 
     * \par<code>A[i]</code>
     * - Select the <code>i</code>th child element of all <code>A</code>.
     * - Please note, that XPath indexes start at 1.
     * - If there is no such <code>i</code>th element, try to create a new one at index <code>i</code>.
     * - Single elements require <code>i</code> to be 1 otherwise an exception is thrown.
     * - For collections, <code>i</code> must not exceed the number of existing items +1 (appending is allowed, filling gaps not).
     * - If the index is too high, a <code>MissingElementException</code> is thrown, and you can extract the last valid index. (Example: If a collection contains 2 elements and you are requesting the 4th element).
     * 
     * \par <code>A[\@xsi:type="T"]</code>
     *     - Select all child elements named <code>A</code> of type <code>T</code>.
     *     - Currently only typenames without namespace are recognised.
     *     - If there is no such element, create and append a new one.
     * \par <code>A[i][\@xsi:type="T"]</code>
     *     - Select the <code>i</code>th child element named <code>A</code>, which must be of type <code>T</code>.
     *     - If there is no such element at index <code>i</code>, a new one is created and inserted (appended).
     * \par <code>A[\@xsi:type="T"][i]</code>
     *     - Out of all child elements named <code>A</code> of type <code>T</code>, take the <code>i</code>th one.
     *     - If there is no such element at index <code>i</code>, create and append one.
     * 
     * \par <code>A[last()]</code>
     *     - Select the last child element named <code>A</code>.
     *     - In case of no element, create a new one.
     * \par <code>A[last()][\@xsi:type="T"]</code>
     *     - Select the last child element named <code>A</code>, which must be of type <code>T</code>.
     *     - For collections: in case of no such type, create and append a new one.
     *     - For single elements: if an element with different type exists, return the empty list.
     * 
     * \par <code>A[\@xsi:type="T"][last()]</code>
     *     - Out of all child elements named <code>A</code> of type <code>T</code>, take the last one.
     *     - For collections: in case of no such type, create and append a new one.
     *     - For single elements: if an element with different type exists, return the empty list.
     * 
     * \par <code>A[position() \> last()]</code>
     *     - Force creation of a child element named <code>A</code> with its default type.
     *     - For collections: create and append the default element.
     *     - For single elements: if any element exists, an exception is thrown.
     *
     * \par <code>A[position() \> last()][\@xsi:type="T"]</code>
     *     - Force creation of a child element named <code>A</code> of type <code>T</code>.
     *     - For collections: create and append the element.
     *     - For single elements: if any element exists, an exception is thrown.
     *
     * \par <code>A[\@xsi:type="T"][position() \> last()]</code>
     *     - Force creation of a child element named <code>A</code> of type <code>T</code>.
     *     - Behaves the same way as the previous expression.
     * 
     * 
     * Supported XPath expressions for GetFromXPath<br>
     * The same expressions like GetOrCreateFromXPath plus
     * \par <code>A[position() \< i] </code>
     *     - Select all child elements named <code>A</code> at indexes lower than <code>i</code>.
     *     - <code>i</code> must be greater than -1, otherwise an exception is thrown.
     * \par <code>A[position() \> i] </code>
     *     - Select all child elements named <code>A</code> at indexes greater than <code>i</code>.
     *     - <code>i</code> must be greater than -1, otherwise an exception is thrown.
     * \par <code>A[position() \> i | position() \< j] </code>
     *     - Select all child elements named <code>A</code> at indexes greater than <code>i</code> and lower than <code>j</code>.
     *     - <code>i</code> and <code>j</code> must be greater than -1, otherwise an exception is thrown.
     *     - If <code>i</code> is greater than <code>j</code> then an empty list is returned.
     *
     * These three expressions can be combined with the <code>\@xsi:type</code> attribute to get a more restricted query.
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL) = 0;

    
// no includefile for extension defined 
// file Mp7JrsUserDescriptionType_ExtMethodDef.h


// no includefile for extension defined 
// file Mp7JrsUserDescriptionType_ExtPropDef.h

};




/*
 * Generated class Mp7JrsUserDescriptionType<br>
 * Located at: mpeg7-2004.xsd, line 2786<br>
 * Classified: Class<br>
 * Derived from: ContentManagementType (Class)<br>
 * Defined in mpeg7-2004.xsd, line 2786.<br>
 */
class MP7JRS_EXPORT Mp7JrsUserDescriptionType :
        public IMp7JrsUserDescriptionType
{
    friend class Mp7JrsUserDescriptionTypeFactory; // constructs objects

public:
    /*
     * Retrieves the base class of Mp7JrsUserDescriptionType
     
     * @return Mp7JrsPtr< Mp7JrsContentManagementType >
     */
    virtual Mp7JrsPtr< Mp7JrsContentManagementType > GetBase() const;
        /* @name Sequence

         */
        //@{
    /*
     * Get User element.
     * @return Mp7JrsPtr< Mp7JrsNode >
     * Possible types (within this library):
     * <ul>
     * <li>IMp7JrsOrganizationPtr</li>
     * <li>IMp7JrsPersonGroupPtr</li>
     * <li>IMp7JrsPersonPtr</li>
     * </ul>
     */
    virtual Mp7JrsPtr< Mp7JrsNode > GetUser() const;

    virtual bool IsValidUser() const;

    /*
     * Set User element.
     * @param item const Mp7JrsPtr< Mp7JrsNode > &
     * Valid types for item:
     * <ul>
     * <li>IMp7JrsOrganizationPtr</li>
     * <li>IMp7JrsPersonGroupPtr</li>
     * <li>IMp7JrsPersonPtr</li>
     * </ul>
     */
    virtual void SetUser(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate User element.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidateUser();

    /*
     * Get UserPreferences element.
     * @return Mp7JrsUserDescriptionType_UserPreferences_CollectionPtr
     */
    virtual Mp7JrsUserDescriptionType_UserPreferences_CollectionPtr GetUserPreferences() const;

    /*
     * Set UserPreferences element.
     * @param item const Mp7JrsUserDescriptionType_UserPreferences_CollectionPtr &
     */
    // Mandatory collection
    virtual void SetUserPreferences(const Mp7JrsUserDescriptionType_UserPreferences_CollectionPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

    /*
     * Get UsageHistory element.
     * @return Mp7JrsUserDescriptionType_UsageHistory_CollectionPtr
     */
    virtual Mp7JrsUserDescriptionType_UsageHistory_CollectionPtr GetUsageHistory() const;

    /*
     * Set UsageHistory element.
     * @param item const Mp7JrsUserDescriptionType_UsageHistory_CollectionPtr &
     */
    // Mandatory collection
    virtual void SetUsageHistory(const Mp7JrsUserDescriptionType_UsageHistory_CollectionPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

        //@}
        /* @name Sequence

         */
        //@{
    /*
     * Get DescriptionMetadata element.
<br>
     * (Inherited from Mp7JrsCompleteDescriptionType)
     * @return Mp7JrsDescriptionMetadataPtr
     */
    virtual Mp7JrsDescriptionMetadataPtr GetDescriptionMetadata() const;

    virtual bool IsValidDescriptionMetadata() const;

    /*
     * Set DescriptionMetadata element.
<br>
     * (Inherited from Mp7JrsCompleteDescriptionType)
     * @param item const Mp7JrsDescriptionMetadataPtr &
     */
    virtual void SetDescriptionMetadata(const Mp7JrsDescriptionMetadataPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate DescriptionMetadata element.
     * (Inherited from Mp7JrsCompleteDescriptionType)
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidateDescriptionMetadata();

    /*
     * Get Relationships element.
<br>
     * (Inherited from Mp7JrsCompleteDescriptionType)
     * @return Mp7JrsCompleteDescriptionType_Relationships_CollectionPtr
     */
    virtual Mp7JrsCompleteDescriptionType_Relationships_CollectionPtr GetRelationships() const;

    /*
     * Set Relationships element.
<br>
     * (Inherited from Mp7JrsCompleteDescriptionType)
     * @param item const Mp7JrsCompleteDescriptionType_Relationships_CollectionPtr &
     */
    // Mandatory collection
    virtual void SetRelationships(const Mp7JrsCompleteDescriptionType_Relationships_CollectionPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

    /*
     * Get OrderingKey element.
<br>
     * (Inherited from Mp7JrsCompleteDescriptionType)
     * @return Mp7JrsCompleteDescriptionType_OrderingKey_CollectionPtr
     */
    virtual Mp7JrsCompleteDescriptionType_OrderingKey_CollectionPtr GetOrderingKey() const;

    /*
     * Set OrderingKey element.
<br>
     * (Inherited from Mp7JrsCompleteDescriptionType)
     * @param item const Mp7JrsCompleteDescriptionType_OrderingKey_CollectionPtr &
     */
    // Mandatory collection
    virtual void SetOrderingKey(const Mp7JrsCompleteDescriptionType_OrderingKey_CollectionPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

        //@}
    /*
     * Gets or creates Mp7JrsNodePtr child elements of UserDescriptionType.
     * Currently this type contains 6 child elements of type "Mp7JrsNodePtr".

     * <ul>
     * <li>DescriptionMetadata</li>
     * <li>urn:mpeg:mpeg7:schema:2004:Relationships</li>
     * <li>urn:mpeg:mpeg7:schema:2004:OrderingKey</li>
     * <li>User</li>
     * <li>urn:mpeg:mpeg7:schema:2004:UserPreferences</li>
     * <li>urn:mpeg:mpeg7:schema:2004:UsageHistory</li>
     * </ul>
     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     * \par<code>A</code> 
     * - Select all child elements named <code>A</code>.
     * - In case of no element, create a new one.
     * - For collections: all subsequent XPath expressions are applied on all collection items (e.g. <code>A/B</code> query or create, childnodes <code>B</code> of all <code>A</code>s). 
     * \par<code>A[i]</code>
     * - Select the <code>i</code>th child element of all <code>A</code>.
     * - Please note, that XPath indexes start at 1.
     * - If there is no such <code>i</code>th element, try to create a new one at index <code>i</code>.
     * - Single elements require <code>i</code> to be 1 otherwise an exception is thrown.
     * - For collections, <code>i</code> must not exceed the number of existing items +1 (appending is allowed, filling gaps not).
     * - If the index is too high, a <code>MissingElementException</code> is thrown, and you can extract the last valid index. (Example: If a collection contains 2 elements and you are requesting the 4th element).
     * 
     * \par <code>A[\@xsi:type="T"]</code>
     *     - Select all child elements named <code>A</code> of type <code>T</code>.
     *     - Currently only typenames without namespace are recognised.
     *     - If there is no such element, create and append a new one.
     * \par <code>A[i][\@xsi:type="T"]</code>
     *     - Select the <code>i</code>th child element named <code>A</code>, which must be of type <code>T</code>.
     *     - If there is no such element at index <code>i</code>, a new one is created and inserted (appended).
     * \par <code>A[\@xsi:type="T"][i]</code>
     *     - Out of all child elements named <code>A</code> of type <code>T</code>, take the <code>i</code>th one.
     *     - If there is no such element at index <code>i</code>, create and append one.
     * 
     * \par <code>A[last()]</code>
     *     - Select the last child element named <code>A</code>.
     *     - In case of no element, create a new one.
     * \par <code>A[last()][\@xsi:type="T"]</code>
     *     - Select the last child element named <code>A</code>, which must be of type <code>T</code>.
     *     - For collections: in case of no such type, create and append a new one.
     *     - For single elements: if an element with different type exists, return the empty list.
     * 
     * \par <code>A[\@xsi:type="T"][last()]</code>
     *     - Out of all child elements named <code>A</code> of type <code>T</code>, take the last one.
     *     - For collections: in case of no such type, create and append a new one.
     *     - For single elements: if an element with different type exists, return the empty list.
     * 
     * \par <code>A[position() \> last()]</code>
     *     - Force creation of a child element named <code>A</code> with its default type.
     *     - For collections: create and append the default element.
     *     - For single elements: if any element exists, an exception is thrown.
     *
     * \par <code>A[position() \> last()][\@xsi:type="T"]</code>
     *     - Force creation of a child element named <code>A</code> of type <code>T</code>.
     *     - For collections: create and append the element.
     *     - For single elements: if any element exists, an exception is thrown.
     *
     * \par <code>A[\@xsi:type="T"][position() \> last()]</code>
     *     - Force creation of a child element named <code>A</code> of type <code>T</code>.
     *     - Behaves the same way as the previous expression.
     * 
     * 
     * Supported XPath expressions for GetFromXPath<br>
     * The same expressions like GetOrCreateFromXPath plus
     * \par <code>A[position() \< i] </code>
     *     - Select all child elements named <code>A</code> at indexes lower than <code>i</code>.
     *     - <code>i</code> must be greater than -1, otherwise an exception is thrown.
     * \par <code>A[position() \> i] </code>
     *     - Select all child elements named <code>A</code> at indexes greater than <code>i</code>.
     *     - <code>i</code> must be greater than -1, otherwise an exception is thrown.
     * \par <code>A[position() \> i | position() \< j] </code>
     *     - Select all child elements named <code>A</code> at indexes greater than <code>i</code> and lower than <code>j</code>.
     *     - <code>i</code> and <code>j</code> must be greater than -1, otherwise an exception is thrown.
     *     - If <code>i</code> is greater than <code>j</code> then an empty list is returned.
     *
     * These three expressions can be combined with the <code>\@xsi:type</code> attribute to get a more restricted query.
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL);

    

    /* Get the class name.
     * @return The name of this class.
     */
    virtual XMLCh * ToText() const;

    /* Parse this type.
     * @param txt buffer to parse from.
     * @return true if parsing was successful, false otherwise.
     */
    virtual bool Parse(const XMLCh * const txt);

    /* Parse the content from a string.  Note, this is purely for ComplexTypes with SimpleContent and SimpleTypes.
     * @param txt buffer to parse from.
     * @return true if parsing was successful, false otherwise.
     */
	virtual bool ContentFromString(const XMLCh * const txt);

    /* Convert the content to string.  Note, this is purely for ComplexTypes with SimpleContent and SimpleTypes. All other types may return NULL or the class name.
     * @return the allocated XMLCh * if conversion was successful. The caller must free memory using Mp7JrsUtil.deallocate().
     */
	virtual XMLCh * ContentToString() const;


    /* Serialialize this object to a DOM tree.
     * @param doc DOM document.
     * @param parent Parent node in DOM tree.
     */
    virtual void Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem = NULL);

    /* Deserialize this object from a DOM tree.
     * @param doc DOM document.
     * @param parent Parent element in DOM tree.
     * @param current Current element in DOM tree.
     */
    virtual bool Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current);

    /* Creates and inserts a Childnode of the specified Type
     * @param type The tag type of the child to be created (tag name).
     * @param xsitype The xsi type of the child to be created if available.
     * @return the created child with the correct dynamic type for the specified type.
     */
    virtual Mp7JrsNodePtr CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype);
         
// no includefile for extension defined 
// file Mp7JrsUserDescriptionType_ExtMyMethodDef.h


public:

    virtual ~Mp7JrsUserDescriptionType();

protected:
    Mp7JrsUserDescriptionType();
    virtual void DeepCopy(const Mp7JrsNodePtr &original);

    virtual Mp7JrsNodeEnum GetAllChildElements() const;

    virtual Mp7JrsNodePtr GetBaseRoot();
    virtual const Mp7JrsNodePtr GetBaseRoot() const;

private:

    // Base Class
    Mp7JrsPtr< Mp7JrsContentManagementType > m_Base;

    Mp7JrsPtr< Mp7JrsNode > m_User;
    bool m_User_Exist; // For optional elements 
    Mp7JrsUserDescriptionType_UserPreferences_CollectionPtr m_UserPreferences;
    Mp7JrsUserDescriptionType_UsageHistory_CollectionPtr m_UsageHistory;

// no includefile for extension defined 
// file Mp7JrsUserDescriptionType_ExtMyPropDef.h


public:
    virtual void Init(); // Called by inheritees to recursively update m_myself
protected:
    void Cleanup();
};

#endif // _2597MP7JRSUSERDESCRIPTIONTYPE_H

