/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/


// ClassType_H.template
#ifndef _2731MP7JRSWORDFORMTYPE_H
#define _2731MP7JRSWORDFORMTYPE_H

#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOM.hpp>

#include "Mp7JrsDefines.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsWordFormTypeFactory.h"
#include "Mp7JrsNode.h"
#include "Mp7JrsConvert.h"


// no includefile for extension defined 
// file Mp7JrsWordFormType_ExtInclude.h


class IMp7JrsWordFormType;
class IMp7JrsWordFormType_terms_CollectionType;
class IMp7JrsWordFormType_type_CollectionType;
typedef Mp7JrsPtr< IMp7JrsWordFormType_terms_CollectionType > Mp7JrsWordFormType_terms_CollectionPtr;
typedef Mp7JrsPtr< IMp7JrsWordFormType_type_CollectionType > Mp7JrsWordFormType_type_CollectionPtr;
typedef Mp7JrsPtr< IMp7JrsWordFormType> Mp7JrsWordFormPtr;

/** 
 * Generated interface IMp7JrsWordFormType for class Mp7JrsWordFormType<br>
 * Located at: mpeg7-2004.xsd, line 3819<br>
 * Classified: Class<br>
 * Derived from: string (String)<br>
 */
class MP7JRS_EXPORT IMp7JrsWordFormType 
 :
        public Mp7JrsNode
{
public:
    // TODO: make these protected?
    IMp7JrsWordFormType();
    virtual ~IMp7JrsWordFormType();
    /**
     * Sets the content of Mp7JrsWordFormType     * @param item XMLCh *
     */
    virtual void SetContent(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Retrieves the content of Mp7JrsWordFormType 
     * @return XMLCh *
     */
    virtual XMLCh * GetContent() const = 0;

    /**
     * Get terms attribute.
     * @return Mp7JrsWordFormType_terms_CollectionPtr
     */
    virtual Mp7JrsWordFormType_terms_CollectionPtr Getterms() const = 0;

    /**
     * Get terms attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool Existterms() const = 0;

    /**
     * Set terms attribute.
     * @param item const Mp7JrsWordFormType_terms_CollectionPtr &
     */
    virtual void Setterms(const Mp7JrsWordFormType_terms_CollectionPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate terms attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void Invalidateterms() = 0;

    /**
     * Get id attribute.
     * @return XMLCh *
     */
    virtual XMLCh * Getid() const = 0;

    /**
     * Get id attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool Existid() const = 0;

    /**
     * Set id attribute.
     * @param item XMLCh *
     */
    virtual void Setid(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate id attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void Invalidateid() = 0;

    /**
     * Get equal attribute.
     * @return XMLCh *
     */
    virtual XMLCh * Getequal() const = 0;

    /**
     * Get equal attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool Existequal() const = 0;

    /**
     * Set equal attribute.
     * @param item XMLCh *
     */
    virtual void Setequal(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate equal attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void Invalidateequal() = 0;

    /**
     * Get type attribute.
     * @return Mp7JrsWordFormType_type_CollectionPtr
     */
    virtual Mp7JrsWordFormType_type_CollectionPtr Gettype() const = 0;

    /**
     * Get type attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool Existtype() const = 0;

    /**
     * Set type attribute.
     * @param item const Mp7JrsWordFormType_type_CollectionPtr &
     */
    virtual void Settype(const Mp7JrsWordFormType_type_CollectionPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate type attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void Invalidatetype() = 0;

    /**
     * Get baseForm attribute.
     * @return XMLCh *
     */
    virtual XMLCh * GetbaseForm() const = 0;

    /**
     * Get baseForm attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExistbaseForm() const = 0;

    /**
     * Set baseForm attribute.
     * @param item XMLCh *
     */
    virtual void SetbaseForm(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate baseForm attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidatebaseForm() = 0;

    /**
     * Gets or creates Mp7JrsNodePtr child elements of WordFormType.
     * Currently just supports rudimentary XPath expressions as WordFormType contains no child elements of type "Mp7JrsNodePtr" at all.

     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL) = 0;

    
// no includefile for extension defined 
// file Mp7JrsWordFormType_ExtMethodDef.h


// no includefile for extension defined 
// file Mp7JrsWordFormType_ExtPropDef.h

};




/*
 * Generated class Mp7JrsWordFormType<br>
 * Located at: mpeg7-2004.xsd, line 3819<br>
 * Classified: Class<br>
 * Derived from: string (String)<br>
 * Defined in mpeg7-2004.xsd, line 3819.<br>
 */
class MP7JRS_EXPORT Mp7JrsWordFormType :
        public IMp7JrsWordFormType
{
    friend class Mp7JrsWordFormTypeFactory; // constructs objects

public:
    /*
     * Sets the content of Mp7JrsWordFormType     * @param item XMLCh *
     */
    virtual void SetContent(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Retrieves the content of Mp7JrsWordFormType 
     * @return XMLCh *
     */
    virtual XMLCh * GetContent() const;

    /*
     * Get terms attribute.
     * @return Mp7JrsWordFormType_terms_CollectionPtr
     */
    virtual Mp7JrsWordFormType_terms_CollectionPtr Getterms() const;

    /*
     * Get terms attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool Existterms() const;

    /*
     * Set terms attribute.
     * @param item const Mp7JrsWordFormType_terms_CollectionPtr &
     */
    virtual void Setterms(const Mp7JrsWordFormType_terms_CollectionPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate terms attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void Invalidateterms();

    /*
     * Get id attribute.
     * @return XMLCh *
     */
    virtual XMLCh * Getid() const;

    /*
     * Get id attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool Existid() const;

    /*
     * Set id attribute.
     * @param item XMLCh *
     */
    virtual void Setid(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate id attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void Invalidateid();

    /*
     * Get equal attribute.
     * @return XMLCh *
     */
    virtual XMLCh * Getequal() const;

    /*
     * Get equal attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool Existequal() const;

    /*
     * Set equal attribute.
     * @param item XMLCh *
     */
    virtual void Setequal(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate equal attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void Invalidateequal();

    /*
     * Get type attribute.
     * @return Mp7JrsWordFormType_type_CollectionPtr
     */
    virtual Mp7JrsWordFormType_type_CollectionPtr Gettype() const;

    /*
     * Get type attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool Existtype() const;

    /*
     * Set type attribute.
     * @param item const Mp7JrsWordFormType_type_CollectionPtr &
     */
    virtual void Settype(const Mp7JrsWordFormType_type_CollectionPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate type attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void Invalidatetype();

    /*
     * Get baseForm attribute.
     * @return XMLCh *
     */
    virtual XMLCh * GetbaseForm() const;

    /*
     * Get baseForm attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExistbaseForm() const;

    /*
     * Set baseForm attribute.
     * @param item XMLCh *
     */
    virtual void SetbaseForm(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate baseForm attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidatebaseForm();

    /*
     * Gets or creates Mp7JrsNodePtr child elements of WordFormType.
     * Currently just supports rudimentary XPath expressions as WordFormType contains no child elements of type "Mp7JrsNodePtr" at all.

     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL);

    

    /* Get the class name.
     * @return The name of this class.
     */
    virtual XMLCh * ToText() const;

    /* Parse this type.
     * @param txt buffer to parse from.
     * @return true if parsing was successful, false otherwise.
     */
    virtual bool Parse(const XMLCh * const txt);

    /* Parse the content from a string.  Note, this is purely for ComplexTypes with SimpleContent and SimpleTypes.
     * @param txt buffer to parse from.
     * @return true if parsing was successful, false otherwise.
     */
	virtual bool ContentFromString(const XMLCh * const txt);

    /* Convert the content to string.  Note, this is purely for ComplexTypes with SimpleContent and SimpleTypes. All other types may return NULL or the class name.
     * @return the allocated XMLCh * if conversion was successful. The caller must free memory using Mp7JrsUtil.deallocate().
     */
	virtual XMLCh * ContentToString() const;


    /* Serialialize this object to a DOM tree.
     * @param doc DOM document.
     * @param parent Parent node in DOM tree.
     */
    virtual void Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem = NULL);

    /* Deserialize this object from a DOM tree.
     * @param doc DOM document.
     * @param parent Parent element in DOM tree.
     * @param current Current element in DOM tree.
     */
    virtual bool Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current);

    /* Creates and inserts a Childnode of the specified Type
     * @param type The tag type of the child to be created (tag name).
     * @param xsitype The xsi type of the child to be created if available.
     * @return the created child with the correct dynamic type for the specified type.
     */
    virtual Mp7JrsNodePtr CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype);
         
// no includefile for extension defined 
// file Mp7JrsWordFormType_ExtMyMethodDef.h


public:

    virtual ~Mp7JrsWordFormType();

protected:
    Mp7JrsWordFormType();
    virtual void DeepCopy(const Mp7JrsNodePtr &original);

    virtual Mp7JrsNodeEnum GetAllChildElements() const;

    virtual Mp7JrsNodePtr GetBaseRoot();
    virtual const Mp7JrsNodePtr GetBaseRoot() const;

private:
    Mp7JrsWordFormType_terms_CollectionPtr m_terms;
    bool m_terms_Exist;
    XMLCh * m_id;
    bool m_id_Exist;
    XMLCh * m_equal;
    bool m_equal_Exist;
    Mp7JrsWordFormType_type_CollectionPtr m_type;
    bool m_type_Exist;
    XMLCh * m_baseForm;
    bool m_baseForm_Exist;

    // Base String
    XMLCh * m_Base;


// no includefile for extension defined 
// file Mp7JrsWordFormType_ExtMyPropDef.h


public:
    virtual void Init(); // Called by inheritees to recursively update m_myself
protected:
    void Cleanup();
};

#endif // _2731MP7JRSWORDFORMTYPE_H

