/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/


// PatternType_H

#ifndef _2475MP7JRSTIMEPOINTTYPE_H
#define _2475MP7JRSTIMEPOINTTYPE_H


#include "Mp7JrsDefines.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsNode.h"
#include "Mp7JrstimePointTypeFactory.h"
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOM.hpp>
#include "Mp7JrsConvert.h"

// begin extension included
// file Mp7JrstimePointType_ExtInclude.h

class Mp7JrsdurationType;
// end extension included



/**
 * Generated interface IMp7JrstimePointType for Pattern Mp7JrstimePointType
 */
class MP7JRS_EXPORT IMp7JrstimePointType :
	public Mp7JrsNode
{
public:
	virtual int GetTimepointSign() const = 0;

	virtual void SetTimepointSign(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

	virtual int GetYear() const = 0;

	virtual void SetYear(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

	virtual int GetMonth() const = 0;

	virtual void SetMonth(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

	virtual int GetDay() const = 0;

	virtual void SetDay(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

	virtual bool GetDate_Exist() const = 0;

	virtual void SetDate_Exist(bool val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

	virtual int GetHours() const = 0;

	virtual void SetHours(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

	virtual int GetMinutes() const = 0;

	virtual void SetMinutes(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

	virtual int GetSeconds() const = 0;

	virtual void SetSeconds(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

	virtual int GetNumberOfFractions() const = 0;

	virtual void SetNumberOfFractions(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

	virtual bool GetTime_Exist() const = 0;

	virtual void SetTime_Exist(bool val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

	virtual int GetFractionsPerSecond() const = 0;

	virtual void SetFractionsPerSecond(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

	virtual int GetTimezoneSign() const = 0;

	virtual void SetTimezoneSign(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

	virtual int GetTimezoneHours() const = 0;

	virtual void SetTimezoneHours(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

	virtual int GetTimezoneMinutes() const = 0;

	virtual void SetTimezoneMinutes(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

	virtual bool GetTimezone_Exist() const = 0;

	virtual void SetTimezone_Exist(bool val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


// begin extension included
// file Mp7JrstimePointType_ExtMethodDef.h


public:

	// IMPORTANT:
	// Years and months ARE IGNORED now in all of the calculations!!!!

	/** Get total number of fractions.
	  * @return Get the total number of fractions with respect to the timepoint's number of fractions/second. */
	virtual int GetTotalNrFractions();

	/** Set total number of fractions.
	  * @param fractions Total number of fractions with respect to the timepoint's number of fractions/second. */
	virtual void SetTotalNrFractions(int fractions, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);
	
	/** Add a duration to this timepoint */
	virtual void Add(Mp7JrsdurationPtr duration, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	/** Subtract a timepoint from this one.
	  * @return The duration representing the difference of the two time points. */
	virtual Mp7JrsdurationPtr Subtract(Mp7JrstimePointPtr timePoint, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


// end extension included


};

/*
 * Generated pattern Mp7JrstimePointType<br>
 * Located at: mpeg7-2004.xsd, line 3493<br>
 * Regular expression: <br>
 * (\-?\d+(\-\d{2}(\-\d{2})?)?)?(T\d{2}(:\d{2}(:\d{2}(:\d+)?)?)?)?(F\d+)?((\-|\+)\d{2}:\d{2})?<br>
 */
class MP7JRS_EXPORT Mp7JrstimePointType :
	public IMp7JrstimePointType
{
	friend class Mp7JrstimePointTypeFactory;
	
public: 
	virtual int GetTimepointSign() const;

	virtual void SetTimepointSign(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	virtual int GetYear() const;

	virtual void SetYear(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	virtual int GetMonth() const;

	virtual void SetMonth(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	virtual int GetDay() const;

	virtual void SetDay(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	virtual bool GetDate_Exist() const;

	virtual void SetDate_Exist(bool val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	virtual int GetHours() const;

	virtual void SetHours(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	virtual int GetMinutes() const;

	virtual void SetMinutes(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	virtual int GetSeconds() const;

	virtual void SetSeconds(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	virtual int GetNumberOfFractions() const;

	virtual void SetNumberOfFractions(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	virtual bool GetTime_Exist() const;

	virtual void SetTime_Exist(bool val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	virtual int GetFractionsPerSecond() const;

	virtual void SetFractionsPerSecond(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	virtual int GetTimezoneSign() const;

	virtual void SetTimezoneSign(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	virtual int GetTimezoneHours() const;

	virtual void SetTimezoneHours(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	virtual int GetTimezoneMinutes() const;

	virtual void SetTimezoneMinutes(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	virtual bool GetTimezone_Exist() const;

	virtual void SetTimezone_Exist(bool val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


	virtual XMLCh * ToText() const;
	virtual bool Parse(const XMLCh * const txt);
	virtual bool ContentFromString(const XMLCh * const txt);

	virtual XMLCh * ContentToString() const;

	virtual void Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem = NULL);
	virtual bool Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent,  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current);

// no includefile for extension defined 
// file Mp7JrstimePointType_ExtMyMethodDef.h


	virtual ~Mp7JrstimePointType();

protected:
	Mp7JrstimePointType();
	virtual void DeepCopy(const Mp7JrsNodePtr &original);
	virtual Mp7JrsNodeEnum GetAllChildElements() const { return Mp7JrsNodeEnum(); }

	virtual Mp7JrsNodePtr GetBaseRoot();
	virtual const Mp7JrsNodePtr GetBaseRoot() const;

private:
	int m_TimepointSign;
	int m_Year;
	int m_Month;
	int m_Day;
	bool m_Date_Exist;
	int m_Hours;
	int m_Minutes;
	int m_Seconds;
	int m_NumberOfFractions;
	bool m_Time_Exist;
	int m_FractionsPerSecond;
	int m_TimezoneSign;
	int m_TimezoneHours;
	int m_TimezoneMinutes;
	bool m_Timezone_Exist;

// no includefile for extension defined 
// file Mp7JrstimePointType_ExtPropDef.h


protected:
	void Init();
	void Cleanup();
	
protected:
    const XMLCh * const m_nsURI; // For namespace handling
};

#endif // _2475MP7JRSTIMEPOINTTYPE_H

