
// The following macros define the minimum required platform.  The minimum required platform
// is the earliest version of Windows, Internet Explorer etc. that has the necessary features to run 
// your application.  The macros work by enabling all features available on platform versions up to and 
// including the version specified.

// Modify the following defines if you have to target a platform prior to the ones specified below.
// Refer to MSDN for the latest info on corresponding values for different platforms.
#ifndef _WIN32_WINNT            // Specifies that the minimum required platform is Windows Vista.
#define _WIN32_WINNT 0x0600     // Change this to the appropriate value to target other versions of Windows.
#endif

#include <stdio.h>
#include <xercesc/util/PlatformUtils.hpp>

#include "Mp7JrsEverything.h"
#include "Mp7JrsArchive.h"
#include "Mp7JrsException.h"

int main(int argc, char** argv)
{
	// Initialise Xerces
	XMLPlatformUtils::Initialize();
	// Initialise MPEG-7 JRS DAVP
	Mp7JrsFactory::Initialize();

	Mp7JrsArchive archive;

	archive.SetEncoding("UTF-8");
	archive.SetDoValidation(true);
	archive.SetSchemaLocation("urn:mpeg:mpeg7:schema:2004", "mpeg7", "davp-2009.xsd");

	try
	{
		Mp7JrsNodePtr root = archive.FromFile("test.xml");
	}
	catch(Mp7JrsException & e)
	{
		const char * msg = e.GetMessage();
		printf("ERROR: %s\n", msg);
	}
	catch(...)
	{
		printf("ERROR: %s\n", "Unknown error");
	}
	// Cleanup MPEG-7 JRS DAVP
	Mp7JrsFactory::Terminate();
	// Cleanup Xerces
	XMLPlatformUtils::Terminate();

	return 0;
}