 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsAudioBPMType_ExtImplInclude.h



#include "Mp7JrsAudioLLDScalarType.h"
#include "Mp7JrszeroToOneType.h"
#include "Mp7JrsAudioLLDScalarType_SeriesOfScalar_CollectionType.h"
#include "Mp7JrsintegerVector.h"
#include "Mp7JrsAudioBPMType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsAudioLLDScalarType_SeriesOfScalar_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:SeriesOfScalar

#include <assert.h>
IMp7JrsAudioBPMType::IMp7JrsAudioBPMType()
{

// no includefile for extension defined 
// file Mp7JrsAudioBPMType_ExtPropInit.h

}

IMp7JrsAudioBPMType::~IMp7JrsAudioBPMType()
{
// no includefile for extension defined 
// file Mp7JrsAudioBPMType_ExtPropCleanup.h

}

Mp7JrsAudioBPMType::Mp7JrsAudioBPMType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsAudioBPMType::~Mp7JrsAudioBPMType()
{
	Cleanup();
}

void Mp7JrsAudioBPMType::Init()
{
	// Init base
	m_Base = CreateAudioLLDScalarType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_loLimit = 0.0; // Value
	m_loLimit_Exist = false;
	m_hiLimit = 0.0; // Value
	m_hiLimit_Exist = false;



// no includefile for extension defined 
// file Mp7JrsAudioBPMType_ExtMyPropInit.h

}

void Mp7JrsAudioBPMType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsAudioBPMType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
}

void Mp7JrsAudioBPMType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use AudioBPMTypePtr, since we
	// might need GetBase(), which isn't defined in IAudioBPMType
    const Mp7JrsPtr< Mp7JrsAudioBPMType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsAudioLLDScalarPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsAudioBPMType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	if (tmp->ExistloLimit())
	{
		this->SetloLimit(tmp->GetloLimit());
	}
	else
	{
		InvalidateloLimit();
	}
	}
	{
	if (tmp->ExisthiLimit())
	{
		this->SethiLimit(tmp->GethiLimit());
	}
	else
	{
		InvalidatehiLimit();
	}
	}
}

Mp7JrsNodePtr Mp7JrsAudioBPMType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsAudioBPMType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsAudioLLDScalarType > Mp7JrsAudioBPMType::GetBase() const
{
	return m_Base;
}

float Mp7JrsAudioBPMType::GetloLimit() const
{
	return m_loLimit;
}

bool Mp7JrsAudioBPMType::ExistloLimit() const
{
	return m_loLimit_Exist;
}
void Mp7JrsAudioBPMType::SetloLimit(float item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioBPMType::SetloLimit().");
	}
    m_loLimit = item;
	m_loLimit_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsAudioBPMType::InvalidateloLimit()
{
	m_loLimit_Exist = false;
}
float Mp7JrsAudioBPMType::GethiLimit() const
{
	return m_hiLimit;
}

bool Mp7JrsAudioBPMType::ExisthiLimit() const
{
	return m_hiLimit_Exist;
}
void Mp7JrsAudioBPMType::SethiLimit(float item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioBPMType::SethiLimit().");
	}
    m_hiLimit = item;
	m_hiLimit_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsAudioBPMType::InvalidatehiLimit()
{
	m_hiLimit_Exist = false;
}
Mp7JrszeroToOnePtr Mp7JrsAudioBPMType::Getconfidence() const
{
	return GetBase()->Getconfidence();
}

bool Mp7JrsAudioBPMType::Existconfidence() const
{
	return GetBase()->Existconfidence();
}
void Mp7JrsAudioBPMType::Setconfidence(const Mp7JrszeroToOnePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioBPMType::Setconfidence().");
	}
	GetBase()->Setconfidence(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsAudioBPMType::Invalidateconfidence()
{
	GetBase()->Invalidateconfidence();
}
float Mp7JrsAudioBPMType::GetScalar() const
{
	return GetBase()->GetScalar();
}

Mp7JrsAudioLLDScalarType_SeriesOfScalar_CollectionPtr Mp7JrsAudioBPMType::GetSeriesOfScalar() const
{
	return GetBase()->GetSeriesOfScalar();
}

// implementing setter for choice 
/* element */
void Mp7JrsAudioBPMType::SetScalar(float item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioBPMType::SetScalar().");
	}
	GetBase()->SetScalar(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsAudioBPMType::SetSeriesOfScalar(const Mp7JrsAudioLLDScalarType_SeriesOfScalar_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioBPMType::SetSeriesOfScalar().");
	}
	GetBase()->SetSeriesOfScalar(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsintegerVectorPtr Mp7JrsAudioBPMType::Getchannels() const
{
	return GetBase()->GetBase()->Getchannels();
}

bool Mp7JrsAudioBPMType::Existchannels() const
{
	return GetBase()->GetBase()->Existchannels();
}
void Mp7JrsAudioBPMType::Setchannels(const Mp7JrsintegerVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioBPMType::Setchannels().");
	}
	GetBase()->GetBase()->Setchannels(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsAudioBPMType::Invalidatechannels()
{
	GetBase()->GetBase()->Invalidatechannels();
}

Mp7JrsNodeEnum Mp7JrsAudioBPMType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for AudioBPMType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "AudioBPMType");
	}		
	
	return result;
}

XMLCh * Mp7JrsAudioBPMType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsAudioBPMType::Parse(const XMLCh * const txt)
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return false;
}

bool Mp7JrsAudioBPMType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsAudioBPMType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsAudioBPMType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsAudioBPMType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("AudioBPMType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_loLimit_Exist)
	{
	// Value
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_loLimit);
		element->setAttributeNS(X(""), X("loLimit"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_hiLimit_Exist)
	{
	// Value
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_hiLimit);
		element->setAttributeNS(X(""), X("hiLimit"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:

}

bool Mp7JrsAudioBPMType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("loLimit")))
	{
        // deserialize value type
        this->SetloLimit(Mp7JrsConvert::TextToFloat(parent->getAttributeNS(X(""), X("loLimit"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("loLimit")))
	{
        // deserialize value type
        this->SetloLimit(Mp7JrsConvert::TextToFloat(parent->getAttribute(X("loLimit"))));
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("hiLimit")))
	{
        // deserialize value type
        this->SethiLimit(Mp7JrsConvert::TextToFloat(parent->getAttributeNS(X(""), X("hiLimit"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("hiLimit")))
	{
        // deserialize value type
        this->SethiLimit(Mp7JrsConvert::TextToFloat(parent->getAttribute(X("hiLimit"))));
        * current = parent;
    }

	// Extensionbase is Mp7JrsAudioLLDScalarType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
// no includefile for extension defined 
// file Mp7JrsAudioBPMType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsAudioBPMType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsAudioBPMType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetSeriesOfScalar() != Mp7JrsNodePtr())
		result.Insert(GetSeriesOfScalar());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsAudioBPMType_ExtMethodImpl.h


