 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsAudioLLDVectorType_ExtImplInclude.h



#include "Mp7JrsAudioDType.h"
#include "Mp7JrsfloatVector.h"
#include "Mp7JrsAudioLLDVectorType_SeriesOfVector_CollectionType.h"
#include "Mp7JrsintegerVector.h"
#include "Mp7JrsAudioLLDVectorType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsAudioLLDVectorType_SeriesOfVector_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:SeriesOfVector

#include <assert.h>
IMp7JrsAudioLLDVectorType::IMp7JrsAudioLLDVectorType()
{

// no includefile for extension defined 
// file Mp7JrsAudioLLDVectorType_ExtPropInit.h

}

IMp7JrsAudioLLDVectorType::~IMp7JrsAudioLLDVectorType()
{
// no includefile for extension defined 
// file Mp7JrsAudioLLDVectorType_ExtPropCleanup.h

}

Mp7JrsAudioLLDVectorType::Mp7JrsAudioLLDVectorType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsAudioLLDVectorType::~Mp7JrsAudioLLDVectorType()
{
	Cleanup();
}

void Mp7JrsAudioLLDVectorType::Init()
{
	// Init base
	m_Base = CreateAudioDType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_Vector = Mp7JrsfloatVectorPtr(); // Collection
	m_SeriesOfVector = Mp7JrsAudioLLDVectorType_SeriesOfVector_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsAudioLLDVectorType_ExtMyPropInit.h

}

void Mp7JrsAudioLLDVectorType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsAudioLLDVectorType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_Vector);
	// Mp7JrsFactory::DeleteObject(m_SeriesOfVector);
}

void Mp7JrsAudioLLDVectorType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use AudioLLDVectorTypePtr, since we
	// might need GetBase(), which isn't defined in IAudioLLDVectorType
    const Mp7JrsPtr< Mp7JrsAudioLLDVectorType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsAudioDPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsAudioLLDVectorType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_Vector);
		this->SetVector(Mp7JrsFactory::CloneObject(tmp->GetVector()));
		// Mp7JrsFactory::DeleteObject(m_SeriesOfVector);
		this->SetSeriesOfVector(Mp7JrsFactory::CloneObject(tmp->GetSeriesOfVector()));
}

Mp7JrsNodePtr Mp7JrsAudioLLDVectorType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsAudioLLDVectorType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsAudioDType > Mp7JrsAudioLLDVectorType::GetBase() const
{
	return m_Base;
}

Mp7JrsfloatVectorPtr Mp7JrsAudioLLDVectorType::GetVector() const
{
	return m_Vector;
}

Mp7JrsAudioLLDVectorType_SeriesOfVector_CollectionPtr Mp7JrsAudioLLDVectorType::GetSeriesOfVector() const
{
	return m_SeriesOfVector;
}

// implementing setter for choice 
/* element */
void Mp7JrsAudioLLDVectorType::SetVector(const Mp7JrsfloatVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioLLDVectorType::SetVector().");
	}
	if (m_Vector != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Vector);
		m_Vector = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Vector) m_Vector->SetParent(m_myself.getPointer());
		if(m_Vector != Mp7JrsfloatVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Vector->SetContentName(XMLString::transcode("Vector"));
		}
	// Mp7JrsFactory::DeleteObject(m_SeriesOfVector);
	m_SeriesOfVector = Mp7JrsAudioLLDVectorType_SeriesOfVector_CollectionPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsAudioLLDVectorType::SetSeriesOfVector(const Mp7JrsAudioLLDVectorType_SeriesOfVector_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioLLDVectorType::SetSeriesOfVector().");
	}
	if (m_SeriesOfVector != item)
	{
		// Mp7JrsFactory::DeleteObject(m_SeriesOfVector);
		m_SeriesOfVector = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_SeriesOfVector) m_SeriesOfVector->SetParent(m_myself.getPointer());
		if(m_SeriesOfVector != Mp7JrsAudioLLDVectorType_SeriesOfVector_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_SeriesOfVector->SetContentName(XMLString::transcode("SeriesOfVector"));
		}
	// Mp7JrsFactory::DeleteObject(m_Vector);
	m_Vector = Mp7JrsfloatVectorPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsintegerVectorPtr Mp7JrsAudioLLDVectorType::Getchannels() const
{
	return GetBase()->Getchannels();
}

bool Mp7JrsAudioLLDVectorType::Existchannels() const
{
	return GetBase()->Existchannels();
}
void Mp7JrsAudioLLDVectorType::Setchannels(const Mp7JrsintegerVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioLLDVectorType::Setchannels().");
	}
	GetBase()->Setchannels(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsAudioLLDVectorType::Invalidatechannels()
{
	GetBase()->Invalidatechannels();
}

Mp7JrsNodeEnum Mp7JrsAudioLLDVectorType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Vector")) == 0)
	{
		// Vector is simple element floatVector
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetVector()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:floatVector")))) != empty)
			{
				// Is type allowed
				Mp7JrsfloatVectorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetVector(p, client);
					if((p = GetVector()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("SeriesOfVector")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:SeriesOfVector is item of type AudioLLDVectorType_SeriesOfVector_LocalType
		// in element collection AudioLLDVectorType_SeriesOfVector_CollectionType
		
		context->Found = true;
		Mp7JrsAudioLLDVectorType_SeriesOfVector_CollectionPtr coll = GetSeriesOfVector();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateAudioLLDVectorType_SeriesOfVector_CollectionType;
				SetSeriesOfVector(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:AudioLLDVectorType_SeriesOfVector_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsAudioLLDVectorType_SeriesOfVector_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for AudioLLDVectorType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "AudioLLDVectorType");
	}		
	
	return result;
}

XMLCh * Mp7JrsAudioLLDVectorType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsAudioLLDVectorType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}

bool Mp7JrsAudioLLDVectorType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsAudioLLDVectorType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsAudioLLDVectorType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsAudioLLDVectorType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("AudioLLDVectorType"));
	// Element serialization:
	if (m_Vector != Mp7JrsfloatVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Vector->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_SeriesOfVector != Mp7JrsAudioLLDVectorType_SeriesOfVector_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_SeriesOfVector->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsAudioLLDVectorType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsAudioDType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
	do // Deserialize choice just one time!
    {
        // Deserialise Element ValCollectionType
        if((parent != NULL) && (XMLString::compareString(parent->getNodeName()
			, X("Vector")) == 0))
        {
            // Deserialize factory type
            Mp7JrsfloatVectorPtr tmp = CreatefloatVector;
            tmp->Deserialize(doc, parent, current);
            this->SetVector(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("SeriesOfVector"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("SeriesOfVector")) == 0))
        {
            // Deserialize factory type
            Mp7JrsAudioLLDVectorType_SeriesOfVector_CollectionPtr tmp = CreateAudioLLDVectorType_SeriesOfVector_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetSeriesOfVector(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
// no includefile for extension defined 
// file Mp7JrsAudioLLDVectorType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsAudioLLDVectorType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Vector")) == 0))
  {
    Mp7JrsfloatVectorPtr tmp = CreatefloatVector;
    this->SetVector(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("SeriesOfVector")) == 0))
  {
    Mp7JrsAudioLLDVectorType_SeriesOfVector_CollectionPtr tmp = CreateAudioLLDVectorType_SeriesOfVector_CollectionType;
    this->SetSeriesOfVector(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsAudioLLDVectorType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetVector() != Mp7JrsNodePtr())
		result.Insert(GetVector());
	if (GetSeriesOfVector() != Mp7JrsNodePtr())
		result.Insert(GetSeriesOfVector());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsAudioLLDVectorType_ExtMethodImpl.h


