 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsAudioSegmentType_LocalType0_ExtImplInclude.h



#include "Mp7JrsAudioSegmentTemporalDecompositionType.h"
#include "Mp7JrsAudioSegmentMediaSourceDecompositionType.h"
#include "Mp7JrsAudioSegmentType_LocalType0.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"

#include <assert.h>
IMp7JrsAudioSegmentType_LocalType0::IMp7JrsAudioSegmentType_LocalType0()
{

// no includefile for extension defined 
// file Mp7JrsAudioSegmentType_LocalType0_ExtPropInit.h

}

IMp7JrsAudioSegmentType_LocalType0::~IMp7JrsAudioSegmentType_LocalType0()
{
// no includefile for extension defined 
// file Mp7JrsAudioSegmentType_LocalType0_ExtPropCleanup.h

}

Mp7JrsAudioSegmentType_LocalType0::Mp7JrsAudioSegmentType_LocalType0()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsAudioSegmentType_LocalType0::~Mp7JrsAudioSegmentType_LocalType0()
{
	Cleanup();
}

void Mp7JrsAudioSegmentType_LocalType0::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_TemporalDecomposition = Mp7JrsAudioSegmentTemporalDecompositionPtr(); // Class
	m_MediaSourceDecomposition = Mp7JrsAudioSegmentMediaSourceDecompositionPtr(); // Class


// no includefile for extension defined 
// file Mp7JrsAudioSegmentType_LocalType0_ExtMyPropInit.h

}

void Mp7JrsAudioSegmentType_LocalType0::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsAudioSegmentType_LocalType0_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_TemporalDecomposition);
	// Mp7JrsFactory::DeleteObject(m_MediaSourceDecomposition);
}

void Mp7JrsAudioSegmentType_LocalType0::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use AudioSegmentType_LocalType0Ptr, since we
	// might need GetBase(), which isn't defined in IAudioSegmentType_LocalType0
    const Mp7JrsPtr< Mp7JrsAudioSegmentType_LocalType0 > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		// Mp7JrsFactory::DeleteObject(m_TemporalDecomposition);
		this->SetTemporalDecomposition(Mp7JrsFactory::CloneObject(tmp->GetTemporalDecomposition()));
		// Mp7JrsFactory::DeleteObject(m_MediaSourceDecomposition);
		this->SetMediaSourceDecomposition(Mp7JrsFactory::CloneObject(tmp->GetMediaSourceDecomposition()));
}

Mp7JrsNodePtr Mp7JrsAudioSegmentType_LocalType0::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsAudioSegmentType_LocalType0::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsAudioSegmentTemporalDecompositionPtr Mp7JrsAudioSegmentType_LocalType0::GetTemporalDecomposition() const
{
	return m_TemporalDecomposition;
}

Mp7JrsAudioSegmentMediaSourceDecompositionPtr Mp7JrsAudioSegmentType_LocalType0::GetMediaSourceDecomposition() const
{
	return m_MediaSourceDecomposition;
}

// implementing setter for choice 
/* element */
void Mp7JrsAudioSegmentType_LocalType0::SetTemporalDecomposition(const Mp7JrsAudioSegmentTemporalDecompositionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioSegmentType_LocalType0::SetTemporalDecomposition().");
	}
	if (m_TemporalDecomposition != item)
	{
		// Mp7JrsFactory::DeleteObject(m_TemporalDecomposition);
		m_TemporalDecomposition = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_TemporalDecomposition) m_TemporalDecomposition->SetParent(m_myself.getPointer());
		if (m_TemporalDecomposition && m_TemporalDecomposition->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:AudioSegmentTemporalDecompositionType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_TemporalDecomposition->UseTypeAttribute = true;
		}
		if(m_TemporalDecomposition != Mp7JrsAudioSegmentTemporalDecompositionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_TemporalDecomposition->SetContentName(XMLString::transcode("TemporalDecomposition"));
		}
	// Mp7JrsFactory::DeleteObject(m_MediaSourceDecomposition);
	m_MediaSourceDecomposition = Mp7JrsAudioSegmentMediaSourceDecompositionPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsAudioSegmentType_LocalType0::SetMediaSourceDecomposition(const Mp7JrsAudioSegmentMediaSourceDecompositionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioSegmentType_LocalType0::SetMediaSourceDecomposition().");
	}
	if (m_MediaSourceDecomposition != item)
	{
		// Mp7JrsFactory::DeleteObject(m_MediaSourceDecomposition);
		m_MediaSourceDecomposition = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_MediaSourceDecomposition) m_MediaSourceDecomposition->SetParent(m_myself.getPointer());
		if (m_MediaSourceDecomposition && m_MediaSourceDecomposition->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:AudioSegmentMediaSourceDecompositionType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_MediaSourceDecomposition->UseTypeAttribute = true;
		}
		if(m_MediaSourceDecomposition != Mp7JrsAudioSegmentMediaSourceDecompositionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_MediaSourceDecomposition->SetContentName(XMLString::transcode("MediaSourceDecomposition"));
		}
	// Mp7JrsFactory::DeleteObject(m_TemporalDecomposition);
	m_TemporalDecomposition = Mp7JrsAudioSegmentTemporalDecompositionPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

// Type: AudioSegmentType_LocalType0: Only types with an element name can have GetOrCreate()
// Mp7JrsNodeEnum Mp7JrsAudioSegmentType_LocalType0::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
//

XMLCh * Mp7JrsAudioSegmentType_LocalType0::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsAudioSegmentType_LocalType0::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsAudioSegmentType_LocalType0::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// Serialize Choice of Sequence collection item
	element = parent;


	// Element serialization:
	// Class
	
	if (m_TemporalDecomposition != Mp7JrsAudioSegmentTemporalDecompositionPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_TemporalDecomposition->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("TemporalDecomposition"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_TemporalDecomposition->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_MediaSourceDecomposition != Mp7JrsAudioSegmentMediaSourceDecompositionPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_MediaSourceDecomposition->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("MediaSourceDecomposition"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_MediaSourceDecomposition->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsAudioSegmentType_LocalType0::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	do // Deserialize choice just one time!
    {
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("TemporalDecomposition"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("TemporalDecomposition")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateAudioSegmentTemporalDecompositionType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetTemporalDecomposition(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("MediaSourceDecomposition"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("MediaSourceDecomposition")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateAudioSegmentMediaSourceDecompositionType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetMediaSourceDecomposition(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
// no includefile for extension defined 
// file Mp7JrsAudioSegmentType_LocalType0_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsAudioSegmentType_LocalType0::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("TemporalDecomposition")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateAudioSegmentTemporalDecompositionType;
    }
    this->SetTemporalDecomposition(child);
  }
  if (XMLString::compareString(elementname, X("MediaSourceDecomposition")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateAudioSegmentMediaSourceDecompositionType;
    }
    this->SetMediaSourceDecomposition(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsAudioSegmentType_LocalType0::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetTemporalDecomposition() != Mp7JrsNodePtr())
		result.Insert(GetTemporalDecomposition());
	if (GetMediaSourceDecomposition() != Mp7JrsNodePtr())
		result.Insert(GetMediaSourceDecomposition());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsAudioSegmentType_LocalType0_ExtMethodImpl.h


