 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsColorTemperatureType_ExtImplInclude.h



#include "Mp7JrsVisualDType.h"
#include "Mp7Jrsunsigned6.h"
#include "Mp7JrsColorTemperatureType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsColorTemperatureType::IMp7JrsColorTemperatureType()
{

// no includefile for extension defined 
// file Mp7JrsColorTemperatureType_ExtPropInit.h

}

IMp7JrsColorTemperatureType::~IMp7JrsColorTemperatureType()
{
// no includefile for extension defined 
// file Mp7JrsColorTemperatureType_ExtPropCleanup.h

}

Mp7JrsColorTemperatureType::Mp7JrsColorTemperatureType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsColorTemperatureType::~Mp7JrsColorTemperatureType()
{
	Cleanup();
}

void Mp7JrsColorTemperatureType::Init()
{
	// Init base
	m_Base = CreateVisualDType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_BrowsingCategory = Mp7JrsColorTemperatureType_BrowsingCategory_LocalType::UninitializedEnumeration; // Enumeration
	m_SubRangeIndex = Mp7Jrsunsigned6Ptr(); // Class with content 


// no includefile for extension defined 
// file Mp7JrsColorTemperatureType_ExtMyPropInit.h

}

void Mp7JrsColorTemperatureType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsColorTemperatureType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_SubRangeIndex);
}

void Mp7JrsColorTemperatureType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use ColorTemperatureTypePtr, since we
	// might need GetBase(), which isn't defined in IColorTemperatureType
    const Mp7JrsPtr< Mp7JrsColorTemperatureType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsVisualDPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsColorTemperatureType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		this->SetBrowsingCategory(tmp->GetBrowsingCategory());
		// Mp7JrsFactory::DeleteObject(m_SubRangeIndex);
		this->SetSubRangeIndex(Mp7JrsFactory::CloneObject(tmp->GetSubRangeIndex()));
}

Mp7JrsNodePtr Mp7JrsColorTemperatureType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsColorTemperatureType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsVisualDType > Mp7JrsColorTemperatureType::GetBase() const
{
	return m_Base;
}

Mp7JrsColorTemperatureType_BrowsingCategory_LocalType::Enumeration Mp7JrsColorTemperatureType::GetBrowsingCategory() const
{
	return m_BrowsingCategory;
}

Mp7Jrsunsigned6Ptr Mp7JrsColorTemperatureType::GetSubRangeIndex() const
{
	return m_SubRangeIndex;
}

void Mp7JrsColorTemperatureType::SetBrowsingCategory(Mp7JrsColorTemperatureType_BrowsingCategory_LocalType::Enumeration item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsColorTemperatureType::SetBrowsingCategory().");
	}
	if (m_BrowsingCategory != item)
	{
		// nothing to free here, hopefully
		m_BrowsingCategory = item;
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsColorTemperatureType::SetSubRangeIndex(const Mp7Jrsunsigned6Ptr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsColorTemperatureType::SetSubRangeIndex().");
	}
	if (m_SubRangeIndex != item)
	{
		// Mp7JrsFactory::DeleteObject(m_SubRangeIndex);
		m_SubRangeIndex = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_SubRangeIndex) m_SubRangeIndex->SetParent(m_myself.getPointer());
		if (m_SubRangeIndex && m_SubRangeIndex->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:unsigned6"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_SubRangeIndex->UseTypeAttribute = true;
		}
		if(m_SubRangeIndex != Mp7Jrsunsigned6Ptr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_SubRangeIndex->SetContentName(XMLString::transcode("SubRangeIndex"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsColorTemperatureType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("SubRangeIndex")) == 0)
	{
		// SubRangeIndex is simple element unsigned6
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetSubRangeIndex()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:unsigned6")))) != empty)
			{
				// Is type allowed
				Mp7Jrsunsigned6Ptr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetSubRangeIndex(p, client);
					if((p = GetSubRangeIndex()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for ColorTemperatureType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "ColorTemperatureType");
	}		
	
	return result;
}

XMLCh * Mp7JrsColorTemperatureType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsColorTemperatureType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}

bool Mp7JrsColorTemperatureType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsColorTemperatureType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsColorTemperatureType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsColorTemperatureType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("ColorTemperatureType"));
	// Element serialization:
	if(m_BrowsingCategory != Mp7JrsColorTemperatureType_BrowsingCategory_LocalType::UninitializedEnumeration) // Enumeration
	{
		XMLCh * tmp = Mp7JrsColorTemperatureType_BrowsingCategory_LocalType::ToText(m_BrowsingCategory);
		Mp7JrsUtil::SerializeTextNode(element, doc, tmp, X("urn:mpeg:mpeg7:schema:2004"), X("BrowsingCategory"), false);
		XMLString::release(&tmp);
	}
	// Class with content		
	
	if (m_SubRangeIndex != Mp7Jrsunsigned6Ptr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_SubRangeIndex->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("SubRangeIndex"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_SubRangeIndex->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsColorTemperatureType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsVisualDType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Deserialize enumeration element
	if(Mp7JrsUtil::HasNodeName(parent, X("BrowsingCategory"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("BrowsingCategory")) == 0))
        {
            Mp7JrsColorTemperatureType_BrowsingCategory_LocalType::Enumeration tmp = Mp7JrsColorTemperatureType_BrowsingCategory_LocalType::Parse(Mp7JrsUtil::GetElementText(parent));
            if(tmp == Mp7JrsColorTemperatureType_BrowsingCategory_LocalType::UninitializedEnumeration)
            {
                    return false;
            }
            this->SetBrowsingCategory(tmp);
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("SubRangeIndex"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("SubRangeIndex")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = Createunsigned6;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetSubRangeIndex(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsColorTemperatureType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsColorTemperatureType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("SubRangeIndex")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = Createunsigned6;
    }
    this->SetSubRangeIndex(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsColorTemperatureType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetSubRangeIndex() != Mp7JrsNodePtr())
		result.Insert(GetSubRangeIndex());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsColorTemperatureType_ExtMethodImpl.h


