 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsCompleteDescriptionType_ExtImplInclude.h



#include "Mp7JrsDescriptionMetadataType.h"
#include "Mp7JrsCompleteDescriptionType_Relationships_CollectionType.h"
#include "Mp7JrsCompleteDescriptionType_OrderingKey_CollectionType.h"
#include "Mp7JrsCompleteDescriptionType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsGraphType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Relationships
#include "Mp7JrsOrderingKeyType.h" // Element collection urn:mpeg:mpeg7:schema:2004:OrderingKey

#include <assert.h>
IMp7JrsCompleteDescriptionType::IMp7JrsCompleteDescriptionType()
{

// no includefile for extension defined 
// file Mp7JrsCompleteDescriptionType_ExtPropInit.h

}

IMp7JrsCompleteDescriptionType::~IMp7JrsCompleteDescriptionType()
{
// no includefile for extension defined 
// file Mp7JrsCompleteDescriptionType_ExtPropCleanup.h

}

Mp7JrsCompleteDescriptionType::Mp7JrsCompleteDescriptionType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsCompleteDescriptionType::~Mp7JrsCompleteDescriptionType()
{
	Cleanup();
}

void Mp7JrsCompleteDescriptionType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_DescriptionMetadata = Mp7JrsDescriptionMetadataPtr(); // Class
	m_DescriptionMetadata_Exist = false;
	m_Relationships = Mp7JrsCompleteDescriptionType_Relationships_CollectionPtr(); // Collection
	m_OrderingKey = Mp7JrsCompleteDescriptionType_OrderingKey_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsCompleteDescriptionType_ExtMyPropInit.h

}

void Mp7JrsCompleteDescriptionType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsCompleteDescriptionType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_DescriptionMetadata);
	// Mp7JrsFactory::DeleteObject(m_Relationships);
	// Mp7JrsFactory::DeleteObject(m_OrderingKey);
}

void Mp7JrsCompleteDescriptionType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use CompleteDescriptionTypePtr, since we
	// might need GetBase(), which isn't defined in ICompleteDescriptionType
    const Mp7JrsPtr< Mp7JrsCompleteDescriptionType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	if (tmp->IsValidDescriptionMetadata())
	{
		// Mp7JrsFactory::DeleteObject(m_DescriptionMetadata);
		this->SetDescriptionMetadata(Mp7JrsFactory::CloneObject(tmp->GetDescriptionMetadata()));
	}
	else
	{
		InvalidateDescriptionMetadata();
	}
		// Mp7JrsFactory::DeleteObject(m_Relationships);
		this->SetRelationships(Mp7JrsFactory::CloneObject(tmp->GetRelationships()));
		// Mp7JrsFactory::DeleteObject(m_OrderingKey);
		this->SetOrderingKey(Mp7JrsFactory::CloneObject(tmp->GetOrderingKey()));
}

Mp7JrsNodePtr Mp7JrsCompleteDescriptionType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsCompleteDescriptionType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsDescriptionMetadataPtr Mp7JrsCompleteDescriptionType::GetDescriptionMetadata() const
{
	return m_DescriptionMetadata;
}

// Element is optional
bool Mp7JrsCompleteDescriptionType::IsValidDescriptionMetadata() const
{
	return m_DescriptionMetadata_Exist;
}

Mp7JrsCompleteDescriptionType_Relationships_CollectionPtr Mp7JrsCompleteDescriptionType::GetRelationships() const
{
	return m_Relationships;
}

Mp7JrsCompleteDescriptionType_OrderingKey_CollectionPtr Mp7JrsCompleteDescriptionType::GetOrderingKey() const
{
	return m_OrderingKey;
}

void Mp7JrsCompleteDescriptionType::SetDescriptionMetadata(const Mp7JrsDescriptionMetadataPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCompleteDescriptionType::SetDescriptionMetadata().");
	}
	if (m_DescriptionMetadata != item || m_DescriptionMetadata_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_DescriptionMetadata);
		m_DescriptionMetadata = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_DescriptionMetadata) m_DescriptionMetadata->SetParent(m_myself.getPointer());
		if (m_DescriptionMetadata && m_DescriptionMetadata->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:DescriptionMetadataType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_DescriptionMetadata->UseTypeAttribute = true;
		}
		m_DescriptionMetadata_Exist = true;
		if(m_DescriptionMetadata != Mp7JrsDescriptionMetadataPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_DescriptionMetadata->SetContentName(XMLString::transcode("DescriptionMetadata"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsCompleteDescriptionType::InvalidateDescriptionMetadata()
{
	m_DescriptionMetadata_Exist = false;
}
void Mp7JrsCompleteDescriptionType::SetRelationships(const Mp7JrsCompleteDescriptionType_Relationships_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCompleteDescriptionType::SetRelationships().");
	}
	if (m_Relationships != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Relationships);
		m_Relationships = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Relationships) m_Relationships->SetParent(m_myself.getPointer());
		if(m_Relationships != Mp7JrsCompleteDescriptionType_Relationships_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Relationships->SetContentName(XMLString::transcode("Relationships"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsCompleteDescriptionType::SetOrderingKey(const Mp7JrsCompleteDescriptionType_OrderingKey_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCompleteDescriptionType::SetOrderingKey().");
	}
	if (m_OrderingKey != item)
	{
		// Mp7JrsFactory::DeleteObject(m_OrderingKey);
		m_OrderingKey = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_OrderingKey) m_OrderingKey->SetParent(m_myself.getPointer());
		if(m_OrderingKey != Mp7JrsCompleteDescriptionType_OrderingKey_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_OrderingKey->SetContentName(XMLString::transcode("OrderingKey"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsCompleteDescriptionType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("DescriptionMetadata")) == 0)
	{
		// DescriptionMetadata is simple element DescriptionMetadataType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetDescriptionMetadata()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:DescriptionMetadataType")))) != empty)
			{
				// Is type allowed
				Mp7JrsDescriptionMetadataPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetDescriptionMetadata(p, client);
					if((p = GetDescriptionMetadata()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Relationships")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Relationships is item of type GraphType
		// in element collection CompleteDescriptionType_Relationships_CollectionType
		
		context->Found = true;
		Mp7JrsCompleteDescriptionType_Relationships_CollectionPtr coll = GetRelationships();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateCompleteDescriptionType_Relationships_CollectionType;
				SetRelationships(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:GraphType")))) != empty)
			{
				// Is type allowed
				Mp7JrsGraphPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("OrderingKey")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:OrderingKey is item of type OrderingKeyType
		// in element collection CompleteDescriptionType_OrderingKey_CollectionType
		
		context->Found = true;
		Mp7JrsCompleteDescriptionType_OrderingKey_CollectionPtr coll = GetOrderingKey();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateCompleteDescriptionType_OrderingKey_CollectionType;
				SetOrderingKey(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:OrderingKeyType")))) != empty)
			{
				// Is type allowed
				Mp7JrsOrderingKeyPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for CompleteDescriptionType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "CompleteDescriptionType");
	}		
	
	return result;
}

XMLCh * Mp7JrsCompleteDescriptionType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsCompleteDescriptionType::Parse(const XMLCh * const txt)
{
	return false;
}

bool Mp7JrsCompleteDescriptionType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	// Mp7JrsCompleteDescriptionType has no attributes so forward it to Parse()
	return Parse(txt);
}
XMLCh* Mp7JrsCompleteDescriptionType::ContentToString() const
{
	// Mp7JrsCompleteDescriptionType has no attributes so forward it to ToText()
	return ToText();
}

void Mp7JrsCompleteDescriptionType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("CompleteDescriptionType"));
	// Element serialization:
	// Class
	
	if (m_DescriptionMetadata != Mp7JrsDescriptionMetadataPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_DescriptionMetadata->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("DescriptionMetadata"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_DescriptionMetadata->Serialize(doc, element, &elem);
	}
	if (m_Relationships != Mp7JrsCompleteDescriptionType_Relationships_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Relationships->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_OrderingKey != Mp7JrsCompleteDescriptionType_OrderingKey_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_OrderingKey->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsCompleteDescriptionType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	parent = Mp7JrsUtil::GetNextChild(parent);
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("DescriptionMetadata"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("DescriptionMetadata")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateDescriptionMetadataType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetDescriptionMetadata(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Relationships"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Relationships")) == 0))
        {
            // Deserialize factory type
            Mp7JrsCompleteDescriptionType_Relationships_CollectionPtr tmp = CreateCompleteDescriptionType_Relationships_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetRelationships(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("OrderingKey"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("OrderingKey")) == 0))
        {
            // Deserialize factory type
            Mp7JrsCompleteDescriptionType_OrderingKey_CollectionPtr tmp = CreateCompleteDescriptionType_OrderingKey_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetOrderingKey(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsCompleteDescriptionType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsCompleteDescriptionType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("DescriptionMetadata")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateDescriptionMetadataType;
    }
    this->SetDescriptionMetadata(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Relationships")) == 0))
  {
    Mp7JrsCompleteDescriptionType_Relationships_CollectionPtr tmp = CreateCompleteDescriptionType_Relationships_CollectionType;
    this->SetRelationships(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("OrderingKey")) == 0))
  {
    Mp7JrsCompleteDescriptionType_OrderingKey_CollectionPtr tmp = CreateCompleteDescriptionType_OrderingKey_CollectionType;
    this->SetOrderingKey(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsCompleteDescriptionType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetDescriptionMetadata() != Mp7JrsNodePtr())
		result.Insert(GetDescriptionMetadata());
	if (GetRelationships() != Mp7JrsNodePtr())
		result.Insert(GetRelationships());
	if (GetOrderingKey() != Mp7JrsNodePtr())
		result.Insert(GetOrderingKey());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsCompleteDescriptionType_ExtMethodImpl.h


