 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsCreationDescriptionType_ExtImplInclude.h



#include "Mp7JrsContentManagementType.h"
#include "Mp7JrsCreationDescriptionType_CreationInformation_CollectionType.h"
#include "Mp7JrsDescriptionMetadataType.h"
#include "Mp7JrsCompleteDescriptionType_Relationships_CollectionType.h"
#include "Mp7JrsCompleteDescriptionType_OrderingKey_CollectionType.h"
#include "Mp7JrsCreationDescriptionType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsGraphType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Relationships
#include "Mp7JrsOrderingKeyType.h" // Element collection urn:mpeg:mpeg7:schema:2004:OrderingKey
#include "Mp7JrsCreationInformationType.h" // Element collection urn:mpeg:mpeg7:schema:2004:CreationInformation

#include <assert.h>
IMp7JrsCreationDescriptionType::IMp7JrsCreationDescriptionType()
{

// no includefile for extension defined 
// file Mp7JrsCreationDescriptionType_ExtPropInit.h

}

IMp7JrsCreationDescriptionType::~IMp7JrsCreationDescriptionType()
{
// no includefile for extension defined 
// file Mp7JrsCreationDescriptionType_ExtPropCleanup.h

}

Mp7JrsCreationDescriptionType::Mp7JrsCreationDescriptionType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsCreationDescriptionType::~Mp7JrsCreationDescriptionType()
{
	Cleanup();
}

void Mp7JrsCreationDescriptionType::Init()
{
	// Init base
	m_Base = CreateContentManagementType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_CreationInformation = Mp7JrsCreationDescriptionType_CreationInformation_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsCreationDescriptionType_ExtMyPropInit.h

}

void Mp7JrsCreationDescriptionType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsCreationDescriptionType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_CreationInformation);
}

void Mp7JrsCreationDescriptionType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use CreationDescriptionTypePtr, since we
	// might need GetBase(), which isn't defined in ICreationDescriptionType
    const Mp7JrsPtr< Mp7JrsCreationDescriptionType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsContentManagementPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsCreationDescriptionType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_CreationInformation);
		this->SetCreationInformation(Mp7JrsFactory::CloneObject(tmp->GetCreationInformation()));
}

Mp7JrsNodePtr Mp7JrsCreationDescriptionType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsCreationDescriptionType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsContentManagementType > Mp7JrsCreationDescriptionType::GetBase() const
{
	return m_Base;
}

Mp7JrsCreationDescriptionType_CreationInformation_CollectionPtr Mp7JrsCreationDescriptionType::GetCreationInformation() const
{
	return m_CreationInformation;
}

void Mp7JrsCreationDescriptionType::SetCreationInformation(const Mp7JrsCreationDescriptionType_CreationInformation_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCreationDescriptionType::SetCreationInformation().");
	}
	if (m_CreationInformation != item)
	{
		// Mp7JrsFactory::DeleteObject(m_CreationInformation);
		m_CreationInformation = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_CreationInformation) m_CreationInformation->SetParent(m_myself.getPointer());
		if(m_CreationInformation != Mp7JrsCreationDescriptionType_CreationInformation_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_CreationInformation->SetContentName(XMLString::transcode("CreationInformation"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsDescriptionMetadataPtr Mp7JrsCreationDescriptionType::GetDescriptionMetadata() const
{
	return GetBase()->GetBase()->GetDescriptionMetadata();
}

// Element is optional
bool Mp7JrsCreationDescriptionType::IsValidDescriptionMetadata() const
{
	return GetBase()->GetBase()->IsValidDescriptionMetadata();
}

Mp7JrsCompleteDescriptionType_Relationships_CollectionPtr Mp7JrsCreationDescriptionType::GetRelationships() const
{
	return GetBase()->GetBase()->GetRelationships();
}

Mp7JrsCompleteDescriptionType_OrderingKey_CollectionPtr Mp7JrsCreationDescriptionType::GetOrderingKey() const
{
	return GetBase()->GetBase()->GetOrderingKey();
}

void Mp7JrsCreationDescriptionType::SetDescriptionMetadata(const Mp7JrsDescriptionMetadataPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCreationDescriptionType::SetDescriptionMetadata().");
	}
	GetBase()->GetBase()->SetDescriptionMetadata(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsCreationDescriptionType::InvalidateDescriptionMetadata()
{
	GetBase()->GetBase()->InvalidateDescriptionMetadata();
}
void Mp7JrsCreationDescriptionType::SetRelationships(const Mp7JrsCompleteDescriptionType_Relationships_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCreationDescriptionType::SetRelationships().");
	}
	GetBase()->GetBase()->SetRelationships(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsCreationDescriptionType::SetOrderingKey(const Mp7JrsCompleteDescriptionType_OrderingKey_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCreationDescriptionType::SetOrderingKey().");
	}
	GetBase()->GetBase()->SetOrderingKey(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsCreationDescriptionType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("CreationInformation")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:CreationInformation is item of type CreationInformationType
		// in element collection CreationDescriptionType_CreationInformation_CollectionType
		
		context->Found = true;
		Mp7JrsCreationDescriptionType_CreationInformation_CollectionPtr coll = GetCreationInformation();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateCreationDescriptionType_CreationInformation_CollectionType;
				SetCreationInformation(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:CreationInformationType")))) != empty)
			{
				// Is type allowed
				Mp7JrsCreationInformationPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for CreationDescriptionType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "CreationDescriptionType");
	}		
	
	return result;
}

XMLCh * Mp7JrsCreationDescriptionType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsCreationDescriptionType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}

bool Mp7JrsCreationDescriptionType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsCreationDescriptionType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsCreationDescriptionType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsCreationDescriptionType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("CreationDescriptionType"));
	// Element serialization:
	if (m_CreationInformation != Mp7JrsCreationDescriptionType_CreationInformation_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_CreationInformation->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsCreationDescriptionType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsContentManagementType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("CreationInformation"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("CreationInformation")) == 0))
        {
            // Deserialize factory type
            Mp7JrsCreationDescriptionType_CreationInformation_CollectionPtr tmp = CreateCreationDescriptionType_CreationInformation_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetCreationInformation(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsCreationDescriptionType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsCreationDescriptionType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("CreationInformation")) == 0))
  {
    Mp7JrsCreationDescriptionType_CreationInformation_CollectionPtr tmp = CreateCreationDescriptionType_CreationInformation_CollectionType;
    this->SetCreationInformation(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsCreationDescriptionType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetCreationInformation() != Mp7JrsNodePtr())
		result.Insert(GetCreationInformation());
	if (GetDescriptionMetadata() != Mp7JrsNodePtr())
		result.Insert(GetDescriptionMetadata());
	if (GetRelationships() != Mp7JrsNodePtr())
		result.Insert(GetRelationships());
	if (GetOrderingKey() != Mp7JrsNodePtr())
		result.Insert(GetOrderingKey());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsCreationDescriptionType_ExtMethodImpl.h


