 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsCreationToolType_ExtImplInclude.h



#include "Mp7JrsTermUseType.h"
#include "Mp7JrsCreationToolType_Setting_CollectionType.h"
#include "Mp7JrsCreationToolType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsCreationToolType_Setting_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Setting

#include <assert.h>
IMp7JrsCreationToolType::IMp7JrsCreationToolType()
{

// no includefile for extension defined 
// file Mp7JrsCreationToolType_ExtPropInit.h

}

IMp7JrsCreationToolType::~IMp7JrsCreationToolType()
{
// no includefile for extension defined 
// file Mp7JrsCreationToolType_ExtPropCleanup.h

}

Mp7JrsCreationToolType::Mp7JrsCreationToolType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsCreationToolType::~Mp7JrsCreationToolType()
{
	Cleanup();
}

void Mp7JrsCreationToolType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_Tool = Mp7JrsTermUsePtr(); // Class
	m_Setting = Mp7JrsCreationToolType_Setting_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsCreationToolType_ExtMyPropInit.h

}

void Mp7JrsCreationToolType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsCreationToolType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Tool);
	// Mp7JrsFactory::DeleteObject(m_Setting);
}

void Mp7JrsCreationToolType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use CreationToolTypePtr, since we
	// might need GetBase(), which isn't defined in ICreationToolType
    const Mp7JrsPtr< Mp7JrsCreationToolType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		// Mp7JrsFactory::DeleteObject(m_Tool);
		this->SetTool(Mp7JrsFactory::CloneObject(tmp->GetTool()));
		// Mp7JrsFactory::DeleteObject(m_Setting);
		this->SetSetting(Mp7JrsFactory::CloneObject(tmp->GetSetting()));
}

Mp7JrsNodePtr Mp7JrsCreationToolType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsCreationToolType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsTermUsePtr Mp7JrsCreationToolType::GetTool() const
{
	return m_Tool;
}

Mp7JrsCreationToolType_Setting_CollectionPtr Mp7JrsCreationToolType::GetSetting() const
{
	return m_Setting;
}

void Mp7JrsCreationToolType::SetTool(const Mp7JrsTermUsePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCreationToolType::SetTool().");
	}
	if (m_Tool != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Tool);
		m_Tool = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Tool) m_Tool->SetParent(m_myself.getPointer());
		if (m_Tool && m_Tool->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TermUseType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Tool->UseTypeAttribute = true;
		}
		if(m_Tool != Mp7JrsTermUsePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Tool->SetContentName(XMLString::transcode("Tool"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsCreationToolType::SetSetting(const Mp7JrsCreationToolType_Setting_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCreationToolType::SetSetting().");
	}
	if (m_Setting != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Setting);
		m_Setting = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Setting) m_Setting->SetParent(m_myself.getPointer());
		if(m_Setting != Mp7JrsCreationToolType_Setting_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Setting->SetContentName(XMLString::transcode("Setting"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsCreationToolType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Tool")) == 0)
	{
		// Tool is simple element TermUseType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetTool()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TermUseType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTermUsePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetTool(p, client);
					if((p = GetTool()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Setting")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Setting is item of type CreationToolType_Setting_LocalType
		// in element collection CreationToolType_Setting_CollectionType
		
		context->Found = true;
		Mp7JrsCreationToolType_Setting_CollectionPtr coll = GetSetting();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateCreationToolType_Setting_CollectionType;
				SetSetting(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:CreationToolType_Setting_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsCreationToolType_Setting_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for CreationToolType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "CreationToolType");
	}		
	
	return result;
}

XMLCh * Mp7JrsCreationToolType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsCreationToolType::Parse(const XMLCh * const txt)
{
	return false;
}

bool Mp7JrsCreationToolType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	// Mp7JrsCreationToolType has no attributes so forward it to Parse()
	return Parse(txt);
}
XMLCh* Mp7JrsCreationToolType::ContentToString() const
{
	// Mp7JrsCreationToolType has no attributes so forward it to ToText()
	return ToText();
}

void Mp7JrsCreationToolType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("CreationToolType"));
	// Element serialization:
	// Class
	
	if (m_Tool != Mp7JrsTermUsePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Tool->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Tool"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Tool->Serialize(doc, element, &elem);
	}
	if (m_Setting != Mp7JrsCreationToolType_Setting_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Setting->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsCreationToolType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	parent = Mp7JrsUtil::GetNextChild(parent);
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Tool"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Tool")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateTermUseType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetTool(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Setting"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Setting")) == 0))
        {
            // Deserialize factory type
            Mp7JrsCreationToolType_Setting_CollectionPtr tmp = CreateCreationToolType_Setting_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetSetting(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsCreationToolType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsCreationToolType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("Tool")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateTermUseType;
    }
    this->SetTool(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Setting")) == 0))
  {
    Mp7JrsCreationToolType_Setting_CollectionPtr tmp = CreateCreationToolType_Setting_CollectionType;
    this->SetSetting(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsCreationToolType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetTool() != Mp7JrsNodePtr())
		result.Insert(GetTool());
	if (GetSetting() != Mp7JrsNodePtr())
		result.Insert(GetSetting());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsCreationToolType_ExtMethodImpl.h


