 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsDoubleMatrixType_ExtImplInclude.h



#include "Mp7JrsdoubleVector.h"
#include "Mp7Jrsdim_LocalType.h"
#include "Mp7JrsDoubleMatrixType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsDoubleMatrixType::IMp7JrsDoubleMatrixType()
{

// no includefile for extension defined 
// file Mp7JrsDoubleMatrixType_ExtPropInit.h

}

IMp7JrsDoubleMatrixType::~IMp7JrsDoubleMatrixType()
{
// no includefile for extension defined 
// file Mp7JrsDoubleMatrixType_ExtPropCleanup.h

}

Mp7JrsDoubleMatrixType::Mp7JrsDoubleMatrixType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsDoubleMatrixType::~Mp7JrsDoubleMatrixType()
{
	Cleanup();
}

void Mp7JrsDoubleMatrixType::Init()
{
	// Init base
	m_Base = CreatedoubleVector;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_dim = Mp7Jrsdim_LocalPtr(); // Create emtpy class ptr



// no includefile for extension defined 
// file Mp7JrsDoubleMatrixType_ExtMyPropInit.h

}

void Mp7JrsDoubleMatrixType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsDoubleMatrixType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_dim);
}

void Mp7JrsDoubleMatrixType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use DoubleMatrixTypePtr, since we
	// might need GetBase(), which isn't defined in IDoubleMatrixType
    const Mp7JrsPtr< Mp7JrsDoubleMatrixType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsdoubleVectorPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsDoubleMatrixType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	// Mp7JrsFactory::DeleteObject(m_dim);
		this->Setdim(Mp7JrsFactory::CloneObject(tmp->Getdim()));
	}
}

Mp7JrsNodePtr Mp7JrsDoubleMatrixType::GetBaseRoot()
{
	return GetBase();
}

const Mp7JrsNodePtr Mp7JrsDoubleMatrixType::GetBaseRoot() const
{
	return GetBase();
}


Mp7JrsPtr< Mp7JrsdoubleVector > Mp7JrsDoubleMatrixType::GetBase() const
{
	return m_Base;
}

Mp7JrsPtr< Mp7JrsdoubleVector > Mp7JrsDoubleMatrixType::GetBaseCollection() const
{
	return GetBase();
}

void Mp7JrsDoubleMatrixType::Initialize(unsigned int maxElems)
{
	GetBase()->Initialize(maxElems);
}

void Mp7JrsDoubleMatrixType::addElement(const double &toAdd, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDoubleMatrixType::addElement().");
	}
	GetBase()->addElement(toAdd);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsDoubleMatrixType::setElementAt(const double &toSet, unsigned int setAt, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDoubleMatrixType::setElementAt().");
	}
	GetBase()->setElementAt(toSet, setAt);
	Mp7JrsClientManager::SendUpdateNotification(
	m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
	client, m_myself.getPointer());
}

void Mp7JrsDoubleMatrixType::insertElementAt(const double &toInsert, unsigned int insertAt, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDoubleMatrixType::insertElementAt().");
	}
	GetBase()->insertElementAt(toInsert, insertAt);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsDoubleMatrixType::removeAllElements(Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDoubleMatrixType::removeAllElements().");
	}
	GetBase()->removeAllElements();
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsDoubleMatrixType::removeElementAt(unsigned int removeAt, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDoubleMatrixType::removeElementAt().");
	}
	GetBase()->removeElementAt(removeAt);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

bool Mp7JrsDoubleMatrixType::containsElement(const double &toCheck)
{
	return GetBase()->containsElement(toCheck);
}

unsigned int Mp7JrsDoubleMatrixType::curCapacity() const
{
	return GetBase()->curCapacity();
}

const double Mp7JrsDoubleMatrixType::elementAt(unsigned int getAt) const
{
	return GetBase()->elementAt(getAt);
}

double Mp7JrsDoubleMatrixType::elementAt(unsigned int getAt)
{
	return GetBase()->elementAt(getAt);
}


unsigned int Mp7JrsDoubleMatrixType::size() const
{
	return GetBase()->size();
}

void Mp7JrsDoubleMatrixType::ensureExtraCapacity(unsigned int length)
{
	GetBase()->ensureExtraCapacity(length);
}

int Mp7JrsDoubleMatrixType::elementIndexOf(const double &toCheck) const
{
	return GetBase()->elementIndexOf(toCheck);
}

Mp7Jrsdim_LocalPtr Mp7JrsDoubleMatrixType::Getdim() const
{
	return m_dim;
}

void Mp7JrsDoubleMatrixType::Setdim(const Mp7Jrsdim_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDoubleMatrixType::Setdim().");
	}
    m_dim = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_dim) m_dim->SetParent(m_myself.getPointer());
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsDoubleMatrixType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	if(isentrypoint)
	{
		// No Mp7JrsNodePtr subelement to get or create for DoubleMatrixType
		Mp7JrsXPathParseContext::ErrorNoSubElement(context, "DoubleMatrixType");
	}
	
	return result;
}

XMLCh * Mp7JrsDoubleMatrixType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsDoubleMatrixType::Parse(const XMLCh * const txt)
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return false;
}

bool Mp7JrsDoubleMatrixType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsDoubleMatrixType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsDoubleMatrixType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsDoubleMatrixType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("DoubleMatrixType"));
    // Attribute Serialization:
	// Class
	if (m_dim != Mp7Jrsdim_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_dim->ToText();
		element->setAttributeNS(X("urn:mpeg:mpeg7:schema:2004"), X("dim"), tmp);
		XMLString::release(&tmp);
	}
	// Element serialization:

}

bool Mp7JrsDoubleMatrixType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X("urn:mpeg:mpeg7:schema:2004"), X("dim")))
	{
        // Deserialize class type
		Mp7Jrsdim_LocalPtr tmp = Createdim_LocalType;
		tmp->Parse(parent->getAttributeNS(X("urn:mpeg:mpeg7:schema:2004"), X("dim")));
        this->Setdim(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("dim")))
	{
		// Deserialize class type
        Mp7Jrsdim_LocalPtr tmp = Createdim_LocalType;
        tmp->Parse(parent->getAttribute(X("dim")));
        this->Setdim(tmp);
        * current = parent;
    }

  // Extensionbase is Mp7JrsdoubleVector
  // Deserialize cce factory type
  if(m_Base->Deserialize(doc, parent, current))
  {
    found = true;
    parent = * current;
    parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
  }
// no includefile for extension defined 
// file Mp7JrsDoubleMatrixType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsDoubleMatrixType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsDoubleMatrixType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsDoubleMatrixType_ExtMethodImpl.h


