 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsElectronicAddressType_ExtImplInclude.h



#include "Mp7JrsElectronicAddressType_Telephone_CollectionType.h"
#include "Mp7JrsElectronicAddressType_Fax_CollectionType.h"
#include "Mp7JrsElectronicAddressType_Email_CollectionType.h"
#include "Mp7JrsElectronicAddressType_Url_CollectionType.h"
#include "Mp7JrsElectronicAddressType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsElectronicAddressType::IMp7JrsElectronicAddressType()
{

// no includefile for extension defined 
// file Mp7JrsElectronicAddressType_ExtPropInit.h

}

IMp7JrsElectronicAddressType::~IMp7JrsElectronicAddressType()
{
// no includefile for extension defined 
// file Mp7JrsElectronicAddressType_ExtPropCleanup.h

}

Mp7JrsElectronicAddressType::Mp7JrsElectronicAddressType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsElectronicAddressType::~Mp7JrsElectronicAddressType()
{
	Cleanup();
}

void Mp7JrsElectronicAddressType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_Telephone = Mp7JrsElectronicAddressType_Telephone_CollectionPtr(); // Collection
	m_Fax = Mp7JrsElectronicAddressType_Fax_CollectionPtr(); // Collection
	m_Email = Mp7JrsElectronicAddressType_Email_CollectionPtr(); // Collection
	m_Url = Mp7JrsElectronicAddressType_Url_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsElectronicAddressType_ExtMyPropInit.h

}

void Mp7JrsElectronicAddressType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsElectronicAddressType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Telephone);
	// Mp7JrsFactory::DeleteObject(m_Fax);
	// Mp7JrsFactory::DeleteObject(m_Email);
	// Mp7JrsFactory::DeleteObject(m_Url);
}

void Mp7JrsElectronicAddressType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use ElectronicAddressTypePtr, since we
	// might need GetBase(), which isn't defined in IElectronicAddressType
    const Mp7JrsPtr< Mp7JrsElectronicAddressType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		// Mp7JrsFactory::DeleteObject(m_Telephone);
		this->SetTelephone(Mp7JrsFactory::CloneObject(tmp->GetTelephone()));
		// Mp7JrsFactory::DeleteObject(m_Fax);
		this->SetFax(Mp7JrsFactory::CloneObject(tmp->GetFax()));
		// Mp7JrsFactory::DeleteObject(m_Email);
		this->SetEmail(Mp7JrsFactory::CloneObject(tmp->GetEmail()));
		// Mp7JrsFactory::DeleteObject(m_Url);
		this->SetUrl(Mp7JrsFactory::CloneObject(tmp->GetUrl()));
}

Mp7JrsNodePtr Mp7JrsElectronicAddressType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsElectronicAddressType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsElectronicAddressType_Telephone_CollectionPtr Mp7JrsElectronicAddressType::GetTelephone() const
{
	return m_Telephone;
}

Mp7JrsElectronicAddressType_Fax_CollectionPtr Mp7JrsElectronicAddressType::GetFax() const
{
	return m_Fax;
}

Mp7JrsElectronicAddressType_Email_CollectionPtr Mp7JrsElectronicAddressType::GetEmail() const
{
	return m_Email;
}

Mp7JrsElectronicAddressType_Url_CollectionPtr Mp7JrsElectronicAddressType::GetUrl() const
{
	return m_Url;
}

void Mp7JrsElectronicAddressType::SetTelephone(const Mp7JrsElectronicAddressType_Telephone_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsElectronicAddressType::SetTelephone().");
	}
	if (m_Telephone != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Telephone);
		m_Telephone = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Telephone) m_Telephone->SetParent(m_myself.getPointer());
		if(m_Telephone != Mp7JrsElectronicAddressType_Telephone_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Telephone->SetContentName(XMLString::transcode("Telephone"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsElectronicAddressType::SetFax(const Mp7JrsElectronicAddressType_Fax_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsElectronicAddressType::SetFax().");
	}
	if (m_Fax != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Fax);
		m_Fax = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Fax) m_Fax->SetParent(m_myself.getPointer());
		if(m_Fax != Mp7JrsElectronicAddressType_Fax_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Fax->SetContentName(XMLString::transcode("Fax"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsElectronicAddressType::SetEmail(const Mp7JrsElectronicAddressType_Email_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsElectronicAddressType::SetEmail().");
	}
	if (m_Email != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Email);
		m_Email = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Email) m_Email->SetParent(m_myself.getPointer());
		if(m_Email != Mp7JrsElectronicAddressType_Email_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Email->SetContentName(XMLString::transcode("Email"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsElectronicAddressType::SetUrl(const Mp7JrsElectronicAddressType_Url_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsElectronicAddressType::SetUrl().");
	}
	if (m_Url != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Url);
		m_Url = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Url) m_Url->SetParent(m_myself.getPointer());
		if(m_Url != Mp7JrsElectronicAddressType_Url_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Url->SetContentName(XMLString::transcode("Url"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsElectronicAddressType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Telephone")) == 0)
	{
		// Some primitive nodes types cannot be handled
		Mp7JrsXPathParseContext::ErrorUnsupportedType(context, "XMLCh *");
	}
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Fax")) == 0)
	{
		// Some primitive nodes types cannot be handled
		Mp7JrsXPathParseContext::ErrorUnsupportedType(context, "XMLCh *");
	}
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Email")) == 0)
	{
		// Some primitive nodes types cannot be handled
		Mp7JrsXPathParseContext::ErrorUnsupportedType(context, "XMLCh *");
	}
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Url")) == 0)
	{
		// Some primitive nodes types cannot be handled
		Mp7JrsXPathParseContext::ErrorUnsupportedType(context, "XMLCh *");
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for ElectronicAddressType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "ElectronicAddressType");
	}		
	
	return result;
}

XMLCh * Mp7JrsElectronicAddressType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsElectronicAddressType::Parse(const XMLCh * const txt)
{
	return false;
}

bool Mp7JrsElectronicAddressType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	// Mp7JrsElectronicAddressType has no attributes so forward it to Parse()
	return Parse(txt);
}
XMLCh* Mp7JrsElectronicAddressType::ContentToString() const
{
	// Mp7JrsElectronicAddressType has no attributes so forward it to ToText()
	return ToText();
}

void Mp7JrsElectronicAddressType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("ElectronicAddressType"));
	// Element serialization:
	if (m_Telephone != Mp7JrsElectronicAddressType_Telephone_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Telephone->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_Fax != Mp7JrsElectronicAddressType_Fax_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Fax->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_Email != Mp7JrsElectronicAddressType_Email_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Email->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_Url != Mp7JrsElectronicAddressType_Url_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Url->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsElectronicAddressType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	parent = Mp7JrsUtil::GetNextChild(parent);
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Telephone"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Telephone")) == 0))
        {
            // Deserialize factory type
            Mp7JrsElectronicAddressType_Telephone_CollectionPtr tmp = CreateElectronicAddressType_Telephone_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetTelephone(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Fax"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Fax")) == 0))
        {
            // Deserialize factory type
            Mp7JrsElectronicAddressType_Fax_CollectionPtr tmp = CreateElectronicAddressType_Fax_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetFax(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Email"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Email")) == 0))
        {
            // Deserialize factory type
            Mp7JrsElectronicAddressType_Email_CollectionPtr tmp = CreateElectronicAddressType_Email_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetEmail(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Url"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Url")) == 0))
        {
            // Deserialize factory type
            Mp7JrsElectronicAddressType_Url_CollectionPtr tmp = CreateElectronicAddressType_Url_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetUrl(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsElectronicAddressType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsElectronicAddressType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Telephone")) == 0))
  {
    Mp7JrsElectronicAddressType_Telephone_CollectionPtr tmp = CreateElectronicAddressType_Telephone_CollectionType;
    this->SetTelephone(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Fax")) == 0))
  {
    Mp7JrsElectronicAddressType_Fax_CollectionPtr tmp = CreateElectronicAddressType_Fax_CollectionType;
    this->SetFax(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Email")) == 0))
  {
    Mp7JrsElectronicAddressType_Email_CollectionPtr tmp = CreateElectronicAddressType_Email_CollectionType;
    this->SetEmail(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Url")) == 0))
  {
    Mp7JrsElectronicAddressType_Url_CollectionPtr tmp = CreateElectronicAddressType_Url_CollectionType;
    this->SetUrl(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsElectronicAddressType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetTelephone() != Mp7JrsNodePtr())
		result.Insert(GetTelephone());
	if (GetFax() != Mp7JrsNodePtr())
		result.Insert(GetFax());
	if (GetEmail() != Mp7JrsNodePtr())
		result.Insert(GetEmail());
	if (GetUrl() != Mp7JrsNodePtr())
		result.Insert(GetUrl());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsElectronicAddressType_ExtMethodImpl.h


