 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsFilter1DType_ExtImplInclude.h



#include "Mp7JrsFilterType.h"
#include "Mp7JrsDoubleMatrixType.h"
#include "Mp7JrsFilter1DType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsFilter1DType::IMp7JrsFilter1DType()
{

// no includefile for extension defined 
// file Mp7JrsFilter1DType_ExtPropInit.h

}

IMp7JrsFilter1DType::~IMp7JrsFilter1DType()
{
// no includefile for extension defined 
// file Mp7JrsFilter1DType_ExtPropCleanup.h

}

Mp7JrsFilter1DType::Mp7JrsFilter1DType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsFilter1DType::~Mp7JrsFilter1DType()
{
	Cleanup();
}

void Mp7JrsFilter1DType::Init()
{
	// Init base
	m_Base = CreateFilterType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_phase = 0; // Value
	m_phase_Default = 0;
	m_phase_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_Terms = Mp7JrsDoubleMatrixPtr(); // Class


// no includefile for extension defined 
// file Mp7JrsFilter1DType_ExtMyPropInit.h

}

void Mp7JrsFilter1DType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsFilter1DType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_Terms);
}

void Mp7JrsFilter1DType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use Filter1DTypePtr, since we
	// might need GetBase(), which isn't defined in IFilter1DType
    const Mp7JrsPtr< Mp7JrsFilter1DType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsFilterPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsFilter1DType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	if (tmp->Existphase())
	{
		this->Setphase(tmp->Getphase());
	}
	else
	{
		Invalidatephase();
	}
	}
		// Mp7JrsFactory::DeleteObject(m_Terms);
		this->SetTerms(Mp7JrsFactory::CloneObject(tmp->GetTerms()));
}

Mp7JrsNodePtr Mp7JrsFilter1DType::GetBaseRoot()
{
	return GetBase();
}

const Mp7JrsNodePtr Mp7JrsFilter1DType::GetBaseRoot() const
{
	return GetBase();
}


Mp7JrsPtr< Mp7JrsFilterType > Mp7JrsFilter1DType::GetBase() const
{
	return m_Base;
}

int Mp7JrsFilter1DType::Getphase() const
{
	if (this->Existphase()) {
		return m_phase;
	} else {
		return m_phase_Default;
	}
}

bool Mp7JrsFilter1DType::Existphase() const
{
	return m_phase_Exist;
}
void Mp7JrsFilter1DType::Setphase(int item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFilter1DType::Setphase().");
	}
    m_phase = item;
	m_phase_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFilter1DType::Invalidatephase()
{
	m_phase_Exist = false;
}
Mp7JrsDoubleMatrixPtr Mp7JrsFilter1DType::GetTerms() const
{
	return m_Terms;
}

void Mp7JrsFilter1DType::SetTerms(const Mp7JrsDoubleMatrixPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFilter1DType::SetTerms().");
	}
	if (m_Terms != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Terms);
		m_Terms = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Terms) m_Terms->SetParent(m_myself.getPointer());
		if (m_Terms && m_Terms->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:DoubleMatrixType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Terms->UseTypeAttribute = true;
		}
		if(m_Terms != Mp7JrsDoubleMatrixPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Terms->SetContentName(XMLString::transcode("Terms"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

unsigned Mp7JrsFilter1DType::Getdim() const
{
	return GetBase()->Getdim();
}

bool Mp7JrsFilter1DType::Existdim() const
{
	return GetBase()->Existdim();
}
void Mp7JrsFilter1DType::Setdim(unsigned item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFilter1DType::Setdim().");
	}
	GetBase()->Setdim(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFilter1DType::Invalidatedim()
{
	GetBase()->Invalidatedim();
}

Mp7JrsNodeEnum Mp7JrsFilter1DType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Terms")) == 0)
	{
		// Terms is simple element DoubleMatrixType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetTerms()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:DoubleMatrixType")))) != empty)
			{
				// Is type allowed
				Mp7JrsDoubleMatrixPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetTerms(p, client);
					if((p = GetTerms()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for Filter1DType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "Filter1DType");
	}		
	
	return result;
}

XMLCh * Mp7JrsFilter1DType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsFilter1DType::Parse(const XMLCh * const txt)
{
	return false;
}

bool Mp7JrsFilter1DType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsFilter1DType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsFilter1DType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsFilter1DType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("Filter1DType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_phase_Exist)
	{
	// Value
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_phase);
		element->setAttributeNS(X(""), X("phase"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:
	// Class
	
	if (m_Terms != Mp7JrsDoubleMatrixPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Terms->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Terms"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Terms->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsFilter1DType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("phase")))
	{
        // deserialize value type
        this->Setphase(Mp7JrsConvert::TextToInt(parent->getAttributeNS(X(""), X("phase"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("phase")))
	{
        // deserialize value type
        this->Setphase(Mp7JrsConvert::TextToInt(parent->getAttribute(X("phase"))));
        * current = parent;
    }

	// Extensionbase is Mp7JrsFilterType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Terms"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Terms")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateDoubleMatrixType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetTerms(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsFilter1DType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsFilter1DType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("Terms")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateDoubleMatrixType;
    }
    this->SetTerms(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsFilter1DType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetTerms() != Mp7JrsNodePtr())
		result.Insert(GetTerms());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsFilter1DType_ExtMethodImpl.h


