 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsGeographicPointType_ExtImplInclude.h



#include "Mp7JrsGeographicPointType_longitude_LocalType.h"
#include "Mp7JrsGeographicPointType_latitude_LocalType.h"
#include "Mp7JrsGeographicPointType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsGeographicPointType::IMp7JrsGeographicPointType()
{

// no includefile for extension defined 
// file Mp7JrsGeographicPointType_ExtPropInit.h

}

IMp7JrsGeographicPointType::~IMp7JrsGeographicPointType()
{
// no includefile for extension defined 
// file Mp7JrsGeographicPointType_ExtPropCleanup.h

}

Mp7JrsGeographicPointType::Mp7JrsGeographicPointType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsGeographicPointType::~Mp7JrsGeographicPointType()
{
	Cleanup();
}

void Mp7JrsGeographicPointType::Init()
{

	// Init attributes
	m_longitude = CreateGeographicPointType_longitude_LocalType; // Create a valid class
	m_longitude->SetContent(-180.0); // Use Value initialiser (xxx2)
	m_latitude = CreateGeographicPointType_latitude_LocalType; // Create a valid class
	m_latitude->SetContent(-90.0); // Use Value initialiser (xxx2)
	m_altitude = 0.0; // Value
	m_altitude_Exist = false;



// no includefile for extension defined 
// file Mp7JrsGeographicPointType_ExtMyPropInit.h

}

void Mp7JrsGeographicPointType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsGeographicPointType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_longitude);
	// Mp7JrsFactory::DeleteObject(m_latitude);
}

void Mp7JrsGeographicPointType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use GeographicPointTypePtr, since we
	// might need GetBase(), which isn't defined in IGeographicPointType
    const Mp7JrsPtr< Mp7JrsGeographicPointType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	{
	// Mp7JrsFactory::DeleteObject(m_longitude);
		this->Setlongitude(Mp7JrsFactory::CloneObject(tmp->Getlongitude()));
	}
	{
	// Mp7JrsFactory::DeleteObject(m_latitude);
		this->Setlatitude(Mp7JrsFactory::CloneObject(tmp->Getlatitude()));
	}
	{
	if (tmp->Existaltitude())
	{
		this->Setaltitude(tmp->Getaltitude());
	}
	else
	{
		Invalidatealtitude();
	}
	}
}

Mp7JrsNodePtr Mp7JrsGeographicPointType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsGeographicPointType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsGeographicPointType_longitude_LocalPtr Mp7JrsGeographicPointType::Getlongitude() const
{
	return m_longitude;
}

void Mp7JrsGeographicPointType::Setlongitude(const Mp7JrsGeographicPointType_longitude_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGeographicPointType::Setlongitude().");
	}
    m_longitude = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_longitude) m_longitude->SetParent(m_myself.getPointer());
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsGeographicPointType_latitude_LocalPtr Mp7JrsGeographicPointType::Getlatitude() const
{
	return m_latitude;
}

void Mp7JrsGeographicPointType::Setlatitude(const Mp7JrsGeographicPointType_latitude_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGeographicPointType::Setlatitude().");
	}
    m_latitude = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_latitude) m_latitude->SetParent(m_myself.getPointer());
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

double Mp7JrsGeographicPointType::Getaltitude() const
{
	return m_altitude;
}

bool Mp7JrsGeographicPointType::Existaltitude() const
{
	return m_altitude_Exist;
}
void Mp7JrsGeographicPointType::Setaltitude(double item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGeographicPointType::Setaltitude().");
	}
    m_altitude = item;
	m_altitude_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGeographicPointType::Invalidatealtitude()
{
	m_altitude_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsGeographicPointType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	// Only rudimentary expressions allowed for this type with no child elements
	return result;
}	

XMLCh * Mp7JrsGeographicPointType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsGeographicPointType::Parse(const XMLCh * const txt)
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return false;
}

bool Mp7JrsGeographicPointType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	// Mp7JrsGeographicPointType with workaround via Deserialise()
	return Mp7JrsXPathParseContext::ContentFromString(this, txt);
}
XMLCh* Mp7JrsGeographicPointType::ContentToString() const
{
	// Mp7JrsGeographicPointType with workaround via Serialise()
	return Mp7JrsXPathParseContext::ContentToString(this);
}

void Mp7JrsGeographicPointType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("GeographicPointType"));
    // Attribute Serialization:
	// Class
	if (m_longitude != Mp7JrsGeographicPointType_longitude_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_longitude->ToText();
		element->setAttributeNS(X(""), X("longitude"), tmp);
		XMLString::release(&tmp);
	}
    // Attribute Serialization:
	// Class
	if (m_latitude != Mp7JrsGeographicPointType_latitude_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_latitude->ToText();
		element->setAttributeNS(X(""), X("latitude"), tmp);
		XMLString::release(&tmp);
	}
    // Attribute Serialization:
		

	// Optional attriute
	if(m_altitude_Exist)
	{
	// Value
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_altitude);
		element->setAttributeNS(X(""), X("altitude"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:

}

bool Mp7JrsGeographicPointType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("longitude")))
	{
        // Deserialize class type
		Mp7JrsGeographicPointType_longitude_LocalPtr tmp = CreateGeographicPointType_longitude_LocalType;
		tmp->Parse(parent->getAttributeNS(X(""), X("longitude")));
        this->Setlongitude(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("longitude")))
	{
		// Deserialize class type
        Mp7JrsGeographicPointType_longitude_LocalPtr tmp = CreateGeographicPointType_longitude_LocalType;
        tmp->Parse(parent->getAttribute(X("longitude")));
        this->Setlongitude(tmp);
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("latitude")))
	{
        // Deserialize class type
		Mp7JrsGeographicPointType_latitude_LocalPtr tmp = CreateGeographicPointType_latitude_LocalType;
		tmp->Parse(parent->getAttributeNS(X(""), X("latitude")));
        this->Setlatitude(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("latitude")))
	{
		// Deserialize class type
        Mp7JrsGeographicPointType_latitude_LocalPtr tmp = CreateGeographicPointType_latitude_LocalType;
        tmp->Parse(parent->getAttribute(X("latitude")));
        this->Setlatitude(tmp);
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("altitude")))
	{
        // deserialize value type
        this->Setaltitude(Mp7JrsConvert::TextToDouble(parent->getAttributeNS(X(""), X("altitude"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("altitude")))
	{
        // deserialize value type
        this->Setaltitude(Mp7JrsConvert::TextToDouble(parent->getAttribute(X("altitude"))));
        * current = parent;
    }

// no includefile for extension defined 
// file Mp7JrsGeographicPointType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsGeographicPointType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsGeographicPointType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsGeographicPointType_ExtMethodImpl.h


