 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsGraphType_LocalType_ExtImplInclude.h



#include "Mp7JrsGraphType_Node_LocalType.h"
#include "Mp7JrsRelationType.h"
#include "Mp7JrsGraphType_LocalType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"

#include <assert.h>
IMp7JrsGraphType_LocalType::IMp7JrsGraphType_LocalType()
{

// no includefile for extension defined 
// file Mp7JrsGraphType_LocalType_ExtPropInit.h

}

IMp7JrsGraphType_LocalType::~IMp7JrsGraphType_LocalType()
{
// no includefile for extension defined 
// file Mp7JrsGraphType_LocalType_ExtPropCleanup.h

}

Mp7JrsGraphType_LocalType::Mp7JrsGraphType_LocalType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsGraphType_LocalType::~Mp7JrsGraphType_LocalType()
{
	Cleanup();
}

void Mp7JrsGraphType_LocalType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_Node = Mp7JrsGraphType_Node_LocalPtr(); // Class
	m_Relation = Mp7JrsRelationPtr(); // Class


// no includefile for extension defined 
// file Mp7JrsGraphType_LocalType_ExtMyPropInit.h

}

void Mp7JrsGraphType_LocalType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsGraphType_LocalType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Node);
	// Mp7JrsFactory::DeleteObject(m_Relation);
}

void Mp7JrsGraphType_LocalType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use GraphType_LocalTypePtr, since we
	// might need GetBase(), which isn't defined in IGraphType_LocalType
    const Mp7JrsPtr< Mp7JrsGraphType_LocalType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		// Mp7JrsFactory::DeleteObject(m_Node);
		this->SetNode(Mp7JrsFactory::CloneObject(tmp->GetNode()));
		// Mp7JrsFactory::DeleteObject(m_Relation);
		this->SetRelation(Mp7JrsFactory::CloneObject(tmp->GetRelation()));
}

Mp7JrsNodePtr Mp7JrsGraphType_LocalType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsGraphType_LocalType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsGraphType_Node_LocalPtr Mp7JrsGraphType_LocalType::GetNode() const
{
	return m_Node;
}

Mp7JrsRelationPtr Mp7JrsGraphType_LocalType::GetRelation() const
{
	return m_Relation;
}

// implementing setter for choice 
/* element */
void Mp7JrsGraphType_LocalType::SetNode(const Mp7JrsGraphType_Node_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGraphType_LocalType::SetNode().");
	}
	if (m_Node != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Node);
		m_Node = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Node) m_Node->SetParent(m_myself.getPointer());
		if (m_Node && m_Node->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:GraphType_Node_LocalType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Node->UseTypeAttribute = true;
		}
		if(m_Node != Mp7JrsGraphType_Node_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Node->SetContentName(XMLString::transcode("Node"));
		}
	// Mp7JrsFactory::DeleteObject(m_Relation);
	m_Relation = Mp7JrsRelationPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsGraphType_LocalType::SetRelation(const Mp7JrsRelationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGraphType_LocalType::SetRelation().");
	}
	if (m_Relation != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Relation);
		m_Relation = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Relation) m_Relation->SetParent(m_myself.getPointer());
		if (m_Relation && m_Relation->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RelationType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Relation->UseTypeAttribute = true;
		}
		if(m_Relation != Mp7JrsRelationPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Relation->SetContentName(XMLString::transcode("Relation"));
		}
	// Mp7JrsFactory::DeleteObject(m_Node);
	m_Node = Mp7JrsGraphType_Node_LocalPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

// Type: GraphType_LocalType: Only types with an element name can have GetOrCreate()
// Mp7JrsNodeEnum Mp7JrsGraphType_LocalType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
//

XMLCh * Mp7JrsGraphType_LocalType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsGraphType_LocalType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsGraphType_LocalType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// Serialize Choice of Sequence collection item
	element = parent;


	// Element serialization:
	// Class
	
	if (m_Node != Mp7JrsGraphType_Node_LocalPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Node->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Node"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Node->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Relation != Mp7JrsRelationPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Relation->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Relation"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Relation->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsGraphType_LocalType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	do // Deserialize choice just one time!
    {
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Node"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Node")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateGraphType_Node_LocalType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetNode(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Relation"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Relation")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateRelationType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetRelation(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
// no includefile for extension defined 
// file Mp7JrsGraphType_LocalType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsGraphType_LocalType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("Node")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateGraphType_Node_LocalType;
    }
    this->SetNode(child);
  }
  if (XMLString::compareString(elementname, X("Relation")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateRelationType;
    }
    this->SetRelation(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsGraphType_LocalType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetNode() != Mp7JrsNodePtr())
		result.Insert(GetNode());
	if (GetRelation() != Mp7JrsNodePtr())
		result.Insert(GetRelation());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsGraphType_LocalType_ExtMethodImpl.h


