 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsImageLocatorType_ExtImplInclude.h



#include "Mp7JrsMediaLocatorType.h"
#include "Mp7JrsmediaTimePointType.h"
#include "Mp7JrsMediaRelTimePointType.h"
#include "Mp7JrsMediaRelIncrTimePointType.h"
#include "Mp7JrsImageLocatorType_BytePosition_LocalType.h"
#include "Mp7JrsInlineMediaType.h"
#include "Mp7JrsImageLocatorType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsImageLocatorType::IMp7JrsImageLocatorType()
{

// no includefile for extension defined 
// file Mp7JrsImageLocatorType_ExtPropInit.h

}

IMp7JrsImageLocatorType::~IMp7JrsImageLocatorType()
{
// no includefile for extension defined 
// file Mp7JrsImageLocatorType_ExtPropCleanup.h

}

Mp7JrsImageLocatorType::Mp7JrsImageLocatorType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsImageLocatorType::~Mp7JrsImageLocatorType()
{
	Cleanup();
}

void Mp7JrsImageLocatorType::Init()
{
	// Init base
	m_Base = CreateMediaLocatorType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_MediaTimePoint = Mp7JrsmediaTimePointPtr(); // Pattern
	m_MediaRelTimePoint = Mp7JrsMediaRelTimePointPtr(); // Class
	m_MediaRelIncrTimePoint = Mp7JrsMediaRelIncrTimePointPtr(); // Class with content 
	m_BytePosition = Mp7JrsImageLocatorType_BytePosition_LocalPtr(); // Class


// no includefile for extension defined 
// file Mp7JrsImageLocatorType_ExtMyPropInit.h

}

void Mp7JrsImageLocatorType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsImageLocatorType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_MediaTimePoint);
	// Mp7JrsFactory::DeleteObject(m_MediaRelTimePoint);
	// Mp7JrsFactory::DeleteObject(m_MediaRelIncrTimePoint);
	// Mp7JrsFactory::DeleteObject(m_BytePosition);
}

void Mp7JrsImageLocatorType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use ImageLocatorTypePtr, since we
	// might need GetBase(), which isn't defined in IImageLocatorType
    const Mp7JrsPtr< Mp7JrsImageLocatorType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsMediaLocatorPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsImageLocatorType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_MediaTimePoint);
		this->SetMediaTimePoint(Mp7JrsFactory::CloneObject(tmp->GetMediaTimePoint()));
		// Mp7JrsFactory::DeleteObject(m_MediaRelTimePoint);
		this->SetMediaRelTimePoint(Mp7JrsFactory::CloneObject(tmp->GetMediaRelTimePoint()));
		// Mp7JrsFactory::DeleteObject(m_MediaRelIncrTimePoint);
		this->SetMediaRelIncrTimePoint(Mp7JrsFactory::CloneObject(tmp->GetMediaRelIncrTimePoint()));
		// Mp7JrsFactory::DeleteObject(m_BytePosition);
		this->SetBytePosition(Mp7JrsFactory::CloneObject(tmp->GetBytePosition()));
}

Mp7JrsNodePtr Mp7JrsImageLocatorType::GetBaseRoot()
{
	return GetBase();
}

const Mp7JrsNodePtr Mp7JrsImageLocatorType::GetBaseRoot() const
{
	return GetBase();
}


Mp7JrsPtr< Mp7JrsMediaLocatorType > Mp7JrsImageLocatorType::GetBase() const
{
	return m_Base;
}

Mp7JrsmediaTimePointPtr Mp7JrsImageLocatorType::GetMediaTimePoint() const
{
	return m_MediaTimePoint;
}

Mp7JrsMediaRelTimePointPtr Mp7JrsImageLocatorType::GetMediaRelTimePoint() const
{
	return m_MediaRelTimePoint;
}

Mp7JrsMediaRelIncrTimePointPtr Mp7JrsImageLocatorType::GetMediaRelIncrTimePoint() const
{
	return m_MediaRelIncrTimePoint;
}

Mp7JrsImageLocatorType_BytePosition_LocalPtr Mp7JrsImageLocatorType::GetBytePosition() const
{
	return m_BytePosition;
}

// implementing setter for choice 
/* element */
void Mp7JrsImageLocatorType::SetMediaTimePoint(const Mp7JrsmediaTimePointPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsImageLocatorType::SetMediaTimePoint().");
	}
	if (m_MediaTimePoint != item)
	{
		// Mp7JrsFactory::DeleteObject(m_MediaTimePoint);
		m_MediaTimePoint = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_MediaTimePoint) m_MediaTimePoint->SetParent(m_myself.getPointer());
		if (m_MediaTimePoint && m_MediaTimePoint->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:mediaTimePointType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_MediaTimePoint->UseTypeAttribute = true;
		}
		if(m_MediaTimePoint != Mp7JrsmediaTimePointPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_MediaTimePoint->SetContentName(XMLString::transcode("MediaTimePoint"));
		}
	// Mp7JrsFactory::DeleteObject(m_MediaRelTimePoint);
	m_MediaRelTimePoint = Mp7JrsMediaRelTimePointPtr();
	// Mp7JrsFactory::DeleteObject(m_MediaRelIncrTimePoint);
	m_MediaRelIncrTimePoint = Mp7JrsMediaRelIncrTimePointPtr();
	// Mp7JrsFactory::DeleteObject(m_BytePosition);
	m_BytePosition = Mp7JrsImageLocatorType_BytePosition_LocalPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsImageLocatorType::SetMediaRelTimePoint(const Mp7JrsMediaRelTimePointPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsImageLocatorType::SetMediaRelTimePoint().");
	}
	if (m_MediaRelTimePoint != item)
	{
		// Mp7JrsFactory::DeleteObject(m_MediaRelTimePoint);
		m_MediaRelTimePoint = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_MediaRelTimePoint) m_MediaRelTimePoint->SetParent(m_myself.getPointer());
		if (m_MediaRelTimePoint && m_MediaRelTimePoint->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaRelTimePointType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_MediaRelTimePoint->UseTypeAttribute = true;
		}
		if(m_MediaRelTimePoint != Mp7JrsMediaRelTimePointPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_MediaRelTimePoint->SetContentName(XMLString::transcode("MediaRelTimePoint"));
		}
	// Mp7JrsFactory::DeleteObject(m_MediaTimePoint);
	m_MediaTimePoint = Mp7JrsmediaTimePointPtr();
	// Mp7JrsFactory::DeleteObject(m_MediaRelIncrTimePoint);
	m_MediaRelIncrTimePoint = Mp7JrsMediaRelIncrTimePointPtr();
	// Mp7JrsFactory::DeleteObject(m_BytePosition);
	m_BytePosition = Mp7JrsImageLocatorType_BytePosition_LocalPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsImageLocatorType::SetMediaRelIncrTimePoint(const Mp7JrsMediaRelIncrTimePointPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsImageLocatorType::SetMediaRelIncrTimePoint().");
	}
	if (m_MediaRelIncrTimePoint != item)
	{
		// Mp7JrsFactory::DeleteObject(m_MediaRelIncrTimePoint);
		m_MediaRelIncrTimePoint = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_MediaRelIncrTimePoint) m_MediaRelIncrTimePoint->SetParent(m_myself.getPointer());
		if (m_MediaRelIncrTimePoint && m_MediaRelIncrTimePoint->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaRelIncrTimePointType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_MediaRelIncrTimePoint->UseTypeAttribute = true;
		}
		if(m_MediaRelIncrTimePoint != Mp7JrsMediaRelIncrTimePointPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_MediaRelIncrTimePoint->SetContentName(XMLString::transcode("MediaRelIncrTimePoint"));
		}
	// Mp7JrsFactory::DeleteObject(m_MediaTimePoint);
	m_MediaTimePoint = Mp7JrsmediaTimePointPtr();
	// Mp7JrsFactory::DeleteObject(m_MediaRelTimePoint);
	m_MediaRelTimePoint = Mp7JrsMediaRelTimePointPtr();
	// Mp7JrsFactory::DeleteObject(m_BytePosition);
	m_BytePosition = Mp7JrsImageLocatorType_BytePosition_LocalPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsImageLocatorType::SetBytePosition(const Mp7JrsImageLocatorType_BytePosition_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsImageLocatorType::SetBytePosition().");
	}
	if (m_BytePosition != item)
	{
		// Mp7JrsFactory::DeleteObject(m_BytePosition);
		m_BytePosition = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_BytePosition) m_BytePosition->SetParent(m_myself.getPointer());
		if (m_BytePosition && m_BytePosition->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ImageLocatorType_BytePosition_LocalType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_BytePosition->UseTypeAttribute = true;
		}
		if(m_BytePosition != Mp7JrsImageLocatorType_BytePosition_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_BytePosition->SetContentName(XMLString::transcode("BytePosition"));
		}
	// Mp7JrsFactory::DeleteObject(m_MediaTimePoint);
	m_MediaTimePoint = Mp7JrsmediaTimePointPtr();
	// Mp7JrsFactory::DeleteObject(m_MediaRelTimePoint);
	m_MediaRelTimePoint = Mp7JrsMediaRelTimePointPtr();
	// Mp7JrsFactory::DeleteObject(m_MediaRelIncrTimePoint);
	m_MediaRelIncrTimePoint = Mp7JrsMediaRelIncrTimePointPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsImageLocatorType::GetMediaUri() const
{
	return GetBase()->GetMediaUri();
}

Mp7JrsInlineMediaPtr Mp7JrsImageLocatorType::GetInlineMedia() const
{
	return GetBase()->GetInlineMedia();
}

unsigned Mp7JrsImageLocatorType::GetStreamID() const
{
	return GetBase()->GetStreamID();
}

// Element is optional
bool Mp7JrsImageLocatorType::IsValidStreamID() const
{
	return GetBase()->IsValidStreamID();
}

	// TODO Implement proper cleanup for nested choices (e.g. CompositionShotEditinbTemporalDecompositionType_LocalType)
// implementing setter for choice 
/* element */
void Mp7JrsImageLocatorType::SetMediaUri(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsImageLocatorType::SetMediaUri().");
	}
	GetBase()->SetMediaUri(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsImageLocatorType::SetInlineMedia(const Mp7JrsInlineMediaPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsImageLocatorType::SetInlineMedia().");
	}
	GetBase()->SetInlineMedia(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsImageLocatorType::SetStreamID(unsigned item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsImageLocatorType::SetStreamID().");
	}
	GetBase()->SetStreamID(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsImageLocatorType::InvalidateStreamID()
{
	GetBase()->InvalidateStreamID();
}

Mp7JrsNodeEnum Mp7JrsImageLocatorType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("MediaTimePoint")) == 0)
	{
		// MediaTimePoint is simple element mediaTimePointType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMediaTimePoint()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:mediaTimePointType")))) != empty)
			{
				// Is type allowed
				Mp7JrsmediaTimePointPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMediaTimePoint(p, client);
					if((p = GetMediaTimePoint()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("MediaRelTimePoint")) == 0)
	{
		// MediaRelTimePoint is simple element MediaRelTimePointType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMediaRelTimePoint()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaRelTimePointType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMediaRelTimePointPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMediaRelTimePoint(p, client);
					if((p = GetMediaRelTimePoint()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("MediaRelIncrTimePoint")) == 0)
	{
		// MediaRelIncrTimePoint is simple element MediaRelIncrTimePointType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMediaRelIncrTimePoint()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaRelIncrTimePointType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMediaRelIncrTimePointPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMediaRelIncrTimePoint(p, client);
					if((p = GetMediaRelIncrTimePoint()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("BytePosition")) == 0)
	{
		// BytePosition is simple element ImageLocatorType_BytePosition_LocalType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetBytePosition()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ImageLocatorType_BytePosition_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsImageLocatorType_BytePosition_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetBytePosition(p, client);
					if((p = GetBytePosition()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for ImageLocatorType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "ImageLocatorType");
	}		
	
	return result;
}

XMLCh * Mp7JrsImageLocatorType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsImageLocatorType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}

bool Mp7JrsImageLocatorType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsImageLocatorType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsImageLocatorType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsImageLocatorType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("ImageLocatorType"));
	// Element serialization:
	if(m_MediaTimePoint != Mp7JrsmediaTimePointPtr()) // Pattern FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Pattern
		// CON contentname already set in element setter -- not needed here
//		XMLCh * contentname = XMLString::transcode("MediaTimePoint");
//		m_MediaTimePoint->SetContentName(contentname);
		m_MediaTimePoint->UseTypeAttribute = this->UseTypeAttribute;
		m_MediaTimePoint->Serialize(doc, element);
	}
	// Class
	
	if (m_MediaRelTimePoint != Mp7JrsMediaRelTimePointPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_MediaRelTimePoint->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("MediaRelTimePoint"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_MediaRelTimePoint->Serialize(doc, element, &elem);
	}
	// Class with content		
	
	if (m_MediaRelIncrTimePoint != Mp7JrsMediaRelIncrTimePointPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_MediaRelIncrTimePoint->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("MediaRelIncrTimePoint"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_MediaRelIncrTimePoint->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_BytePosition != Mp7JrsImageLocatorType_BytePosition_LocalPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_BytePosition->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("BytePosition"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_BytePosition->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsImageLocatorType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsMediaLocatorType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
	do // Deserialize choice just one time!
    {
        // Deserialize pattern element
	if(Mp7JrsUtil::HasNodeName(parent, X("MediaTimePoint"), X("urn:mpeg:mpeg7:schema:2004")))
//	        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("MediaTimePoint")) == 0))
        {
            Mp7JrsmediaTimePointPtr tmp = CreatemediaTimePointType;
            tmp->Deserialize(doc, parent, current);
            this->SetMediaTimePoint(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("MediaRelTimePoint"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("MediaRelTimePoint")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateMediaRelTimePointType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetMediaRelTimePoint(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("MediaRelIncrTimePoint"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("MediaRelIncrTimePoint")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateMediaRelIncrTimePointType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetMediaRelIncrTimePoint(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("BytePosition"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("BytePosition")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateImageLocatorType_BytePosition_LocalType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetBytePosition(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
// no includefile for extension defined 
// file Mp7JrsImageLocatorType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsImageLocatorType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("MediaTimePoint")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreatemediaTimePointType;
    }
    this->SetMediaTimePoint(child);
  }
  if (XMLString::compareString(elementname, X("MediaRelTimePoint")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateMediaRelTimePointType;
    }
    this->SetMediaRelTimePoint(child);
  }
  if (XMLString::compareString(elementname, X("MediaRelIncrTimePoint")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateMediaRelIncrTimePointType;
    }
    this->SetMediaRelIncrTimePoint(child);
  }
  if (XMLString::compareString(elementname, X("BytePosition")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateImageLocatorType_BytePosition_LocalType;
    }
    this->SetBytePosition(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsImageLocatorType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetMediaTimePoint() != Mp7JrsNodePtr())
		result.Insert(GetMediaTimePoint());
	if (GetMediaRelTimePoint() != Mp7JrsNodePtr())
		result.Insert(GetMediaRelTimePoint());
	if (GetMediaRelIncrTimePoint() != Mp7JrsNodePtr())
		result.Insert(GetMediaRelIncrTimePoint());
	if (GetBytePosition() != Mp7JrsNodePtr())
		result.Insert(GetBytePosition());
	if (GetInlineMedia() != Mp7JrsNodePtr())
		result.Insert(GetInlineMedia());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsImageLocatorType_ExtMethodImpl.h


