 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsInkSegmentType_LocalType1_ExtImplInclude.h



#include "Mp7JrsInkSegmentSpatialDecompositionType.h"
#include "Mp7JrsInkSegmentTemporalDecompositionType.h"
#include "Mp7JrsInkSegmentType_LocalType1.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"

#include <assert.h>
IMp7JrsInkSegmentType_LocalType1::IMp7JrsInkSegmentType_LocalType1()
{

// no includefile for extension defined 
// file Mp7JrsInkSegmentType_LocalType1_ExtPropInit.h

}

IMp7JrsInkSegmentType_LocalType1::~IMp7JrsInkSegmentType_LocalType1()
{
// no includefile for extension defined 
// file Mp7JrsInkSegmentType_LocalType1_ExtPropCleanup.h

}

Mp7JrsInkSegmentType_LocalType1::Mp7JrsInkSegmentType_LocalType1()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsInkSegmentType_LocalType1::~Mp7JrsInkSegmentType_LocalType1()
{
	Cleanup();
}

void Mp7JrsInkSegmentType_LocalType1::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_SpatialDecomposition = Mp7JrsInkSegmentSpatialDecompositionPtr(); // Class
	m_TemporalDecomposition = Mp7JrsInkSegmentTemporalDecompositionPtr(); // Class


// no includefile for extension defined 
// file Mp7JrsInkSegmentType_LocalType1_ExtMyPropInit.h

}

void Mp7JrsInkSegmentType_LocalType1::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsInkSegmentType_LocalType1_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_SpatialDecomposition);
	// Mp7JrsFactory::DeleteObject(m_TemporalDecomposition);
}

void Mp7JrsInkSegmentType_LocalType1::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use InkSegmentType_LocalType1Ptr, since we
	// might need GetBase(), which isn't defined in IInkSegmentType_LocalType1
    const Mp7JrsPtr< Mp7JrsInkSegmentType_LocalType1 > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		// Mp7JrsFactory::DeleteObject(m_SpatialDecomposition);
		this->SetSpatialDecomposition(Mp7JrsFactory::CloneObject(tmp->GetSpatialDecomposition()));
		// Mp7JrsFactory::DeleteObject(m_TemporalDecomposition);
		this->SetTemporalDecomposition(Mp7JrsFactory::CloneObject(tmp->GetTemporalDecomposition()));
}

Mp7JrsNodePtr Mp7JrsInkSegmentType_LocalType1::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsInkSegmentType_LocalType1::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsInkSegmentSpatialDecompositionPtr Mp7JrsInkSegmentType_LocalType1::GetSpatialDecomposition() const
{
	return m_SpatialDecomposition;
}

Mp7JrsInkSegmentTemporalDecompositionPtr Mp7JrsInkSegmentType_LocalType1::GetTemporalDecomposition() const
{
	return m_TemporalDecomposition;
}

// implementing setter for choice 
/* element */
void Mp7JrsInkSegmentType_LocalType1::SetSpatialDecomposition(const Mp7JrsInkSegmentSpatialDecompositionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsInkSegmentType_LocalType1::SetSpatialDecomposition().");
	}
	if (m_SpatialDecomposition != item)
	{
		// Mp7JrsFactory::DeleteObject(m_SpatialDecomposition);
		m_SpatialDecomposition = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_SpatialDecomposition) m_SpatialDecomposition->SetParent(m_myself.getPointer());
		if (m_SpatialDecomposition && m_SpatialDecomposition->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:InkSegmentSpatialDecompositionType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_SpatialDecomposition->UseTypeAttribute = true;
		}
		if(m_SpatialDecomposition != Mp7JrsInkSegmentSpatialDecompositionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_SpatialDecomposition->SetContentName(XMLString::transcode("SpatialDecomposition"));
		}
	// Mp7JrsFactory::DeleteObject(m_TemporalDecomposition);
	m_TemporalDecomposition = Mp7JrsInkSegmentTemporalDecompositionPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsInkSegmentType_LocalType1::SetTemporalDecomposition(const Mp7JrsInkSegmentTemporalDecompositionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsInkSegmentType_LocalType1::SetTemporalDecomposition().");
	}
	if (m_TemporalDecomposition != item)
	{
		// Mp7JrsFactory::DeleteObject(m_TemporalDecomposition);
		m_TemporalDecomposition = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_TemporalDecomposition) m_TemporalDecomposition->SetParent(m_myself.getPointer());
		if (m_TemporalDecomposition && m_TemporalDecomposition->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:InkSegmentTemporalDecompositionType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_TemporalDecomposition->UseTypeAttribute = true;
		}
		if(m_TemporalDecomposition != Mp7JrsInkSegmentTemporalDecompositionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_TemporalDecomposition->SetContentName(XMLString::transcode("TemporalDecomposition"));
		}
	// Mp7JrsFactory::DeleteObject(m_SpatialDecomposition);
	m_SpatialDecomposition = Mp7JrsInkSegmentSpatialDecompositionPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

// Type: InkSegmentType_LocalType1: Only types with an element name can have GetOrCreate()
// Mp7JrsNodeEnum Mp7JrsInkSegmentType_LocalType1::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
//

XMLCh * Mp7JrsInkSegmentType_LocalType1::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsInkSegmentType_LocalType1::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsInkSegmentType_LocalType1::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// Serialize Choice of Sequence collection item
	element = parent;


	// Element serialization:
	// Class
	
	if (m_SpatialDecomposition != Mp7JrsInkSegmentSpatialDecompositionPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_SpatialDecomposition->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("SpatialDecomposition"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_SpatialDecomposition->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_TemporalDecomposition != Mp7JrsInkSegmentTemporalDecompositionPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_TemporalDecomposition->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("TemporalDecomposition"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_TemporalDecomposition->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsInkSegmentType_LocalType1::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	do // Deserialize choice just one time!
    {
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("SpatialDecomposition"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("SpatialDecomposition")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateInkSegmentSpatialDecompositionType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetSpatialDecomposition(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("TemporalDecomposition"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("TemporalDecomposition")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateInkSegmentTemporalDecompositionType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetTemporalDecomposition(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
// no includefile for extension defined 
// file Mp7JrsInkSegmentType_LocalType1_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsInkSegmentType_LocalType1::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("SpatialDecomposition")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateInkSegmentSpatialDecompositionType;
    }
    this->SetSpatialDecomposition(child);
  }
  if (XMLString::compareString(elementname, X("TemporalDecomposition")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateInkSegmentTemporalDecompositionType;
    }
    this->SetTemporalDecomposition(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsInkSegmentType_LocalType1::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetSpatialDecomposition() != Mp7JrsNodePtr())
		result.Insert(GetSpatialDecomposition());
	if (GetTemporalDecomposition() != Mp7JrsNodePtr())
		result.Insert(GetTemporalDecomposition());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsInkSegmentType_LocalType1_ExtMethodImpl.h


