 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsMediaIncrDurationType_ExtImplInclude.h



#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsMediaIncrDurationType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsMediaIncrDurationType::IMp7JrsMediaIncrDurationType()
{

// no includefile for extension defined 
// file Mp7JrsMediaIncrDurationType_ExtPropInit.h

}

IMp7JrsMediaIncrDurationType::~IMp7JrsMediaIncrDurationType()
{
// no includefile for extension defined 
// file Mp7JrsMediaIncrDurationType_ExtPropCleanup.h

}

Mp7JrsMediaIncrDurationType::Mp7JrsMediaIncrDurationType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsMediaIncrDurationType::~Mp7JrsMediaIncrDurationType()
{
	Cleanup();
}

void Mp7JrsMediaIncrDurationType::Init()
{
	// Init base
	m_Base = 0;

	// Init attributes
	m_mediaTimeUnit = Mp7JrsmediaDurationPtr(); // Pattern
	m_mediaTimeUnit_Exist = false;



// no includefile for extension defined 
// file Mp7JrsMediaIncrDurationType_ExtMyPropInit.h

}

void Mp7JrsMediaIncrDurationType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsMediaIncrDurationType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_mediaTimeUnit); 
}

void Mp7JrsMediaIncrDurationType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use MediaIncrDurationTypePtr, since we
	// might need GetBase(), which isn't defined in IMediaIncrDurationType
    const Mp7JrsPtr< Mp7JrsMediaIncrDurationType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	m_Base = tmp->GetContent();
	{
	// Mp7JrsFactory::DeleteObject(m_mediaTimeUnit); 
	if (tmp->ExistmediaTimeUnit())
	{
		this->SetmediaTimeUnit(Mp7JrsFactory::CloneObject(tmp->GetmediaTimeUnit()));
	}
	else
	{
		InvalidatemediaTimeUnit();
	}
	}
}

Mp7JrsNodePtr Mp7JrsMediaIncrDurationType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsMediaIncrDurationType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


void Mp7JrsMediaIncrDurationType::SetContent(int item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaIncrDurationType::SetContent().");
	}
    if (m_Base != item)
    {
		m_Base = item;
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

int Mp7JrsMediaIncrDurationType::GetContent() const
{
	return m_Base;
}
Mp7JrsmediaDurationPtr Mp7JrsMediaIncrDurationType::GetmediaTimeUnit() const
{
	return m_mediaTimeUnit;
}

bool Mp7JrsMediaIncrDurationType::ExistmediaTimeUnit() const
{
	return m_mediaTimeUnit_Exist;
}
void Mp7JrsMediaIncrDurationType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaIncrDurationType::SetmediaTimeUnit().");
	}
    m_mediaTimeUnit = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_mediaTimeUnit) m_mediaTimeUnit->SetParent(m_myself.getPointer());
	m_mediaTimeUnit_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaIncrDurationType::InvalidatemediaTimeUnit()
{
	m_mediaTimeUnit_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsMediaIncrDurationType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	if(isentrypoint)
	{
		// No Mp7JrsNodePtr subelement to get or create for MediaIncrDurationType
		Mp7JrsXPathParseContext::ErrorNoSubElement(context, "MediaIncrDurationType");
	}
	
	return result;
}

XMLCh * Mp7JrsMediaIncrDurationType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsMediaIncrDurationType::Parse(const XMLCh * const txt)
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return false;
}

bool Mp7JrsMediaIncrDurationType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	// Mp7JrsMediaIncrDurationType with workaround via Deserialise()
	return Mp7JrsXPathParseContext::ContentFromString(this, txt);
}
XMLCh* Mp7JrsMediaIncrDurationType::ContentToString() const
{
	// Mp7JrsMediaIncrDurationType with workaround via Serialise()
	return Mp7JrsXPathParseContext::ContentToString(this);
}

void Mp7JrsMediaIncrDurationType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


    {
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_Base);
		element->appendChild(doc->createTextNode(tmp));
		XMLString::release(&tmp);
    }

	assert(element);
// no includefile for extension defined 
// file Mp7JrsMediaIncrDurationType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("MediaIncrDurationType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_mediaTimeUnit_Exist)
	{
	// Pattern
	if (m_mediaTimeUnit != Mp7JrsmediaDurationPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_mediaTimeUnit->ToText();
		element->setAttributeNS(X(""), X("mediaTimeUnit"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:

}

bool Mp7JrsMediaIncrDurationType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("mediaTimeUnit")))
	{
        Mp7JrsmediaDurationPtr tmp = CreatemediaDurationType;
        tmp->Parse(parent->getAttributeNS(X(""), X("mediaTimeUnit")));
        this->SetmediaTimeUnit(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("mediaTimeUnit")))
	{
        Mp7JrsmediaDurationPtr tmp = CreatemediaDurationType;
        tmp->Parse(parent->getAttribute(X("mediaTimeUnit")));
        this->SetmediaTimeUnit(tmp);
        * current = parent;
    }

  m_Base = Mp7JrsConvert::TextToInt(Mp7JrsUtil::GetElementText(parent));
  found = true;
// no includefile for extension defined 
// file Mp7JrsMediaIncrDurationType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsMediaIncrDurationType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsMediaIncrDurationType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsMediaIncrDurationType_ExtMethodImpl.h


