 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsMediaProfileType_ExtImplInclude.h



#include "Mp7JrsDSType.h"
#include "Mp7JrsMediaProfileType_ComponentMediaProfile_CollectionType.h"
#include "Mp7JrsMediaFormatType.h"
#include "Mp7JrsMediaTranscodingHintsType.h"
#include "Mp7JrsMediaQualityType.h"
#include "Mp7JrsMediaProfileType_MediaInstance_CollectionType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsMediaProfileType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsMediaProfileType.h" // Element collection urn:mpeg:mpeg7:schema:2004:ComponentMediaProfile
#include "Mp7JrsMediaInstanceType.h" // Element collection urn:mpeg:mpeg7:schema:2004:MediaInstance

#include <assert.h>
IMp7JrsMediaProfileType::IMp7JrsMediaProfileType()
{

// no includefile for extension defined 
// file Mp7JrsMediaProfileType_ExtPropInit.h

}

IMp7JrsMediaProfileType::~IMp7JrsMediaProfileType()
{
// no includefile for extension defined 
// file Mp7JrsMediaProfileType_ExtPropCleanup.h

}

Mp7JrsMediaProfileType::Mp7JrsMediaProfileType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsMediaProfileType::~Mp7JrsMediaProfileType()
{
	Cleanup();
}

void Mp7JrsMediaProfileType::Init()
{
	// Init base
	m_Base = CreateDSType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_master = false; // Value
	m_master_Default = false;
	m_master_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_ComponentMediaProfile = Mp7JrsMediaProfileType_ComponentMediaProfile_CollectionPtr(); // Collection
	m_MediaFormat = Mp7JrsMediaFormatPtr(); // Class
	m_MediaFormat_Exist = false;
	m_MediaTranscodingHints = Mp7JrsMediaTranscodingHintsPtr(); // Class
	m_MediaTranscodingHints_Exist = false;
	m_MediaQuality = Mp7JrsMediaQualityPtr(); // Class
	m_MediaQuality_Exist = false;
	m_MediaInstance = Mp7JrsMediaProfileType_MediaInstance_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsMediaProfileType_ExtMyPropInit.h

}

void Mp7JrsMediaProfileType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsMediaProfileType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_ComponentMediaProfile);
	// Mp7JrsFactory::DeleteObject(m_MediaFormat);
	// Mp7JrsFactory::DeleteObject(m_MediaTranscodingHints);
	// Mp7JrsFactory::DeleteObject(m_MediaQuality);
	// Mp7JrsFactory::DeleteObject(m_MediaInstance);
}

void Mp7JrsMediaProfileType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use MediaProfileTypePtr, since we
	// might need GetBase(), which isn't defined in IMediaProfileType
    const Mp7JrsPtr< Mp7JrsMediaProfileType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsDSPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsMediaProfileType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	if (tmp->Existmaster())
	{
		this->Setmaster(tmp->Getmaster());
	}
	else
	{
		Invalidatemaster();
	}
	}
		// Mp7JrsFactory::DeleteObject(m_ComponentMediaProfile);
		this->SetComponentMediaProfile(Mp7JrsFactory::CloneObject(tmp->GetComponentMediaProfile()));
	if (tmp->IsValidMediaFormat())
	{
		// Mp7JrsFactory::DeleteObject(m_MediaFormat);
		this->SetMediaFormat(Mp7JrsFactory::CloneObject(tmp->GetMediaFormat()));
	}
	else
	{
		InvalidateMediaFormat();
	}
	if (tmp->IsValidMediaTranscodingHints())
	{
		// Mp7JrsFactory::DeleteObject(m_MediaTranscodingHints);
		this->SetMediaTranscodingHints(Mp7JrsFactory::CloneObject(tmp->GetMediaTranscodingHints()));
	}
	else
	{
		InvalidateMediaTranscodingHints();
	}
	if (tmp->IsValidMediaQuality())
	{
		// Mp7JrsFactory::DeleteObject(m_MediaQuality);
		this->SetMediaQuality(Mp7JrsFactory::CloneObject(tmp->GetMediaQuality()));
	}
	else
	{
		InvalidateMediaQuality();
	}
		// Mp7JrsFactory::DeleteObject(m_MediaInstance);
		this->SetMediaInstance(Mp7JrsFactory::CloneObject(tmp->GetMediaInstance()));
}

Mp7JrsNodePtr Mp7JrsMediaProfileType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsMediaProfileType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsDSType > Mp7JrsMediaProfileType::GetBase() const
{
	return m_Base;
}

bool Mp7JrsMediaProfileType::Getmaster() const
{
	if (this->Existmaster()) {
		return m_master;
	} else {
		return m_master_Default;
	}
}

bool Mp7JrsMediaProfileType::Existmaster() const
{
	return m_master_Exist;
}
void Mp7JrsMediaProfileType::Setmaster(bool item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaProfileType::Setmaster().");
	}
    m_master = item;
	m_master_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaProfileType::Invalidatemaster()
{
	m_master_Exist = false;
}
Mp7JrsMediaProfileType_ComponentMediaProfile_CollectionPtr Mp7JrsMediaProfileType::GetComponentMediaProfile() const
{
	return m_ComponentMediaProfile;
}

Mp7JrsMediaFormatPtr Mp7JrsMediaProfileType::GetMediaFormat() const
{
	return m_MediaFormat;
}

// Element is optional
bool Mp7JrsMediaProfileType::IsValidMediaFormat() const
{
	return m_MediaFormat_Exist;
}

Mp7JrsMediaTranscodingHintsPtr Mp7JrsMediaProfileType::GetMediaTranscodingHints() const
{
	return m_MediaTranscodingHints;
}

// Element is optional
bool Mp7JrsMediaProfileType::IsValidMediaTranscodingHints() const
{
	return m_MediaTranscodingHints_Exist;
}

Mp7JrsMediaQualityPtr Mp7JrsMediaProfileType::GetMediaQuality() const
{
	return m_MediaQuality;
}

// Element is optional
bool Mp7JrsMediaProfileType::IsValidMediaQuality() const
{
	return m_MediaQuality_Exist;
}

Mp7JrsMediaProfileType_MediaInstance_CollectionPtr Mp7JrsMediaProfileType::GetMediaInstance() const
{
	return m_MediaInstance;
}

void Mp7JrsMediaProfileType::SetComponentMediaProfile(const Mp7JrsMediaProfileType_ComponentMediaProfile_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaProfileType::SetComponentMediaProfile().");
	}
	if (m_ComponentMediaProfile != item)
	{
		// Mp7JrsFactory::DeleteObject(m_ComponentMediaProfile);
		m_ComponentMediaProfile = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ComponentMediaProfile) m_ComponentMediaProfile->SetParent(m_myself.getPointer());
		if(m_ComponentMediaProfile != Mp7JrsMediaProfileType_ComponentMediaProfile_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ComponentMediaProfile->SetContentName(XMLString::transcode("ComponentMediaProfile"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaProfileType::SetMediaFormat(const Mp7JrsMediaFormatPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaProfileType::SetMediaFormat().");
	}
	if (m_MediaFormat != item || m_MediaFormat_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_MediaFormat);
		m_MediaFormat = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_MediaFormat) m_MediaFormat->SetParent(m_myself.getPointer());
		if (m_MediaFormat && m_MediaFormat->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaFormatType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_MediaFormat->UseTypeAttribute = true;
		}
		m_MediaFormat_Exist = true;
		if(m_MediaFormat != Mp7JrsMediaFormatPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_MediaFormat->SetContentName(XMLString::transcode("MediaFormat"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaProfileType::InvalidateMediaFormat()
{
	m_MediaFormat_Exist = false;
}
void Mp7JrsMediaProfileType::SetMediaTranscodingHints(const Mp7JrsMediaTranscodingHintsPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaProfileType::SetMediaTranscodingHints().");
	}
	if (m_MediaTranscodingHints != item || m_MediaTranscodingHints_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_MediaTranscodingHints);
		m_MediaTranscodingHints = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_MediaTranscodingHints) m_MediaTranscodingHints->SetParent(m_myself.getPointer());
		if (m_MediaTranscodingHints && m_MediaTranscodingHints->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaTranscodingHintsType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_MediaTranscodingHints->UseTypeAttribute = true;
		}
		m_MediaTranscodingHints_Exist = true;
		if(m_MediaTranscodingHints != Mp7JrsMediaTranscodingHintsPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_MediaTranscodingHints->SetContentName(XMLString::transcode("MediaTranscodingHints"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaProfileType::InvalidateMediaTranscodingHints()
{
	m_MediaTranscodingHints_Exist = false;
}
void Mp7JrsMediaProfileType::SetMediaQuality(const Mp7JrsMediaQualityPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaProfileType::SetMediaQuality().");
	}
	if (m_MediaQuality != item || m_MediaQuality_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_MediaQuality);
		m_MediaQuality = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_MediaQuality) m_MediaQuality->SetParent(m_myself.getPointer());
		if (m_MediaQuality && m_MediaQuality->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaQualityType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_MediaQuality->UseTypeAttribute = true;
		}
		m_MediaQuality_Exist = true;
		if(m_MediaQuality != Mp7JrsMediaQualityPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_MediaQuality->SetContentName(XMLString::transcode("MediaQuality"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaProfileType::InvalidateMediaQuality()
{
	m_MediaQuality_Exist = false;
}
void Mp7JrsMediaProfileType::SetMediaInstance(const Mp7JrsMediaProfileType_MediaInstance_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaProfileType::SetMediaInstance().");
	}
	if (m_MediaInstance != item)
	{
		// Mp7JrsFactory::DeleteObject(m_MediaInstance);
		m_MediaInstance = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_MediaInstance) m_MediaInstance->SetParent(m_myself.getPointer());
		if(m_MediaInstance != Mp7JrsMediaProfileType_MediaInstance_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_MediaInstance->SetContentName(XMLString::transcode("MediaInstance"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsMediaProfileType::Getid() const
{
	return GetBase()->Getid();
}

bool Mp7JrsMediaProfileType::Existid() const
{
	return GetBase()->Existid();
}
void Mp7JrsMediaProfileType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaProfileType::Setid().");
	}
	GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaProfileType::Invalidateid()
{
	GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsMediaProfileType::GettimeBase() const
{
	return GetBase()->GettimeBase();
}

bool Mp7JrsMediaProfileType::ExisttimeBase() const
{
	return GetBase()->ExisttimeBase();
}
void Mp7JrsMediaProfileType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaProfileType::SettimeBase().");
	}
	GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaProfileType::InvalidatetimeBase()
{
	GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsMediaProfileType::GettimeUnit() const
{
	return GetBase()->GettimeUnit();
}

bool Mp7JrsMediaProfileType::ExisttimeUnit() const
{
	return GetBase()->ExisttimeUnit();
}
void Mp7JrsMediaProfileType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaProfileType::SettimeUnit().");
	}
	GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaProfileType::InvalidatetimeUnit()
{
	GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsMediaProfileType::GetmediaTimeBase() const
{
	return GetBase()->GetmediaTimeBase();
}

bool Mp7JrsMediaProfileType::ExistmediaTimeBase() const
{
	return GetBase()->ExistmediaTimeBase();
}
void Mp7JrsMediaProfileType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaProfileType::SetmediaTimeBase().");
	}
	GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaProfileType::InvalidatemediaTimeBase()
{
	GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsMediaProfileType::GetmediaTimeUnit() const
{
	return GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsMediaProfileType::ExistmediaTimeUnit() const
{
	return GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsMediaProfileType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaProfileType::SetmediaTimeUnit().");
	}
	GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaProfileType::InvalidatemediaTimeUnit()
{
	GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsMediaProfileType::GetHeader() const
{
	return GetBase()->GetHeader();
}

void Mp7JrsMediaProfileType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaProfileType::SetHeader().");
	}
	GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsMediaProfileType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("ComponentMediaProfile")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:ComponentMediaProfile is item of type MediaProfileType
		// in element collection MediaProfileType_ComponentMediaProfile_CollectionType
		
		context->Found = true;
		Mp7JrsMediaProfileType_ComponentMediaProfile_CollectionPtr coll = GetComponentMediaProfile();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateMediaProfileType_ComponentMediaProfile_CollectionType;
				SetComponentMediaProfile(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaProfileType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMediaProfilePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("MediaFormat")) == 0)
	{
		// MediaFormat is simple element MediaFormatType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMediaFormat()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaFormatType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMediaFormatPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMediaFormat(p, client);
					if((p = GetMediaFormat()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("MediaTranscodingHints")) == 0)
	{
		// MediaTranscodingHints is simple element MediaTranscodingHintsType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMediaTranscodingHints()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaTranscodingHintsType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMediaTranscodingHintsPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMediaTranscodingHints(p, client);
					if((p = GetMediaTranscodingHints()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("MediaQuality")) == 0)
	{
		// MediaQuality is simple element MediaQualityType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMediaQuality()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaQualityType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMediaQualityPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMediaQuality(p, client);
					if((p = GetMediaQuality()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("MediaInstance")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:MediaInstance is item of type MediaInstanceType
		// in element collection MediaProfileType_MediaInstance_CollectionType
		
		context->Found = true;
		Mp7JrsMediaProfileType_MediaInstance_CollectionPtr coll = GetMediaInstance();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateMediaProfileType_MediaInstance_CollectionType;
				SetMediaInstance(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaInstanceType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMediaInstancePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for MediaProfileType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "MediaProfileType");
	}		
	
	return result;
}

XMLCh * Mp7JrsMediaProfileType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsMediaProfileType::Parse(const XMLCh * const txt)
{
	return false;
}

bool Mp7JrsMediaProfileType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsMediaProfileType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsMediaProfileType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsMediaProfileType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("MediaProfileType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_master_Exist)
	{
	// Value
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_master);
		element->setAttributeNS(X(""), X("master"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:
	if (m_ComponentMediaProfile != Mp7JrsMediaProfileType_ComponentMediaProfile_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_ComponentMediaProfile->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	// Class
	
	if (m_MediaFormat != Mp7JrsMediaFormatPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_MediaFormat->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("MediaFormat"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_MediaFormat->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_MediaTranscodingHints != Mp7JrsMediaTranscodingHintsPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_MediaTranscodingHints->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("MediaTranscodingHints"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_MediaTranscodingHints->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_MediaQuality != Mp7JrsMediaQualityPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_MediaQuality->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("MediaQuality"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_MediaQuality->Serialize(doc, element, &elem);
	}
	if (m_MediaInstance != Mp7JrsMediaProfileType_MediaInstance_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_MediaInstance->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsMediaProfileType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("master")))
	{
        // deserialize value type
        this->Setmaster(Mp7JrsConvert::TextToBool(parent->getAttributeNS(X(""), X("master"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("master")))
	{
        // deserialize value type
        this->Setmaster(Mp7JrsConvert::TextToBool(parent->getAttribute(X("master"))));
        * current = parent;
    }

	// Extensionbase is Mp7JrsDSType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("ComponentMediaProfile"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("ComponentMediaProfile")) == 0))
        {
            // Deserialize factory type
            Mp7JrsMediaProfileType_ComponentMediaProfile_CollectionPtr tmp = CreateMediaProfileType_ComponentMediaProfile_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetComponentMediaProfile(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("MediaFormat"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("MediaFormat")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateMediaFormatType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetMediaFormat(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("MediaTranscodingHints"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("MediaTranscodingHints")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateMediaTranscodingHintsType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetMediaTranscodingHints(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("MediaQuality"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("MediaQuality")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateMediaQualityType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetMediaQuality(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("MediaInstance"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("MediaInstance")) == 0))
        {
            // Deserialize factory type
            Mp7JrsMediaProfileType_MediaInstance_CollectionPtr tmp = CreateMediaProfileType_MediaInstance_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetMediaInstance(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsMediaProfileType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsMediaProfileType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("ComponentMediaProfile")) == 0))
  {
    Mp7JrsMediaProfileType_ComponentMediaProfile_CollectionPtr tmp = CreateMediaProfileType_ComponentMediaProfile_CollectionType;
    this->SetComponentMediaProfile(tmp);
    child = tmp;
  }
  if (XMLString::compareString(elementname, X("MediaFormat")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateMediaFormatType;
    }
    this->SetMediaFormat(child);
  }
  if (XMLString::compareString(elementname, X("MediaTranscodingHints")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateMediaTranscodingHintsType;
    }
    this->SetMediaTranscodingHints(child);
  }
  if (XMLString::compareString(elementname, X("MediaQuality")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateMediaQualityType;
    }
    this->SetMediaQuality(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("MediaInstance")) == 0))
  {
    Mp7JrsMediaProfileType_MediaInstance_CollectionPtr tmp = CreateMediaProfileType_MediaInstance_CollectionType;
    this->SetMediaInstance(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsMediaProfileType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetComponentMediaProfile() != Mp7JrsNodePtr())
		result.Insert(GetComponentMediaProfile());
	if (GetMediaFormat() != Mp7JrsNodePtr())
		result.Insert(GetMediaFormat());
	if (GetMediaTranscodingHints() != Mp7JrsNodePtr())
		result.Insert(GetMediaTranscodingHints());
	if (GetMediaQuality() != Mp7JrsNodePtr())
		result.Insert(GetMediaQuality());
	if (GetMediaInstance() != Mp7JrsNodePtr())
		result.Insert(GetMediaInstance());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsMediaProfileType_ExtMethodImpl.h


