 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsMediaReviewType_ExtImplInclude.h



#include "Mp7JrsRatingType.h"
#include "Mp7JrsMediaReviewType_FreeTextReview_CollectionType.h"
#include "Mp7JrsRelatedMaterialType.h"
#include "Mp7JrsAgentType.h"
#include "Mp7JrsMediaReviewType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsTextualType.h" // Element collection urn:mpeg:mpeg7:schema:2004:FreeTextReview

#include <assert.h>
IMp7JrsMediaReviewType::IMp7JrsMediaReviewType()
{

// no includefile for extension defined 
// file Mp7JrsMediaReviewType_ExtPropInit.h

}

IMp7JrsMediaReviewType::~IMp7JrsMediaReviewType()
{
// no includefile for extension defined 
// file Mp7JrsMediaReviewType_ExtPropCleanup.h

}

Mp7JrsMediaReviewType::Mp7JrsMediaReviewType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsMediaReviewType::~Mp7JrsMediaReviewType()
{
	Cleanup();
}

void Mp7JrsMediaReviewType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_Rating = Mp7JrsRatingPtr(); // Class
	m_Rating_Exist = false;
	m_FreeTextReview = Mp7JrsMediaReviewType_FreeTextReview_CollectionPtr(); // Collection
	m_ReviewReference = Mp7JrsRelatedMaterialPtr(); // Class
	m_ReviewReference_Exist = false;
	m_Reviewer = Mp7JrsPtr< Mp7JrsNode >(); // Class
	m_Reviewer_Exist = false;


// no includefile for extension defined 
// file Mp7JrsMediaReviewType_ExtMyPropInit.h

}

void Mp7JrsMediaReviewType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsMediaReviewType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Rating);
	// Mp7JrsFactory::DeleteObject(m_FreeTextReview);
	// Mp7JrsFactory::DeleteObject(m_ReviewReference);
	// Mp7JrsFactory::DeleteObject(m_Reviewer);
}

void Mp7JrsMediaReviewType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use MediaReviewTypePtr, since we
	// might need GetBase(), which isn't defined in IMediaReviewType
    const Mp7JrsPtr< Mp7JrsMediaReviewType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	if (tmp->IsValidRating())
	{
		// Mp7JrsFactory::DeleteObject(m_Rating);
		this->SetRating(Mp7JrsFactory::CloneObject(tmp->GetRating()));
	}
	else
	{
		InvalidateRating();
	}
		// Mp7JrsFactory::DeleteObject(m_FreeTextReview);
		this->SetFreeTextReview(Mp7JrsFactory::CloneObject(tmp->GetFreeTextReview()));
	if (tmp->IsValidReviewReference())
	{
		// Mp7JrsFactory::DeleteObject(m_ReviewReference);
		this->SetReviewReference(Mp7JrsFactory::CloneObject(tmp->GetReviewReference()));
	}
	else
	{
		InvalidateReviewReference();
	}
	if (tmp->IsValidReviewer())
	{
		// Mp7JrsFactory::DeleteObject(m_Reviewer);
		this->SetReviewer(Mp7JrsFactory::CloneObject(tmp->GetReviewer()));
	}
	else
	{
		InvalidateReviewer();
	}
}

Mp7JrsNodePtr Mp7JrsMediaReviewType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsMediaReviewType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsRatingPtr Mp7JrsMediaReviewType::GetRating() const
{
	return m_Rating;
}

// Element is optional
bool Mp7JrsMediaReviewType::IsValidRating() const
{
	return m_Rating_Exist;
}

Mp7JrsMediaReviewType_FreeTextReview_CollectionPtr Mp7JrsMediaReviewType::GetFreeTextReview() const
{
	return m_FreeTextReview;
}

Mp7JrsRelatedMaterialPtr Mp7JrsMediaReviewType::GetReviewReference() const
{
	return m_ReviewReference;
}

// Element is optional
bool Mp7JrsMediaReviewType::IsValidReviewReference() const
{
	return m_ReviewReference_Exist;
}

Mp7JrsPtr< Mp7JrsNode > Mp7JrsMediaReviewType::GetReviewer() const
{
	return m_Reviewer;
}

// Element is optional
bool Mp7JrsMediaReviewType::IsValidReviewer() const
{
	return m_Reviewer_Exist;
}

void Mp7JrsMediaReviewType::SetRating(const Mp7JrsRatingPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaReviewType::SetRating().");
	}
	if (m_Rating != item || m_Rating_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Rating);
		m_Rating = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Rating) m_Rating->SetParent(m_myself.getPointer());
		if (m_Rating && m_Rating->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RatingType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Rating->UseTypeAttribute = true;
		}
		m_Rating_Exist = true;
		if(m_Rating != Mp7JrsRatingPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Rating->SetContentName(XMLString::transcode("Rating"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaReviewType::InvalidateRating()
{
	m_Rating_Exist = false;
}
void Mp7JrsMediaReviewType::SetFreeTextReview(const Mp7JrsMediaReviewType_FreeTextReview_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaReviewType::SetFreeTextReview().");
	}
	if (m_FreeTextReview != item)
	{
		// Mp7JrsFactory::DeleteObject(m_FreeTextReview);
		m_FreeTextReview = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_FreeTextReview) m_FreeTextReview->SetParent(m_myself.getPointer());
		if(m_FreeTextReview != Mp7JrsMediaReviewType_FreeTextReview_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_FreeTextReview->SetContentName(XMLString::transcode("FreeTextReview"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaReviewType::SetReviewReference(const Mp7JrsRelatedMaterialPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaReviewType::SetReviewReference().");
	}
	if (m_ReviewReference != item || m_ReviewReference_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_ReviewReference);
		m_ReviewReference = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ReviewReference) m_ReviewReference->SetParent(m_myself.getPointer());
		if (m_ReviewReference && m_ReviewReference->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RelatedMaterialType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_ReviewReference->UseTypeAttribute = true;
		}
		m_ReviewReference_Exist = true;
		if(m_ReviewReference != Mp7JrsRelatedMaterialPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ReviewReference->SetContentName(XMLString::transcode("ReviewReference"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaReviewType::InvalidateReviewReference()
{
	m_ReviewReference_Exist = false;
}
void Mp7JrsMediaReviewType::SetReviewer(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaReviewType::SetReviewer().");
	}
	if (m_Reviewer != item || m_Reviewer_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Reviewer);
		m_Reviewer = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Reviewer) m_Reviewer->SetParent(m_myself.getPointer());
		// Element is abstract, so set UseTypeAttribute to true
		if (m_Reviewer) m_Reviewer->UseTypeAttribute = true;
		m_Reviewer_Exist = true;
		if(m_Reviewer != Mp7JrsPtr< Mp7JrsNode >()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Reviewer->SetContentName(XMLString::transcode("Reviewer"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaReviewType::InvalidateReviewer()
{
	m_Reviewer_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsMediaReviewType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Rating")) == 0)
	{
		// Rating is simple element RatingType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetRating()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RatingType")))) != empty)
			{
				// Is type allowed
				Mp7JrsRatingPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetRating(p, client);
					if((p = GetRating()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("FreeTextReview")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:FreeTextReview is item of type TextualType
		// in element collection MediaReviewType_FreeTextReview_CollectionType
		
		context->Found = true;
		Mp7JrsMediaReviewType_FreeTextReview_CollectionPtr coll = GetFreeTextReview();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateMediaReviewType_FreeTextReview_CollectionType;
				SetFreeTextReview(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TextualType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTextualPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("ReviewReference")) == 0)
	{
		// ReviewReference is simple element RelatedMaterialType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetReviewReference()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RelatedMaterialType")))) != empty)
			{
				// Is type allowed
				Mp7JrsRelatedMaterialPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetReviewReference(p, client);
					if((p = GetReviewReference()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Reviewer")) == 0)
	{
		// Reviewer is simple abstract element AgentType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetReviewer()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, 0)) != empty)
			{
				// Is type allowed
				Mp7JrsAgentPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetReviewer(p, client);
					if((p = GetReviewer()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for MediaReviewType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "MediaReviewType");
	}		
	
	return result;
}

XMLCh * Mp7JrsMediaReviewType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsMediaReviewType::Parse(const XMLCh * const txt)
{
	return false;
}

bool Mp7JrsMediaReviewType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	// Mp7JrsMediaReviewType has no attributes so forward it to Parse()
	return Parse(txt);
}
XMLCh* Mp7JrsMediaReviewType::ContentToString() const
{
	// Mp7JrsMediaReviewType has no attributes so forward it to ToText()
	return ToText();
}

void Mp7JrsMediaReviewType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("MediaReviewType"));
	// Element serialization:
	// Class
	
	if (m_Rating != Mp7JrsRatingPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Rating->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Rating"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Rating->Serialize(doc, element, &elem);
	}
	if (m_FreeTextReview != Mp7JrsMediaReviewType_FreeTextReview_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_FreeTextReview->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	// Class
	
	if (m_ReviewReference != Mp7JrsRelatedMaterialPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_ReviewReference->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("ReviewReference"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_ReviewReference->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Reviewer != Mp7JrsPtr< Mp7JrsNode >())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Reviewer->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Reviewer"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
			// Abstract type, so use xsitype
			m_Reviewer->UseTypeAttribute = true;
		}
		m_Reviewer->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsMediaReviewType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	parent = Mp7JrsUtil::GetNextChild(parent);
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Rating"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Rating")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateRatingType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetRating(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("FreeTextReview"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("FreeTextReview")) == 0))
        {
            // Deserialize factory type
            Mp7JrsMediaReviewType_FreeTextReview_CollectionPtr tmp = CreateMediaReviewType_FreeTextReview_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetFreeTextReview(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("ReviewReference"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("ReviewReference")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateRelatedMaterialType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetReviewReference(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Reviewer"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Reviewer")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateAgentType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetReviewer(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsMediaReviewType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsMediaReviewType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("Rating")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateRatingType;
    }
    this->SetRating(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("FreeTextReview")) == 0))
  {
    Mp7JrsMediaReviewType_FreeTextReview_CollectionPtr tmp = CreateMediaReviewType_FreeTextReview_CollectionType;
    this->SetFreeTextReview(tmp);
    child = tmp;
  }
  if (XMLString::compareString(elementname, X("ReviewReference")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateRelatedMaterialType;
    }
    this->SetReviewReference(child);
  }
  if (XMLString::compareString(elementname, X("Reviewer")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateAgentType;
    }
    this->SetReviewer(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsMediaReviewType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetRating() != Mp7JrsNodePtr())
		result.Insert(GetRating());
	if (GetFreeTextReview() != Mp7JrsNodePtr())
		result.Insert(GetFreeTextReview());
	if (GetReviewReference() != Mp7JrsNodePtr())
		result.Insert(GetReviewReference());
	if (GetReviewer() != Mp7JrsNodePtr())
		result.Insert(GetReviewer());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsMediaReviewType_ExtMethodImpl.h


