 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsMelodyType_ExtImplInclude.h



#include "Mp7JrsAudioDSType.h"
#include "Mp7JrsMeterType.h"
#include "Mp7JrsscaleType.h"
#include "Mp7JrsKeyType.h"
#include "Mp7JrsMelodyContourType.h"
#include "Mp7JrsMelodyType_MelodySequence_CollectionType.h"
#include "Mp7JrsintegerVector.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsMelodyType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsMelodySequenceType.h" // Element collection urn:mpeg:mpeg7:schema:2004:MelodySequence

#include <assert.h>
IMp7JrsMelodyType::IMp7JrsMelodyType()
{

// no includefile for extension defined 
// file Mp7JrsMelodyType_ExtPropInit.h

}

IMp7JrsMelodyType::~IMp7JrsMelodyType()
{
// no includefile for extension defined 
// file Mp7JrsMelodyType_ExtPropCleanup.h

}

Mp7JrsMelodyType::Mp7JrsMelodyType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsMelodyType::~Mp7JrsMelodyType()
{
	Cleanup();
}

void Mp7JrsMelodyType::Init()
{
	// Init base
	m_Base = CreateAudioDSType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_Meter = Mp7JrsMeterPtr(); // Class
	m_Meter_Exist = false;
	m_Scale = Mp7JrsscalePtr(); // Class
	m_Scale_Exist = false;
	m_Key = Mp7JrsKeyPtr(); // Class
	m_Key_Exist = false;
	m_MelodyContour = Mp7JrsMelodyContourPtr(); // Class
	m_MelodySequence = Mp7JrsMelodyType_MelodySequence_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsMelodyType_ExtMyPropInit.h

}

void Mp7JrsMelodyType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsMelodyType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_Meter);
	// Mp7JrsFactory::DeleteObject(m_Scale);
	// Mp7JrsFactory::DeleteObject(m_Key);
	// Mp7JrsFactory::DeleteObject(m_MelodyContour);
	// Mp7JrsFactory::DeleteObject(m_MelodySequence);
}

void Mp7JrsMelodyType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use MelodyTypePtr, since we
	// might need GetBase(), which isn't defined in IMelodyType
    const Mp7JrsPtr< Mp7JrsMelodyType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsAudioDSPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsMelodyType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	if (tmp->IsValidMeter())
	{
		// Mp7JrsFactory::DeleteObject(m_Meter);
		this->SetMeter(Mp7JrsFactory::CloneObject(tmp->GetMeter()));
	}
	else
	{
		InvalidateMeter();
	}
	if (tmp->IsValidScale())
	{
		// Mp7JrsFactory::DeleteObject(m_Scale);
		this->SetScale(Mp7JrsFactory::CloneObject(tmp->GetScale()));
	}
	else
	{
		InvalidateScale();
	}
	if (tmp->IsValidKey())
	{
		// Mp7JrsFactory::DeleteObject(m_Key);
		this->SetKey(Mp7JrsFactory::CloneObject(tmp->GetKey()));
	}
	else
	{
		InvalidateKey();
	}
		// Mp7JrsFactory::DeleteObject(m_MelodyContour);
		this->SetMelodyContour(Mp7JrsFactory::CloneObject(tmp->GetMelodyContour()));
		// Mp7JrsFactory::DeleteObject(m_MelodySequence);
		this->SetMelodySequence(Mp7JrsFactory::CloneObject(tmp->GetMelodySequence()));
}

Mp7JrsNodePtr Mp7JrsMelodyType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsMelodyType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsAudioDSType > Mp7JrsMelodyType::GetBase() const
{
	return m_Base;
}

Mp7JrsMeterPtr Mp7JrsMelodyType::GetMeter() const
{
	return m_Meter;
}

// Element is optional
bool Mp7JrsMelodyType::IsValidMeter() const
{
	return m_Meter_Exist;
}

Mp7JrsscalePtr Mp7JrsMelodyType::GetScale() const
{
	return m_Scale;
}

// Element is optional
bool Mp7JrsMelodyType::IsValidScale() const
{
	return m_Scale_Exist;
}

Mp7JrsKeyPtr Mp7JrsMelodyType::GetKey() const
{
	return m_Key;
}

// Element is optional
bool Mp7JrsMelodyType::IsValidKey() const
{
	return m_Key_Exist;
}

Mp7JrsMelodyContourPtr Mp7JrsMelodyType::GetMelodyContour() const
{
	return m_MelodyContour;
}

Mp7JrsMelodyType_MelodySequence_CollectionPtr Mp7JrsMelodyType::GetMelodySequence() const
{
	return m_MelodySequence;
}

void Mp7JrsMelodyType::SetMeter(const Mp7JrsMeterPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMelodyType::SetMeter().");
	}
	if (m_Meter != item || m_Meter_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Meter);
		m_Meter = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Meter) m_Meter->SetParent(m_myself.getPointer());
		if (m_Meter && m_Meter->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MeterType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Meter->UseTypeAttribute = true;
		}
		m_Meter_Exist = true;
		if(m_Meter != Mp7JrsMeterPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Meter->SetContentName(XMLString::transcode("Meter"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMelodyType::InvalidateMeter()
{
	m_Meter_Exist = false;
}
void Mp7JrsMelodyType::SetScale(const Mp7JrsscalePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMelodyType::SetScale().");
	}
	if (m_Scale != item || m_Scale_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Scale);
		m_Scale = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Scale) m_Scale->SetParent(m_myself.getPointer());
		if (m_Scale && m_Scale->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:scaleType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Scale->UseTypeAttribute = true;
		}
		m_Scale_Exist = true;
		if(m_Scale != Mp7JrsscalePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Scale->SetContentName(XMLString::transcode("Scale"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMelodyType::InvalidateScale()
{
	m_Scale_Exist = false;
}
void Mp7JrsMelodyType::SetKey(const Mp7JrsKeyPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMelodyType::SetKey().");
	}
	if (m_Key != item || m_Key_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Key);
		m_Key = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Key) m_Key->SetParent(m_myself.getPointer());
		if (m_Key && m_Key->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:KeyType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Key->UseTypeAttribute = true;
		}
		m_Key_Exist = true;
		if(m_Key != Mp7JrsKeyPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Key->SetContentName(XMLString::transcode("Key"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMelodyType::InvalidateKey()
{
	m_Key_Exist = false;
}
	// TODO Implement proper cleanup for nested choices (e.g. CompositionShotEditinbTemporalDecompositionType_LocalType)
// implementing setter for choice 
/* element */
void Mp7JrsMelodyType::SetMelodyContour(const Mp7JrsMelodyContourPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMelodyType::SetMelodyContour().");
	}
	if (m_MelodyContour != item)
	{
		// Mp7JrsFactory::DeleteObject(m_MelodyContour);
		m_MelodyContour = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_MelodyContour) m_MelodyContour->SetParent(m_myself.getPointer());
		if (m_MelodyContour && m_MelodyContour->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MelodyContourType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_MelodyContour->UseTypeAttribute = true;
		}
		if(m_MelodyContour != Mp7JrsMelodyContourPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_MelodyContour->SetContentName(XMLString::transcode("MelodyContour"));
		}
	// Mp7JrsFactory::DeleteObject(m_MelodySequence);
	m_MelodySequence = Mp7JrsMelodyType_MelodySequence_CollectionPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsMelodyType::SetMelodySequence(const Mp7JrsMelodyType_MelodySequence_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMelodyType::SetMelodySequence().");
	}
	if (m_MelodySequence != item)
	{
		// Mp7JrsFactory::DeleteObject(m_MelodySequence);
		m_MelodySequence = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_MelodySequence) m_MelodySequence->SetParent(m_myself.getPointer());
		if(m_MelodySequence != Mp7JrsMelodyType_MelodySequence_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_MelodySequence->SetContentName(XMLString::transcode("MelodySequence"));
		}
	// Mp7JrsFactory::DeleteObject(m_MelodyContour);
	m_MelodyContour = Mp7JrsMelodyContourPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsintegerVectorPtr Mp7JrsMelodyType::Getchannels() const
{
	return GetBase()->Getchannels();
}

bool Mp7JrsMelodyType::Existchannels() const
{
	return GetBase()->Existchannels();
}
void Mp7JrsMelodyType::Setchannels(const Mp7JrsintegerVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMelodyType::Setchannels().");
	}
	GetBase()->Setchannels(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMelodyType::Invalidatechannels()
{
	GetBase()->Invalidatechannels();
}
XMLCh * Mp7JrsMelodyType::Getid() const
{
	return GetBase()->GetBase()->Getid();
}

bool Mp7JrsMelodyType::Existid() const
{
	return GetBase()->GetBase()->Existid();
}
void Mp7JrsMelodyType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMelodyType::Setid().");
	}
	GetBase()->GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMelodyType::Invalidateid()
{
	GetBase()->GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsMelodyType::GettimeBase() const
{
	return GetBase()->GetBase()->GettimeBase();
}

bool Mp7JrsMelodyType::ExisttimeBase() const
{
	return GetBase()->GetBase()->ExisttimeBase();
}
void Mp7JrsMelodyType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMelodyType::SettimeBase().");
	}
	GetBase()->GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMelodyType::InvalidatetimeBase()
{
	GetBase()->GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsMelodyType::GettimeUnit() const
{
	return GetBase()->GetBase()->GettimeUnit();
}

bool Mp7JrsMelodyType::ExisttimeUnit() const
{
	return GetBase()->GetBase()->ExisttimeUnit();
}
void Mp7JrsMelodyType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMelodyType::SettimeUnit().");
	}
	GetBase()->GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMelodyType::InvalidatetimeUnit()
{
	GetBase()->GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsMelodyType::GetmediaTimeBase() const
{
	return GetBase()->GetBase()->GetmediaTimeBase();
}

bool Mp7JrsMelodyType::ExistmediaTimeBase() const
{
	return GetBase()->GetBase()->ExistmediaTimeBase();
}
void Mp7JrsMelodyType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMelodyType::SetmediaTimeBase().");
	}
	GetBase()->GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMelodyType::InvalidatemediaTimeBase()
{
	GetBase()->GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsMelodyType::GetmediaTimeUnit() const
{
	return GetBase()->GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsMelodyType::ExistmediaTimeUnit() const
{
	return GetBase()->GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsMelodyType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMelodyType::SetmediaTimeUnit().");
	}
	GetBase()->GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMelodyType::InvalidatemediaTimeUnit()
{
	GetBase()->GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsMelodyType::GetHeader() const
{
	return GetBase()->GetBase()->GetHeader();
}

void Mp7JrsMelodyType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMelodyType::SetHeader().");
	}
	GetBase()->GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsMelodyType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Meter")) == 0)
	{
		// Meter is simple element MeterType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMeter()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MeterType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMeterPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMeter(p, client);
					if((p = GetMeter()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Scale")) == 0)
	{
		// Scale is simple element scaleType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetScale()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:scaleType")))) != empty)
			{
				// Is type allowed
				Mp7JrsscalePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetScale(p, client);
					if((p = GetScale()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Key")) == 0)
	{
		// Key is simple element KeyType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetKey()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:KeyType")))) != empty)
			{
				// Is type allowed
				Mp7JrsKeyPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetKey(p, client);
					if((p = GetKey()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("MelodyContour")) == 0)
	{
		// MelodyContour is simple element MelodyContourType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMelodyContour()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MelodyContourType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMelodyContourPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMelodyContour(p, client);
					if((p = GetMelodyContour()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("MelodySequence")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:MelodySequence is item of type MelodySequenceType
		// in element collection MelodyType_MelodySequence_CollectionType
		
		context->Found = true;
		Mp7JrsMelodyType_MelodySequence_CollectionPtr coll = GetMelodySequence();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateMelodyType_MelodySequence_CollectionType;
				SetMelodySequence(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MelodySequenceType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMelodySequencePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for MelodyType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "MelodyType");
	}		
	
	return result;
}

XMLCh * Mp7JrsMelodyType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsMelodyType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}

bool Mp7JrsMelodyType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsMelodyType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsMelodyType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsMelodyType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("MelodyType"));
	// Element serialization:
	// Class
	
	if (m_Meter != Mp7JrsMeterPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Meter->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Meter"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Meter->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Scale != Mp7JrsscalePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Scale->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Scale"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Scale->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Key != Mp7JrsKeyPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Key->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Key"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Key->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_MelodyContour != Mp7JrsMelodyContourPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_MelodyContour->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("MelodyContour"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_MelodyContour->Serialize(doc, element, &elem);
	}
	if (m_MelodySequence != Mp7JrsMelodyType_MelodySequence_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_MelodySequence->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsMelodyType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsAudioDSType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Meter"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Meter")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateMeterType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetMeter(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Scale"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Scale")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreatescaleType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetScale(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Key"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Key")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateKeyType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetKey(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
	do // Deserialize choice just one time!
    {
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("MelodyContour"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("MelodyContour")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateMelodyContourType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetMelodyContour(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("MelodySequence"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("MelodySequence")) == 0))
        {
            // Deserialize factory type
            Mp7JrsMelodyType_MelodySequence_CollectionPtr tmp = CreateMelodyType_MelodySequence_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetMelodySequence(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
// no includefile for extension defined 
// file Mp7JrsMelodyType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsMelodyType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("Meter")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateMeterType;
    }
    this->SetMeter(child);
  }
  if (XMLString::compareString(elementname, X("Scale")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreatescaleType;
    }
    this->SetScale(child);
  }
  if (XMLString::compareString(elementname, X("Key")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateKeyType;
    }
    this->SetKey(child);
  }
  if (XMLString::compareString(elementname, X("MelodyContour")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateMelodyContourType;
    }
    this->SetMelodyContour(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("MelodySequence")) == 0))
  {
    Mp7JrsMelodyType_MelodySequence_CollectionPtr tmp = CreateMelodyType_MelodySequence_CollectionType;
    this->SetMelodySequence(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsMelodyType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetMeter() != Mp7JrsNodePtr())
		result.Insert(GetMeter());
	if (GetScale() != Mp7JrsNodePtr())
		result.Insert(GetScale());
	if (GetKey() != Mp7JrsNodePtr())
		result.Insert(GetKey());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	if (GetMelodyContour() != Mp7JrsNodePtr())
		result.Insert(GetMelodyContour());
	if (GetMelodySequence() != Mp7JrsNodePtr())
		result.Insert(GetMelodySequence());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsMelodyType_ExtMethodImpl.h


