 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsMeterType_ExtImplInclude.h



#include "Mp7JrsAudioDType.h"
#include "Mp7JrsMeterType_Numerator_LocalType.h"
#include "Mp7JrsintegerVector.h"
#include "Mp7JrsMeterType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsMeterType::IMp7JrsMeterType()
{

// no includefile for extension defined 
// file Mp7JrsMeterType_ExtPropInit.h

}

IMp7JrsMeterType::~IMp7JrsMeterType()
{
// no includefile for extension defined 
// file Mp7JrsMeterType_ExtPropCleanup.h

}

Mp7JrsMeterType::Mp7JrsMeterType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsMeterType::~Mp7JrsMeterType()
{
	Cleanup();
}

void Mp7JrsMeterType::Init()
{
	// Init base
	m_Base = CreateAudioDType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_Numerator = Mp7JrsMeterType_Numerator_LocalPtr(); // Class with content 
	m_Denominator = Mp7JrsMeterType_Denominator_LocalType::UninitializedEnumeration; // Enumeration


// no includefile for extension defined 
// file Mp7JrsMeterType_ExtMyPropInit.h

}

void Mp7JrsMeterType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsMeterType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_Numerator);
}

void Mp7JrsMeterType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use MeterTypePtr, since we
	// might need GetBase(), which isn't defined in IMeterType
    const Mp7JrsPtr< Mp7JrsMeterType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsAudioDPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsMeterType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_Numerator);
		this->SetNumerator(Mp7JrsFactory::CloneObject(tmp->GetNumerator()));
		this->SetDenominator(tmp->GetDenominator());
}

Mp7JrsNodePtr Mp7JrsMeterType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsMeterType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsAudioDType > Mp7JrsMeterType::GetBase() const
{
	return m_Base;
}

Mp7JrsMeterType_Numerator_LocalPtr Mp7JrsMeterType::GetNumerator() const
{
	return m_Numerator;
}

Mp7JrsMeterType_Denominator_LocalType::Enumeration Mp7JrsMeterType::GetDenominator() const
{
	return m_Denominator;
}

void Mp7JrsMeterType::SetNumerator(const Mp7JrsMeterType_Numerator_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMeterType::SetNumerator().");
	}
	if (m_Numerator != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Numerator);
		m_Numerator = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Numerator) m_Numerator->SetParent(m_myself.getPointer());
		if (m_Numerator && m_Numerator->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MeterType_Numerator_LocalType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Numerator->UseTypeAttribute = true;
		}
		if(m_Numerator != Mp7JrsMeterType_Numerator_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Numerator->SetContentName(XMLString::transcode("Numerator"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMeterType::SetDenominator(Mp7JrsMeterType_Denominator_LocalType::Enumeration item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMeterType::SetDenominator().");
	}
	if (m_Denominator != item)
	{
		// nothing to free here, hopefully
		m_Denominator = item;
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsintegerVectorPtr Mp7JrsMeterType::Getchannels() const
{
	return GetBase()->Getchannels();
}

bool Mp7JrsMeterType::Existchannels() const
{
	return GetBase()->Existchannels();
}
void Mp7JrsMeterType::Setchannels(const Mp7JrsintegerVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMeterType::Setchannels().");
	}
	GetBase()->Setchannels(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMeterType::Invalidatechannels()
{
	GetBase()->Invalidatechannels();
}

Mp7JrsNodeEnum Mp7JrsMeterType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Numerator")) == 0)
	{
		// Numerator is simple element MeterType_Numerator_LocalType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetNumerator()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MeterType_Numerator_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMeterType_Numerator_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetNumerator(p, client);
					if((p = GetNumerator()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for MeterType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "MeterType");
	}		
	
	return result;
}

XMLCh * Mp7JrsMeterType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsMeterType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}

bool Mp7JrsMeterType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsMeterType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsMeterType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsMeterType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("MeterType"));
	// Element serialization:
	// Class with content		
	
	if (m_Numerator != Mp7JrsMeterType_Numerator_LocalPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Numerator->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Numerator"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Numerator->Serialize(doc, element, &elem);
	}
	if(m_Denominator != Mp7JrsMeterType_Denominator_LocalType::UninitializedEnumeration) // Enumeration
	{
		XMLCh * tmp = Mp7JrsMeterType_Denominator_LocalType::ToText(m_Denominator);
		Mp7JrsUtil::SerializeTextNode(element, doc, tmp, X("urn:mpeg:mpeg7:schema:2004"), X("Denominator"), false);
		XMLString::release(&tmp);
	}

}

bool Mp7JrsMeterType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsAudioDType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Numerator"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Numerator")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateMeterType_Numerator_LocalType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetNumerator(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize enumeration element
	if(Mp7JrsUtil::HasNodeName(parent, X("Denominator"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Denominator")) == 0))
        {
            Mp7JrsMeterType_Denominator_LocalType::Enumeration tmp = Mp7JrsMeterType_Denominator_LocalType::Parse(Mp7JrsUtil::GetElementText(parent));
            if(tmp == Mp7JrsMeterType_Denominator_LocalType::UninitializedEnumeration)
            {
                    return false;
            }
            this->SetDenominator(tmp);
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsMeterType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsMeterType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("Numerator")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateMeterType_Numerator_LocalType;
    }
    this->SetNumerator(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsMeterType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetNumerator() != Mp7JrsNodePtr())
		result.Insert(GetNumerator());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsMeterType_ExtMethodImpl.h


