 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsNameComponentType_ExtImplInclude.h



#include "Mp7JrsTextualBaseType.h"
#include "Mp7JrsTextualBaseType_phoneticTranscription_CollectionType.h"
#include "Mp7JrsNameComponentType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsNameComponentType::IMp7JrsNameComponentType()
{

// no includefile for extension defined 
// file Mp7JrsNameComponentType_ExtPropInit.h

}

IMp7JrsNameComponentType::~IMp7JrsNameComponentType()
{
// no includefile for extension defined 
// file Mp7JrsNameComponentType_ExtPropCleanup.h

}

Mp7JrsNameComponentType::Mp7JrsNameComponentType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsNameComponentType::~Mp7JrsNameComponentType()
{
	Cleanup();
}

void Mp7JrsNameComponentType::Init()
{
	// Init base
	m_Base = CreateTextualBaseType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_initial = NULL; // String
	m_initial_Exist = false;
	m_abbrev = NULL; // String
	m_abbrev_Exist = false;



// no includefile for extension defined 
// file Mp7JrsNameComponentType_ExtMyPropInit.h

}

void Mp7JrsNameComponentType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsNameComponentType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	XMLString::release(&m_initial);
	XMLString::release(&m_abbrev);
}

void Mp7JrsNameComponentType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use NameComponentTypePtr, since we
	// might need GetBase(), which isn't defined in INameComponentType
    const Mp7JrsPtr< Mp7JrsNameComponentType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsTextualBasePtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsNameComponentType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	XMLString::release(&m_initial);
	if (tmp->Existinitial())
	{
		this->Setinitial(XMLString::replicate(tmp->Getinitial()));
	}
	else
	{
		Invalidateinitial();
	}
	}
	{
	XMLString::release(&m_abbrev);
	if (tmp->Existabbrev())
	{
		this->Setabbrev(XMLString::replicate(tmp->Getabbrev()));
	}
	else
	{
		Invalidateabbrev();
	}
	}
}

Mp7JrsNodePtr Mp7JrsNameComponentType::GetBaseRoot()
{
	return GetBase();
}

const Mp7JrsNodePtr Mp7JrsNameComponentType::GetBaseRoot() const
{
	return GetBase();
}


Mp7JrsPtr< Mp7JrsTextualBaseType > Mp7JrsNameComponentType::GetBase() const
{
	return m_Base;
}

void Mp7JrsNameComponentType::SetContent(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsNameComponentType::SetContent().");
	}
	GetBase()->SetContent(item);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsNameComponentType::GetContent() const
{
	return GetBase()->GetContent();
}
XMLCh * Mp7JrsNameComponentType::Getinitial() const
{
	return m_initial;
}

bool Mp7JrsNameComponentType::Existinitial() const
{
	return m_initial_Exist;
}
void Mp7JrsNameComponentType::Setinitial(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsNameComponentType::Setinitial().");
	}
    m_initial = item;
	m_initial_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsNameComponentType::Invalidateinitial()
{
	m_initial_Exist = false;
}
XMLCh * Mp7JrsNameComponentType::Getabbrev() const
{
	return m_abbrev;
}

bool Mp7JrsNameComponentType::Existabbrev() const
{
	return m_abbrev_Exist;
}
void Mp7JrsNameComponentType::Setabbrev(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsNameComponentType::Setabbrev().");
	}
    m_abbrev = item;
	m_abbrev_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsNameComponentType::Invalidateabbrev()
{
	m_abbrev_Exist = false;
}
XMLCh * Mp7JrsNameComponentType::Getlang() const
{
	return GetBase()->Getlang();
}

bool Mp7JrsNameComponentType::Existlang() const
{
	return GetBase()->Existlang();
}
void Mp7JrsNameComponentType::Setlang(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsNameComponentType::Setlang().");
	}
	GetBase()->Setlang(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsNameComponentType::Invalidatelang()
{
	GetBase()->Invalidatelang();
}
Mp7JrsTextualBaseType_phoneticTranscription_CollectionPtr Mp7JrsNameComponentType::GetphoneticTranscription() const
{
	return GetBase()->GetphoneticTranscription();
}

bool Mp7JrsNameComponentType::ExistphoneticTranscription() const
{
	return GetBase()->ExistphoneticTranscription();
}
void Mp7JrsNameComponentType::SetphoneticTranscription(const Mp7JrsTextualBaseType_phoneticTranscription_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsNameComponentType::SetphoneticTranscription().");
	}
	GetBase()->SetphoneticTranscription(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsNameComponentType::InvalidatephoneticTranscription()
{
	GetBase()->InvalidatephoneticTranscription();
}
Mp7JrsphoneticAlphabetType::Enumeration Mp7JrsNameComponentType::GetphoneticAlphabet() const
{
	return GetBase()->GetphoneticAlphabet();
}

bool Mp7JrsNameComponentType::ExistphoneticAlphabet() const
{
	return GetBase()->ExistphoneticAlphabet();
}
void Mp7JrsNameComponentType::SetphoneticAlphabet(Mp7JrsphoneticAlphabetType::Enumeration item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsNameComponentType::SetphoneticAlphabet().");
	}
	GetBase()->SetphoneticAlphabet(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsNameComponentType::InvalidatephoneticAlphabet()
{
	GetBase()->InvalidatephoneticAlphabet();
}

Mp7JrsNodeEnum Mp7JrsNameComponentType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	if(isentrypoint)
	{
		// No Mp7JrsNodePtr subelement to get or create for NameComponentType
		Mp7JrsXPathParseContext::ErrorNoSubElement(context, "NameComponentType");
	}
	
	return result;
}

XMLCh * Mp7JrsNameComponentType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsNameComponentType::Parse(const XMLCh * const txt)
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return false;
}

bool Mp7JrsNameComponentType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsNameComponentType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsNameComponentType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsNameComponentType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("NameComponentType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_initial_Exist)
	{
	// String
	if(m_initial != (XMLCh *)NULL)
	{
		element->setAttributeNS(X(""), X("initial"), m_initial);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_abbrev_Exist)
	{
	// String
	if(m_abbrev != (XMLCh *)NULL)
	{
		element->setAttributeNS(X(""), X("abbrev"), m_abbrev);
	}
		
	} // Optional attriute
	
	// Element serialization:

}

bool Mp7JrsNameComponentType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("initial")))
	{
        // Deserialize string type
        this->Setinitial(Mp7JrsConvert::TextToString(parent->getAttributeNS(X(""), X("initial"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("initial")))
	{
        // Deserialize string type
        this->Setinitial(Mp7JrsConvert::TextToString(parent->getAttribute(X("initial"))));
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("abbrev")))
	{
        // Deserialize string type
        this->Setabbrev(Mp7JrsConvert::TextToString(parent->getAttributeNS(X(""), X("abbrev"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("abbrev")))
	{
        // Deserialize string type
        this->Setabbrev(Mp7JrsConvert::TextToString(parent->getAttribute(X("abbrev"))));
        * current = parent;
    }

	// Extensionbase is Mp7JrsTextualBaseType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
// no includefile for extension defined 
// file Mp7JrsNameComponentType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsNameComponentType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsNameComponentType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsNameComponentType_ExtMethodImpl.h


