 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsOrderingKeyType_ExtImplInclude.h



#include "Mp7JrsHeaderType.h"
#include "Mp7JrsOrderingKeyType_Selector_LocalType.h"
#include "Mp7JrsOrderingKeyType_Field_CollectionType.h"
#include "Mp7JrsOrderingKeyType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsOrderingKeyType_Field_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Field

#include <assert.h>
IMp7JrsOrderingKeyType::IMp7JrsOrderingKeyType()
{

// no includefile for extension defined 
// file Mp7JrsOrderingKeyType_ExtPropInit.h

}

IMp7JrsOrderingKeyType::~IMp7JrsOrderingKeyType()
{
// no includefile for extension defined 
// file Mp7JrsOrderingKeyType_ExtPropCleanup.h

}

Mp7JrsOrderingKeyType::Mp7JrsOrderingKeyType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsOrderingKeyType::~Mp7JrsOrderingKeyType()
{
	Cleanup();
}

void Mp7JrsOrderingKeyType::Init()
{
	// Init base
	m_Base = CreateHeaderType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_name = NULL; // String
	m_name_Exist = false;
	m_semantics = NULL; // String
	m_semantics_Exist = false;
	m_direction = Mp7JrsOrderingKeyType_direction_LocalType::UninitializedEnumeration;
// old	m_direction = Mp7JrsOrderingKeyType_direction_LocalType::Enumeration::;
// old	m_direction_Default = Mp7JrsOrderingKeyType_direction_LocalType::Enumeration::descending;
	m_direction_Default = Mp7JrsOrderingKeyType_direction_LocalType::descending;
	m_direction_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_Selector = Mp7JrsOrderingKeyType_Selector_LocalPtr(); // Class
	m_Field = Mp7JrsOrderingKeyType_Field_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsOrderingKeyType_ExtMyPropInit.h

}

void Mp7JrsOrderingKeyType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsOrderingKeyType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	XMLString::release(&m_name);
	XMLString::release(&m_semantics);
	// Mp7JrsFactory::DeleteObject(m_Selector);
	// Mp7JrsFactory::DeleteObject(m_Field);
}

void Mp7JrsOrderingKeyType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use OrderingKeyTypePtr, since we
	// might need GetBase(), which isn't defined in IOrderingKeyType
    const Mp7JrsPtr< Mp7JrsOrderingKeyType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsHeaderPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsOrderingKeyType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	XMLString::release(&m_name);
	if (tmp->Existname())
	{
		this->Setname(XMLString::replicate(tmp->Getname()));
	}
	else
	{
		Invalidatename();
	}
	}
	{
	XMLString::release(&m_semantics);
	if (tmp->Existsemantics())
	{
		this->Setsemantics(XMLString::replicate(tmp->Getsemantics()));
	}
	else
	{
		Invalidatesemantics();
	}
	}
	{
	if (tmp->Existdirection())
	{
		this->Setdirection(tmp->Getdirection());
	}
	else
	{
		Invalidatedirection();
	}
	}
		// Mp7JrsFactory::DeleteObject(m_Selector);
		this->SetSelector(Mp7JrsFactory::CloneObject(tmp->GetSelector()));
		// Mp7JrsFactory::DeleteObject(m_Field);
		this->SetField(Mp7JrsFactory::CloneObject(tmp->GetField()));
}

Mp7JrsNodePtr Mp7JrsOrderingKeyType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsOrderingKeyType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsHeaderType > Mp7JrsOrderingKeyType::GetBase() const
{
	return m_Base;
}

XMLCh * Mp7JrsOrderingKeyType::Getname() const
{
	return m_name;
}

bool Mp7JrsOrderingKeyType::Existname() const
{
	return m_name_Exist;
}
void Mp7JrsOrderingKeyType::Setname(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsOrderingKeyType::Setname().");
	}
    m_name = item;
	m_name_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsOrderingKeyType::Invalidatename()
{
	m_name_Exist = false;
}
XMLCh * Mp7JrsOrderingKeyType::Getsemantics() const
{
	return m_semantics;
}

bool Mp7JrsOrderingKeyType::Existsemantics() const
{
	return m_semantics_Exist;
}
void Mp7JrsOrderingKeyType::Setsemantics(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsOrderingKeyType::Setsemantics().");
	}
    m_semantics = item;
	m_semantics_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsOrderingKeyType::Invalidatesemantics()
{
	m_semantics_Exist = false;
}
Mp7JrsOrderingKeyType_direction_LocalType::Enumeration Mp7JrsOrderingKeyType::Getdirection() const
{
	if (this->Existdirection()) {
		return m_direction;
	} else {
		return m_direction_Default;
	}
}

bool Mp7JrsOrderingKeyType::Existdirection() const
{
	return m_direction_Exist;
}
void Mp7JrsOrderingKeyType::Setdirection(Mp7JrsOrderingKeyType_direction_LocalType::Enumeration item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsOrderingKeyType::Setdirection().");
	}
    m_direction = item;
	m_direction_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsOrderingKeyType::Invalidatedirection()
{
	m_direction_Exist = false;
}
Mp7JrsOrderingKeyType_Selector_LocalPtr Mp7JrsOrderingKeyType::GetSelector() const
{
	return m_Selector;
}

Mp7JrsOrderingKeyType_Field_CollectionPtr Mp7JrsOrderingKeyType::GetField() const
{
	return m_Field;
}

void Mp7JrsOrderingKeyType::SetSelector(const Mp7JrsOrderingKeyType_Selector_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsOrderingKeyType::SetSelector().");
	}
	if (m_Selector != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Selector);
		m_Selector = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Selector) m_Selector->SetParent(m_myself.getPointer());
		if (m_Selector && m_Selector->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:OrderingKeyType_Selector_LocalType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Selector->UseTypeAttribute = true;
		}
		if(m_Selector != Mp7JrsOrderingKeyType_Selector_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Selector->SetContentName(XMLString::transcode("Selector"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsOrderingKeyType::SetField(const Mp7JrsOrderingKeyType_Field_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsOrderingKeyType::SetField().");
	}
	if (m_Field != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Field);
		m_Field = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Field) m_Field->SetParent(m_myself.getPointer());
		if(m_Field != Mp7JrsOrderingKeyType_Field_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Field->SetContentName(XMLString::transcode("Field"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsOrderingKeyType::Getid() const
{
	return GetBase()->Getid();
}

bool Mp7JrsOrderingKeyType::Existid() const
{
	return GetBase()->Existid();
}
void Mp7JrsOrderingKeyType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsOrderingKeyType::Setid().");
	}
	GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsOrderingKeyType::Invalidateid()
{
	GetBase()->Invalidateid();
}

Mp7JrsNodeEnum Mp7JrsOrderingKeyType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Selector")) == 0)
	{
		// Selector is simple element OrderingKeyType_Selector_LocalType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetSelector()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:OrderingKeyType_Selector_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsOrderingKeyType_Selector_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetSelector(p, client);
					if((p = GetSelector()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Field")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Field is item of type OrderingKeyType_Field_LocalType
		// in element collection OrderingKeyType_Field_CollectionType
		
		context->Found = true;
		Mp7JrsOrderingKeyType_Field_CollectionPtr coll = GetField();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateOrderingKeyType_Field_CollectionType;
				SetField(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:OrderingKeyType_Field_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsOrderingKeyType_Field_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for OrderingKeyType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "OrderingKeyType");
	}		
	
	return result;
}

XMLCh * Mp7JrsOrderingKeyType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsOrderingKeyType::Parse(const XMLCh * const txt)
{
	return false;
}

bool Mp7JrsOrderingKeyType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsOrderingKeyType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsOrderingKeyType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsOrderingKeyType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("OrderingKeyType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_name_Exist)
	{
	// String
	if(m_name != (XMLCh *)NULL)
	{
		element->setAttributeNS(X(""), X("name"), m_name);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_semantics_Exist)
	{
	// String
	if(m_semantics != (XMLCh *)NULL)
	{
		element->setAttributeNS(X(""), X("semantics"), m_semantics);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_direction_Exist)
	{
	// Enumeration
	if(m_direction != Mp7JrsOrderingKeyType_direction_LocalType::UninitializedEnumeration)
	{
		XMLCh * tmp = Mp7JrsOrderingKeyType_direction_LocalType::ToText(m_direction);
		element->setAttributeNS(X(""), X("direction"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:
	// Class
	
	if (m_Selector != Mp7JrsOrderingKeyType_Selector_LocalPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Selector->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Selector"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Selector->Serialize(doc, element, &elem);
	}
	if (m_Field != Mp7JrsOrderingKeyType_Field_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Field->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsOrderingKeyType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("name")))
	{
        // Deserialize string type
        this->Setname(Mp7JrsConvert::TextToString(parent->getAttributeNS(X(""), X("name"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("name")))
	{
        // Deserialize string type
        this->Setname(Mp7JrsConvert::TextToString(parent->getAttribute(X("name"))));
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("semantics")))
	{
        // Deserialize string type
        this->Setsemantics(Mp7JrsConvert::TextToString(parent->getAttributeNS(X(""), X("semantics"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("semantics")))
	{
        // Deserialize string type
        this->Setsemantics(Mp7JrsConvert::TextToString(parent->getAttribute(X("semantics"))));
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("direction")))
	{
        this->Setdirection(Mp7JrsOrderingKeyType_direction_LocalType::Parse(parent->getAttributeNS(X(""), X("direction"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("direction")))
	{
        this->Setdirection(Mp7JrsOrderingKeyType_direction_LocalType::Parse(parent->getAttribute(X("direction"))));
        * current = parent;
    }

	// Extensionbase is Mp7JrsHeaderType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Selector"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Selector")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateOrderingKeyType_Selector_LocalType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetSelector(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Field"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Field")) == 0))
        {
            // Deserialize factory type
            Mp7JrsOrderingKeyType_Field_CollectionPtr tmp = CreateOrderingKeyType_Field_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetField(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsOrderingKeyType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsOrderingKeyType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("Selector")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateOrderingKeyType_Selector_LocalType;
    }
    this->SetSelector(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Field")) == 0))
  {
    Mp7JrsOrderingKeyType_Field_CollectionPtr tmp = CreateOrderingKeyType_Field_CollectionType;
    this->SetField(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsOrderingKeyType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetSelector() != Mp7JrsNodePtr())
		result.Insert(GetSelector());
	if (GetField() != Mp7JrsNodePtr())
		result.Insert(GetField());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsOrderingKeyType_ExtMethodImpl.h


