 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsPackageType_LocalType_ExtImplInclude.h



#include "Mp7JrsPackageType_Scheme_LocalType.h"
#include "Mp7JrsPackageType.h"
#include "Mp7JrsPackageType_LocalType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"

#include <assert.h>
IMp7JrsPackageType_LocalType::IMp7JrsPackageType_LocalType()
{

// no includefile for extension defined 
// file Mp7JrsPackageType_LocalType_ExtPropInit.h

}

IMp7JrsPackageType_LocalType::~IMp7JrsPackageType_LocalType()
{
// no includefile for extension defined 
// file Mp7JrsPackageType_LocalType_ExtPropCleanup.h

}

Mp7JrsPackageType_LocalType::Mp7JrsPackageType_LocalType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsPackageType_LocalType::~Mp7JrsPackageType_LocalType()
{
	Cleanup();
}

void Mp7JrsPackageType_LocalType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_Scheme = Mp7JrsPackageType_Scheme_LocalPtr(); // Class
	m_Package = Mp7JrsPackagePtr(); // Class


// no includefile for extension defined 
// file Mp7JrsPackageType_LocalType_ExtMyPropInit.h

}

void Mp7JrsPackageType_LocalType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsPackageType_LocalType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Scheme);
	// Mp7JrsFactory::DeleteObject(m_Package);
}

void Mp7JrsPackageType_LocalType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use PackageType_LocalTypePtr, since we
	// might need GetBase(), which isn't defined in IPackageType_LocalType
    const Mp7JrsPtr< Mp7JrsPackageType_LocalType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		// Mp7JrsFactory::DeleteObject(m_Scheme);
		this->SetScheme(Mp7JrsFactory::CloneObject(tmp->GetScheme()));
		// Mp7JrsFactory::DeleteObject(m_Package);
		this->SetPackage(Mp7JrsFactory::CloneObject(tmp->GetPackage()));
}

Mp7JrsNodePtr Mp7JrsPackageType_LocalType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsPackageType_LocalType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsPackageType_Scheme_LocalPtr Mp7JrsPackageType_LocalType::GetScheme() const
{
	return m_Scheme;
}

Mp7JrsPackagePtr Mp7JrsPackageType_LocalType::GetPackage() const
{
	return m_Package;
}

// implementing setter for choice 
/* element */
void Mp7JrsPackageType_LocalType::SetScheme(const Mp7JrsPackageType_Scheme_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPackageType_LocalType::SetScheme().");
	}
	if (m_Scheme != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Scheme);
		m_Scheme = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Scheme) m_Scheme->SetParent(m_myself.getPointer());
		if (m_Scheme && m_Scheme->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:PackageType_Scheme_LocalType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Scheme->UseTypeAttribute = true;
		}
		if(m_Scheme != Mp7JrsPackageType_Scheme_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Scheme->SetContentName(XMLString::transcode("Scheme"));
		}
	// Mp7JrsFactory::DeleteObject(m_Package);
	m_Package = Mp7JrsPackagePtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsPackageType_LocalType::SetPackage(const Mp7JrsPackagePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPackageType_LocalType::SetPackage().");
	}
	if (m_Package != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Package);
		m_Package = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Package) m_Package->SetParent(m_myself.getPointer());
		if (m_Package && m_Package->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:PackageType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Package->UseTypeAttribute = true;
		}
		if(m_Package != Mp7JrsPackagePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Package->SetContentName(XMLString::transcode("Package"));
		}
	// Mp7JrsFactory::DeleteObject(m_Scheme);
	m_Scheme = Mp7JrsPackageType_Scheme_LocalPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

// Type: PackageType_LocalType: Only types with an element name can have GetOrCreate()
// Mp7JrsNodeEnum Mp7JrsPackageType_LocalType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
//

XMLCh * Mp7JrsPackageType_LocalType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsPackageType_LocalType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsPackageType_LocalType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// Serialize Choice of Sequence collection item
	element = parent;


	// Element serialization:
	// Class
	
	if (m_Scheme != Mp7JrsPackageType_Scheme_LocalPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Scheme->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Scheme"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Scheme->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Package != Mp7JrsPackagePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Package->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Package"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Package->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsPackageType_LocalType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	do // Deserialize choice just one time!
    {
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Scheme"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Scheme")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreatePackageType_Scheme_LocalType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetScheme(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Package"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Package")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreatePackageType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetPackage(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
// no includefile for extension defined 
// file Mp7JrsPackageType_LocalType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsPackageType_LocalType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("Scheme")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreatePackageType_Scheme_LocalType;
    }
    this->SetScheme(child);
  }
  if (XMLString::compareString(elementname, X("Package")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreatePackageType;
    }
    this->SetPackage(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsPackageType_LocalType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetScheme() != Mp7JrsNodePtr())
		result.Insert(GetScheme());
	if (GetPackage() != Mp7JrsNodePtr())
		result.Insert(GetPackage());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsPackageType_LocalType_ExtMethodImpl.h


