 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsParentalGuidanceType_ExtImplInclude.h



#include "Mp7JrsControlledTermUseType.h"
#include "Mp7JrsParentalGuidanceType_Region_CollectionType.h"
#include "Mp7JrsParentalGuidanceType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsregionCode.h" // Element collection urn:mpeg:mpeg7:schema:2004:Region

#include <assert.h>
IMp7JrsParentalGuidanceType::IMp7JrsParentalGuidanceType()
{

// no includefile for extension defined 
// file Mp7JrsParentalGuidanceType_ExtPropInit.h

}

IMp7JrsParentalGuidanceType::~IMp7JrsParentalGuidanceType()
{
// no includefile for extension defined 
// file Mp7JrsParentalGuidanceType_ExtPropCleanup.h

}

Mp7JrsParentalGuidanceType::Mp7JrsParentalGuidanceType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsParentalGuidanceType::~Mp7JrsParentalGuidanceType()
{
	Cleanup();
}

void Mp7JrsParentalGuidanceType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_ParentalRating = Mp7JrsControlledTermUsePtr(); // Class
	m_MinimumAge = (0); // Value

	m_Region = Mp7JrsParentalGuidanceType_Region_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsParentalGuidanceType_ExtMyPropInit.h

}

void Mp7JrsParentalGuidanceType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsParentalGuidanceType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_ParentalRating);
	// Mp7JrsFactory::DeleteObject(m_Region);
}

void Mp7JrsParentalGuidanceType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use ParentalGuidanceTypePtr, since we
	// might need GetBase(), which isn't defined in IParentalGuidanceType
    const Mp7JrsPtr< Mp7JrsParentalGuidanceType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		// Mp7JrsFactory::DeleteObject(m_ParentalRating);
		this->SetParentalRating(Mp7JrsFactory::CloneObject(tmp->GetParentalRating()));
		this->SetMinimumAge(tmp->GetMinimumAge());
		// Mp7JrsFactory::DeleteObject(m_Region);
		this->SetRegion(Mp7JrsFactory::CloneObject(tmp->GetRegion()));
}

Mp7JrsNodePtr Mp7JrsParentalGuidanceType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsParentalGuidanceType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsControlledTermUsePtr Mp7JrsParentalGuidanceType::GetParentalRating() const
{
	return m_ParentalRating;
}

unsigned Mp7JrsParentalGuidanceType::GetMinimumAge() const
{
	return m_MinimumAge;
}

Mp7JrsParentalGuidanceType_Region_CollectionPtr Mp7JrsParentalGuidanceType::GetRegion() const
{
	return m_Region;
}

	// TODO Implement proper cleanup for nested choices (e.g. CompositionShotEditinbTemporalDecompositionType_LocalType)
// implementing setter for choice 
/* element */
void Mp7JrsParentalGuidanceType::SetParentalRating(const Mp7JrsControlledTermUsePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsParentalGuidanceType::SetParentalRating().");
	}
	if (m_ParentalRating != item)
	{
		// Mp7JrsFactory::DeleteObject(m_ParentalRating);
		m_ParentalRating = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ParentalRating) m_ParentalRating->SetParent(m_myself.getPointer());
		if (m_ParentalRating && m_ParentalRating->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ControlledTermUseType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_ParentalRating->UseTypeAttribute = true;
		}
		if(m_ParentalRating != Mp7JrsControlledTermUsePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ParentalRating->SetContentName(XMLString::transcode("ParentalRating"));
		}
	
	m_MinimumAge_Valid = false;
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsParentalGuidanceType::SetMinimumAge(unsigned item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsParentalGuidanceType::SetMinimumAge().");
	}
	if (m_MinimumAge != item)
	{
		m_MinimumAge = item;
		m_MinimumAge_Valid = true;
	// Mp7JrsFactory::DeleteObject(m_ParentalRating);
	m_ParentalRating = Mp7JrsControlledTermUsePtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsParentalGuidanceType::SetRegion(const Mp7JrsParentalGuidanceType_Region_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsParentalGuidanceType::SetRegion().");
	}
	if (m_Region != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Region);
		m_Region = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Region) m_Region->SetParent(m_myself.getPointer());
		if(m_Region != Mp7JrsParentalGuidanceType_Region_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Region->SetContentName(XMLString::transcode("Region"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsParentalGuidanceType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("ParentalRating")) == 0)
	{
		// ParentalRating is simple element ControlledTermUseType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetParentalRating()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ControlledTermUseType")))) != empty)
			{
				// Is type allowed
				Mp7JrsControlledTermUsePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetParentalRating(p, client);
					if((p = GetParentalRating()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Region")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Region is item of type regionCode
		// in element collection ParentalGuidanceType_Region_CollectionType
		
		context->Found = true;
		Mp7JrsParentalGuidanceType_Region_CollectionPtr coll = GetRegion();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateParentalGuidanceType_Region_CollectionType;
				SetRegion(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:regionCode")))) != empty)
			{
				// Is type allowed
				Mp7JrsregionCodePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for ParentalGuidanceType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "ParentalGuidanceType");
	}		
	
	return result;
}

XMLCh * Mp7JrsParentalGuidanceType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsParentalGuidanceType::Parse(const XMLCh * const txt)
{
	return false;
}

bool Mp7JrsParentalGuidanceType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	// Mp7JrsParentalGuidanceType has no attributes so forward it to Parse()
	return Parse(txt);
}
XMLCh* Mp7JrsParentalGuidanceType::ContentToString() const
{
	// Mp7JrsParentalGuidanceType has no attributes so forward it to ToText()
	return ToText();
}

void Mp7JrsParentalGuidanceType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("ParentalGuidanceType"));
	// Element serialization:
	// Class
	
	if (m_ParentalRating != Mp7JrsControlledTermUsePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_ParentalRating->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("ParentalRating"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_ParentalRating->Serialize(doc, element, &elem);
	}
	if(m_MinimumAge_Valid) // Value
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_MinimumAge);
		Mp7JrsUtil::SerializeTextNode(element, doc, tmp, X("urn:mpeg:mpeg7:schema:2004"), X("MinimumAge"), false);
		XMLString::release(&tmp);
	}
	if (m_Region != Mp7JrsParentalGuidanceType_Region_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Region->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsParentalGuidanceType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	parent = Mp7JrsUtil::GetNextChild(parent);
	do // Deserialize choice just one time!
    {
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("ParentalRating"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("ParentalRating")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateControlledTermUseType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetParentalRating(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	// Deserialize value element
	if(Mp7JrsUtil::HasNodeName(parent, X("MinimumAge"), X("urn:mpeg:mpeg7:schema:2004")))
	{
		XMLCh * tmp = Mp7JrsUtil::GetElementText(parent);
		this->SetMinimumAge(Mp7JrsConvert::TextToUnsignedInt(tmp));
		XMLString::release(&tmp);
		found = true;
		* current = parent;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		continue;       // For choices
	}
	} while(false);
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Region"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Region")) == 0))
        {
            // Deserialize factory type
            Mp7JrsParentalGuidanceType_Region_CollectionPtr tmp = CreateParentalGuidanceType_Region_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetRegion(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsParentalGuidanceType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsParentalGuidanceType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("ParentalRating")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateControlledTermUseType;
    }
    this->SetParentalRating(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Region")) == 0))
  {
    Mp7JrsParentalGuidanceType_Region_CollectionPtr tmp = CreateParentalGuidanceType_Region_CollectionType;
    this->SetRegion(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsParentalGuidanceType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetRegion() != Mp7JrsNodePtr())
		result.Insert(GetRegion());
	if (GetParentalRating() != Mp7JrsNodePtr())
		result.Insert(GetParentalRating());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsParentalGuidanceType_ExtMethodImpl.h


