 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsProbabilityDistributionType_ExtImplInclude.h



#include "Mp7JrsProbabilityModelType.h"
#include "Mp7JrsDoubleMatrixType.h"
#include "Mp7JrsProbabilityDistributionType_Moment_CollectionType.h"
#include "Mp7JrsProbabilityDistributionType_Cumulant_CollectionType.h"
#include "Mp7JrszeroToOneType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsProbabilityDistributionType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsProbabilityDistributionType_Moment_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Moment
#include "Mp7JrsProbabilityDistributionType_Cumulant_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Cumulant

#include <assert.h>
IMp7JrsProbabilityDistributionType::IMp7JrsProbabilityDistributionType()
{

// no includefile for extension defined 
// file Mp7JrsProbabilityDistributionType_ExtPropInit.h

}

IMp7JrsProbabilityDistributionType::~IMp7JrsProbabilityDistributionType()
{
// no includefile for extension defined 
// file Mp7JrsProbabilityDistributionType_ExtPropCleanup.h

}

Mp7JrsProbabilityDistributionType::Mp7JrsProbabilityDistributionType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsProbabilityDistributionType::~Mp7JrsProbabilityDistributionType()
{
	Cleanup();
}

void Mp7JrsProbabilityDistributionType::Init()
{
	// Init base
	m_Base = CreateProbabilityModelType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_dim = 0; // Value
	m_dim_Default = 1;
	m_dim_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_Mean = Mp7JrsDoubleMatrixPtr(); // Class
	m_Mean_Exist = false;
	m_Variance = Mp7JrsDoubleMatrixPtr(); // Class
	m_Variance_Exist = false;
	m_Min = Mp7JrsDoubleMatrixPtr(); // Class
	m_Min_Exist = false;
	m_Max = Mp7JrsDoubleMatrixPtr(); // Class
	m_Max_Exist = false;
	m_Mode = Mp7JrsDoubleMatrixPtr(); // Class
	m_Mode_Exist = false;
	m_Median = Mp7JrsDoubleMatrixPtr(); // Class
	m_Median_Exist = false;
	m_Moment = Mp7JrsProbabilityDistributionType_Moment_CollectionPtr(); // Collection
	m_Cumulant = Mp7JrsProbabilityDistributionType_Cumulant_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsProbabilityDistributionType_ExtMyPropInit.h

}

void Mp7JrsProbabilityDistributionType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsProbabilityDistributionType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_Mean);
	// Mp7JrsFactory::DeleteObject(m_Variance);
	// Mp7JrsFactory::DeleteObject(m_Min);
	// Mp7JrsFactory::DeleteObject(m_Max);
	// Mp7JrsFactory::DeleteObject(m_Mode);
	// Mp7JrsFactory::DeleteObject(m_Median);
	// Mp7JrsFactory::DeleteObject(m_Moment);
	// Mp7JrsFactory::DeleteObject(m_Cumulant);
}

void Mp7JrsProbabilityDistributionType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use ProbabilityDistributionTypePtr, since we
	// might need GetBase(), which isn't defined in IProbabilityDistributionType
    const Mp7JrsPtr< Mp7JrsProbabilityDistributionType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsProbabilityModelPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsProbabilityDistributionType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	if (tmp->Existdim())
	{
		this->Setdim(tmp->Getdim());
	}
	else
	{
		Invalidatedim();
	}
	}
	if (tmp->IsValidMean())
	{
		// Mp7JrsFactory::DeleteObject(m_Mean);
		this->SetMean(Mp7JrsFactory::CloneObject(tmp->GetMean()));
	}
	else
	{
		InvalidateMean();
	}
	if (tmp->IsValidVariance())
	{
		// Mp7JrsFactory::DeleteObject(m_Variance);
		this->SetVariance(Mp7JrsFactory::CloneObject(tmp->GetVariance()));
	}
	else
	{
		InvalidateVariance();
	}
	if (tmp->IsValidMin())
	{
		// Mp7JrsFactory::DeleteObject(m_Min);
		this->SetMin(Mp7JrsFactory::CloneObject(tmp->GetMin()));
	}
	else
	{
		InvalidateMin();
	}
	if (tmp->IsValidMax())
	{
		// Mp7JrsFactory::DeleteObject(m_Max);
		this->SetMax(Mp7JrsFactory::CloneObject(tmp->GetMax()));
	}
	else
	{
		InvalidateMax();
	}
	if (tmp->IsValidMode())
	{
		// Mp7JrsFactory::DeleteObject(m_Mode);
		this->SetMode(Mp7JrsFactory::CloneObject(tmp->GetMode()));
	}
	else
	{
		InvalidateMode();
	}
	if (tmp->IsValidMedian())
	{
		// Mp7JrsFactory::DeleteObject(m_Median);
		this->SetMedian(Mp7JrsFactory::CloneObject(tmp->GetMedian()));
	}
	else
	{
		InvalidateMedian();
	}
		// Mp7JrsFactory::DeleteObject(m_Moment);
		this->SetMoment(Mp7JrsFactory::CloneObject(tmp->GetMoment()));
		// Mp7JrsFactory::DeleteObject(m_Cumulant);
		this->SetCumulant(Mp7JrsFactory::CloneObject(tmp->GetCumulant()));
}

Mp7JrsNodePtr Mp7JrsProbabilityDistributionType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsProbabilityDistributionType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsProbabilityModelType > Mp7JrsProbabilityDistributionType::GetBase() const
{
	return m_Base;
}

unsigned Mp7JrsProbabilityDistributionType::Getdim() const
{
	if (this->Existdim()) {
		return m_dim;
	} else {
		return m_dim_Default;
	}
}

bool Mp7JrsProbabilityDistributionType::Existdim() const
{
	return m_dim_Exist;
}
void Mp7JrsProbabilityDistributionType::Setdim(unsigned item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityDistributionType::Setdim().");
	}
    m_dim = item;
	m_dim_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityDistributionType::Invalidatedim()
{
	m_dim_Exist = false;
}
Mp7JrsDoubleMatrixPtr Mp7JrsProbabilityDistributionType::GetMean() const
{
	return m_Mean;
}

// Element is optional
bool Mp7JrsProbabilityDistributionType::IsValidMean() const
{
	return m_Mean_Exist;
}

Mp7JrsDoubleMatrixPtr Mp7JrsProbabilityDistributionType::GetVariance() const
{
	return m_Variance;
}

// Element is optional
bool Mp7JrsProbabilityDistributionType::IsValidVariance() const
{
	return m_Variance_Exist;
}

Mp7JrsDoubleMatrixPtr Mp7JrsProbabilityDistributionType::GetMin() const
{
	return m_Min;
}

// Element is optional
bool Mp7JrsProbabilityDistributionType::IsValidMin() const
{
	return m_Min_Exist;
}

Mp7JrsDoubleMatrixPtr Mp7JrsProbabilityDistributionType::GetMax() const
{
	return m_Max;
}

// Element is optional
bool Mp7JrsProbabilityDistributionType::IsValidMax() const
{
	return m_Max_Exist;
}

Mp7JrsDoubleMatrixPtr Mp7JrsProbabilityDistributionType::GetMode() const
{
	return m_Mode;
}

// Element is optional
bool Mp7JrsProbabilityDistributionType::IsValidMode() const
{
	return m_Mode_Exist;
}

Mp7JrsDoubleMatrixPtr Mp7JrsProbabilityDistributionType::GetMedian() const
{
	return m_Median;
}

// Element is optional
bool Mp7JrsProbabilityDistributionType::IsValidMedian() const
{
	return m_Median_Exist;
}

Mp7JrsProbabilityDistributionType_Moment_CollectionPtr Mp7JrsProbabilityDistributionType::GetMoment() const
{
	return m_Moment;
}

Mp7JrsProbabilityDistributionType_Cumulant_CollectionPtr Mp7JrsProbabilityDistributionType::GetCumulant() const
{
	return m_Cumulant;
}

void Mp7JrsProbabilityDistributionType::SetMean(const Mp7JrsDoubleMatrixPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityDistributionType::SetMean().");
	}
	if (m_Mean != item || m_Mean_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Mean);
		m_Mean = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Mean) m_Mean->SetParent(m_myself.getPointer());
		if (m_Mean && m_Mean->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:DoubleMatrixType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Mean->UseTypeAttribute = true;
		}
		m_Mean_Exist = true;
		if(m_Mean != Mp7JrsDoubleMatrixPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Mean->SetContentName(XMLString::transcode("Mean"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityDistributionType::InvalidateMean()
{
	m_Mean_Exist = false;
}
void Mp7JrsProbabilityDistributionType::SetVariance(const Mp7JrsDoubleMatrixPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityDistributionType::SetVariance().");
	}
	if (m_Variance != item || m_Variance_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Variance);
		m_Variance = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Variance) m_Variance->SetParent(m_myself.getPointer());
		if (m_Variance && m_Variance->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:DoubleMatrixType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Variance->UseTypeAttribute = true;
		}
		m_Variance_Exist = true;
		if(m_Variance != Mp7JrsDoubleMatrixPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Variance->SetContentName(XMLString::transcode("Variance"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityDistributionType::InvalidateVariance()
{
	m_Variance_Exist = false;
}
void Mp7JrsProbabilityDistributionType::SetMin(const Mp7JrsDoubleMatrixPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityDistributionType::SetMin().");
	}
	if (m_Min != item || m_Min_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Min);
		m_Min = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Min) m_Min->SetParent(m_myself.getPointer());
		if (m_Min && m_Min->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:DoubleMatrixType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Min->UseTypeAttribute = true;
		}
		m_Min_Exist = true;
		if(m_Min != Mp7JrsDoubleMatrixPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Min->SetContentName(XMLString::transcode("Min"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityDistributionType::InvalidateMin()
{
	m_Min_Exist = false;
}
void Mp7JrsProbabilityDistributionType::SetMax(const Mp7JrsDoubleMatrixPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityDistributionType::SetMax().");
	}
	if (m_Max != item || m_Max_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Max);
		m_Max = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Max) m_Max->SetParent(m_myself.getPointer());
		if (m_Max && m_Max->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:DoubleMatrixType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Max->UseTypeAttribute = true;
		}
		m_Max_Exist = true;
		if(m_Max != Mp7JrsDoubleMatrixPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Max->SetContentName(XMLString::transcode("Max"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityDistributionType::InvalidateMax()
{
	m_Max_Exist = false;
}
void Mp7JrsProbabilityDistributionType::SetMode(const Mp7JrsDoubleMatrixPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityDistributionType::SetMode().");
	}
	if (m_Mode != item || m_Mode_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Mode);
		m_Mode = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Mode) m_Mode->SetParent(m_myself.getPointer());
		if (m_Mode && m_Mode->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:DoubleMatrixType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Mode->UseTypeAttribute = true;
		}
		m_Mode_Exist = true;
		if(m_Mode != Mp7JrsDoubleMatrixPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Mode->SetContentName(XMLString::transcode("Mode"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityDistributionType::InvalidateMode()
{
	m_Mode_Exist = false;
}
void Mp7JrsProbabilityDistributionType::SetMedian(const Mp7JrsDoubleMatrixPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityDistributionType::SetMedian().");
	}
	if (m_Median != item || m_Median_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Median);
		m_Median = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Median) m_Median->SetParent(m_myself.getPointer());
		if (m_Median && m_Median->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:DoubleMatrixType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Median->UseTypeAttribute = true;
		}
		m_Median_Exist = true;
		if(m_Median != Mp7JrsDoubleMatrixPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Median->SetContentName(XMLString::transcode("Median"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityDistributionType::InvalidateMedian()
{
	m_Median_Exist = false;
}
void Mp7JrsProbabilityDistributionType::SetMoment(const Mp7JrsProbabilityDistributionType_Moment_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityDistributionType::SetMoment().");
	}
	if (m_Moment != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Moment);
		m_Moment = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Moment) m_Moment->SetParent(m_myself.getPointer());
		if(m_Moment != Mp7JrsProbabilityDistributionType_Moment_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Moment->SetContentName(XMLString::transcode("Moment"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityDistributionType::SetCumulant(const Mp7JrsProbabilityDistributionType_Cumulant_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityDistributionType::SetCumulant().");
	}
	if (m_Cumulant != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Cumulant);
		m_Cumulant = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Cumulant) m_Cumulant->SetParent(m_myself.getPointer());
		if(m_Cumulant != Mp7JrsProbabilityDistributionType_Cumulant_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Cumulant->SetContentName(XMLString::transcode("Cumulant"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrszeroToOnePtr Mp7JrsProbabilityDistributionType::Getconfidence() const
{
	return GetBase()->GetBase()->Getconfidence();
}

bool Mp7JrsProbabilityDistributionType::Existconfidence() const
{
	return GetBase()->GetBase()->Existconfidence();
}
void Mp7JrsProbabilityDistributionType::Setconfidence(const Mp7JrszeroToOnePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityDistributionType::Setconfidence().");
	}
	GetBase()->GetBase()->Setconfidence(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityDistributionType::Invalidateconfidence()
{
	GetBase()->GetBase()->Invalidateconfidence();
}
Mp7JrszeroToOnePtr Mp7JrsProbabilityDistributionType::Getreliability() const
{
	return GetBase()->GetBase()->Getreliability();
}

bool Mp7JrsProbabilityDistributionType::Existreliability() const
{
	return GetBase()->GetBase()->Existreliability();
}
void Mp7JrsProbabilityDistributionType::Setreliability(const Mp7JrszeroToOnePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityDistributionType::Setreliability().");
	}
	GetBase()->GetBase()->Setreliability(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityDistributionType::Invalidatereliability()
{
	GetBase()->GetBase()->Invalidatereliability();
}
XMLCh * Mp7JrsProbabilityDistributionType::Getid() const
{
	return GetBase()->GetBase()->GetBase()->Getid();
}

bool Mp7JrsProbabilityDistributionType::Existid() const
{
	return GetBase()->GetBase()->GetBase()->Existid();
}
void Mp7JrsProbabilityDistributionType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityDistributionType::Setid().");
	}
	GetBase()->GetBase()->GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityDistributionType::Invalidateid()
{
	GetBase()->GetBase()->GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsProbabilityDistributionType::GettimeBase() const
{
	return GetBase()->GetBase()->GetBase()->GettimeBase();
}

bool Mp7JrsProbabilityDistributionType::ExisttimeBase() const
{
	return GetBase()->GetBase()->GetBase()->ExisttimeBase();
}
void Mp7JrsProbabilityDistributionType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityDistributionType::SettimeBase().");
	}
	GetBase()->GetBase()->GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityDistributionType::InvalidatetimeBase()
{
	GetBase()->GetBase()->GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsProbabilityDistributionType::GettimeUnit() const
{
	return GetBase()->GetBase()->GetBase()->GettimeUnit();
}

bool Mp7JrsProbabilityDistributionType::ExisttimeUnit() const
{
	return GetBase()->GetBase()->GetBase()->ExisttimeUnit();
}
void Mp7JrsProbabilityDistributionType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityDistributionType::SettimeUnit().");
	}
	GetBase()->GetBase()->GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityDistributionType::InvalidatetimeUnit()
{
	GetBase()->GetBase()->GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsProbabilityDistributionType::GetmediaTimeBase() const
{
	return GetBase()->GetBase()->GetBase()->GetmediaTimeBase();
}

bool Mp7JrsProbabilityDistributionType::ExistmediaTimeBase() const
{
	return GetBase()->GetBase()->GetBase()->ExistmediaTimeBase();
}
void Mp7JrsProbabilityDistributionType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityDistributionType::SetmediaTimeBase().");
	}
	GetBase()->GetBase()->GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityDistributionType::InvalidatemediaTimeBase()
{
	GetBase()->GetBase()->GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsProbabilityDistributionType::GetmediaTimeUnit() const
{
	return GetBase()->GetBase()->GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsProbabilityDistributionType::ExistmediaTimeUnit() const
{
	return GetBase()->GetBase()->GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsProbabilityDistributionType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityDistributionType::SetmediaTimeUnit().");
	}
	GetBase()->GetBase()->GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityDistributionType::InvalidatemediaTimeUnit()
{
	GetBase()->GetBase()->GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsProbabilityDistributionType::GetHeader() const
{
	return GetBase()->GetBase()->GetBase()->GetHeader();
}

void Mp7JrsProbabilityDistributionType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityDistributionType::SetHeader().");
	}
	GetBase()->GetBase()->GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsProbabilityDistributionType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Mean")) == 0)
	{
		// Mean is simple element DoubleMatrixType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMean()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:DoubleMatrixType")))) != empty)
			{
				// Is type allowed
				Mp7JrsDoubleMatrixPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMean(p, client);
					if((p = GetMean()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Variance")) == 0)
	{
		// Variance is simple element DoubleMatrixType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetVariance()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:DoubleMatrixType")))) != empty)
			{
				// Is type allowed
				Mp7JrsDoubleMatrixPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetVariance(p, client);
					if((p = GetVariance()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Min")) == 0)
	{
		// Min is simple element DoubleMatrixType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMin()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:DoubleMatrixType")))) != empty)
			{
				// Is type allowed
				Mp7JrsDoubleMatrixPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMin(p, client);
					if((p = GetMin()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Max")) == 0)
	{
		// Max is simple element DoubleMatrixType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMax()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:DoubleMatrixType")))) != empty)
			{
				// Is type allowed
				Mp7JrsDoubleMatrixPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMax(p, client);
					if((p = GetMax()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Mode")) == 0)
	{
		// Mode is simple element DoubleMatrixType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMode()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:DoubleMatrixType")))) != empty)
			{
				// Is type allowed
				Mp7JrsDoubleMatrixPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMode(p, client);
					if((p = GetMode()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Median")) == 0)
	{
		// Median is simple element DoubleMatrixType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMedian()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:DoubleMatrixType")))) != empty)
			{
				// Is type allowed
				Mp7JrsDoubleMatrixPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMedian(p, client);
					if((p = GetMedian()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Moment")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Moment is item of type ProbabilityDistributionType_Moment_LocalType
		// in element collection ProbabilityDistributionType_Moment_CollectionType
		
		context->Found = true;
		Mp7JrsProbabilityDistributionType_Moment_CollectionPtr coll = GetMoment();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateProbabilityDistributionType_Moment_CollectionType;
				SetMoment(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ProbabilityDistributionType_Moment_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsProbabilityDistributionType_Moment_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Cumulant")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Cumulant is item of type ProbabilityDistributionType_Cumulant_LocalType
		// in element collection ProbabilityDistributionType_Cumulant_CollectionType
		
		context->Found = true;
		Mp7JrsProbabilityDistributionType_Cumulant_CollectionPtr coll = GetCumulant();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateProbabilityDistributionType_Cumulant_CollectionType;
				SetCumulant(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ProbabilityDistributionType_Cumulant_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsProbabilityDistributionType_Cumulant_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for ProbabilityDistributionType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "ProbabilityDistributionType");
	}		
	
	return result;
}

XMLCh * Mp7JrsProbabilityDistributionType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsProbabilityDistributionType::Parse(const XMLCh * const txt)
{
	return false;
}

bool Mp7JrsProbabilityDistributionType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsProbabilityDistributionType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsProbabilityDistributionType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsProbabilityDistributionType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("ProbabilityDistributionType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_dim_Exist)
	{
	// Value
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_dim);
		element->setAttributeNS(X(""), X("dim"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:
	// Class
	
	if (m_Mean != Mp7JrsDoubleMatrixPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Mean->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Mean"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Mean->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Variance != Mp7JrsDoubleMatrixPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Variance->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Variance"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Variance->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Min != Mp7JrsDoubleMatrixPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Min->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Min"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Min->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Max != Mp7JrsDoubleMatrixPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Max->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Max"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Max->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Mode != Mp7JrsDoubleMatrixPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Mode->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Mode"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Mode->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Median != Mp7JrsDoubleMatrixPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Median->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Median"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Median->Serialize(doc, element, &elem);
	}
	if (m_Moment != Mp7JrsProbabilityDistributionType_Moment_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Moment->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_Cumulant != Mp7JrsProbabilityDistributionType_Cumulant_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Cumulant->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsProbabilityDistributionType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("dim")))
	{
        // deserialize value type
        this->Setdim(Mp7JrsConvert::TextToUnsignedInt(parent->getAttributeNS(X(""), X("dim"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("dim")))
	{
        // deserialize value type
        this->Setdim(Mp7JrsConvert::TextToUnsignedInt(parent->getAttribute(X("dim"))));
        * current = parent;
    }

	// Extensionbase is Mp7JrsProbabilityModelType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Mean"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Mean")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateDoubleMatrixType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetMean(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Variance"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Variance")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateDoubleMatrixType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetVariance(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Min"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Min")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateDoubleMatrixType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetMin(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Max"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Max")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateDoubleMatrixType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetMax(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Mode"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Mode")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateDoubleMatrixType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetMode(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Median"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Median")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateDoubleMatrixType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetMedian(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Moment"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Moment")) == 0))
        {
            // Deserialize factory type
            Mp7JrsProbabilityDistributionType_Moment_CollectionPtr tmp = CreateProbabilityDistributionType_Moment_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetMoment(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Cumulant"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Cumulant")) == 0))
        {
            // Deserialize factory type
            Mp7JrsProbabilityDistributionType_Cumulant_CollectionPtr tmp = CreateProbabilityDistributionType_Cumulant_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetCumulant(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsProbabilityDistributionType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsProbabilityDistributionType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("Mean")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateDoubleMatrixType;
    }
    this->SetMean(child);
  }
  if (XMLString::compareString(elementname, X("Variance")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateDoubleMatrixType;
    }
    this->SetVariance(child);
  }
  if (XMLString::compareString(elementname, X("Min")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateDoubleMatrixType;
    }
    this->SetMin(child);
  }
  if (XMLString::compareString(elementname, X("Max")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateDoubleMatrixType;
    }
    this->SetMax(child);
  }
  if (XMLString::compareString(elementname, X("Mode")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateDoubleMatrixType;
    }
    this->SetMode(child);
  }
  if (XMLString::compareString(elementname, X("Median")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateDoubleMatrixType;
    }
    this->SetMedian(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Moment")) == 0))
  {
    Mp7JrsProbabilityDistributionType_Moment_CollectionPtr tmp = CreateProbabilityDistributionType_Moment_CollectionType;
    this->SetMoment(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Cumulant")) == 0))
  {
    Mp7JrsProbabilityDistributionType_Cumulant_CollectionPtr tmp = CreateProbabilityDistributionType_Cumulant_CollectionType;
    this->SetCumulant(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsProbabilityDistributionType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetMean() != Mp7JrsNodePtr())
		result.Insert(GetMean());
	if (GetVariance() != Mp7JrsNodePtr())
		result.Insert(GetVariance());
	if (GetMin() != Mp7JrsNodePtr())
		result.Insert(GetMin());
	if (GetMax() != Mp7JrsNodePtr())
		result.Insert(GetMax());
	if (GetMode() != Mp7JrsNodePtr())
		result.Insert(GetMode());
	if (GetMedian() != Mp7JrsNodePtr())
		result.Insert(GetMedian());
	if (GetMoment() != Mp7JrsNodePtr())
		result.Insert(GetMoment());
	if (GetCumulant() != Mp7JrsNodePtr())
		result.Insert(GetCumulant());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsProbabilityDistributionType_ExtMethodImpl.h


