 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsProbabilityModelClassType_ExtImplInclude.h



#include "Mp7JrsAnalyticModelType.h"
#include "Mp7JrsProbabilityModelClassType_CollectionType.h"
#include "Mp7JrsAnalyticModelType_CollectionType.h"
#include "Mp7JrszeroToOneType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsProbabilityModelClassType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsAnalyticModelType_LocalType.h" // Choice collection Label
#include "Mp7JrsAnalyticModelType_Label_LocalType.h" // Choice collection element Label
#include "Mp7JrsAnalyticModelType_Semantics_LocalType.h" // Choice collection element Semantics
#include "Mp7JrsProbabilityModelClassType_LocalType.h" // Sequence collection DescriptorModel
#include "Mp7JrsDescriptorModelType.h" // Sequence collection element DescriptorModel
#include "Mp7JrsProbabilityModelType.h" // Sequence collection element ProbabilityModel

#include <assert.h>
IMp7JrsProbabilityModelClassType::IMp7JrsProbabilityModelClassType()
{

// no includefile for extension defined 
// file Mp7JrsProbabilityModelClassType_ExtPropInit.h

}

IMp7JrsProbabilityModelClassType::~IMp7JrsProbabilityModelClassType()
{
// no includefile for extension defined 
// file Mp7JrsProbabilityModelClassType_ExtPropCleanup.h

}

Mp7JrsProbabilityModelClassType::Mp7JrsProbabilityModelClassType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsProbabilityModelClassType::~Mp7JrsProbabilityModelClassType()
{
	Cleanup();
}

void Mp7JrsProbabilityModelClassType::Init()
{
	// Init base
	m_Base = CreateAnalyticModelType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_ProbabilityModelClassType_LocalType = Mp7JrsProbabilityModelClassType_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsProbabilityModelClassType_ExtMyPropInit.h

}

void Mp7JrsProbabilityModelClassType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsProbabilityModelClassType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_ProbabilityModelClassType_LocalType);
}

void Mp7JrsProbabilityModelClassType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use ProbabilityModelClassTypePtr, since we
	// might need GetBase(), which isn't defined in IProbabilityModelClassType
    const Mp7JrsPtr< Mp7JrsProbabilityModelClassType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsAnalyticModelPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsProbabilityModelClassType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_ProbabilityModelClassType_LocalType);
		this->SetProbabilityModelClassType_LocalType(Mp7JrsFactory::CloneObject(tmp->GetProbabilityModelClassType_LocalType()));
}

Mp7JrsNodePtr Mp7JrsProbabilityModelClassType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsProbabilityModelClassType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsAnalyticModelType > Mp7JrsProbabilityModelClassType::GetBase() const
{
	return m_Base;
}

Mp7JrsProbabilityModelClassType_CollectionPtr Mp7JrsProbabilityModelClassType::GetProbabilityModelClassType_LocalType() const
{
	return m_ProbabilityModelClassType_LocalType;
}

void Mp7JrsProbabilityModelClassType::SetProbabilityModelClassType_LocalType(const Mp7JrsProbabilityModelClassType_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityModelClassType::SetProbabilityModelClassType_LocalType().");
	}
	if (m_ProbabilityModelClassType_LocalType != item)
	{
		// Mp7JrsFactory::DeleteObject(m_ProbabilityModelClassType_LocalType);
		m_ProbabilityModelClassType_LocalType = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ProbabilityModelClassType_LocalType) m_ProbabilityModelClassType_LocalType->SetParent(m_myself.getPointer());
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsAnalyticModelType_function_LocalType::Enumeration Mp7JrsProbabilityModelClassType::Getfunction() const
{
	return GetBase()->Getfunction();
}

bool Mp7JrsProbabilityModelClassType::Existfunction() const
{
	return GetBase()->Existfunction();
}
void Mp7JrsProbabilityModelClassType::Setfunction(Mp7JrsAnalyticModelType_function_LocalType::Enumeration item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityModelClassType::Setfunction().");
	}
	GetBase()->Setfunction(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityModelClassType::Invalidatefunction()
{
	GetBase()->Invalidatefunction();
}
Mp7JrsAnalyticModelType_CollectionPtr Mp7JrsProbabilityModelClassType::GetAnalyticModelType_LocalType() const
{
	return GetBase()->GetAnalyticModelType_LocalType();
}

void Mp7JrsProbabilityModelClassType::SetAnalyticModelType_LocalType(const Mp7JrsAnalyticModelType_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityModelClassType::SetAnalyticModelType_LocalType().");
	}
	GetBase()->SetAnalyticModelType_LocalType(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrszeroToOnePtr Mp7JrsProbabilityModelClassType::Getconfidence() const
{
	return GetBase()->GetBase()->Getconfidence();
}

bool Mp7JrsProbabilityModelClassType::Existconfidence() const
{
	return GetBase()->GetBase()->Existconfidence();
}
void Mp7JrsProbabilityModelClassType::Setconfidence(const Mp7JrszeroToOnePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityModelClassType::Setconfidence().");
	}
	GetBase()->GetBase()->Setconfidence(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityModelClassType::Invalidateconfidence()
{
	GetBase()->GetBase()->Invalidateconfidence();
}
Mp7JrszeroToOnePtr Mp7JrsProbabilityModelClassType::Getreliability() const
{
	return GetBase()->GetBase()->Getreliability();
}

bool Mp7JrsProbabilityModelClassType::Existreliability() const
{
	return GetBase()->GetBase()->Existreliability();
}
void Mp7JrsProbabilityModelClassType::Setreliability(const Mp7JrszeroToOnePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityModelClassType::Setreliability().");
	}
	GetBase()->GetBase()->Setreliability(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityModelClassType::Invalidatereliability()
{
	GetBase()->GetBase()->Invalidatereliability();
}
XMLCh * Mp7JrsProbabilityModelClassType::Getid() const
{
	return GetBase()->GetBase()->GetBase()->Getid();
}

bool Mp7JrsProbabilityModelClassType::Existid() const
{
	return GetBase()->GetBase()->GetBase()->Existid();
}
void Mp7JrsProbabilityModelClassType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityModelClassType::Setid().");
	}
	GetBase()->GetBase()->GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityModelClassType::Invalidateid()
{
	GetBase()->GetBase()->GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsProbabilityModelClassType::GettimeBase() const
{
	return GetBase()->GetBase()->GetBase()->GettimeBase();
}

bool Mp7JrsProbabilityModelClassType::ExisttimeBase() const
{
	return GetBase()->GetBase()->GetBase()->ExisttimeBase();
}
void Mp7JrsProbabilityModelClassType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityModelClassType::SettimeBase().");
	}
	GetBase()->GetBase()->GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityModelClassType::InvalidatetimeBase()
{
	GetBase()->GetBase()->GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsProbabilityModelClassType::GettimeUnit() const
{
	return GetBase()->GetBase()->GetBase()->GettimeUnit();
}

bool Mp7JrsProbabilityModelClassType::ExisttimeUnit() const
{
	return GetBase()->GetBase()->GetBase()->ExisttimeUnit();
}
void Mp7JrsProbabilityModelClassType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityModelClassType::SettimeUnit().");
	}
	GetBase()->GetBase()->GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityModelClassType::InvalidatetimeUnit()
{
	GetBase()->GetBase()->GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsProbabilityModelClassType::GetmediaTimeBase() const
{
	return GetBase()->GetBase()->GetBase()->GetmediaTimeBase();
}

bool Mp7JrsProbabilityModelClassType::ExistmediaTimeBase() const
{
	return GetBase()->GetBase()->GetBase()->ExistmediaTimeBase();
}
void Mp7JrsProbabilityModelClassType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityModelClassType::SetmediaTimeBase().");
	}
	GetBase()->GetBase()->GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityModelClassType::InvalidatemediaTimeBase()
{
	GetBase()->GetBase()->GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsProbabilityModelClassType::GetmediaTimeUnit() const
{
	return GetBase()->GetBase()->GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsProbabilityModelClassType::ExistmediaTimeUnit() const
{
	return GetBase()->GetBase()->GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsProbabilityModelClassType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityModelClassType::SetmediaTimeUnit().");
	}
	GetBase()->GetBase()->GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityModelClassType::InvalidatemediaTimeUnit()
{
	GetBase()->GetBase()->GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsProbabilityModelClassType::GetHeader() const
{
	return GetBase()->GetBase()->GetBase()->GetHeader();
}

void Mp7JrsProbabilityModelClassType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityModelClassType::SetHeader().");
	}
	GetBase()->GetBase()->GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsProbabilityModelClassType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("DescriptorModel")) == 0)
	{
		// DescriptorModel is contained in itemtype ProbabilityModelClassType_LocalType
		// in sequence collection ProbabilityModelClassType_CollectionType

		context->Found = true;
		Mp7JrsProbabilityModelClassType_CollectionPtr coll = GetProbabilityModelClassType_LocalType();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateProbabilityModelClassType_CollectionType;
				SetProbabilityModelClassType_LocalType(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = ((Mp7JrsProbabilityModelClassType_LocalPtr)coll->elementAt(i))->GetDescriptorModel()) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				// context->FreeIndexList.addElement(i); // FTT We don't allow to shift elems between different sequence items.
				++i;
			}
		}
		int start, stop;
		if(Mp7JrsXPathParseContext::PostProcessSequenceCollection(context, i, start, stop, result))
		{
			if(i >= 0)
			{
				Mp7JrsProbabilityModelClassType_LocalPtr item = CreateProbabilityModelClassType_LocalType;
				coll->insertElementAt(item, i, client);
			}
			else // Just reorganise the existing nodes and then we can insert without creating another sequence item.
			{
				int j;
				if(start < stop)
				{
					for(j = start; j < stop; ++j)
					{
						((Mp7JrsProbabilityModelClassType_LocalPtr)coll->elementAt(j))->SetDescriptorModel(
						((Mp7JrsProbabilityModelClassType_LocalPtr)coll->elementAt(j+1))->GetDescriptorModel());
					}
				}
				else
				{
					for(j = start; j > stop; --j)
					{
						((Mp7JrsProbabilityModelClassType_LocalPtr)coll->elementAt(j))->SetDescriptorModel(
						((Mp7JrsProbabilityModelClassType_LocalPtr)coll->elementAt(j-1))->GetDescriptorModel());
					}
				}
				i = stop;
			}
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:DescriptorModelType")))) != empty)
			{
				// Is type allowed
				Mp7JrsDescriptorModelPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					((Mp7JrsProbabilityModelClassType_LocalPtr)coll->elementAt(i))->SetDescriptorModel(p, client);
					 // Maybe some other restrictions are waiting...
					if((p = ((Mp7JrsProbabilityModelClassType_LocalPtr)coll->elementAt(i))->GetDescriptorModel()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;

	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("ProbabilityModel")) == 0)
	{
		// ProbabilityModel is contained in itemtype ProbabilityModelClassType_LocalType
		// in sequence collection ProbabilityModelClassType_CollectionType

		context->Found = true;
		Mp7JrsProbabilityModelClassType_CollectionPtr coll = GetProbabilityModelClassType_LocalType();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateProbabilityModelClassType_CollectionType;
				SetProbabilityModelClassType_LocalType(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = ((Mp7JrsProbabilityModelClassType_LocalPtr)coll->elementAt(i))->GetProbabilityModel()) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				// context->FreeIndexList.addElement(i); // FTT We don't allow to shift elems between different sequence items.
				++i;
			}
		}
		int start, stop;
		if(Mp7JrsXPathParseContext::PostProcessSequenceCollection(context, i, start, stop, result))
		{
			if(i >= 0)
			{
				Mp7JrsProbabilityModelClassType_LocalPtr item = CreateProbabilityModelClassType_LocalType;
				coll->insertElementAt(item, i, client);
			}
			else // Just reorganise the existing nodes and then we can insert without creating another sequence item.
			{
				int j;
				if(start < stop)
				{
					for(j = start; j < stop; ++j)
					{
						((Mp7JrsProbabilityModelClassType_LocalPtr)coll->elementAt(j))->SetProbabilityModel(
						((Mp7JrsProbabilityModelClassType_LocalPtr)coll->elementAt(j+1))->GetProbabilityModel());
					}
				}
				else
				{
					for(j = start; j > stop; --j)
					{
						((Mp7JrsProbabilityModelClassType_LocalPtr)coll->elementAt(j))->SetProbabilityModel(
						((Mp7JrsProbabilityModelClassType_LocalPtr)coll->elementAt(j-1))->GetProbabilityModel());
					}
				}
				i = stop;
			}
			if((p = Mp7JrsXPathParseContext::CreateNode(context, 0)) != empty)
			{
				// Is type allowed
				Mp7JrsProbabilityModelPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					((Mp7JrsProbabilityModelClassType_LocalPtr)coll->elementAt(i))->SetProbabilityModel(p, client);
					 // Maybe some other restrictions are waiting...
					if((p = ((Mp7JrsProbabilityModelClassType_LocalPtr)coll->elementAt(i))->GetProbabilityModel()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;

	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for ProbabilityModelClassType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "ProbabilityModelClassType");
	}		
	
	return result;
}

XMLCh * Mp7JrsProbabilityModelClassType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsProbabilityModelClassType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}

bool Mp7JrsProbabilityModelClassType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsProbabilityModelClassType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsProbabilityModelClassType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsProbabilityModelClassType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("ProbabilityModelClassType"));
	// Element serialization:
	if (m_ProbabilityModelClassType_LocalType != Mp7JrsProbabilityModelClassType_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_ProbabilityModelClassType_LocalType->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsProbabilityModelClassType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsAnalyticModelType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Sequence Collection
        if((parent != NULL) && (
// Old            (XMLString::compareString(parent->getNodeName(), X("urn:mpeg:mpeg7:schema:2004:DescriptorModel")) == 0)
            (Mp7JrsUtil::HasNodeName(parent, X("DescriptorModel"), X("urn:mpeg:mpeg7:schema:2004")))
                ||
// Old            (XMLString::compareString(parent->getNodeName(), X("urn:mpeg:mpeg7:schema:2004:ProbabilityModel")) == 0)
            (Mp7JrsUtil::HasNodeName(parent, X("ProbabilityModel"), X("urn:mpeg:mpeg7:schema:2004")))
                ))
        {
            // Deserialize factory type
            Mp7JrsProbabilityModelClassType_CollectionPtr tmp = CreateProbabilityModelClassType_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetProbabilityModelClassType_LocalType(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsProbabilityModelClassType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsProbabilityModelClassType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  // Non-Choice Collection
    // local type, so no need to check explicitly for a name (BAW 05 04 2004)
  {
    Mp7JrsProbabilityModelClassType_CollectionPtr tmp = CreateProbabilityModelClassType_CollectionType;
    this->SetProbabilityModelClassType_LocalType(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsProbabilityModelClassType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetProbabilityModelClassType_LocalType() != Mp7JrsNodePtr())
		result.Insert(GetProbabilityModelClassType_LocalType());
	if (GetAnalyticModelType_LocalType() != Mp7JrsNodePtr())
		result.Insert(GetAnalyticModelType_LocalType());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsProbabilityModelClassType_ExtMethodImpl.h


