 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsSemanticStateType_ExtImplInclude.h



#include "Mp7JrsSemanticBaseType.h"
#include "Mp7JrsSemanticStateType_AttributeValuePair_CollectionType0.h"
#include "Mp7JrsSemanticTimeType.h"
#include "Mp7JrsSemanticPlaceType.h"
#include "Mp7JrsAbstractionLevelType.h"
#include "Mp7JrsSemanticBaseType_Label_CollectionType.h"
#include "Mp7JrsTextAnnotationType.h"
#include "Mp7JrsSemanticBaseType_Property_CollectionType.h"
#include "Mp7JrsSemanticBaseType_MediaOccurrence_CollectionType.h"
#include "Mp7JrsSemanticBaseType_Relation_CollectionType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsSemanticStateType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsTermUseType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Label
#include "Mp7JrsSemanticBaseType_MediaOccurrence_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:MediaOccurrence
#include "Mp7JrsRelationType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Relation
#include "Mp7JrsSemanticStateType_AttributeValuePair_LocalType0.h" // Element collection urn:mpeg:mpeg7:schema:2004:AttributeValuePair

#include <assert.h>
IMp7JrsSemanticStateType::IMp7JrsSemanticStateType()
{

// no includefile for extension defined 
// file Mp7JrsSemanticStateType_ExtPropInit.h

}

IMp7JrsSemanticStateType::~IMp7JrsSemanticStateType()
{
// no includefile for extension defined 
// file Mp7JrsSemanticStateType_ExtPropCleanup.h

}

Mp7JrsSemanticStateType::Mp7JrsSemanticStateType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsSemanticStateType::~Mp7JrsSemanticStateType()
{
	Cleanup();
}

void Mp7JrsSemanticStateType::Init()
{
	// Init base
	m_Base = CreateSemanticBaseType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_AttributeValuePair = Mp7JrsSemanticStateType_AttributeValuePair_Collection0Ptr(); // Collection
	m_SemanticTime = Mp7JrsSemanticTimePtr(); // Class
	m_SemanticTime_Exist = false;
	m_SemanticPlace = Mp7JrsSemanticPlacePtr(); // Class
	m_SemanticPlace_Exist = false;


// no includefile for extension defined 
// file Mp7JrsSemanticStateType_ExtMyPropInit.h

}

void Mp7JrsSemanticStateType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsSemanticStateType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_AttributeValuePair);
	// Mp7JrsFactory::DeleteObject(m_SemanticTime);
	// Mp7JrsFactory::DeleteObject(m_SemanticPlace);
}

void Mp7JrsSemanticStateType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use SemanticStateTypePtr, since we
	// might need GetBase(), which isn't defined in ISemanticStateType
    const Mp7JrsPtr< Mp7JrsSemanticStateType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsSemanticBasePtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsSemanticStateType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_AttributeValuePair);
		this->SetAttributeValuePair(Mp7JrsFactory::CloneObject(tmp->GetAttributeValuePair()));
	if (tmp->IsValidSemanticTime())
	{
		// Mp7JrsFactory::DeleteObject(m_SemanticTime);
		this->SetSemanticTime(Mp7JrsFactory::CloneObject(tmp->GetSemanticTime()));
	}
	else
	{
		InvalidateSemanticTime();
	}
	if (tmp->IsValidSemanticPlace())
	{
		// Mp7JrsFactory::DeleteObject(m_SemanticPlace);
		this->SetSemanticPlace(Mp7JrsFactory::CloneObject(tmp->GetSemanticPlace()));
	}
	else
	{
		InvalidateSemanticPlace();
	}
}

Mp7JrsNodePtr Mp7JrsSemanticStateType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsSemanticStateType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsSemanticBaseType > Mp7JrsSemanticStateType::GetBase() const
{
	return m_Base;
}

Mp7JrsSemanticStateType_AttributeValuePair_Collection0Ptr Mp7JrsSemanticStateType::GetAttributeValuePair() const
{
	return m_AttributeValuePair;
}

Mp7JrsSemanticTimePtr Mp7JrsSemanticStateType::GetSemanticTime() const
{
	return m_SemanticTime;
}

// Element is optional
bool Mp7JrsSemanticStateType::IsValidSemanticTime() const
{
	return m_SemanticTime_Exist;
}

Mp7JrsSemanticPlacePtr Mp7JrsSemanticStateType::GetSemanticPlace() const
{
	return m_SemanticPlace;
}

// Element is optional
bool Mp7JrsSemanticStateType::IsValidSemanticPlace() const
{
	return m_SemanticPlace_Exist;
}

void Mp7JrsSemanticStateType::SetAttributeValuePair(const Mp7JrsSemanticStateType_AttributeValuePair_Collection0Ptr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSemanticStateType::SetAttributeValuePair().");
	}
	if (m_AttributeValuePair != item)
	{
		// Mp7JrsFactory::DeleteObject(m_AttributeValuePair);
		m_AttributeValuePair = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_AttributeValuePair) m_AttributeValuePair->SetParent(m_myself.getPointer());
		if(m_AttributeValuePair != Mp7JrsSemanticStateType_AttributeValuePair_Collection0Ptr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_AttributeValuePair->SetContentName(XMLString::transcode("AttributeValuePair"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSemanticStateType::SetSemanticTime(const Mp7JrsSemanticTimePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSemanticStateType::SetSemanticTime().");
	}
	if (m_SemanticTime != item || m_SemanticTime_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_SemanticTime);
		m_SemanticTime = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_SemanticTime) m_SemanticTime->SetParent(m_myself.getPointer());
		if (m_SemanticTime && m_SemanticTime->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SemanticTimeType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_SemanticTime->UseTypeAttribute = true;
		}
		m_SemanticTime_Exist = true;
		if(m_SemanticTime != Mp7JrsSemanticTimePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_SemanticTime->SetContentName(XMLString::transcode("SemanticTime"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSemanticStateType::InvalidateSemanticTime()
{
	m_SemanticTime_Exist = false;
}
void Mp7JrsSemanticStateType::SetSemanticPlace(const Mp7JrsSemanticPlacePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSemanticStateType::SetSemanticPlace().");
	}
	if (m_SemanticPlace != item || m_SemanticPlace_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_SemanticPlace);
		m_SemanticPlace = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_SemanticPlace) m_SemanticPlace->SetParent(m_myself.getPointer());
		if (m_SemanticPlace && m_SemanticPlace->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SemanticPlaceType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_SemanticPlace->UseTypeAttribute = true;
		}
		m_SemanticPlace_Exist = true;
		if(m_SemanticPlace != Mp7JrsSemanticPlacePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_SemanticPlace->SetContentName(XMLString::transcode("SemanticPlace"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSemanticStateType::InvalidateSemanticPlace()
{
	m_SemanticPlace_Exist = false;
}
Mp7JrsAbstractionLevelPtr Mp7JrsSemanticStateType::GetAbstractionLevel() const
{
	return GetBase()->GetAbstractionLevel();
}

// Element is optional
bool Mp7JrsSemanticStateType::IsValidAbstractionLevel() const
{
	return GetBase()->IsValidAbstractionLevel();
}

Mp7JrsSemanticBaseType_Label_CollectionPtr Mp7JrsSemanticStateType::GetLabel() const
{
	return GetBase()->GetLabel();
}

Mp7JrsTextAnnotationPtr Mp7JrsSemanticStateType::GetDefinition() const
{
	return GetBase()->GetDefinition();
}

// Element is optional
bool Mp7JrsSemanticStateType::IsValidDefinition() const
{
	return GetBase()->IsValidDefinition();
}

Mp7JrsSemanticBaseType_Property_CollectionPtr Mp7JrsSemanticStateType::GetProperty() const
{
	return GetBase()->GetProperty();
}

Mp7JrsSemanticBaseType_MediaOccurrence_CollectionPtr Mp7JrsSemanticStateType::GetMediaOccurrence() const
{
	return GetBase()->GetMediaOccurrence();
}

Mp7JrsSemanticBaseType_Relation_CollectionPtr Mp7JrsSemanticStateType::GetRelation() const
{
	return GetBase()->GetRelation();
}

void Mp7JrsSemanticStateType::SetAbstractionLevel(const Mp7JrsAbstractionLevelPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSemanticStateType::SetAbstractionLevel().");
	}
	GetBase()->SetAbstractionLevel(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSemanticStateType::InvalidateAbstractionLevel()
{
	GetBase()->InvalidateAbstractionLevel();
}
void Mp7JrsSemanticStateType::SetLabel(const Mp7JrsSemanticBaseType_Label_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSemanticStateType::SetLabel().");
	}
	GetBase()->SetLabel(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSemanticStateType::SetDefinition(const Mp7JrsTextAnnotationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSemanticStateType::SetDefinition().");
	}
	GetBase()->SetDefinition(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSemanticStateType::InvalidateDefinition()
{
	GetBase()->InvalidateDefinition();
}
void Mp7JrsSemanticStateType::SetProperty(const Mp7JrsSemanticBaseType_Property_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSemanticStateType::SetProperty().");
	}
	GetBase()->SetProperty(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSemanticStateType::SetMediaOccurrence(const Mp7JrsSemanticBaseType_MediaOccurrence_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSemanticStateType::SetMediaOccurrence().");
	}
	GetBase()->SetMediaOccurrence(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSemanticStateType::SetRelation(const Mp7JrsSemanticBaseType_Relation_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSemanticStateType::SetRelation().");
	}
	GetBase()->SetRelation(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsSemanticStateType::Getid() const
{
	return GetBase()->GetBase()->Getid();
}

bool Mp7JrsSemanticStateType::Existid() const
{
	return GetBase()->GetBase()->Existid();
}
void Mp7JrsSemanticStateType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSemanticStateType::Setid().");
	}
	GetBase()->GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSemanticStateType::Invalidateid()
{
	GetBase()->GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsSemanticStateType::GettimeBase() const
{
	return GetBase()->GetBase()->GettimeBase();
}

bool Mp7JrsSemanticStateType::ExisttimeBase() const
{
	return GetBase()->GetBase()->ExisttimeBase();
}
void Mp7JrsSemanticStateType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSemanticStateType::SettimeBase().");
	}
	GetBase()->GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSemanticStateType::InvalidatetimeBase()
{
	GetBase()->GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsSemanticStateType::GettimeUnit() const
{
	return GetBase()->GetBase()->GettimeUnit();
}

bool Mp7JrsSemanticStateType::ExisttimeUnit() const
{
	return GetBase()->GetBase()->ExisttimeUnit();
}
void Mp7JrsSemanticStateType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSemanticStateType::SettimeUnit().");
	}
	GetBase()->GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSemanticStateType::InvalidatetimeUnit()
{
	GetBase()->GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsSemanticStateType::GetmediaTimeBase() const
{
	return GetBase()->GetBase()->GetmediaTimeBase();
}

bool Mp7JrsSemanticStateType::ExistmediaTimeBase() const
{
	return GetBase()->GetBase()->ExistmediaTimeBase();
}
void Mp7JrsSemanticStateType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSemanticStateType::SetmediaTimeBase().");
	}
	GetBase()->GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSemanticStateType::InvalidatemediaTimeBase()
{
	GetBase()->GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsSemanticStateType::GetmediaTimeUnit() const
{
	return GetBase()->GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsSemanticStateType::ExistmediaTimeUnit() const
{
	return GetBase()->GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsSemanticStateType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSemanticStateType::SetmediaTimeUnit().");
	}
	GetBase()->GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSemanticStateType::InvalidatemediaTimeUnit()
{
	GetBase()->GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsSemanticStateType::GetHeader() const
{
	return GetBase()->GetBase()->GetHeader();
}

void Mp7JrsSemanticStateType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSemanticStateType::SetHeader().");
	}
	GetBase()->GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsSemanticStateType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("AttributeValuePair")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:AttributeValuePair is item of type SemanticStateType_AttributeValuePair_LocalType0
		// in element collection SemanticStateType_AttributeValuePair_CollectionType0
		
		context->Found = true;
		Mp7JrsSemanticStateType_AttributeValuePair_Collection0Ptr coll = GetAttributeValuePair();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateSemanticStateType_AttributeValuePair_CollectionType0;
				SetAttributeValuePair(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SemanticStateType_AttributeValuePair_LocalType0")))) != empty)
			{
				// Is type allowed
				Mp7JrsSemanticStateType_AttributeValuePair_Local0Ptr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("SemanticTime")) == 0)
	{
		// SemanticTime is simple element SemanticTimeType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetSemanticTime()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SemanticTimeType")))) != empty)
			{
				// Is type allowed
				Mp7JrsSemanticTimePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetSemanticTime(p, client);
					if((p = GetSemanticTime()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("SemanticPlace")) == 0)
	{
		// SemanticPlace is simple element SemanticPlaceType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetSemanticPlace()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SemanticPlaceType")))) != empty)
			{
				// Is type allowed
				Mp7JrsSemanticPlacePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetSemanticPlace(p, client);
					if((p = GetSemanticPlace()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for SemanticStateType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "SemanticStateType");
	}		
	
	return result;
}

XMLCh * Mp7JrsSemanticStateType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsSemanticStateType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}

bool Mp7JrsSemanticStateType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsSemanticStateType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsSemanticStateType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsSemanticStateType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("SemanticStateType"));
	// Element serialization:
	if (m_AttributeValuePair != Mp7JrsSemanticStateType_AttributeValuePair_Collection0Ptr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_AttributeValuePair->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	// Class
	
	if (m_SemanticTime != Mp7JrsSemanticTimePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_SemanticTime->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("SemanticTime"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_SemanticTime->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_SemanticPlace != Mp7JrsSemanticPlacePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_SemanticPlace->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("SemanticPlace"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_SemanticPlace->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsSemanticStateType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsSemanticBaseType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("AttributeValuePair"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("AttributeValuePair")) == 0))
        {
            // Deserialize factory type
            Mp7JrsSemanticStateType_AttributeValuePair_Collection0Ptr tmp = CreateSemanticStateType_AttributeValuePair_CollectionType0;
            tmp->Deserialize(doc, parent, current);
            this->SetAttributeValuePair(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("SemanticTime"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("SemanticTime")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateSemanticTimeType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetSemanticTime(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("SemanticPlace"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("SemanticPlace")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateSemanticPlaceType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetSemanticPlace(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsSemanticStateType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsSemanticStateType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("AttributeValuePair")) == 0))
  {
    Mp7JrsSemanticStateType_AttributeValuePair_Collection0Ptr tmp = CreateSemanticStateType_AttributeValuePair_CollectionType0;
    this->SetAttributeValuePair(tmp);
    child = tmp;
  }
  if (XMLString::compareString(elementname, X("SemanticTime")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateSemanticTimeType;
    }
    this->SetSemanticTime(child);
  }
  if (XMLString::compareString(elementname, X("SemanticPlace")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateSemanticPlaceType;
    }
    this->SetSemanticPlace(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsSemanticStateType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetAttributeValuePair() != Mp7JrsNodePtr())
		result.Insert(GetAttributeValuePair());
	if (GetSemanticTime() != Mp7JrsNodePtr())
		result.Insert(GetSemanticTime());
	if (GetSemanticPlace() != Mp7JrsNodePtr())
		result.Insert(GetSemanticPlace());
	if (GetAbstractionLevel() != Mp7JrsNodePtr())
		result.Insert(GetAbstractionLevel());
	if (GetLabel() != Mp7JrsNodePtr())
		result.Insert(GetLabel());
	if (GetDefinition() != Mp7JrsNodePtr())
		result.Insert(GetDefinition());
	if (GetProperty() != Mp7JrsNodePtr())
		result.Insert(GetProperty());
	if (GetMediaOccurrence() != Mp7JrsNodePtr())
		result.Insert(GetMediaOccurrence());
	if (GetRelation() != Mp7JrsNodePtr())
		result.Insert(GetRelation());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsSemanticStateType_ExtMethodImpl.h


