 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsSeriesOfScalarBinaryType_ExtImplInclude.h



#include "Mp7JrsSeriesOfScalarType.h"
#include "Mp7JrsFloatMatrixType.h"
#include "Mp7JrsfloatVector.h"
#include "Mp7JrsScalableSeriesType_Scaling_CollectionType.h"
#include "Mp7JrsSeriesOfScalarBinaryType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsScalableSeriesType_Scaling_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Scaling

#include <assert.h>
IMp7JrsSeriesOfScalarBinaryType::IMp7JrsSeriesOfScalarBinaryType()
{

// no includefile for extension defined 
// file Mp7JrsSeriesOfScalarBinaryType_ExtPropInit.h

}

IMp7JrsSeriesOfScalarBinaryType::~IMp7JrsSeriesOfScalarBinaryType()
{
// no includefile for extension defined 
// file Mp7JrsSeriesOfScalarBinaryType_ExtPropCleanup.h

}

Mp7JrsSeriesOfScalarBinaryType::Mp7JrsSeriesOfScalarBinaryType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsSeriesOfScalarBinaryType::~Mp7JrsSeriesOfScalarBinaryType()
{
	Cleanup();
}

void Mp7JrsSeriesOfScalarBinaryType::Init()
{
	// Init base
	m_Base = CreateSeriesOfScalarType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_VarianceScalewise = Mp7JrsFloatMatrixPtr(); // Class
	m_VarianceScalewise_Exist = false;


// no includefile for extension defined 
// file Mp7JrsSeriesOfScalarBinaryType_ExtMyPropInit.h

}

void Mp7JrsSeriesOfScalarBinaryType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsSeriesOfScalarBinaryType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_VarianceScalewise);
}

void Mp7JrsSeriesOfScalarBinaryType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use SeriesOfScalarBinaryTypePtr, since we
	// might need GetBase(), which isn't defined in ISeriesOfScalarBinaryType
    const Mp7JrsPtr< Mp7JrsSeriesOfScalarBinaryType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsSeriesOfScalarPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsSeriesOfScalarBinaryType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	if (tmp->IsValidVarianceScalewise())
	{
		// Mp7JrsFactory::DeleteObject(m_VarianceScalewise);
		this->SetVarianceScalewise(Mp7JrsFactory::CloneObject(tmp->GetVarianceScalewise()));
	}
	else
	{
		InvalidateVarianceScalewise();
	}
}

Mp7JrsNodePtr Mp7JrsSeriesOfScalarBinaryType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsSeriesOfScalarBinaryType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsSeriesOfScalarType > Mp7JrsSeriesOfScalarBinaryType::GetBase() const
{
	return m_Base;
}

Mp7JrsFloatMatrixPtr Mp7JrsSeriesOfScalarBinaryType::GetVarianceScalewise() const
{
	return m_VarianceScalewise;
}

// Element is optional
bool Mp7JrsSeriesOfScalarBinaryType::IsValidVarianceScalewise() const
{
	return m_VarianceScalewise_Exist;
}

void Mp7JrsSeriesOfScalarBinaryType::SetVarianceScalewise(const Mp7JrsFloatMatrixPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfScalarBinaryType::SetVarianceScalewise().");
	}
	if (m_VarianceScalewise != item || m_VarianceScalewise_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_VarianceScalewise);
		m_VarianceScalewise = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_VarianceScalewise) m_VarianceScalewise->SetParent(m_myself.getPointer());
		if (m_VarianceScalewise && m_VarianceScalewise->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FloatMatrixType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_VarianceScalewise->UseTypeAttribute = true;
		}
		m_VarianceScalewise_Exist = true;
		if(m_VarianceScalewise != Mp7JrsFloatMatrixPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_VarianceScalewise->SetContentName(XMLString::transcode("VarianceScalewise"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfScalarBinaryType::InvalidateVarianceScalewise()
{
	m_VarianceScalewise_Exist = false;
}
Mp7JrsfloatVectorPtr Mp7JrsSeriesOfScalarBinaryType::GetRaw() const
{
	return GetBase()->GetRaw();
}

// Element is optional
bool Mp7JrsSeriesOfScalarBinaryType::IsValidRaw() const
{
	return GetBase()->IsValidRaw();
}

Mp7JrsfloatVectorPtr Mp7JrsSeriesOfScalarBinaryType::GetMin() const
{
	return GetBase()->GetMin();
}

// Element is optional
bool Mp7JrsSeriesOfScalarBinaryType::IsValidMin() const
{
	return GetBase()->IsValidMin();
}

Mp7JrsfloatVectorPtr Mp7JrsSeriesOfScalarBinaryType::GetMax() const
{
	return GetBase()->GetMax();
}

// Element is optional
bool Mp7JrsSeriesOfScalarBinaryType::IsValidMax() const
{
	return GetBase()->IsValidMax();
}

Mp7JrsfloatVectorPtr Mp7JrsSeriesOfScalarBinaryType::GetMean() const
{
	return GetBase()->GetMean();
}

// Element is optional
bool Mp7JrsSeriesOfScalarBinaryType::IsValidMean() const
{
	return GetBase()->IsValidMean();
}

Mp7JrsfloatVectorPtr Mp7JrsSeriesOfScalarBinaryType::GetRandom() const
{
	return GetBase()->GetRandom();
}

// Element is optional
bool Mp7JrsSeriesOfScalarBinaryType::IsValidRandom() const
{
	return GetBase()->IsValidRandom();
}

Mp7JrsfloatVectorPtr Mp7JrsSeriesOfScalarBinaryType::GetFirst() const
{
	return GetBase()->GetFirst();
}

// Element is optional
bool Mp7JrsSeriesOfScalarBinaryType::IsValidFirst() const
{
	return GetBase()->IsValidFirst();
}

Mp7JrsfloatVectorPtr Mp7JrsSeriesOfScalarBinaryType::GetLast() const
{
	return GetBase()->GetLast();
}

// Element is optional
bool Mp7JrsSeriesOfScalarBinaryType::IsValidLast() const
{
	return GetBase()->IsValidLast();
}

Mp7JrsfloatVectorPtr Mp7JrsSeriesOfScalarBinaryType::GetVariance() const
{
	return GetBase()->GetVariance();
}

// Element is optional
bool Mp7JrsSeriesOfScalarBinaryType::IsValidVariance() const
{
	return GetBase()->IsValidVariance();
}

Mp7JrsfloatVectorPtr Mp7JrsSeriesOfScalarBinaryType::GetWeight() const
{
	return GetBase()->GetWeight();
}

// Element is optional
bool Mp7JrsSeriesOfScalarBinaryType::IsValidWeight() const
{
	return GetBase()->IsValidWeight();
}

void Mp7JrsSeriesOfScalarBinaryType::SetRaw(const Mp7JrsfloatVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfScalarBinaryType::SetRaw().");
	}
	GetBase()->SetRaw(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfScalarBinaryType::InvalidateRaw()
{
	GetBase()->InvalidateRaw();
}
void Mp7JrsSeriesOfScalarBinaryType::SetMin(const Mp7JrsfloatVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfScalarBinaryType::SetMin().");
	}
	GetBase()->SetMin(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfScalarBinaryType::InvalidateMin()
{
	GetBase()->InvalidateMin();
}
void Mp7JrsSeriesOfScalarBinaryType::SetMax(const Mp7JrsfloatVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfScalarBinaryType::SetMax().");
	}
	GetBase()->SetMax(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfScalarBinaryType::InvalidateMax()
{
	GetBase()->InvalidateMax();
}
void Mp7JrsSeriesOfScalarBinaryType::SetMean(const Mp7JrsfloatVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfScalarBinaryType::SetMean().");
	}
	GetBase()->SetMean(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfScalarBinaryType::InvalidateMean()
{
	GetBase()->InvalidateMean();
}
void Mp7JrsSeriesOfScalarBinaryType::SetRandom(const Mp7JrsfloatVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfScalarBinaryType::SetRandom().");
	}
	GetBase()->SetRandom(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfScalarBinaryType::InvalidateRandom()
{
	GetBase()->InvalidateRandom();
}
void Mp7JrsSeriesOfScalarBinaryType::SetFirst(const Mp7JrsfloatVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfScalarBinaryType::SetFirst().");
	}
	GetBase()->SetFirst(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfScalarBinaryType::InvalidateFirst()
{
	GetBase()->InvalidateFirst();
}
void Mp7JrsSeriesOfScalarBinaryType::SetLast(const Mp7JrsfloatVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfScalarBinaryType::SetLast().");
	}
	GetBase()->SetLast(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfScalarBinaryType::InvalidateLast()
{
	GetBase()->InvalidateLast();
}
void Mp7JrsSeriesOfScalarBinaryType::SetVariance(const Mp7JrsfloatVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfScalarBinaryType::SetVariance().");
	}
	GetBase()->SetVariance(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfScalarBinaryType::InvalidateVariance()
{
	GetBase()->InvalidateVariance();
}
void Mp7JrsSeriesOfScalarBinaryType::SetWeight(const Mp7JrsfloatVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfScalarBinaryType::SetWeight().");
	}
	GetBase()->SetWeight(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfScalarBinaryType::InvalidateWeight()
{
	GetBase()->InvalidateWeight();
}
unsigned Mp7JrsSeriesOfScalarBinaryType::GettotalNumOfSamples() const
{
	return GetBase()->GetBase()->GettotalNumOfSamples();
}

void Mp7JrsSeriesOfScalarBinaryType::SettotalNumOfSamples(unsigned item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfScalarBinaryType::SettotalNumOfSamples().");
	}
	GetBase()->GetBase()->SettotalNumOfSamples(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsScalableSeriesType_Scaling_CollectionPtr Mp7JrsSeriesOfScalarBinaryType::GetScaling() const
{
	return GetBase()->GetBase()->GetScaling();
}

void Mp7JrsSeriesOfScalarBinaryType::SetScaling(const Mp7JrsScalableSeriesType_Scaling_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfScalarBinaryType::SetScaling().");
	}
	GetBase()->GetBase()->SetScaling(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsSeriesOfScalarBinaryType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("VarianceScalewise")) == 0)
	{
		// VarianceScalewise is simple element FloatMatrixType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetVarianceScalewise()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FloatMatrixType")))) != empty)
			{
				// Is type allowed
				Mp7JrsFloatMatrixPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetVarianceScalewise(p, client);
					if((p = GetVarianceScalewise()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for SeriesOfScalarBinaryType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "SeriesOfScalarBinaryType");
	}		
	
	return result;
}

XMLCh * Mp7JrsSeriesOfScalarBinaryType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsSeriesOfScalarBinaryType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}

bool Mp7JrsSeriesOfScalarBinaryType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsSeriesOfScalarBinaryType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsSeriesOfScalarBinaryType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsSeriesOfScalarBinaryType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("SeriesOfScalarBinaryType"));
	// Element serialization:
	// Class
	
	if (m_VarianceScalewise != Mp7JrsFloatMatrixPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_VarianceScalewise->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("VarianceScalewise"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_VarianceScalewise->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsSeriesOfScalarBinaryType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsSeriesOfScalarType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("VarianceScalewise"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("VarianceScalewise")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateFloatMatrixType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetVarianceScalewise(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsSeriesOfScalarBinaryType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsSeriesOfScalarBinaryType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("VarianceScalewise")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateFloatMatrixType;
    }
    this->SetVarianceScalewise(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsSeriesOfScalarBinaryType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetVarianceScalewise() != Mp7JrsNodePtr())
		result.Insert(GetVarianceScalewise());
	if (GetRaw() != Mp7JrsNodePtr())
		result.Insert(GetRaw());
	if (GetMin() != Mp7JrsNodePtr())
		result.Insert(GetMin());
	if (GetMax() != Mp7JrsNodePtr())
		result.Insert(GetMax());
	if (GetMean() != Mp7JrsNodePtr())
		result.Insert(GetMean());
	if (GetRandom() != Mp7JrsNodePtr())
		result.Insert(GetRandom());
	if (GetFirst() != Mp7JrsNodePtr())
		result.Insert(GetFirst());
	if (GetLast() != Mp7JrsNodePtr())
		result.Insert(GetLast());
	if (GetVariance() != Mp7JrsNodePtr())
		result.Insert(GetVariance());
	if (GetWeight() != Mp7JrsNodePtr())
		result.Insert(GetWeight());
	if (GetScaling() != Mp7JrsNodePtr())
		result.Insert(GetScaling());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsSeriesOfScalarBinaryType_ExtMethodImpl.h


