 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsSeriesOfScalarType_ExtImplInclude.h



#include "Mp7JrsScalableSeriesType.h"
#include "Mp7JrsfloatVector.h"
#include "Mp7JrsScalableSeriesType_Scaling_CollectionType.h"
#include "Mp7JrsSeriesOfScalarType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsScalableSeriesType_Scaling_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Scaling

#include <assert.h>
IMp7JrsSeriesOfScalarType::IMp7JrsSeriesOfScalarType()
{

// no includefile for extension defined 
// file Mp7JrsSeriesOfScalarType_ExtPropInit.h

}

IMp7JrsSeriesOfScalarType::~IMp7JrsSeriesOfScalarType()
{
// no includefile for extension defined 
// file Mp7JrsSeriesOfScalarType_ExtPropCleanup.h

}

Mp7JrsSeriesOfScalarType::Mp7JrsSeriesOfScalarType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsSeriesOfScalarType::~Mp7JrsSeriesOfScalarType()
{
	Cleanup();
}

void Mp7JrsSeriesOfScalarType::Init()
{
	// Init base
	m_Base = CreateScalableSeriesType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_Raw = Mp7JrsfloatVectorPtr(); // Collection
	m_Raw_Exist = false;
	m_Min = Mp7JrsfloatVectorPtr(); // Collection
	m_Min_Exist = false;
	m_Max = Mp7JrsfloatVectorPtr(); // Collection
	m_Max_Exist = false;
	m_Mean = Mp7JrsfloatVectorPtr(); // Collection
	m_Mean_Exist = false;
	m_Random = Mp7JrsfloatVectorPtr(); // Collection
	m_Random_Exist = false;
	m_First = Mp7JrsfloatVectorPtr(); // Collection
	m_First_Exist = false;
	m_Last = Mp7JrsfloatVectorPtr(); // Collection
	m_Last_Exist = false;
	m_Variance = Mp7JrsfloatVectorPtr(); // Collection
	m_Variance_Exist = false;
	m_Weight = Mp7JrsfloatVectorPtr(); // Collection
	m_Weight_Exist = false;


// no includefile for extension defined 
// file Mp7JrsSeriesOfScalarType_ExtMyPropInit.h

}

void Mp7JrsSeriesOfScalarType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsSeriesOfScalarType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_Raw);
	// Mp7JrsFactory::DeleteObject(m_Min);
	// Mp7JrsFactory::DeleteObject(m_Max);
	// Mp7JrsFactory::DeleteObject(m_Mean);
	// Mp7JrsFactory::DeleteObject(m_Random);
	// Mp7JrsFactory::DeleteObject(m_First);
	// Mp7JrsFactory::DeleteObject(m_Last);
	// Mp7JrsFactory::DeleteObject(m_Variance);
	// Mp7JrsFactory::DeleteObject(m_Weight);
}

void Mp7JrsSeriesOfScalarType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use SeriesOfScalarTypePtr, since we
	// might need GetBase(), which isn't defined in ISeriesOfScalarType
    const Mp7JrsPtr< Mp7JrsSeriesOfScalarType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsScalableSeriesPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsSeriesOfScalarType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	if (tmp->IsValidRaw())
	{
		// Mp7JrsFactory::DeleteObject(m_Raw);
		this->SetRaw(Mp7JrsFactory::CloneObject(tmp->GetRaw()));
	}
	else
	{
		InvalidateRaw();
	}
	if (tmp->IsValidMin())
	{
		// Mp7JrsFactory::DeleteObject(m_Min);
		this->SetMin(Mp7JrsFactory::CloneObject(tmp->GetMin()));
	}
	else
	{
		InvalidateMin();
	}
	if (tmp->IsValidMax())
	{
		// Mp7JrsFactory::DeleteObject(m_Max);
		this->SetMax(Mp7JrsFactory::CloneObject(tmp->GetMax()));
	}
	else
	{
		InvalidateMax();
	}
	if (tmp->IsValidMean())
	{
		// Mp7JrsFactory::DeleteObject(m_Mean);
		this->SetMean(Mp7JrsFactory::CloneObject(tmp->GetMean()));
	}
	else
	{
		InvalidateMean();
	}
	if (tmp->IsValidRandom())
	{
		// Mp7JrsFactory::DeleteObject(m_Random);
		this->SetRandom(Mp7JrsFactory::CloneObject(tmp->GetRandom()));
	}
	else
	{
		InvalidateRandom();
	}
	if (tmp->IsValidFirst())
	{
		// Mp7JrsFactory::DeleteObject(m_First);
		this->SetFirst(Mp7JrsFactory::CloneObject(tmp->GetFirst()));
	}
	else
	{
		InvalidateFirst();
	}
	if (tmp->IsValidLast())
	{
		// Mp7JrsFactory::DeleteObject(m_Last);
		this->SetLast(Mp7JrsFactory::CloneObject(tmp->GetLast()));
	}
	else
	{
		InvalidateLast();
	}
	if (tmp->IsValidVariance())
	{
		// Mp7JrsFactory::DeleteObject(m_Variance);
		this->SetVariance(Mp7JrsFactory::CloneObject(tmp->GetVariance()));
	}
	else
	{
		InvalidateVariance();
	}
	if (tmp->IsValidWeight())
	{
		// Mp7JrsFactory::DeleteObject(m_Weight);
		this->SetWeight(Mp7JrsFactory::CloneObject(tmp->GetWeight()));
	}
	else
	{
		InvalidateWeight();
	}
}

Mp7JrsNodePtr Mp7JrsSeriesOfScalarType::GetBaseRoot()
{
	return GetBase();
}

const Mp7JrsNodePtr Mp7JrsSeriesOfScalarType::GetBaseRoot() const
{
	return GetBase();
}


Mp7JrsPtr< Mp7JrsScalableSeriesType > Mp7JrsSeriesOfScalarType::GetBase() const
{
	return m_Base;
}

Mp7JrsfloatVectorPtr Mp7JrsSeriesOfScalarType::GetRaw() const
{
	return m_Raw;
}

// Element is optional
bool Mp7JrsSeriesOfScalarType::IsValidRaw() const
{
	return m_Raw_Exist;
}

Mp7JrsfloatVectorPtr Mp7JrsSeriesOfScalarType::GetMin() const
{
	return m_Min;
}

// Element is optional
bool Mp7JrsSeriesOfScalarType::IsValidMin() const
{
	return m_Min_Exist;
}

Mp7JrsfloatVectorPtr Mp7JrsSeriesOfScalarType::GetMax() const
{
	return m_Max;
}

// Element is optional
bool Mp7JrsSeriesOfScalarType::IsValidMax() const
{
	return m_Max_Exist;
}

Mp7JrsfloatVectorPtr Mp7JrsSeriesOfScalarType::GetMean() const
{
	return m_Mean;
}

// Element is optional
bool Mp7JrsSeriesOfScalarType::IsValidMean() const
{
	return m_Mean_Exist;
}

Mp7JrsfloatVectorPtr Mp7JrsSeriesOfScalarType::GetRandom() const
{
	return m_Random;
}

// Element is optional
bool Mp7JrsSeriesOfScalarType::IsValidRandom() const
{
	return m_Random_Exist;
}

Mp7JrsfloatVectorPtr Mp7JrsSeriesOfScalarType::GetFirst() const
{
	return m_First;
}

// Element is optional
bool Mp7JrsSeriesOfScalarType::IsValidFirst() const
{
	return m_First_Exist;
}

Mp7JrsfloatVectorPtr Mp7JrsSeriesOfScalarType::GetLast() const
{
	return m_Last;
}

// Element is optional
bool Mp7JrsSeriesOfScalarType::IsValidLast() const
{
	return m_Last_Exist;
}

Mp7JrsfloatVectorPtr Mp7JrsSeriesOfScalarType::GetVariance() const
{
	return m_Variance;
}

// Element is optional
bool Mp7JrsSeriesOfScalarType::IsValidVariance() const
{
	return m_Variance_Exist;
}

Mp7JrsfloatVectorPtr Mp7JrsSeriesOfScalarType::GetWeight() const
{
	return m_Weight;
}

// Element is optional
bool Mp7JrsSeriesOfScalarType::IsValidWeight() const
{
	return m_Weight_Exist;
}

void Mp7JrsSeriesOfScalarType::SetRaw(const Mp7JrsfloatVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfScalarType::SetRaw().");
	}
	if (m_Raw != item || m_Raw_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Raw);
		m_Raw = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Raw) m_Raw->SetParent(m_myself.getPointer());
		m_Raw_Exist = true;
		if(m_Raw != Mp7JrsfloatVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Raw->SetContentName(XMLString::transcode("Raw"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfScalarType::InvalidateRaw()
{
	m_Raw_Exist = false;
}
void Mp7JrsSeriesOfScalarType::SetMin(const Mp7JrsfloatVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfScalarType::SetMin().");
	}
	if (m_Min != item || m_Min_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Min);
		m_Min = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Min) m_Min->SetParent(m_myself.getPointer());
		m_Min_Exist = true;
		if(m_Min != Mp7JrsfloatVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Min->SetContentName(XMLString::transcode("Min"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfScalarType::InvalidateMin()
{
	m_Min_Exist = false;
}
void Mp7JrsSeriesOfScalarType::SetMax(const Mp7JrsfloatVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfScalarType::SetMax().");
	}
	if (m_Max != item || m_Max_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Max);
		m_Max = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Max) m_Max->SetParent(m_myself.getPointer());
		m_Max_Exist = true;
		if(m_Max != Mp7JrsfloatVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Max->SetContentName(XMLString::transcode("Max"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfScalarType::InvalidateMax()
{
	m_Max_Exist = false;
}
void Mp7JrsSeriesOfScalarType::SetMean(const Mp7JrsfloatVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfScalarType::SetMean().");
	}
	if (m_Mean != item || m_Mean_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Mean);
		m_Mean = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Mean) m_Mean->SetParent(m_myself.getPointer());
		m_Mean_Exist = true;
		if(m_Mean != Mp7JrsfloatVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Mean->SetContentName(XMLString::transcode("Mean"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfScalarType::InvalidateMean()
{
	m_Mean_Exist = false;
}
void Mp7JrsSeriesOfScalarType::SetRandom(const Mp7JrsfloatVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfScalarType::SetRandom().");
	}
	if (m_Random != item || m_Random_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Random);
		m_Random = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Random) m_Random->SetParent(m_myself.getPointer());
		m_Random_Exist = true;
		if(m_Random != Mp7JrsfloatVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Random->SetContentName(XMLString::transcode("Random"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfScalarType::InvalidateRandom()
{
	m_Random_Exist = false;
}
void Mp7JrsSeriesOfScalarType::SetFirst(const Mp7JrsfloatVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfScalarType::SetFirst().");
	}
	if (m_First != item || m_First_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_First);
		m_First = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_First) m_First->SetParent(m_myself.getPointer());
		m_First_Exist = true;
		if(m_First != Mp7JrsfloatVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_First->SetContentName(XMLString::transcode("First"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfScalarType::InvalidateFirst()
{
	m_First_Exist = false;
}
void Mp7JrsSeriesOfScalarType::SetLast(const Mp7JrsfloatVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfScalarType::SetLast().");
	}
	if (m_Last != item || m_Last_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Last);
		m_Last = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Last) m_Last->SetParent(m_myself.getPointer());
		m_Last_Exist = true;
		if(m_Last != Mp7JrsfloatVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Last->SetContentName(XMLString::transcode("Last"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfScalarType::InvalidateLast()
{
	m_Last_Exist = false;
}
void Mp7JrsSeriesOfScalarType::SetVariance(const Mp7JrsfloatVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfScalarType::SetVariance().");
	}
	if (m_Variance != item || m_Variance_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Variance);
		m_Variance = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Variance) m_Variance->SetParent(m_myself.getPointer());
		m_Variance_Exist = true;
		if(m_Variance != Mp7JrsfloatVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Variance->SetContentName(XMLString::transcode("Variance"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfScalarType::InvalidateVariance()
{
	m_Variance_Exist = false;
}
void Mp7JrsSeriesOfScalarType::SetWeight(const Mp7JrsfloatVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfScalarType::SetWeight().");
	}
	if (m_Weight != item || m_Weight_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Weight);
		m_Weight = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Weight) m_Weight->SetParent(m_myself.getPointer());
		m_Weight_Exist = true;
		if(m_Weight != Mp7JrsfloatVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Weight->SetContentName(XMLString::transcode("Weight"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfScalarType::InvalidateWeight()
{
	m_Weight_Exist = false;
}
unsigned Mp7JrsSeriesOfScalarType::GettotalNumOfSamples() const
{
	return GetBase()->GettotalNumOfSamples();
}

void Mp7JrsSeriesOfScalarType::SettotalNumOfSamples(unsigned item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfScalarType::SettotalNumOfSamples().");
	}
	GetBase()->SettotalNumOfSamples(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsScalableSeriesType_Scaling_CollectionPtr Mp7JrsSeriesOfScalarType::GetScaling() const
{
	return GetBase()->GetScaling();
}

void Mp7JrsSeriesOfScalarType::SetScaling(const Mp7JrsScalableSeriesType_Scaling_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfScalarType::SetScaling().");
	}
	GetBase()->SetScaling(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsSeriesOfScalarType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Raw")) == 0)
	{
		// Raw is simple element floatVector
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetRaw()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:floatVector")))) != empty)
			{
				// Is type allowed
				Mp7JrsfloatVectorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetRaw(p, client);
					if((p = GetRaw()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Min")) == 0)
	{
		// Min is simple element floatVector
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMin()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:floatVector")))) != empty)
			{
				// Is type allowed
				Mp7JrsfloatVectorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMin(p, client);
					if((p = GetMin()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Max")) == 0)
	{
		// Max is simple element floatVector
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMax()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:floatVector")))) != empty)
			{
				// Is type allowed
				Mp7JrsfloatVectorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMax(p, client);
					if((p = GetMax()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Mean")) == 0)
	{
		// Mean is simple element floatVector
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMean()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:floatVector")))) != empty)
			{
				// Is type allowed
				Mp7JrsfloatVectorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMean(p, client);
					if((p = GetMean()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Random")) == 0)
	{
		// Random is simple element floatVector
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetRandom()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:floatVector")))) != empty)
			{
				// Is type allowed
				Mp7JrsfloatVectorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetRandom(p, client);
					if((p = GetRandom()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("First")) == 0)
	{
		// First is simple element floatVector
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetFirst()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:floatVector")))) != empty)
			{
				// Is type allowed
				Mp7JrsfloatVectorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetFirst(p, client);
					if((p = GetFirst()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Last")) == 0)
	{
		// Last is simple element floatVector
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetLast()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:floatVector")))) != empty)
			{
				// Is type allowed
				Mp7JrsfloatVectorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetLast(p, client);
					if((p = GetLast()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Variance")) == 0)
	{
		// Variance is simple element floatVector
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetVariance()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:floatVector")))) != empty)
			{
				// Is type allowed
				Mp7JrsfloatVectorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetVariance(p, client);
					if((p = GetVariance()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Weight")) == 0)
	{
		// Weight is simple element floatVector
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetWeight()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:floatVector")))) != empty)
			{
				// Is type allowed
				Mp7JrsfloatVectorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetWeight(p, client);
					if((p = GetWeight()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for SeriesOfScalarType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "SeriesOfScalarType");
	}		
	
	return result;
}

XMLCh * Mp7JrsSeriesOfScalarType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsSeriesOfScalarType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}

bool Mp7JrsSeriesOfScalarType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsSeriesOfScalarType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsSeriesOfScalarType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsSeriesOfScalarType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("SeriesOfScalarType"));
	// Element serialization:
	if (m_Raw != Mp7JrsfloatVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Raw->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_Min != Mp7JrsfloatVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Min->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_Max != Mp7JrsfloatVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Max->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_Mean != Mp7JrsfloatVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Mean->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_Random != Mp7JrsfloatVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Random->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_First != Mp7JrsfloatVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_First->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_Last != Mp7JrsfloatVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Last->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_Variance != Mp7JrsfloatVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Variance->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_Weight != Mp7JrsfloatVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Weight->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsSeriesOfScalarType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsScalableSeriesType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Deserialise Element ValCollectionType
        if((parent != NULL) && (XMLString::compareString(parent->getNodeName()
			, X("Raw")) == 0))
        {
            // Deserialize factory type
            Mp7JrsfloatVectorPtr tmp = CreatefloatVector;
            tmp->Deserialize(doc, parent, current);
            this->SetRaw(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialise Element ValCollectionType
        if((parent != NULL) && (XMLString::compareString(parent->getNodeName()
			, X("Min")) == 0))
        {
            // Deserialize factory type
            Mp7JrsfloatVectorPtr tmp = CreatefloatVector;
            tmp->Deserialize(doc, parent, current);
            this->SetMin(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialise Element ValCollectionType
        if((parent != NULL) && (XMLString::compareString(parent->getNodeName()
			, X("Max")) == 0))
        {
            // Deserialize factory type
            Mp7JrsfloatVectorPtr tmp = CreatefloatVector;
            tmp->Deserialize(doc, parent, current);
            this->SetMax(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialise Element ValCollectionType
        if((parent != NULL) && (XMLString::compareString(parent->getNodeName()
			, X("Mean")) == 0))
        {
            // Deserialize factory type
            Mp7JrsfloatVectorPtr tmp = CreatefloatVector;
            tmp->Deserialize(doc, parent, current);
            this->SetMean(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialise Element ValCollectionType
        if((parent != NULL) && (XMLString::compareString(parent->getNodeName()
			, X("Random")) == 0))
        {
            // Deserialize factory type
            Mp7JrsfloatVectorPtr tmp = CreatefloatVector;
            tmp->Deserialize(doc, parent, current);
            this->SetRandom(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialise Element ValCollectionType
        if((parent != NULL) && (XMLString::compareString(parent->getNodeName()
			, X("First")) == 0))
        {
            // Deserialize factory type
            Mp7JrsfloatVectorPtr tmp = CreatefloatVector;
            tmp->Deserialize(doc, parent, current);
            this->SetFirst(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialise Element ValCollectionType
        if((parent != NULL) && (XMLString::compareString(parent->getNodeName()
			, X("Last")) == 0))
        {
            // Deserialize factory type
            Mp7JrsfloatVectorPtr tmp = CreatefloatVector;
            tmp->Deserialize(doc, parent, current);
            this->SetLast(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialise Element ValCollectionType
        if((parent != NULL) && (XMLString::compareString(parent->getNodeName()
			, X("Variance")) == 0))
        {
            // Deserialize factory type
            Mp7JrsfloatVectorPtr tmp = CreatefloatVector;
            tmp->Deserialize(doc, parent, current);
            this->SetVariance(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialise Element ValCollectionType
        if((parent != NULL) && (XMLString::compareString(parent->getNodeName()
			, X("Weight")) == 0))
        {
            // Deserialize factory type
            Mp7JrsfloatVectorPtr tmp = CreatefloatVector;
            tmp->Deserialize(doc, parent, current);
            this->SetWeight(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsSeriesOfScalarType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsSeriesOfScalarType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Raw")) == 0))
  {
    Mp7JrsfloatVectorPtr tmp = CreatefloatVector;
    this->SetRaw(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Min")) == 0))
  {
    Mp7JrsfloatVectorPtr tmp = CreatefloatVector;
    this->SetMin(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Max")) == 0))
  {
    Mp7JrsfloatVectorPtr tmp = CreatefloatVector;
    this->SetMax(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Mean")) == 0))
  {
    Mp7JrsfloatVectorPtr tmp = CreatefloatVector;
    this->SetMean(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Random")) == 0))
  {
    Mp7JrsfloatVectorPtr tmp = CreatefloatVector;
    this->SetRandom(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("First")) == 0))
  {
    Mp7JrsfloatVectorPtr tmp = CreatefloatVector;
    this->SetFirst(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Last")) == 0))
  {
    Mp7JrsfloatVectorPtr tmp = CreatefloatVector;
    this->SetLast(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Variance")) == 0))
  {
    Mp7JrsfloatVectorPtr tmp = CreatefloatVector;
    this->SetVariance(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Weight")) == 0))
  {
    Mp7JrsfloatVectorPtr tmp = CreatefloatVector;
    this->SetWeight(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsSeriesOfScalarType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetRaw() != Mp7JrsNodePtr())
		result.Insert(GetRaw());
	if (GetMin() != Mp7JrsNodePtr())
		result.Insert(GetMin());
	if (GetMax() != Mp7JrsNodePtr())
		result.Insert(GetMax());
	if (GetMean() != Mp7JrsNodePtr())
		result.Insert(GetMean());
	if (GetRandom() != Mp7JrsNodePtr())
		result.Insert(GetRandom());
	if (GetFirst() != Mp7JrsNodePtr())
		result.Insert(GetFirst());
	if (GetLast() != Mp7JrsNodePtr())
		result.Insert(GetLast());
	if (GetVariance() != Mp7JrsNodePtr())
		result.Insert(GetVariance());
	if (GetWeight() != Mp7JrsNodePtr())
		result.Insert(GetWeight());
	if (GetScaling() != Mp7JrsNodePtr())
		result.Insert(GetScaling());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsSeriesOfScalarType_ExtMethodImpl.h


