 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsShapeVariationType_ExtImplInclude.h



#include "Mp7JrsVisualDType.h"
#include "Mp7JrsShapeVariationType_StaticShapeVariation_LocalType.h"
#include "Mp7JrsShapeVariationType_DynamicShapeVariation_LocalType.h"
#include "Mp7JrsShapeVariationType_StatisticalVariation_LocalType.h"
#include "Mp7JrsShapeVariationType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsShapeVariationType::IMp7JrsShapeVariationType()
{

// no includefile for extension defined 
// file Mp7JrsShapeVariationType_ExtPropInit.h

}

IMp7JrsShapeVariationType::~IMp7JrsShapeVariationType()
{
// no includefile for extension defined 
// file Mp7JrsShapeVariationType_ExtPropCleanup.h

}

Mp7JrsShapeVariationType::Mp7JrsShapeVariationType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsShapeVariationType::~Mp7JrsShapeVariationType()
{
	Cleanup();
}

void Mp7JrsShapeVariationType::Init()
{
	// Init base
	m_Base = CreateVisualDType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_StaticShapeVariation = Mp7JrsShapeVariationType_StaticShapeVariation_LocalPtr(); // Class
	m_DynamicShapeVariation = Mp7JrsShapeVariationType_DynamicShapeVariation_LocalPtr(); // Class
	m_StatisticalVariation = Mp7JrsShapeVariationType_StatisticalVariation_LocalPtr(); // Class


// no includefile for extension defined 
// file Mp7JrsShapeVariationType_ExtMyPropInit.h

}

void Mp7JrsShapeVariationType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsShapeVariationType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_StaticShapeVariation);
	// Mp7JrsFactory::DeleteObject(m_DynamicShapeVariation);
	// Mp7JrsFactory::DeleteObject(m_StatisticalVariation);
}

void Mp7JrsShapeVariationType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use ShapeVariationTypePtr, since we
	// might need GetBase(), which isn't defined in IShapeVariationType
    const Mp7JrsPtr< Mp7JrsShapeVariationType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsVisualDPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsShapeVariationType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_StaticShapeVariation);
		this->SetStaticShapeVariation(Mp7JrsFactory::CloneObject(tmp->GetStaticShapeVariation()));
		// Mp7JrsFactory::DeleteObject(m_DynamicShapeVariation);
		this->SetDynamicShapeVariation(Mp7JrsFactory::CloneObject(tmp->GetDynamicShapeVariation()));
		// Mp7JrsFactory::DeleteObject(m_StatisticalVariation);
		this->SetStatisticalVariation(Mp7JrsFactory::CloneObject(tmp->GetStatisticalVariation()));
}

Mp7JrsNodePtr Mp7JrsShapeVariationType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsShapeVariationType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsVisualDType > Mp7JrsShapeVariationType::GetBase() const
{
	return m_Base;
}

Mp7JrsShapeVariationType_StaticShapeVariation_LocalPtr Mp7JrsShapeVariationType::GetStaticShapeVariation() const
{
	return m_StaticShapeVariation;
}

Mp7JrsShapeVariationType_DynamicShapeVariation_LocalPtr Mp7JrsShapeVariationType::GetDynamicShapeVariation() const
{
	return m_DynamicShapeVariation;
}

Mp7JrsShapeVariationType_StatisticalVariation_LocalPtr Mp7JrsShapeVariationType::GetStatisticalVariation() const
{
	return m_StatisticalVariation;
}

void Mp7JrsShapeVariationType::SetStaticShapeVariation(const Mp7JrsShapeVariationType_StaticShapeVariation_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsShapeVariationType::SetStaticShapeVariation().");
	}
	if (m_StaticShapeVariation != item)
	{
		// Mp7JrsFactory::DeleteObject(m_StaticShapeVariation);
		m_StaticShapeVariation = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_StaticShapeVariation) m_StaticShapeVariation->SetParent(m_myself.getPointer());
		if (m_StaticShapeVariation && m_StaticShapeVariation->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ShapeVariationType_StaticShapeVariation_LocalType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_StaticShapeVariation->UseTypeAttribute = true;
		}
		if(m_StaticShapeVariation != Mp7JrsShapeVariationType_StaticShapeVariation_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_StaticShapeVariation->SetContentName(XMLString::transcode("StaticShapeVariation"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsShapeVariationType::SetDynamicShapeVariation(const Mp7JrsShapeVariationType_DynamicShapeVariation_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsShapeVariationType::SetDynamicShapeVariation().");
	}
	if (m_DynamicShapeVariation != item)
	{
		// Mp7JrsFactory::DeleteObject(m_DynamicShapeVariation);
		m_DynamicShapeVariation = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_DynamicShapeVariation) m_DynamicShapeVariation->SetParent(m_myself.getPointer());
		if (m_DynamicShapeVariation && m_DynamicShapeVariation->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ShapeVariationType_DynamicShapeVariation_LocalType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_DynamicShapeVariation->UseTypeAttribute = true;
		}
		if(m_DynamicShapeVariation != Mp7JrsShapeVariationType_DynamicShapeVariation_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_DynamicShapeVariation->SetContentName(XMLString::transcode("DynamicShapeVariation"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsShapeVariationType::SetStatisticalVariation(const Mp7JrsShapeVariationType_StatisticalVariation_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsShapeVariationType::SetStatisticalVariation().");
	}
	if (m_StatisticalVariation != item)
	{
		// Mp7JrsFactory::DeleteObject(m_StatisticalVariation);
		m_StatisticalVariation = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_StatisticalVariation) m_StatisticalVariation->SetParent(m_myself.getPointer());
		if (m_StatisticalVariation && m_StatisticalVariation->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ShapeVariationType_StatisticalVariation_LocalType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_StatisticalVariation->UseTypeAttribute = true;
		}
		if(m_StatisticalVariation != Mp7JrsShapeVariationType_StatisticalVariation_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_StatisticalVariation->SetContentName(XMLString::transcode("StatisticalVariation"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsShapeVariationType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("StaticShapeVariation")) == 0)
	{
		// StaticShapeVariation is simple element ShapeVariationType_StaticShapeVariation_LocalType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetStaticShapeVariation()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ShapeVariationType_StaticShapeVariation_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsShapeVariationType_StaticShapeVariation_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetStaticShapeVariation(p, client);
					if((p = GetStaticShapeVariation()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("DynamicShapeVariation")) == 0)
	{
		// DynamicShapeVariation is simple element ShapeVariationType_DynamicShapeVariation_LocalType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetDynamicShapeVariation()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ShapeVariationType_DynamicShapeVariation_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsShapeVariationType_DynamicShapeVariation_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetDynamicShapeVariation(p, client);
					if((p = GetDynamicShapeVariation()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("StatisticalVariation")) == 0)
	{
		// StatisticalVariation is simple element ShapeVariationType_StatisticalVariation_LocalType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetStatisticalVariation()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ShapeVariationType_StatisticalVariation_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsShapeVariationType_StatisticalVariation_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetStatisticalVariation(p, client);
					if((p = GetStatisticalVariation()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for ShapeVariationType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "ShapeVariationType");
	}		
	
	return result;
}

XMLCh * Mp7JrsShapeVariationType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsShapeVariationType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}

bool Mp7JrsShapeVariationType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsShapeVariationType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsShapeVariationType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsShapeVariationType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("ShapeVariationType"));
	// Element serialization:
	// Class
	
	if (m_StaticShapeVariation != Mp7JrsShapeVariationType_StaticShapeVariation_LocalPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_StaticShapeVariation->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("StaticShapeVariation"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_StaticShapeVariation->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_DynamicShapeVariation != Mp7JrsShapeVariationType_DynamicShapeVariation_LocalPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_DynamicShapeVariation->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("DynamicShapeVariation"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_DynamicShapeVariation->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_StatisticalVariation != Mp7JrsShapeVariationType_StatisticalVariation_LocalPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_StatisticalVariation->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("StatisticalVariation"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_StatisticalVariation->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsShapeVariationType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsVisualDType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("StaticShapeVariation"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("StaticShapeVariation")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateShapeVariationType_StaticShapeVariation_LocalType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetStaticShapeVariation(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("DynamicShapeVariation"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("DynamicShapeVariation")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateShapeVariationType_DynamicShapeVariation_LocalType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetDynamicShapeVariation(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("StatisticalVariation"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("StatisticalVariation")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateShapeVariationType_StatisticalVariation_LocalType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetStatisticalVariation(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsShapeVariationType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsShapeVariationType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("StaticShapeVariation")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateShapeVariationType_StaticShapeVariation_LocalType;
    }
    this->SetStaticShapeVariation(child);
  }
  if (XMLString::compareString(elementname, X("DynamicShapeVariation")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateShapeVariationType_DynamicShapeVariation_LocalType;
    }
    this->SetDynamicShapeVariation(child);
  }
  if (XMLString::compareString(elementname, X("StatisticalVariation")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateShapeVariationType_StatisticalVariation_LocalType;
    }
    this->SetStatisticalVariation(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsShapeVariationType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetStaticShapeVariation() != Mp7JrsNodePtr())
		result.Insert(GetStaticShapeVariation());
	if (GetDynamicShapeVariation() != Mp7JrsNodePtr())
		result.Insert(GetDynamicShapeVariation());
	if (GetStatisticalVariation() != Mp7JrsNodePtr())
		result.Insert(GetStatisticalVariation());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsShapeVariationType_ExtMethodImpl.h


