 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsSpeakerInfoType_ExtImplInclude.h



#include "Mp7JrsHeaderType.h"
#include "Mp7JrsPersonType.h"
#include "Mp7JrsSpeakerInfoType_WordIndex_LocalType.h"
#include "Mp7JrsSpeakerInfoType_PhoneIndex_LocalType.h"
#include "Mp7JrsSpeakerInfoType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsSpeakerInfoType::IMp7JrsSpeakerInfoType()
{

// no includefile for extension defined 
// file Mp7JrsSpeakerInfoType_ExtPropInit.h

}

IMp7JrsSpeakerInfoType::~IMp7JrsSpeakerInfoType()
{
// no includefile for extension defined 
// file Mp7JrsSpeakerInfoType_ExtPropCleanup.h

}

Mp7JrsSpeakerInfoType::Mp7JrsSpeakerInfoType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsSpeakerInfoType::~Mp7JrsSpeakerInfoType()
{
	Cleanup();
}

void Mp7JrsSpeakerInfoType::Init()
{
	// Init base
	m_Base = CreateHeaderType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_phoneLexiconRef = NULL; // String
	m_phoneLexiconRef_Exist = false;
	m_wordLexiconRef = NULL; // String
	m_wordLexiconRef_Exist = false;
	m_confusionInfoRef = NULL; // String
	m_confusionInfoRef_Exist = false;
	m_descriptionMetadataRef = NULL; // String
	m_descriptionMetadataRef_Exist = false;
	m_provenance = Mp7JrsSpeakerInfoType_provenance_LocalType::UninitializedEnumeration;

	// Init elements (element, union sequence choice all any)
	
	m_SpokenLanguage = XMLString::transcode(""); //  Mandatory String
	m_Person = Mp7JrsPersonPtr(); // Class
	m_Person_Exist = false;
	m_WordIndex = Mp7JrsSpeakerInfoType_WordIndex_LocalPtr(); // Class
	m_WordIndex_Exist = false;
	m_PhoneIndex = Mp7JrsSpeakerInfoType_PhoneIndex_LocalPtr(); // Class
	m_PhoneIndex_Exist = false;


// no includefile for extension defined 
// file Mp7JrsSpeakerInfoType_ExtMyPropInit.h

}

void Mp7JrsSpeakerInfoType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsSpeakerInfoType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	XMLString::release(&m_phoneLexiconRef);
	XMLString::release(&m_wordLexiconRef);
	XMLString::release(&m_confusionInfoRef);
	XMLString::release(&m_descriptionMetadataRef);
	XMLString::release(&this->m_SpokenLanguage);
	this->m_SpokenLanguage = NULL;
	// Mp7JrsFactory::DeleteObject(m_Person);
	// Mp7JrsFactory::DeleteObject(m_WordIndex);
	// Mp7JrsFactory::DeleteObject(m_PhoneIndex);
}

void Mp7JrsSpeakerInfoType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use SpeakerInfoTypePtr, since we
	// might need GetBase(), which isn't defined in ISpeakerInfoType
    const Mp7JrsPtr< Mp7JrsSpeakerInfoType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsHeaderPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsSpeakerInfoType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	XMLString::release(&m_phoneLexiconRef);
	if (tmp->ExistphoneLexiconRef())
	{
		this->SetphoneLexiconRef(XMLString::replicate(tmp->GetphoneLexiconRef()));
	}
	else
	{
		InvalidatephoneLexiconRef();
	}
	}
	{
	XMLString::release(&m_wordLexiconRef);
	if (tmp->ExistwordLexiconRef())
	{
		this->SetwordLexiconRef(XMLString::replicate(tmp->GetwordLexiconRef()));
	}
	else
	{
		InvalidatewordLexiconRef();
	}
	}
	{
	XMLString::release(&m_confusionInfoRef);
	if (tmp->ExistconfusionInfoRef())
	{
		this->SetconfusionInfoRef(XMLString::replicate(tmp->GetconfusionInfoRef()));
	}
	else
	{
		InvalidateconfusionInfoRef();
	}
	}
	{
	XMLString::release(&m_descriptionMetadataRef);
	if (tmp->ExistdescriptionMetadataRef())
	{
		this->SetdescriptionMetadataRef(XMLString::replicate(tmp->GetdescriptionMetadataRef()));
	}
	else
	{
		InvalidatedescriptionMetadataRef();
	}
	}
	{
		this->Setprovenance(tmp->Getprovenance());
	}
		this->SetSpokenLanguage(XMLString::replicate(tmp->GetSpokenLanguage()));
	if (tmp->IsValidPerson())
	{
		// Mp7JrsFactory::DeleteObject(m_Person);
		this->SetPerson(Mp7JrsFactory::CloneObject(tmp->GetPerson()));
	}
	else
	{
		InvalidatePerson();
	}
	if (tmp->IsValidWordIndex())
	{
		// Mp7JrsFactory::DeleteObject(m_WordIndex);
		this->SetWordIndex(Mp7JrsFactory::CloneObject(tmp->GetWordIndex()));
	}
	else
	{
		InvalidateWordIndex();
	}
	if (tmp->IsValidPhoneIndex())
	{
		// Mp7JrsFactory::DeleteObject(m_PhoneIndex);
		this->SetPhoneIndex(Mp7JrsFactory::CloneObject(tmp->GetPhoneIndex()));
	}
	else
	{
		InvalidatePhoneIndex();
	}
}

Mp7JrsNodePtr Mp7JrsSpeakerInfoType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsSpeakerInfoType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsHeaderType > Mp7JrsSpeakerInfoType::GetBase() const
{
	return m_Base;
}

XMLCh * Mp7JrsSpeakerInfoType::GetphoneLexiconRef() const
{
	return m_phoneLexiconRef;
}

bool Mp7JrsSpeakerInfoType::ExistphoneLexiconRef() const
{
	return m_phoneLexiconRef_Exist;
}
void Mp7JrsSpeakerInfoType::SetphoneLexiconRef(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpeakerInfoType::SetphoneLexiconRef().");
	}
    m_phoneLexiconRef = item;
	m_phoneLexiconRef_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSpeakerInfoType::InvalidatephoneLexiconRef()
{
	m_phoneLexiconRef_Exist = false;
}
XMLCh * Mp7JrsSpeakerInfoType::GetwordLexiconRef() const
{
	return m_wordLexiconRef;
}

bool Mp7JrsSpeakerInfoType::ExistwordLexiconRef() const
{
	return m_wordLexiconRef_Exist;
}
void Mp7JrsSpeakerInfoType::SetwordLexiconRef(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpeakerInfoType::SetwordLexiconRef().");
	}
    m_wordLexiconRef = item;
	m_wordLexiconRef_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSpeakerInfoType::InvalidatewordLexiconRef()
{
	m_wordLexiconRef_Exist = false;
}
XMLCh * Mp7JrsSpeakerInfoType::GetconfusionInfoRef() const
{
	return m_confusionInfoRef;
}

bool Mp7JrsSpeakerInfoType::ExistconfusionInfoRef() const
{
	return m_confusionInfoRef_Exist;
}
void Mp7JrsSpeakerInfoType::SetconfusionInfoRef(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpeakerInfoType::SetconfusionInfoRef().");
	}
    m_confusionInfoRef = item;
	m_confusionInfoRef_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSpeakerInfoType::InvalidateconfusionInfoRef()
{
	m_confusionInfoRef_Exist = false;
}
XMLCh * Mp7JrsSpeakerInfoType::GetdescriptionMetadataRef() const
{
	return m_descriptionMetadataRef;
}

bool Mp7JrsSpeakerInfoType::ExistdescriptionMetadataRef() const
{
	return m_descriptionMetadataRef_Exist;
}
void Mp7JrsSpeakerInfoType::SetdescriptionMetadataRef(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpeakerInfoType::SetdescriptionMetadataRef().");
	}
    m_descriptionMetadataRef = item;
	m_descriptionMetadataRef_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSpeakerInfoType::InvalidatedescriptionMetadataRef()
{
	m_descriptionMetadataRef_Exist = false;
}
Mp7JrsSpeakerInfoType_provenance_LocalType::Enumeration Mp7JrsSpeakerInfoType::Getprovenance() const
{
	return m_provenance;
}

void Mp7JrsSpeakerInfoType::Setprovenance(Mp7JrsSpeakerInfoType_provenance_LocalType::Enumeration item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpeakerInfoType::Setprovenance().");
	}
    m_provenance = item;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsSpeakerInfoType::GetSpokenLanguage() const
{
	return m_SpokenLanguage;
}

Mp7JrsPersonPtr Mp7JrsSpeakerInfoType::GetPerson() const
{
	return m_Person;
}

// Element is optional
bool Mp7JrsSpeakerInfoType::IsValidPerson() const
{
	return m_Person_Exist;
}

Mp7JrsSpeakerInfoType_WordIndex_LocalPtr Mp7JrsSpeakerInfoType::GetWordIndex() const
{
	return m_WordIndex;
}

// Element is optional
bool Mp7JrsSpeakerInfoType::IsValidWordIndex() const
{
	return m_WordIndex_Exist;
}

Mp7JrsSpeakerInfoType_PhoneIndex_LocalPtr Mp7JrsSpeakerInfoType::GetPhoneIndex() const
{
	return m_PhoneIndex;
}

// Element is optional
bool Mp7JrsSpeakerInfoType::IsValidPhoneIndex() const
{
	return m_PhoneIndex_Exist;
}

void Mp7JrsSpeakerInfoType::SetSpokenLanguage(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpeakerInfoType::SetSpokenLanguage().");
	}
	if (m_SpokenLanguage != item)
	{
		XMLString::release(&m_SpokenLanguage);
		m_SpokenLanguage = item;
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSpeakerInfoType::SetPerson(const Mp7JrsPersonPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpeakerInfoType::SetPerson().");
	}
	if (m_Person != item || m_Person_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Person);
		m_Person = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Person) m_Person->SetParent(m_myself.getPointer());
		if (m_Person && m_Person->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:PersonType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Person->UseTypeAttribute = true;
		}
		m_Person_Exist = true;
		if(m_Person != Mp7JrsPersonPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Person->SetContentName(XMLString::transcode("Person"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSpeakerInfoType::InvalidatePerson()
{
	m_Person_Exist = false;
}
void Mp7JrsSpeakerInfoType::SetWordIndex(const Mp7JrsSpeakerInfoType_WordIndex_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpeakerInfoType::SetWordIndex().");
	}
	if (m_WordIndex != item || m_WordIndex_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_WordIndex);
		m_WordIndex = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_WordIndex) m_WordIndex->SetParent(m_myself.getPointer());
		if (m_WordIndex && m_WordIndex->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SpeakerInfoType_WordIndex_LocalType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_WordIndex->UseTypeAttribute = true;
		}
		m_WordIndex_Exist = true;
		if(m_WordIndex != Mp7JrsSpeakerInfoType_WordIndex_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_WordIndex->SetContentName(XMLString::transcode("WordIndex"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSpeakerInfoType::InvalidateWordIndex()
{
	m_WordIndex_Exist = false;
}
void Mp7JrsSpeakerInfoType::SetPhoneIndex(const Mp7JrsSpeakerInfoType_PhoneIndex_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpeakerInfoType::SetPhoneIndex().");
	}
	if (m_PhoneIndex != item || m_PhoneIndex_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_PhoneIndex);
		m_PhoneIndex = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_PhoneIndex) m_PhoneIndex->SetParent(m_myself.getPointer());
		if (m_PhoneIndex && m_PhoneIndex->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SpeakerInfoType_PhoneIndex_LocalType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_PhoneIndex->UseTypeAttribute = true;
		}
		m_PhoneIndex_Exist = true;
		if(m_PhoneIndex != Mp7JrsSpeakerInfoType_PhoneIndex_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_PhoneIndex->SetContentName(XMLString::transcode("PhoneIndex"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSpeakerInfoType::InvalidatePhoneIndex()
{
	m_PhoneIndex_Exist = false;
}
XMLCh * Mp7JrsSpeakerInfoType::Getid() const
{
	return GetBase()->Getid();
}

bool Mp7JrsSpeakerInfoType::Existid() const
{
	return GetBase()->Existid();
}
void Mp7JrsSpeakerInfoType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpeakerInfoType::Setid().");
	}
	GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSpeakerInfoType::Invalidateid()
{
	GetBase()->Invalidateid();
}

Mp7JrsNodeEnum Mp7JrsSpeakerInfoType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Person")) == 0)
	{
		// Person is simple element PersonType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetPerson()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:PersonType")))) != empty)
			{
				// Is type allowed
				Mp7JrsPersonPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetPerson(p, client);
					if((p = GetPerson()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("WordIndex")) == 0)
	{
		// WordIndex is simple element SpeakerInfoType_WordIndex_LocalType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetWordIndex()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SpeakerInfoType_WordIndex_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsSpeakerInfoType_WordIndex_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetWordIndex(p, client);
					if((p = GetWordIndex()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("PhoneIndex")) == 0)
	{
		// PhoneIndex is simple element SpeakerInfoType_PhoneIndex_LocalType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetPhoneIndex()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SpeakerInfoType_PhoneIndex_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsSpeakerInfoType_PhoneIndex_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetPhoneIndex(p, client);
					if((p = GetPhoneIndex()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for SpeakerInfoType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "SpeakerInfoType");
	}		
	
	return result;
}

XMLCh * Mp7JrsSpeakerInfoType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsSpeakerInfoType::Parse(const XMLCh * const txt)
{
	return false;
}

bool Mp7JrsSpeakerInfoType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsSpeakerInfoType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsSpeakerInfoType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsSpeakerInfoType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("SpeakerInfoType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_phoneLexiconRef_Exist)
	{
	// String
	if(m_phoneLexiconRef != (XMLCh *)NULL)
	{
		element->setAttributeNS(X(""), X("phoneLexiconRef"), m_phoneLexiconRef);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_wordLexiconRef_Exist)
	{
	// String
	if(m_wordLexiconRef != (XMLCh *)NULL)
	{
		element->setAttributeNS(X(""), X("wordLexiconRef"), m_wordLexiconRef);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_confusionInfoRef_Exist)
	{
	// String
	if(m_confusionInfoRef != (XMLCh *)NULL)
	{
		element->setAttributeNS(X(""), X("confusionInfoRef"), m_confusionInfoRef);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_descriptionMetadataRef_Exist)
	{
	// String
	if(m_descriptionMetadataRef != (XMLCh *)NULL)
	{
		element->setAttributeNS(X(""), X("descriptionMetadataRef"), m_descriptionMetadataRef);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
	// Enumeration
	if(m_provenance != Mp7JrsSpeakerInfoType_provenance_LocalType::UninitializedEnumeration)
	{
		XMLCh * tmp = Mp7JrsSpeakerInfoType_provenance_LocalType::ToText(m_provenance);
		element->setAttributeNS(X(""), X("provenance"), tmp);
		XMLString::release(&tmp);
	}
	// Element serialization:
	 // String
	if(m_SpokenLanguage != (XMLCh*)NULL) Mp7JrsUtil::SerializeTextNode(element, doc, m_SpokenLanguage, X("urn:mpeg:mpeg7:schema:2004"), X("SpokenLanguage"), false);
	// Class
	
	if (m_Person != Mp7JrsPersonPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Person->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Person"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Person->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_WordIndex != Mp7JrsSpeakerInfoType_WordIndex_LocalPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_WordIndex->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("WordIndex"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_WordIndex->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_PhoneIndex != Mp7JrsSpeakerInfoType_PhoneIndex_LocalPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_PhoneIndex->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("PhoneIndex"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_PhoneIndex->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsSpeakerInfoType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("phoneLexiconRef")))
	{
        // Deserialize string type
        this->SetphoneLexiconRef(Mp7JrsConvert::TextToString(parent->getAttributeNS(X(""), X("phoneLexiconRef"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("phoneLexiconRef")))
	{
        // Deserialize string type
        this->SetphoneLexiconRef(Mp7JrsConvert::TextToString(parent->getAttribute(X("phoneLexiconRef"))));
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("wordLexiconRef")))
	{
        // Deserialize string type
        this->SetwordLexiconRef(Mp7JrsConvert::TextToString(parent->getAttributeNS(X(""), X("wordLexiconRef"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("wordLexiconRef")))
	{
        // Deserialize string type
        this->SetwordLexiconRef(Mp7JrsConvert::TextToString(parent->getAttribute(X("wordLexiconRef"))));
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("confusionInfoRef")))
	{
        // Deserialize string type
        this->SetconfusionInfoRef(Mp7JrsConvert::TextToString(parent->getAttributeNS(X(""), X("confusionInfoRef"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("confusionInfoRef")))
	{
        // Deserialize string type
        this->SetconfusionInfoRef(Mp7JrsConvert::TextToString(parent->getAttribute(X("confusionInfoRef"))));
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("descriptionMetadataRef")))
	{
        // Deserialize string type
        this->SetdescriptionMetadataRef(Mp7JrsConvert::TextToString(parent->getAttributeNS(X(""), X("descriptionMetadataRef"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("descriptionMetadataRef")))
	{
        // Deserialize string type
        this->SetdescriptionMetadataRef(Mp7JrsConvert::TextToString(parent->getAttribute(X("descriptionMetadataRef"))));
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("provenance")))
	{
        this->Setprovenance(Mp7JrsSpeakerInfoType_provenance_LocalType::Parse(parent->getAttributeNS(X(""), X("provenance"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("provenance")))
	{
        this->Setprovenance(Mp7JrsSpeakerInfoType_provenance_LocalType::Parse(parent->getAttribute(X("provenance"))));
        * current = parent;
    }

	// Extensionbase is Mp7JrsHeaderType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
	// Deserialize string (XMLCh*) element
	if(Mp7JrsUtil::HasNodeName(parent, X("SpokenLanguage"), X("urn:mpeg:mpeg7:schema:2004")))
	{
		// FTT memleaking		this->SetSpokenLanguage(Mp7JrsConvert::TextToString(Mp7JrsUtil::GetElementText(parent)));
		this->SetSpokenLanguage(Mp7JrsUtil::GetElementText(parent));
		found = true;
		* current = parent;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Person"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Person")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreatePersonType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetPerson(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("WordIndex"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("WordIndex")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateSpeakerInfoType_WordIndex_LocalType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetWordIndex(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("PhoneIndex"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("PhoneIndex")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateSpeakerInfoType_PhoneIndex_LocalType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetPhoneIndex(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsSpeakerInfoType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsSpeakerInfoType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("Person")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreatePersonType;
    }
    this->SetPerson(child);
  }
  if (XMLString::compareString(elementname, X("WordIndex")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateSpeakerInfoType_WordIndex_LocalType;
    }
    this->SetWordIndex(child);
  }
  if (XMLString::compareString(elementname, X("PhoneIndex")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateSpeakerInfoType_PhoneIndex_LocalType;
    }
    this->SetPhoneIndex(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsSpeakerInfoType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetPerson() != Mp7JrsNodePtr())
		result.Insert(GetPerson());
	if (GetWordIndex() != Mp7JrsNodePtr())
		result.Insert(GetWordIndex());
	if (GetPhoneIndex() != Mp7JrsNodePtr())
		result.Insert(GetPhoneIndex());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsSpeakerInfoType_ExtMethodImpl.h


