 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsStateTransitionModelType_ExtImplInclude.h



#include "Mp7JrsFiniteStateModelType.h"
#include "Mp7JrsProbabilityMatrixType.h"
#include "Mp7JrsStateTransitionModelType_State_CollectionType.h"
#include "Mp7JrszeroToOneType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsStateTransitionModelType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsModelStateType.h" // Element collection urn:mpeg:mpeg7:schema:2004:State

#include <assert.h>
IMp7JrsStateTransitionModelType::IMp7JrsStateTransitionModelType()
{

// no includefile for extension defined 
// file Mp7JrsStateTransitionModelType_ExtPropInit.h

}

IMp7JrsStateTransitionModelType::~IMp7JrsStateTransitionModelType()
{
// no includefile for extension defined 
// file Mp7JrsStateTransitionModelType_ExtPropCleanup.h

}

Mp7JrsStateTransitionModelType::Mp7JrsStateTransitionModelType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsStateTransitionModelType::~Mp7JrsStateTransitionModelType()
{
	Cleanup();
}

void Mp7JrsStateTransitionModelType::Init()
{
	// Init base
	m_Base = CreateFiniteStateModelType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_Initial = Mp7JrsProbabilityMatrixPtr(); // Class
	m_Initial_Exist = false;
	m_Transitions = Mp7JrsProbabilityMatrixPtr(); // Class
	m_State = Mp7JrsStateTransitionModelType_State_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsStateTransitionModelType_ExtMyPropInit.h

}

void Mp7JrsStateTransitionModelType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsStateTransitionModelType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_Initial);
	// Mp7JrsFactory::DeleteObject(m_Transitions);
	// Mp7JrsFactory::DeleteObject(m_State);
}

void Mp7JrsStateTransitionModelType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use StateTransitionModelTypePtr, since we
	// might need GetBase(), which isn't defined in IStateTransitionModelType
    const Mp7JrsPtr< Mp7JrsStateTransitionModelType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsFiniteStateModelPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsStateTransitionModelType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	if (tmp->IsValidInitial())
	{
		// Mp7JrsFactory::DeleteObject(m_Initial);
		this->SetInitial(Mp7JrsFactory::CloneObject(tmp->GetInitial()));
	}
	else
	{
		InvalidateInitial();
	}
		// Mp7JrsFactory::DeleteObject(m_Transitions);
		this->SetTransitions(Mp7JrsFactory::CloneObject(tmp->GetTransitions()));
		// Mp7JrsFactory::DeleteObject(m_State);
		this->SetState(Mp7JrsFactory::CloneObject(tmp->GetState()));
}

Mp7JrsNodePtr Mp7JrsStateTransitionModelType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsStateTransitionModelType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsFiniteStateModelType > Mp7JrsStateTransitionModelType::GetBase() const
{
	return m_Base;
}

Mp7JrsProbabilityMatrixPtr Mp7JrsStateTransitionModelType::GetInitial() const
{
	return m_Initial;
}

// Element is optional
bool Mp7JrsStateTransitionModelType::IsValidInitial() const
{
	return m_Initial_Exist;
}

Mp7JrsProbabilityMatrixPtr Mp7JrsStateTransitionModelType::GetTransitions() const
{
	return m_Transitions;
}

Mp7JrsStateTransitionModelType_State_CollectionPtr Mp7JrsStateTransitionModelType::GetState() const
{
	return m_State;
}

void Mp7JrsStateTransitionModelType::SetInitial(const Mp7JrsProbabilityMatrixPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStateTransitionModelType::SetInitial().");
	}
	if (m_Initial != item || m_Initial_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Initial);
		m_Initial = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Initial) m_Initial->SetParent(m_myself.getPointer());
		if (m_Initial && m_Initial->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ProbabilityMatrixType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Initial->UseTypeAttribute = true;
		}
		m_Initial_Exist = true;
		if(m_Initial != Mp7JrsProbabilityMatrixPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Initial->SetContentName(XMLString::transcode("Initial"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsStateTransitionModelType::InvalidateInitial()
{
	m_Initial_Exist = false;
}
void Mp7JrsStateTransitionModelType::SetTransitions(const Mp7JrsProbabilityMatrixPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStateTransitionModelType::SetTransitions().");
	}
	if (m_Transitions != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Transitions);
		m_Transitions = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Transitions) m_Transitions->SetParent(m_myself.getPointer());
		if (m_Transitions && m_Transitions->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ProbabilityMatrixType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Transitions->UseTypeAttribute = true;
		}
		if(m_Transitions != Mp7JrsProbabilityMatrixPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Transitions->SetContentName(XMLString::transcode("Transitions"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsStateTransitionModelType::SetState(const Mp7JrsStateTransitionModelType_State_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStateTransitionModelType::SetState().");
	}
	if (m_State != item)
	{
		// Mp7JrsFactory::DeleteObject(m_State);
		m_State = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_State) m_State->SetParent(m_myself.getPointer());
		if(m_State != Mp7JrsStateTransitionModelType_State_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_State->SetContentName(XMLString::transcode("State"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

unsigned Mp7JrsStateTransitionModelType::GetnumOfStates() const
{
	return GetBase()->GetnumOfStates();
}

void Mp7JrsStateTransitionModelType::SetnumOfStates(unsigned item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStateTransitionModelType::SetnumOfStates().");
	}
	GetBase()->SetnumOfStates(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrszeroToOnePtr Mp7JrsStateTransitionModelType::Getconfidence() const
{
	return GetBase()->GetBase()->GetBase()->Getconfidence();
}

bool Mp7JrsStateTransitionModelType::Existconfidence() const
{
	return GetBase()->GetBase()->GetBase()->Existconfidence();
}
void Mp7JrsStateTransitionModelType::Setconfidence(const Mp7JrszeroToOnePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStateTransitionModelType::Setconfidence().");
	}
	GetBase()->GetBase()->GetBase()->Setconfidence(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsStateTransitionModelType::Invalidateconfidence()
{
	GetBase()->GetBase()->GetBase()->Invalidateconfidence();
}
Mp7JrszeroToOnePtr Mp7JrsStateTransitionModelType::Getreliability() const
{
	return GetBase()->GetBase()->GetBase()->Getreliability();
}

bool Mp7JrsStateTransitionModelType::Existreliability() const
{
	return GetBase()->GetBase()->GetBase()->Existreliability();
}
void Mp7JrsStateTransitionModelType::Setreliability(const Mp7JrszeroToOnePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStateTransitionModelType::Setreliability().");
	}
	GetBase()->GetBase()->GetBase()->Setreliability(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsStateTransitionModelType::Invalidatereliability()
{
	GetBase()->GetBase()->GetBase()->Invalidatereliability();
}
XMLCh * Mp7JrsStateTransitionModelType::Getid() const
{
	return GetBase()->GetBase()->GetBase()->GetBase()->Getid();
}

bool Mp7JrsStateTransitionModelType::Existid() const
{
	return GetBase()->GetBase()->GetBase()->GetBase()->Existid();
}
void Mp7JrsStateTransitionModelType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStateTransitionModelType::Setid().");
	}
	GetBase()->GetBase()->GetBase()->GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsStateTransitionModelType::Invalidateid()
{
	GetBase()->GetBase()->GetBase()->GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsStateTransitionModelType::GettimeBase() const
{
	return GetBase()->GetBase()->GetBase()->GetBase()->GettimeBase();
}

bool Mp7JrsStateTransitionModelType::ExisttimeBase() const
{
	return GetBase()->GetBase()->GetBase()->GetBase()->ExisttimeBase();
}
void Mp7JrsStateTransitionModelType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStateTransitionModelType::SettimeBase().");
	}
	GetBase()->GetBase()->GetBase()->GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsStateTransitionModelType::InvalidatetimeBase()
{
	GetBase()->GetBase()->GetBase()->GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsStateTransitionModelType::GettimeUnit() const
{
	return GetBase()->GetBase()->GetBase()->GetBase()->GettimeUnit();
}

bool Mp7JrsStateTransitionModelType::ExisttimeUnit() const
{
	return GetBase()->GetBase()->GetBase()->GetBase()->ExisttimeUnit();
}
void Mp7JrsStateTransitionModelType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStateTransitionModelType::SettimeUnit().");
	}
	GetBase()->GetBase()->GetBase()->GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsStateTransitionModelType::InvalidatetimeUnit()
{
	GetBase()->GetBase()->GetBase()->GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsStateTransitionModelType::GetmediaTimeBase() const
{
	return GetBase()->GetBase()->GetBase()->GetBase()->GetmediaTimeBase();
}

bool Mp7JrsStateTransitionModelType::ExistmediaTimeBase() const
{
	return GetBase()->GetBase()->GetBase()->GetBase()->ExistmediaTimeBase();
}
void Mp7JrsStateTransitionModelType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStateTransitionModelType::SetmediaTimeBase().");
	}
	GetBase()->GetBase()->GetBase()->GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsStateTransitionModelType::InvalidatemediaTimeBase()
{
	GetBase()->GetBase()->GetBase()->GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsStateTransitionModelType::GetmediaTimeUnit() const
{
	return GetBase()->GetBase()->GetBase()->GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsStateTransitionModelType::ExistmediaTimeUnit() const
{
	return GetBase()->GetBase()->GetBase()->GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsStateTransitionModelType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStateTransitionModelType::SetmediaTimeUnit().");
	}
	GetBase()->GetBase()->GetBase()->GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsStateTransitionModelType::InvalidatemediaTimeUnit()
{
	GetBase()->GetBase()->GetBase()->GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsStateTransitionModelType::GetHeader() const
{
	return GetBase()->GetBase()->GetBase()->GetBase()->GetHeader();
}

void Mp7JrsStateTransitionModelType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStateTransitionModelType::SetHeader().");
	}
	GetBase()->GetBase()->GetBase()->GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsStateTransitionModelType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Initial")) == 0)
	{
		// Initial is simple element ProbabilityMatrixType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetInitial()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ProbabilityMatrixType")))) != empty)
			{
				// Is type allowed
				Mp7JrsProbabilityMatrixPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetInitial(p, client);
					if((p = GetInitial()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Transitions")) == 0)
	{
		// Transitions is simple element ProbabilityMatrixType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetTransitions()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ProbabilityMatrixType")))) != empty)
			{
				// Is type allowed
				Mp7JrsProbabilityMatrixPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetTransitions(p, client);
					if((p = GetTransitions()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("State")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:State is item of type ModelStateType
		// in element collection StateTransitionModelType_State_CollectionType
		
		context->Found = true;
		Mp7JrsStateTransitionModelType_State_CollectionPtr coll = GetState();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateStateTransitionModelType_State_CollectionType;
				SetState(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ModelStateType")))) != empty)
			{
				// Is type allowed
				Mp7JrsModelStatePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for StateTransitionModelType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "StateTransitionModelType");
	}		
	
	return result;
}

XMLCh * Mp7JrsStateTransitionModelType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsStateTransitionModelType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}

bool Mp7JrsStateTransitionModelType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsStateTransitionModelType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsStateTransitionModelType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsStateTransitionModelType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("StateTransitionModelType"));
	// Element serialization:
	// Class
	
	if (m_Initial != Mp7JrsProbabilityMatrixPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Initial->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Initial"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Initial->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Transitions != Mp7JrsProbabilityMatrixPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Transitions->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Transitions"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Transitions->Serialize(doc, element, &elem);
	}
	if (m_State != Mp7JrsStateTransitionModelType_State_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_State->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsStateTransitionModelType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsFiniteStateModelType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Initial"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Initial")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateProbabilityMatrixType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetInitial(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Transitions"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Transitions")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateProbabilityMatrixType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetTransitions(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("State"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("State")) == 0))
        {
            // Deserialize factory type
            Mp7JrsStateTransitionModelType_State_CollectionPtr tmp = CreateStateTransitionModelType_State_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetState(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsStateTransitionModelType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsStateTransitionModelType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("Initial")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateProbabilityMatrixType;
    }
    this->SetInitial(child);
  }
  if (XMLString::compareString(elementname, X("Transitions")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateProbabilityMatrixType;
    }
    this->SetTransitions(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("State")) == 0))
  {
    Mp7JrsStateTransitionModelType_State_CollectionPtr tmp = CreateStateTransitionModelType_State_CollectionType;
    this->SetState(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsStateTransitionModelType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetInitial() != Mp7JrsNodePtr())
		result.Insert(GetInitial());
	if (GetTransitions() != Mp7JrsNodePtr())
		result.Insert(GetTransitions());
	if (GetState() != Mp7JrsNodePtr())
		result.Insert(GetState());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsStateTransitionModelType_ExtMethodImpl.h


