 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsStillRegionFeatureType_ExtImplInclude.h



#include "Mp7JrsVisualDSType.h"
#include "Mp7JrsDominantColorType.h"
#include "Mp7JrsScalableColorType.h"
#include "Mp7JrsColorStructureType.h"
#include "Mp7JrsColorLayoutType.h"
#include "Mp7JrsColorTemperatureType.h"
#include "Mp7JrsIlluminationInvariantColorType.h"
#include "Mp7JrsEdgeHistogramType.h"
#include "Mp7JrsHomogeneousTextureType.h"
#include "Mp7JrsTextureBrowsingType.h"
#include "Mp7JrsRegionShapeType.h"
#include "Mp7JrsContourShapeType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsStillRegionFeatureType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header

#include <assert.h>
IMp7JrsStillRegionFeatureType::IMp7JrsStillRegionFeatureType()
{

// no includefile for extension defined 
// file Mp7JrsStillRegionFeatureType_ExtPropInit.h

}

IMp7JrsStillRegionFeatureType::~IMp7JrsStillRegionFeatureType()
{
// no includefile for extension defined 
// file Mp7JrsStillRegionFeatureType_ExtPropCleanup.h

}

Mp7JrsStillRegionFeatureType::Mp7JrsStillRegionFeatureType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsStillRegionFeatureType::~Mp7JrsStillRegionFeatureType()
{
	Cleanup();
}

void Mp7JrsStillRegionFeatureType::Init()
{
	// Init base
	m_Base = CreateVisualDSType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_DominantColor = Mp7JrsDominantColorPtr(); // Class
	m_ScalableColor = Mp7JrsScalableColorPtr(); // Class
	m_ColorStructure = Mp7JrsColorStructurePtr(); // Class
	m_ColorLayout = Mp7JrsColorLayoutPtr(); // Class
	m_ColorLayout_Exist = false;
	m_ColorTemperature = Mp7JrsColorTemperaturePtr(); // Class
	m_ColorTemperature_Exist = false;
	m_IlluminationCompensatedColor = Mp7JrsIlluminationInvariantColorPtr(); // Class
	m_IlluminationCompensatedColor_Exist = false;
	m_Edge = Mp7JrsEdgeHistogramPtr(); // Class
	m_Edge_Exist = false;
	m_HomogeneousPattern = Mp7JrsHomogeneousTexturePtr(); // Class
	m_TextureBrowsing = Mp7JrsTextureBrowsingPtr(); // Class
	m_ShapeMask = Mp7JrsRegionShapePtr(); // Class
	m_Contour = Mp7JrsContourShapePtr(); // Class


// no includefile for extension defined 
// file Mp7JrsStillRegionFeatureType_ExtMyPropInit.h

}

void Mp7JrsStillRegionFeatureType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsStillRegionFeatureType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_DominantColor);
	// Mp7JrsFactory::DeleteObject(m_ScalableColor);
	// Mp7JrsFactory::DeleteObject(m_ColorStructure);
	// Mp7JrsFactory::DeleteObject(m_ColorLayout);
	// Mp7JrsFactory::DeleteObject(m_ColorTemperature);
	// Mp7JrsFactory::DeleteObject(m_IlluminationCompensatedColor);
	// Mp7JrsFactory::DeleteObject(m_Edge);
	// Mp7JrsFactory::DeleteObject(m_HomogeneousPattern);
	// Mp7JrsFactory::DeleteObject(m_TextureBrowsing);
	// Mp7JrsFactory::DeleteObject(m_ShapeMask);
	// Mp7JrsFactory::DeleteObject(m_Contour);
}

void Mp7JrsStillRegionFeatureType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use StillRegionFeatureTypePtr, since we
	// might need GetBase(), which isn't defined in IStillRegionFeatureType
    const Mp7JrsPtr< Mp7JrsStillRegionFeatureType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsVisualDSPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsStillRegionFeatureType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_DominantColor);
		this->SetDominantColor(Mp7JrsFactory::CloneObject(tmp->GetDominantColor()));
		// Mp7JrsFactory::DeleteObject(m_ScalableColor);
		this->SetScalableColor(Mp7JrsFactory::CloneObject(tmp->GetScalableColor()));
		// Mp7JrsFactory::DeleteObject(m_ColorStructure);
		this->SetColorStructure(Mp7JrsFactory::CloneObject(tmp->GetColorStructure()));
	if (tmp->IsValidColorLayout())
	{
		// Mp7JrsFactory::DeleteObject(m_ColorLayout);
		this->SetColorLayout(Mp7JrsFactory::CloneObject(tmp->GetColorLayout()));
	}
	else
	{
		InvalidateColorLayout();
	}
	if (tmp->IsValidColorTemperature())
	{
		// Mp7JrsFactory::DeleteObject(m_ColorTemperature);
		this->SetColorTemperature(Mp7JrsFactory::CloneObject(tmp->GetColorTemperature()));
	}
	else
	{
		InvalidateColorTemperature();
	}
	if (tmp->IsValidIlluminationCompensatedColor())
	{
		// Mp7JrsFactory::DeleteObject(m_IlluminationCompensatedColor);
		this->SetIlluminationCompensatedColor(Mp7JrsFactory::CloneObject(tmp->GetIlluminationCompensatedColor()));
	}
	else
	{
		InvalidateIlluminationCompensatedColor();
	}
	if (tmp->IsValidEdge())
	{
		// Mp7JrsFactory::DeleteObject(m_Edge);
		this->SetEdge(Mp7JrsFactory::CloneObject(tmp->GetEdge()));
	}
	else
	{
		InvalidateEdge();
	}
		// Mp7JrsFactory::DeleteObject(m_HomogeneousPattern);
		this->SetHomogeneousPattern(Mp7JrsFactory::CloneObject(tmp->GetHomogeneousPattern()));
		// Mp7JrsFactory::DeleteObject(m_TextureBrowsing);
		this->SetTextureBrowsing(Mp7JrsFactory::CloneObject(tmp->GetTextureBrowsing()));
		// Mp7JrsFactory::DeleteObject(m_ShapeMask);
		this->SetShapeMask(Mp7JrsFactory::CloneObject(tmp->GetShapeMask()));
		// Mp7JrsFactory::DeleteObject(m_Contour);
		this->SetContour(Mp7JrsFactory::CloneObject(tmp->GetContour()));
}

Mp7JrsNodePtr Mp7JrsStillRegionFeatureType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsStillRegionFeatureType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsVisualDSType > Mp7JrsStillRegionFeatureType::GetBase() const
{
	return m_Base;
}

Mp7JrsDominantColorPtr Mp7JrsStillRegionFeatureType::GetDominantColor() const
{
	return m_DominantColor;
}

Mp7JrsScalableColorPtr Mp7JrsStillRegionFeatureType::GetScalableColor() const
{
	return m_ScalableColor;
}

Mp7JrsColorStructurePtr Mp7JrsStillRegionFeatureType::GetColorStructure() const
{
	return m_ColorStructure;
}

Mp7JrsColorLayoutPtr Mp7JrsStillRegionFeatureType::GetColorLayout() const
{
	return m_ColorLayout;
}

// Element is optional
bool Mp7JrsStillRegionFeatureType::IsValidColorLayout() const
{
	return m_ColorLayout_Exist;
}

Mp7JrsColorTemperaturePtr Mp7JrsStillRegionFeatureType::GetColorTemperature() const
{
	return m_ColorTemperature;
}

// Element is optional
bool Mp7JrsStillRegionFeatureType::IsValidColorTemperature() const
{
	return m_ColorTemperature_Exist;
}

Mp7JrsIlluminationInvariantColorPtr Mp7JrsStillRegionFeatureType::GetIlluminationCompensatedColor() const
{
	return m_IlluminationCompensatedColor;
}

// Element is optional
bool Mp7JrsStillRegionFeatureType::IsValidIlluminationCompensatedColor() const
{
	return m_IlluminationCompensatedColor_Exist;
}

Mp7JrsEdgeHistogramPtr Mp7JrsStillRegionFeatureType::GetEdge() const
{
	return m_Edge;
}

// Element is optional
bool Mp7JrsStillRegionFeatureType::IsValidEdge() const
{
	return m_Edge_Exist;
}

Mp7JrsHomogeneousTexturePtr Mp7JrsStillRegionFeatureType::GetHomogeneousPattern() const
{
	return m_HomogeneousPattern;
}

Mp7JrsTextureBrowsingPtr Mp7JrsStillRegionFeatureType::GetTextureBrowsing() const
{
	return m_TextureBrowsing;
}

Mp7JrsRegionShapePtr Mp7JrsStillRegionFeatureType::GetShapeMask() const
{
	return m_ShapeMask;
}

Mp7JrsContourShapePtr Mp7JrsStillRegionFeatureType::GetContour() const
{
	return m_Contour;
}

	// TODO Implement proper cleanup for nested choices (e.g. CompositionShotEditinbTemporalDecompositionType_LocalType)
// implementing setter for choice 
/* element */
void Mp7JrsStillRegionFeatureType::SetDominantColor(const Mp7JrsDominantColorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStillRegionFeatureType::SetDominantColor().");
	}
	if (m_DominantColor != item)
	{
		// Mp7JrsFactory::DeleteObject(m_DominantColor);
		m_DominantColor = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_DominantColor) m_DominantColor->SetParent(m_myself.getPointer());
		if (m_DominantColor && m_DominantColor->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:DominantColorType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_DominantColor->UseTypeAttribute = true;
		}
		if(m_DominantColor != Mp7JrsDominantColorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_DominantColor->SetContentName(XMLString::transcode("DominantColor"));
		}
	// Mp7JrsFactory::DeleteObject(m_ScalableColor);
	m_ScalableColor = Mp7JrsScalableColorPtr();
	// Mp7JrsFactory::DeleteObject(m_ColorStructure);
	m_ColorStructure = Mp7JrsColorStructurePtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsStillRegionFeatureType::SetScalableColor(const Mp7JrsScalableColorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStillRegionFeatureType::SetScalableColor().");
	}
	if (m_ScalableColor != item)
	{
		// Mp7JrsFactory::DeleteObject(m_ScalableColor);
		m_ScalableColor = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ScalableColor) m_ScalableColor->SetParent(m_myself.getPointer());
		if (m_ScalableColor && m_ScalableColor->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ScalableColorType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_ScalableColor->UseTypeAttribute = true;
		}
		if(m_ScalableColor != Mp7JrsScalableColorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ScalableColor->SetContentName(XMLString::transcode("ScalableColor"));
		}
	// Mp7JrsFactory::DeleteObject(m_DominantColor);
	m_DominantColor = Mp7JrsDominantColorPtr();
	// Mp7JrsFactory::DeleteObject(m_ColorStructure);
	m_ColorStructure = Mp7JrsColorStructurePtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsStillRegionFeatureType::SetColorStructure(const Mp7JrsColorStructurePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStillRegionFeatureType::SetColorStructure().");
	}
	if (m_ColorStructure != item)
	{
		// Mp7JrsFactory::DeleteObject(m_ColorStructure);
		m_ColorStructure = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ColorStructure) m_ColorStructure->SetParent(m_myself.getPointer());
		if (m_ColorStructure && m_ColorStructure->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ColorStructureType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_ColorStructure->UseTypeAttribute = true;
		}
		if(m_ColorStructure != Mp7JrsColorStructurePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ColorStructure->SetContentName(XMLString::transcode("ColorStructure"));
		}
	// Mp7JrsFactory::DeleteObject(m_DominantColor);
	m_DominantColor = Mp7JrsDominantColorPtr();
	// Mp7JrsFactory::DeleteObject(m_ScalableColor);
	m_ScalableColor = Mp7JrsScalableColorPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsStillRegionFeatureType::SetColorLayout(const Mp7JrsColorLayoutPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStillRegionFeatureType::SetColorLayout().");
	}
	if (m_ColorLayout != item || m_ColorLayout_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_ColorLayout);
		m_ColorLayout = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ColorLayout) m_ColorLayout->SetParent(m_myself.getPointer());
		if (m_ColorLayout && m_ColorLayout->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ColorLayoutType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_ColorLayout->UseTypeAttribute = true;
		}
		m_ColorLayout_Exist = true;
		if(m_ColorLayout != Mp7JrsColorLayoutPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ColorLayout->SetContentName(XMLString::transcode("ColorLayout"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsStillRegionFeatureType::InvalidateColorLayout()
{
	m_ColorLayout_Exist = false;
}
void Mp7JrsStillRegionFeatureType::SetColorTemperature(const Mp7JrsColorTemperaturePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStillRegionFeatureType::SetColorTemperature().");
	}
	if (m_ColorTemperature != item || m_ColorTemperature_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_ColorTemperature);
		m_ColorTemperature = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ColorTemperature) m_ColorTemperature->SetParent(m_myself.getPointer());
		if (m_ColorTemperature && m_ColorTemperature->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ColorTemperatureType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_ColorTemperature->UseTypeAttribute = true;
		}
		m_ColorTemperature_Exist = true;
		if(m_ColorTemperature != Mp7JrsColorTemperaturePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ColorTemperature->SetContentName(XMLString::transcode("ColorTemperature"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsStillRegionFeatureType::InvalidateColorTemperature()
{
	m_ColorTemperature_Exist = false;
}
void Mp7JrsStillRegionFeatureType::SetIlluminationCompensatedColor(const Mp7JrsIlluminationInvariantColorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStillRegionFeatureType::SetIlluminationCompensatedColor().");
	}
	if (m_IlluminationCompensatedColor != item || m_IlluminationCompensatedColor_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_IlluminationCompensatedColor);
		m_IlluminationCompensatedColor = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_IlluminationCompensatedColor) m_IlluminationCompensatedColor->SetParent(m_myself.getPointer());
		if (m_IlluminationCompensatedColor && m_IlluminationCompensatedColor->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:IlluminationInvariantColorType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_IlluminationCompensatedColor->UseTypeAttribute = true;
		}
		m_IlluminationCompensatedColor_Exist = true;
		if(m_IlluminationCompensatedColor != Mp7JrsIlluminationInvariantColorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_IlluminationCompensatedColor->SetContentName(XMLString::transcode("IlluminationCompensatedColor"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsStillRegionFeatureType::InvalidateIlluminationCompensatedColor()
{
	m_IlluminationCompensatedColor_Exist = false;
}
void Mp7JrsStillRegionFeatureType::SetEdge(const Mp7JrsEdgeHistogramPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStillRegionFeatureType::SetEdge().");
	}
	if (m_Edge != item || m_Edge_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Edge);
		m_Edge = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Edge) m_Edge->SetParent(m_myself.getPointer());
		if (m_Edge && m_Edge->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:EdgeHistogramType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Edge->UseTypeAttribute = true;
		}
		m_Edge_Exist = true;
		if(m_Edge != Mp7JrsEdgeHistogramPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Edge->SetContentName(XMLString::transcode("Edge"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsStillRegionFeatureType::InvalidateEdge()
{
	m_Edge_Exist = false;
}
	// TODO Implement proper cleanup for nested choices (e.g. CompositionShotEditinbTemporalDecompositionType_LocalType)
// implementing setter for choice 
/* element */
void Mp7JrsStillRegionFeatureType::SetHomogeneousPattern(const Mp7JrsHomogeneousTexturePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStillRegionFeatureType::SetHomogeneousPattern().");
	}
	if (m_HomogeneousPattern != item)
	{
		// Mp7JrsFactory::DeleteObject(m_HomogeneousPattern);
		m_HomogeneousPattern = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_HomogeneousPattern) m_HomogeneousPattern->SetParent(m_myself.getPointer());
		if (m_HomogeneousPattern && m_HomogeneousPattern->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:HomogeneousTextureType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_HomogeneousPattern->UseTypeAttribute = true;
		}
		if(m_HomogeneousPattern != Mp7JrsHomogeneousTexturePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_HomogeneousPattern->SetContentName(XMLString::transcode("HomogeneousPattern"));
		}
	// Mp7JrsFactory::DeleteObject(m_TextureBrowsing);
	m_TextureBrowsing = Mp7JrsTextureBrowsingPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsStillRegionFeatureType::SetTextureBrowsing(const Mp7JrsTextureBrowsingPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStillRegionFeatureType::SetTextureBrowsing().");
	}
	if (m_TextureBrowsing != item)
	{
		// Mp7JrsFactory::DeleteObject(m_TextureBrowsing);
		m_TextureBrowsing = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_TextureBrowsing) m_TextureBrowsing->SetParent(m_myself.getPointer());
		if (m_TextureBrowsing && m_TextureBrowsing->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TextureBrowsingType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_TextureBrowsing->UseTypeAttribute = true;
		}
		if(m_TextureBrowsing != Mp7JrsTextureBrowsingPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_TextureBrowsing->SetContentName(XMLString::transcode("TextureBrowsing"));
		}
	// Mp7JrsFactory::DeleteObject(m_HomogeneousPattern);
	m_HomogeneousPattern = Mp7JrsHomogeneousTexturePtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

	// TODO Implement proper cleanup for nested choices (e.g. CompositionShotEditinbTemporalDecompositionType_LocalType)
// implementing setter for choice 
/* element */
void Mp7JrsStillRegionFeatureType::SetShapeMask(const Mp7JrsRegionShapePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStillRegionFeatureType::SetShapeMask().");
	}
	if (m_ShapeMask != item)
	{
		// Mp7JrsFactory::DeleteObject(m_ShapeMask);
		m_ShapeMask = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ShapeMask) m_ShapeMask->SetParent(m_myself.getPointer());
		if (m_ShapeMask && m_ShapeMask->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RegionShapeType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_ShapeMask->UseTypeAttribute = true;
		}
		if(m_ShapeMask != Mp7JrsRegionShapePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ShapeMask->SetContentName(XMLString::transcode("ShapeMask"));
		}
	// Mp7JrsFactory::DeleteObject(m_Contour);
	m_Contour = Mp7JrsContourShapePtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsStillRegionFeatureType::SetContour(const Mp7JrsContourShapePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStillRegionFeatureType::SetContour().");
	}
	if (m_Contour != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Contour);
		m_Contour = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Contour) m_Contour->SetParent(m_myself.getPointer());
		if (m_Contour && m_Contour->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ContourShapeType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Contour->UseTypeAttribute = true;
		}
		if(m_Contour != Mp7JrsContourShapePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Contour->SetContentName(XMLString::transcode("Contour"));
		}
	// Mp7JrsFactory::DeleteObject(m_ShapeMask);
	m_ShapeMask = Mp7JrsRegionShapePtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsStillRegionFeatureType::Getid() const
{
	return GetBase()->GetBase()->Getid();
}

bool Mp7JrsStillRegionFeatureType::Existid() const
{
	return GetBase()->GetBase()->Existid();
}
void Mp7JrsStillRegionFeatureType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStillRegionFeatureType::Setid().");
	}
	GetBase()->GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsStillRegionFeatureType::Invalidateid()
{
	GetBase()->GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsStillRegionFeatureType::GettimeBase() const
{
	return GetBase()->GetBase()->GettimeBase();
}

bool Mp7JrsStillRegionFeatureType::ExisttimeBase() const
{
	return GetBase()->GetBase()->ExisttimeBase();
}
void Mp7JrsStillRegionFeatureType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStillRegionFeatureType::SettimeBase().");
	}
	GetBase()->GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsStillRegionFeatureType::InvalidatetimeBase()
{
	GetBase()->GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsStillRegionFeatureType::GettimeUnit() const
{
	return GetBase()->GetBase()->GettimeUnit();
}

bool Mp7JrsStillRegionFeatureType::ExisttimeUnit() const
{
	return GetBase()->GetBase()->ExisttimeUnit();
}
void Mp7JrsStillRegionFeatureType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStillRegionFeatureType::SettimeUnit().");
	}
	GetBase()->GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsStillRegionFeatureType::InvalidatetimeUnit()
{
	GetBase()->GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsStillRegionFeatureType::GetmediaTimeBase() const
{
	return GetBase()->GetBase()->GetmediaTimeBase();
}

bool Mp7JrsStillRegionFeatureType::ExistmediaTimeBase() const
{
	return GetBase()->GetBase()->ExistmediaTimeBase();
}
void Mp7JrsStillRegionFeatureType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStillRegionFeatureType::SetmediaTimeBase().");
	}
	GetBase()->GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsStillRegionFeatureType::InvalidatemediaTimeBase()
{
	GetBase()->GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsStillRegionFeatureType::GetmediaTimeUnit() const
{
	return GetBase()->GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsStillRegionFeatureType::ExistmediaTimeUnit() const
{
	return GetBase()->GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsStillRegionFeatureType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStillRegionFeatureType::SetmediaTimeUnit().");
	}
	GetBase()->GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsStillRegionFeatureType::InvalidatemediaTimeUnit()
{
	GetBase()->GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsStillRegionFeatureType::GetHeader() const
{
	return GetBase()->GetBase()->GetHeader();
}

void Mp7JrsStillRegionFeatureType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStillRegionFeatureType::SetHeader().");
	}
	GetBase()->GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsStillRegionFeatureType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("DominantColor")) == 0)
	{
		// DominantColor is simple element DominantColorType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetDominantColor()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:DominantColorType")))) != empty)
			{
				// Is type allowed
				Mp7JrsDominantColorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetDominantColor(p, client);
					if((p = GetDominantColor()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("ScalableColor")) == 0)
	{
		// ScalableColor is simple element ScalableColorType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetScalableColor()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ScalableColorType")))) != empty)
			{
				// Is type allowed
				Mp7JrsScalableColorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetScalableColor(p, client);
					if((p = GetScalableColor()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("ColorStructure")) == 0)
	{
		// ColorStructure is simple element ColorStructureType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetColorStructure()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ColorStructureType")))) != empty)
			{
				// Is type allowed
				Mp7JrsColorStructurePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetColorStructure(p, client);
					if((p = GetColorStructure()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("ColorLayout")) == 0)
	{
		// ColorLayout is simple element ColorLayoutType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetColorLayout()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ColorLayoutType")))) != empty)
			{
				// Is type allowed
				Mp7JrsColorLayoutPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetColorLayout(p, client);
					if((p = GetColorLayout()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("ColorTemperature")) == 0)
	{
		// ColorTemperature is simple element ColorTemperatureType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetColorTemperature()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ColorTemperatureType")))) != empty)
			{
				// Is type allowed
				Mp7JrsColorTemperaturePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetColorTemperature(p, client);
					if((p = GetColorTemperature()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("IlluminationCompensatedColor")) == 0)
	{
		// IlluminationCompensatedColor is simple element IlluminationInvariantColorType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetIlluminationCompensatedColor()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:IlluminationInvariantColorType")))) != empty)
			{
				// Is type allowed
				Mp7JrsIlluminationInvariantColorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetIlluminationCompensatedColor(p, client);
					if((p = GetIlluminationCompensatedColor()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Edge")) == 0)
	{
		// Edge is simple element EdgeHistogramType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetEdge()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:EdgeHistogramType")))) != empty)
			{
				// Is type allowed
				Mp7JrsEdgeHistogramPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetEdge(p, client);
					if((p = GetEdge()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("HomogeneousPattern")) == 0)
	{
		// HomogeneousPattern is simple element HomogeneousTextureType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetHomogeneousPattern()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:HomogeneousTextureType")))) != empty)
			{
				// Is type allowed
				Mp7JrsHomogeneousTexturePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetHomogeneousPattern(p, client);
					if((p = GetHomogeneousPattern()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("TextureBrowsing")) == 0)
	{
		// TextureBrowsing is simple element TextureBrowsingType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetTextureBrowsing()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TextureBrowsingType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTextureBrowsingPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetTextureBrowsing(p, client);
					if((p = GetTextureBrowsing()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("ShapeMask")) == 0)
	{
		// ShapeMask is simple element RegionShapeType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetShapeMask()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RegionShapeType")))) != empty)
			{
				// Is type allowed
				Mp7JrsRegionShapePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetShapeMask(p, client);
					if((p = GetShapeMask()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Contour")) == 0)
	{
		// Contour is simple element ContourShapeType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetContour()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ContourShapeType")))) != empty)
			{
				// Is type allowed
				Mp7JrsContourShapePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetContour(p, client);
					if((p = GetContour()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for StillRegionFeatureType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "StillRegionFeatureType");
	}		
	
	return result;
}

XMLCh * Mp7JrsStillRegionFeatureType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsStillRegionFeatureType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}

bool Mp7JrsStillRegionFeatureType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsStillRegionFeatureType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsStillRegionFeatureType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsStillRegionFeatureType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("StillRegionFeatureType"));
	// Element serialization:
	// Class
	
	if (m_DominantColor != Mp7JrsDominantColorPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_DominantColor->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("DominantColor"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_DominantColor->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_ScalableColor != Mp7JrsScalableColorPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_ScalableColor->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("ScalableColor"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_ScalableColor->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_ColorStructure != Mp7JrsColorStructurePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_ColorStructure->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("ColorStructure"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_ColorStructure->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_ColorLayout != Mp7JrsColorLayoutPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_ColorLayout->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("ColorLayout"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_ColorLayout->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_ColorTemperature != Mp7JrsColorTemperaturePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_ColorTemperature->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("ColorTemperature"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_ColorTemperature->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_IlluminationCompensatedColor != Mp7JrsIlluminationInvariantColorPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_IlluminationCompensatedColor->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("IlluminationCompensatedColor"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_IlluminationCompensatedColor->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Edge != Mp7JrsEdgeHistogramPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Edge->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Edge"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Edge->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_HomogeneousPattern != Mp7JrsHomogeneousTexturePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_HomogeneousPattern->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("HomogeneousPattern"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_HomogeneousPattern->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_TextureBrowsing != Mp7JrsTextureBrowsingPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_TextureBrowsing->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("TextureBrowsing"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_TextureBrowsing->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_ShapeMask != Mp7JrsRegionShapePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_ShapeMask->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("ShapeMask"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_ShapeMask->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Contour != Mp7JrsContourShapePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Contour->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Contour"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Contour->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsStillRegionFeatureType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsVisualDSType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
	do // Deserialize choice just one time!
    {
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("DominantColor"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("DominantColor")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateDominantColorType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetDominantColor(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("ScalableColor"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("ScalableColor")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateScalableColorType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetScalableColor(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("ColorStructure"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("ColorStructure")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateColorStructureType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetColorStructure(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("ColorLayout"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("ColorLayout")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateColorLayoutType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetColorLayout(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("ColorTemperature"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("ColorTemperature")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateColorTemperatureType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetColorTemperature(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("IlluminationCompensatedColor"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("IlluminationCompensatedColor")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateIlluminationInvariantColorType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetIlluminationCompensatedColor(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Edge"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Edge")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateEdgeHistogramType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetEdge(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
	do // Deserialize choice just one time!
    {
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("HomogeneousPattern"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("HomogeneousPattern")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateHomogeneousTextureType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetHomogeneousPattern(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("TextureBrowsing"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("TextureBrowsing")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateTextureBrowsingType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetTextureBrowsing(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
	do // Deserialize choice just one time!
    {
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("ShapeMask"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("ShapeMask")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateRegionShapeType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetShapeMask(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Contour"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Contour")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateContourShapeType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetContour(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
// no includefile for extension defined 
// file Mp7JrsStillRegionFeatureType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsStillRegionFeatureType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("DominantColor")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateDominantColorType;
    }
    this->SetDominantColor(child);
  }
  if (XMLString::compareString(elementname, X("ScalableColor")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateScalableColorType;
    }
    this->SetScalableColor(child);
  }
  if (XMLString::compareString(elementname, X("ColorStructure")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateColorStructureType;
    }
    this->SetColorStructure(child);
  }
  if (XMLString::compareString(elementname, X("ColorLayout")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateColorLayoutType;
    }
    this->SetColorLayout(child);
  }
  if (XMLString::compareString(elementname, X("ColorTemperature")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateColorTemperatureType;
    }
    this->SetColorTemperature(child);
  }
  if (XMLString::compareString(elementname, X("IlluminationCompensatedColor")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateIlluminationInvariantColorType;
    }
    this->SetIlluminationCompensatedColor(child);
  }
  if (XMLString::compareString(elementname, X("Edge")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateEdgeHistogramType;
    }
    this->SetEdge(child);
  }
  if (XMLString::compareString(elementname, X("HomogeneousPattern")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateHomogeneousTextureType;
    }
    this->SetHomogeneousPattern(child);
  }
  if (XMLString::compareString(elementname, X("TextureBrowsing")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateTextureBrowsingType;
    }
    this->SetTextureBrowsing(child);
  }
  if (XMLString::compareString(elementname, X("ShapeMask")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateRegionShapeType;
    }
    this->SetShapeMask(child);
  }
  if (XMLString::compareString(elementname, X("Contour")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateContourShapeType;
    }
    this->SetContour(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsStillRegionFeatureType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetColorLayout() != Mp7JrsNodePtr())
		result.Insert(GetColorLayout());
	if (GetColorTemperature() != Mp7JrsNodePtr())
		result.Insert(GetColorTemperature());
	if (GetIlluminationCompensatedColor() != Mp7JrsNodePtr())
		result.Insert(GetIlluminationCompensatedColor());
	if (GetEdge() != Mp7JrsNodePtr())
		result.Insert(GetEdge());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	if (GetDominantColor() != Mp7JrsNodePtr())
		result.Insert(GetDominantColor());
	if (GetScalableColor() != Mp7JrsNodePtr())
		result.Insert(GetScalableColor());
	if (GetColorStructure() != Mp7JrsNodePtr())
		result.Insert(GetColorStructure());
	if (GetHomogeneousPattern() != Mp7JrsNodePtr())
		result.Insert(GetHomogeneousPattern());
	if (GetTextureBrowsing() != Mp7JrsNodePtr())
		result.Insert(GetTextureBrowsing());
	if (GetShapeMask() != Mp7JrsNodePtr())
		result.Insert(GetShapeMask());
	if (GetContour() != Mp7JrsNodePtr())
		result.Insert(GetContour());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsStillRegionFeatureType_ExtMethodImpl.h


