 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsStreamSectionType_ExtImplInclude.h



#include "Mp7JrsStreamSectionType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsStreamSectionType::IMp7JrsStreamSectionType()
{

// no includefile for extension defined 
// file Mp7JrsStreamSectionType_ExtPropInit.h

}

IMp7JrsStreamSectionType::~IMp7JrsStreamSectionType()
{
// no includefile for extension defined 
// file Mp7JrsStreamSectionType_ExtPropCleanup.h

}

Mp7JrsStreamSectionType::Mp7JrsStreamSectionType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsStreamSectionType::~Mp7JrsStreamSectionType()
{
	Cleanup();
}

void Mp7JrsStreamSectionType::Init()
{

	// Init attributes
	m_unit = Mp7JrsunitType::UninitializedEnumeration;
	m_unit_Exist = false;
	m_start = 0; // Value
	m_start_Exist = false;
	m_length = 0; // Value
	m_length_Exist = false;



// no includefile for extension defined 
// file Mp7JrsStreamSectionType_ExtMyPropInit.h

}

void Mp7JrsStreamSectionType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsStreamSectionType_ExtMyPropCleanup.h


}

void Mp7JrsStreamSectionType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use StreamSectionTypePtr, since we
	// might need GetBase(), which isn't defined in IStreamSectionType
    const Mp7JrsPtr< Mp7JrsStreamSectionType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	{
	if (tmp->Existunit())
	{
		this->Setunit(tmp->Getunit());
	}
	else
	{
		Invalidateunit();
	}
	}
	{
	if (tmp->Existstart())
	{
		this->Setstart(tmp->Getstart());
	}
	else
	{
		Invalidatestart();
	}
	}
	{
	if (tmp->Existlength())
	{
		this->Setlength(tmp->Getlength());
	}
	else
	{
		Invalidatelength();
	}
	}
}

Mp7JrsNodePtr Mp7JrsStreamSectionType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsStreamSectionType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsunitType::Enumeration Mp7JrsStreamSectionType::Getunit() const
{
	return m_unit;
}

bool Mp7JrsStreamSectionType::Existunit() const
{
	return m_unit_Exist;
}
void Mp7JrsStreamSectionType::Setunit(Mp7JrsunitType::Enumeration item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStreamSectionType::Setunit().");
	}
    m_unit = item;
	m_unit_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsStreamSectionType::Invalidateunit()
{
	m_unit_Exist = false;
}
unsigned Mp7JrsStreamSectionType::Getstart() const
{
	return m_start;
}

bool Mp7JrsStreamSectionType::Existstart() const
{
	return m_start_Exist;
}
void Mp7JrsStreamSectionType::Setstart(unsigned item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStreamSectionType::Setstart().");
	}
    m_start = item;
	m_start_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsStreamSectionType::Invalidatestart()
{
	m_start_Exist = false;
}
unsigned Mp7JrsStreamSectionType::Getlength() const
{
	return m_length;
}

bool Mp7JrsStreamSectionType::Existlength() const
{
	return m_length_Exist;
}
void Mp7JrsStreamSectionType::Setlength(unsigned item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStreamSectionType::Setlength().");
	}
    m_length = item;
	m_length_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsStreamSectionType::Invalidatelength()
{
	m_length_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsStreamSectionType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	// Only rudimentary expressions allowed for this type with no child elements
	return result;
}	

XMLCh * Mp7JrsStreamSectionType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsStreamSectionType::Parse(const XMLCh * const txt)
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return false;
}

bool Mp7JrsStreamSectionType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	// Mp7JrsStreamSectionType with workaround via Deserialise()
	return Mp7JrsXPathParseContext::ContentFromString(this, txt);
}
XMLCh* Mp7JrsStreamSectionType::ContentToString() const
{
	// Mp7JrsStreamSectionType with workaround via Serialise()
	return Mp7JrsXPathParseContext::ContentToString(this);
}

void Mp7JrsStreamSectionType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("StreamSectionType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_unit_Exist)
	{
	// Enumeration
	if(m_unit != Mp7JrsunitType::UninitializedEnumeration)
	{
		XMLCh * tmp = Mp7JrsunitType::ToText(m_unit);
		element->setAttributeNS(X(""), X("unit"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_start_Exist)
	{
	// Value
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_start);
		element->setAttributeNS(X(""), X("start"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_length_Exist)
	{
	// Value
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_length);
		element->setAttributeNS(X(""), X("length"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:

}

bool Mp7JrsStreamSectionType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("unit")))
	{
        this->Setunit(Mp7JrsunitType::Parse(parent->getAttributeNS(X(""), X("unit"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("unit")))
	{
        this->Setunit(Mp7JrsunitType::Parse(parent->getAttribute(X("unit"))));
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("start")))
	{
        // deserialize value type
        this->Setstart(Mp7JrsConvert::TextToUnsignedInt(parent->getAttributeNS(X(""), X("start"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("start")))
	{
        // deserialize value type
        this->Setstart(Mp7JrsConvert::TextToUnsignedInt(parent->getAttribute(X("start"))));
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("length")))
	{
        // deserialize value type
        this->Setlength(Mp7JrsConvert::TextToUnsignedInt(parent->getAttributeNS(X(""), X("length"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("length")))
	{
        // deserialize value type
        this->Setlength(Mp7JrsConvert::TextToUnsignedInt(parent->getAttribute(X("length"))));
        * current = parent;
    }

// no includefile for extension defined 
// file Mp7JrsStreamSectionType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsStreamSectionType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsStreamSectionType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsStreamSectionType_ExtMethodImpl.h


