 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsStructuredAnnotationType_ExtImplInclude.h



#include "Mp7JrsStructuredAnnotationType_Who_CollectionType.h"
#include "Mp7JrsStructuredAnnotationType_WhatObject_CollectionType.h"
#include "Mp7JrsStructuredAnnotationType_WhatAction_CollectionType.h"
#include "Mp7JrsStructuredAnnotationType_Where_CollectionType.h"
#include "Mp7JrsStructuredAnnotationType_When_CollectionType.h"
#include "Mp7JrsStructuredAnnotationType_Why_CollectionType.h"
#include "Mp7JrsStructuredAnnotationType_How_CollectionType.h"
#include "Mp7JrsStructuredAnnotationType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsTermUseType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Who

#include <assert.h>
IMp7JrsStructuredAnnotationType::IMp7JrsStructuredAnnotationType()
{

// no includefile for extension defined 
// file Mp7JrsStructuredAnnotationType_ExtPropInit.h

}

IMp7JrsStructuredAnnotationType::~IMp7JrsStructuredAnnotationType()
{
// no includefile for extension defined 
// file Mp7JrsStructuredAnnotationType_ExtPropCleanup.h

}

Mp7JrsStructuredAnnotationType::Mp7JrsStructuredAnnotationType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsStructuredAnnotationType::~Mp7JrsStructuredAnnotationType()
{
	Cleanup();
}

void Mp7JrsStructuredAnnotationType::Init()
{

	// Init attributes
	m_lang = NULL; // String
	m_lang_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_Who = Mp7JrsStructuredAnnotationType_Who_CollectionPtr(); // Collection
	m_WhatObject = Mp7JrsStructuredAnnotationType_WhatObject_CollectionPtr(); // Collection
	m_WhatAction = Mp7JrsStructuredAnnotationType_WhatAction_CollectionPtr(); // Collection
	m_Where = Mp7JrsStructuredAnnotationType_Where_CollectionPtr(); // Collection
	m_When = Mp7JrsStructuredAnnotationType_When_CollectionPtr(); // Collection
	m_Why = Mp7JrsStructuredAnnotationType_Why_CollectionPtr(); // Collection
	m_How = Mp7JrsStructuredAnnotationType_How_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsStructuredAnnotationType_ExtMyPropInit.h

}

void Mp7JrsStructuredAnnotationType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsStructuredAnnotationType_ExtMyPropCleanup.h


	XMLString::release(&m_lang);
	// Mp7JrsFactory::DeleteObject(m_Who);
	// Mp7JrsFactory::DeleteObject(m_WhatObject);
	// Mp7JrsFactory::DeleteObject(m_WhatAction);
	// Mp7JrsFactory::DeleteObject(m_Where);
	// Mp7JrsFactory::DeleteObject(m_When);
	// Mp7JrsFactory::DeleteObject(m_Why);
	// Mp7JrsFactory::DeleteObject(m_How);
}

void Mp7JrsStructuredAnnotationType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use StructuredAnnotationTypePtr, since we
	// might need GetBase(), which isn't defined in IStructuredAnnotationType
    const Mp7JrsPtr< Mp7JrsStructuredAnnotationType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	{
	XMLString::release(&m_lang);
	if (tmp->Existlang())
	{
		this->Setlang(XMLString::replicate(tmp->Getlang()));
	}
	else
	{
		Invalidatelang();
	}
	}
		// Mp7JrsFactory::DeleteObject(m_Who);
		this->SetWho(Mp7JrsFactory::CloneObject(tmp->GetWho()));
		// Mp7JrsFactory::DeleteObject(m_WhatObject);
		this->SetWhatObject(Mp7JrsFactory::CloneObject(tmp->GetWhatObject()));
		// Mp7JrsFactory::DeleteObject(m_WhatAction);
		this->SetWhatAction(Mp7JrsFactory::CloneObject(tmp->GetWhatAction()));
		// Mp7JrsFactory::DeleteObject(m_Where);
		this->SetWhere(Mp7JrsFactory::CloneObject(tmp->GetWhere()));
		// Mp7JrsFactory::DeleteObject(m_When);
		this->SetWhen(Mp7JrsFactory::CloneObject(tmp->GetWhen()));
		// Mp7JrsFactory::DeleteObject(m_Why);
		this->SetWhy(Mp7JrsFactory::CloneObject(tmp->GetWhy()));
		// Mp7JrsFactory::DeleteObject(m_How);
		this->SetHow(Mp7JrsFactory::CloneObject(tmp->GetHow()));
}

Mp7JrsNodePtr Mp7JrsStructuredAnnotationType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsStructuredAnnotationType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


XMLCh * Mp7JrsStructuredAnnotationType::Getlang() const
{
	return m_lang;
}

bool Mp7JrsStructuredAnnotationType::Existlang() const
{
	return m_lang_Exist;
}
void Mp7JrsStructuredAnnotationType::Setlang(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStructuredAnnotationType::Setlang().");
	}
    m_lang = item;
	m_lang_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsStructuredAnnotationType::Invalidatelang()
{
	m_lang_Exist = false;
}
Mp7JrsStructuredAnnotationType_Who_CollectionPtr Mp7JrsStructuredAnnotationType::GetWho() const
{
	return m_Who;
}

Mp7JrsStructuredAnnotationType_WhatObject_CollectionPtr Mp7JrsStructuredAnnotationType::GetWhatObject() const
{
	return m_WhatObject;
}

Mp7JrsStructuredAnnotationType_WhatAction_CollectionPtr Mp7JrsStructuredAnnotationType::GetWhatAction() const
{
	return m_WhatAction;
}

Mp7JrsStructuredAnnotationType_Where_CollectionPtr Mp7JrsStructuredAnnotationType::GetWhere() const
{
	return m_Where;
}

Mp7JrsStructuredAnnotationType_When_CollectionPtr Mp7JrsStructuredAnnotationType::GetWhen() const
{
	return m_When;
}

Mp7JrsStructuredAnnotationType_Why_CollectionPtr Mp7JrsStructuredAnnotationType::GetWhy() const
{
	return m_Why;
}

Mp7JrsStructuredAnnotationType_How_CollectionPtr Mp7JrsStructuredAnnotationType::GetHow() const
{
	return m_How;
}

void Mp7JrsStructuredAnnotationType::SetWho(const Mp7JrsStructuredAnnotationType_Who_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStructuredAnnotationType::SetWho().");
	}
	if (m_Who != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Who);
		m_Who = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Who) m_Who->SetParent(m_myself.getPointer());
		if(m_Who != Mp7JrsStructuredAnnotationType_Who_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Who->SetContentName(XMLString::transcode("Who"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsStructuredAnnotationType::SetWhatObject(const Mp7JrsStructuredAnnotationType_WhatObject_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStructuredAnnotationType::SetWhatObject().");
	}
	if (m_WhatObject != item)
	{
		// Mp7JrsFactory::DeleteObject(m_WhatObject);
		m_WhatObject = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_WhatObject) m_WhatObject->SetParent(m_myself.getPointer());
		if(m_WhatObject != Mp7JrsStructuredAnnotationType_WhatObject_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_WhatObject->SetContentName(XMLString::transcode("WhatObject"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsStructuredAnnotationType::SetWhatAction(const Mp7JrsStructuredAnnotationType_WhatAction_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStructuredAnnotationType::SetWhatAction().");
	}
	if (m_WhatAction != item)
	{
		// Mp7JrsFactory::DeleteObject(m_WhatAction);
		m_WhatAction = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_WhatAction) m_WhatAction->SetParent(m_myself.getPointer());
		if(m_WhatAction != Mp7JrsStructuredAnnotationType_WhatAction_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_WhatAction->SetContentName(XMLString::transcode("WhatAction"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsStructuredAnnotationType::SetWhere(const Mp7JrsStructuredAnnotationType_Where_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStructuredAnnotationType::SetWhere().");
	}
	if (m_Where != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Where);
		m_Where = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Where) m_Where->SetParent(m_myself.getPointer());
		if(m_Where != Mp7JrsStructuredAnnotationType_Where_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Where->SetContentName(XMLString::transcode("Where"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsStructuredAnnotationType::SetWhen(const Mp7JrsStructuredAnnotationType_When_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStructuredAnnotationType::SetWhen().");
	}
	if (m_When != item)
	{
		// Mp7JrsFactory::DeleteObject(m_When);
		m_When = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_When) m_When->SetParent(m_myself.getPointer());
		if(m_When != Mp7JrsStructuredAnnotationType_When_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_When->SetContentName(XMLString::transcode("When"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsStructuredAnnotationType::SetWhy(const Mp7JrsStructuredAnnotationType_Why_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStructuredAnnotationType::SetWhy().");
	}
	if (m_Why != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Why);
		m_Why = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Why) m_Why->SetParent(m_myself.getPointer());
		if(m_Why != Mp7JrsStructuredAnnotationType_Why_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Why->SetContentName(XMLString::transcode("Why"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsStructuredAnnotationType::SetHow(const Mp7JrsStructuredAnnotationType_How_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStructuredAnnotationType::SetHow().");
	}
	if (m_How != item)
	{
		// Mp7JrsFactory::DeleteObject(m_How);
		m_How = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_How) m_How->SetParent(m_myself.getPointer());
		if(m_How != Mp7JrsStructuredAnnotationType_How_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_How->SetContentName(XMLString::transcode("How"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsStructuredAnnotationType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Who")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Who is item of type TermUseType
		// in element collection StructuredAnnotationType_Who_CollectionType
		
		context->Found = true;
		Mp7JrsStructuredAnnotationType_Who_CollectionPtr coll = GetWho();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateStructuredAnnotationType_Who_CollectionType;
				SetWho(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TermUseType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTermUsePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("WhatObject")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:WhatObject is item of type TermUseType
		// in element collection StructuredAnnotationType_WhatObject_CollectionType
		
		context->Found = true;
		Mp7JrsStructuredAnnotationType_WhatObject_CollectionPtr coll = GetWhatObject();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateStructuredAnnotationType_WhatObject_CollectionType;
				SetWhatObject(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TermUseType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTermUsePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("WhatAction")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:WhatAction is item of type TermUseType
		// in element collection StructuredAnnotationType_WhatAction_CollectionType
		
		context->Found = true;
		Mp7JrsStructuredAnnotationType_WhatAction_CollectionPtr coll = GetWhatAction();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateStructuredAnnotationType_WhatAction_CollectionType;
				SetWhatAction(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TermUseType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTermUsePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Where")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Where is item of type TermUseType
		// in element collection StructuredAnnotationType_Where_CollectionType
		
		context->Found = true;
		Mp7JrsStructuredAnnotationType_Where_CollectionPtr coll = GetWhere();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateStructuredAnnotationType_Where_CollectionType;
				SetWhere(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TermUseType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTermUsePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("When")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:When is item of type TermUseType
		// in element collection StructuredAnnotationType_When_CollectionType
		
		context->Found = true;
		Mp7JrsStructuredAnnotationType_When_CollectionPtr coll = GetWhen();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateStructuredAnnotationType_When_CollectionType;
				SetWhen(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TermUseType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTermUsePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Why")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Why is item of type TermUseType
		// in element collection StructuredAnnotationType_Why_CollectionType
		
		context->Found = true;
		Mp7JrsStructuredAnnotationType_Why_CollectionPtr coll = GetWhy();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateStructuredAnnotationType_Why_CollectionType;
				SetWhy(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TermUseType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTermUsePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("How")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:How is item of type TermUseType
		// in element collection StructuredAnnotationType_How_CollectionType
		
		context->Found = true;
		Mp7JrsStructuredAnnotationType_How_CollectionPtr coll = GetHow();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateStructuredAnnotationType_How_CollectionType;
				SetHow(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TermUseType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTermUsePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for StructuredAnnotationType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "StructuredAnnotationType");
	}		
	
	return result;
}

XMLCh * Mp7JrsStructuredAnnotationType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsStructuredAnnotationType::Parse(const XMLCh * const txt)
{
	return false;
}

bool Mp7JrsStructuredAnnotationType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	// Mp7JrsStructuredAnnotationType with workaround via Deserialise()
	return Mp7JrsXPathParseContext::ContentFromString(this, txt);
}
XMLCh* Mp7JrsStructuredAnnotationType::ContentToString() const
{
	// Mp7JrsStructuredAnnotationType with workaround via Serialise()
	return Mp7JrsXPathParseContext::ContentToString(this);
}

void Mp7JrsStructuredAnnotationType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("StructuredAnnotationType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_lang_Exist)
	{
	// String
	if(m_lang != (XMLCh *)NULL)
	{
		element->setAttributeNS(X("http://www.w3.org/XML/1998/namespace"), X("xml:lang"), m_lang);
	}
		
	} // Optional attriute
	
	// Element serialization:
	if (m_Who != Mp7JrsStructuredAnnotationType_Who_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Who->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_WhatObject != Mp7JrsStructuredAnnotationType_WhatObject_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_WhatObject->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_WhatAction != Mp7JrsStructuredAnnotationType_WhatAction_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_WhatAction->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_Where != Mp7JrsStructuredAnnotationType_Where_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Where->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_When != Mp7JrsStructuredAnnotationType_When_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_When->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_Why != Mp7JrsStructuredAnnotationType_Why_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Why->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_How != Mp7JrsStructuredAnnotationType_How_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_How->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsStructuredAnnotationType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X("http://www.w3.org/XML/1998/namespace"), X("lang")))
	{
        // Deserialize string type
        this->Setlang(Mp7JrsConvert::TextToString(parent->getAttributeNS(X("http://www.w3.org/XML/1998/namespace"), X("lang"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("lang")))
	{
        // Deserialize string type
        this->Setlang(Mp7JrsConvert::TextToString(parent->getAttribute(X("lang"))));
        * current = parent;
    }

	parent = Mp7JrsUtil::GetNextChild(parent);
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Who"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Who")) == 0))
        {
            // Deserialize factory type
            Mp7JrsStructuredAnnotationType_Who_CollectionPtr tmp = CreateStructuredAnnotationType_Who_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetWho(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("WhatObject"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("WhatObject")) == 0))
        {
            // Deserialize factory type
            Mp7JrsStructuredAnnotationType_WhatObject_CollectionPtr tmp = CreateStructuredAnnotationType_WhatObject_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetWhatObject(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("WhatAction"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("WhatAction")) == 0))
        {
            // Deserialize factory type
            Mp7JrsStructuredAnnotationType_WhatAction_CollectionPtr tmp = CreateStructuredAnnotationType_WhatAction_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetWhatAction(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Where"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Where")) == 0))
        {
            // Deserialize factory type
            Mp7JrsStructuredAnnotationType_Where_CollectionPtr tmp = CreateStructuredAnnotationType_Where_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetWhere(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("When"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("When")) == 0))
        {
            // Deserialize factory type
            Mp7JrsStructuredAnnotationType_When_CollectionPtr tmp = CreateStructuredAnnotationType_When_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetWhen(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Why"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Why")) == 0))
        {
            // Deserialize factory type
            Mp7JrsStructuredAnnotationType_Why_CollectionPtr tmp = CreateStructuredAnnotationType_Why_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetWhy(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("How"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("How")) == 0))
        {
            // Deserialize factory type
            Mp7JrsStructuredAnnotationType_How_CollectionPtr tmp = CreateStructuredAnnotationType_How_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetHow(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsStructuredAnnotationType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsStructuredAnnotationType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Who")) == 0))
  {
    Mp7JrsStructuredAnnotationType_Who_CollectionPtr tmp = CreateStructuredAnnotationType_Who_CollectionType;
    this->SetWho(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("WhatObject")) == 0))
  {
    Mp7JrsStructuredAnnotationType_WhatObject_CollectionPtr tmp = CreateStructuredAnnotationType_WhatObject_CollectionType;
    this->SetWhatObject(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("WhatAction")) == 0))
  {
    Mp7JrsStructuredAnnotationType_WhatAction_CollectionPtr tmp = CreateStructuredAnnotationType_WhatAction_CollectionType;
    this->SetWhatAction(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Where")) == 0))
  {
    Mp7JrsStructuredAnnotationType_Where_CollectionPtr tmp = CreateStructuredAnnotationType_Where_CollectionType;
    this->SetWhere(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("When")) == 0))
  {
    Mp7JrsStructuredAnnotationType_When_CollectionPtr tmp = CreateStructuredAnnotationType_When_CollectionType;
    this->SetWhen(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Why")) == 0))
  {
    Mp7JrsStructuredAnnotationType_Why_CollectionPtr tmp = CreateStructuredAnnotationType_Why_CollectionType;
    this->SetWhy(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("How")) == 0))
  {
    Mp7JrsStructuredAnnotationType_How_CollectionPtr tmp = CreateStructuredAnnotationType_How_CollectionType;
    this->SetHow(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsStructuredAnnotationType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetWho() != Mp7JrsNodePtr())
		result.Insert(GetWho());
	if (GetWhatObject() != Mp7JrsNodePtr())
		result.Insert(GetWhatObject());
	if (GetWhatAction() != Mp7JrsNodePtr())
		result.Insert(GetWhatAction());
	if (GetWhere() != Mp7JrsNodePtr())
		result.Insert(GetWhere());
	if (GetWhen() != Mp7JrsNodePtr())
		result.Insert(GetWhen());
	if (GetWhy() != Mp7JrsNodePtr())
		result.Insert(GetWhy());
	if (GetHow() != Mp7JrsNodePtr())
		result.Insert(GetHow());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsStructuredAnnotationType_ExtMethodImpl.h


