 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsSummarySegmentType_ExtImplInclude.h



#include "Mp7JrsDSType.h"
#include "Mp7JrsIDREFS.h"
#include "Mp7JrsSummarySegmentType_Name_CollectionType.h"
#include "Mp7JrsUniqueIDType.h"
#include "Mp7JrsTemporalSegmentLocatorType.h"
#include "Mp7JrsSummarySegmentType_KeyFrame_CollectionType.h"
#include "Mp7JrsSummarySegmentType_KeySound_CollectionType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsSummarySegmentType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsTextualType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Name
#include "Mp7JrsImageLocatorType.h" // Element collection urn:mpeg:mpeg7:schema:2004:KeyFrame

#include <assert.h>
IMp7JrsSummarySegmentType::IMp7JrsSummarySegmentType()
{

// no includefile for extension defined 
// file Mp7JrsSummarySegmentType_ExtPropInit.h

}

IMp7JrsSummarySegmentType::~IMp7JrsSummarySegmentType()
{
// no includefile for extension defined 
// file Mp7JrsSummarySegmentType_ExtPropCleanup.h

}

Mp7JrsSummarySegmentType::Mp7JrsSummarySegmentType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsSummarySegmentType::~Mp7JrsSummarySegmentType()
{
	Cleanup();
}

void Mp7JrsSummarySegmentType::Init()
{
	// Init base
	m_Base = CreateDSType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_themeIDs = Mp7JrsIDREFSPtr(); // Collection
	m_themeIDs_Exist = false;
	m_order = 1; // Value
	m_order_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_Name = Mp7JrsSummarySegmentType_Name_CollectionPtr(); // Collection
	m_SourceID = Mp7JrsUniqueIDPtr(); // Class
	m_SourceID_Exist = false;
	m_KeyAudioVisualClip = Mp7JrsTemporalSegmentLocatorPtr(); // Class
	m_KeyVideoClip = Mp7JrsTemporalSegmentLocatorPtr(); // Class
	m_KeyVideoClip_Exist = false;
	m_KeyAudioClip = Mp7JrsTemporalSegmentLocatorPtr(); // Class
	m_KeyAudioClip_Exist = false;
	m_KeyFrame = Mp7JrsSummarySegmentType_KeyFrame_CollectionPtr(); // Collection
	m_KeySound = Mp7JrsSummarySegmentType_KeySound_CollectionPtr(); // Collection


// begin extension included
// file Mp7JrsSummarySegmentType_ExtMyPropInit.h

	m_Name = CreateSummarySegmentType_Name_CollectionType;

	m_KeyFrame = CreateSummarySegmentType_KeyFrame_CollectionType;
// end extension included

}

void Mp7JrsSummarySegmentType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsSummarySegmentType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_themeIDs);
	// Mp7JrsFactory::DeleteObject(m_Name);
	// Mp7JrsFactory::DeleteObject(m_SourceID);
	// Mp7JrsFactory::DeleteObject(m_KeyAudioVisualClip);
	// Mp7JrsFactory::DeleteObject(m_KeyVideoClip);
	// Mp7JrsFactory::DeleteObject(m_KeyAudioClip);
	// Mp7JrsFactory::DeleteObject(m_KeyFrame);
	// Mp7JrsFactory::DeleteObject(m_KeySound);
}

void Mp7JrsSummarySegmentType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use SummarySegmentTypePtr, since we
	// might need GetBase(), which isn't defined in ISummarySegmentType
    const Mp7JrsPtr< Mp7JrsSummarySegmentType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsDSPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsSummarySegmentType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	// Mp7JrsFactory::DeleteObject(m_themeIDs);
	if (tmp->ExistthemeIDs())
	{
		this->SetthemeIDs(Mp7JrsFactory::CloneObject(tmp->GetthemeIDs()));
	}
	else
	{
		InvalidatethemeIDs();
	}
	}
	{
	if (tmp->Existorder())
	{
		this->Setorder(tmp->Getorder());
	}
	else
	{
		Invalidateorder();
	}
	}
		// Mp7JrsFactory::DeleteObject(m_Name);
		this->SetName(Mp7JrsFactory::CloneObject(tmp->GetName()));
	if (tmp->IsValidSourceID())
	{
		// Mp7JrsFactory::DeleteObject(m_SourceID);
		this->SetSourceID(Mp7JrsFactory::CloneObject(tmp->GetSourceID()));
	}
	else
	{
		InvalidateSourceID();
	}
		// Mp7JrsFactory::DeleteObject(m_KeyAudioVisualClip);
		this->SetKeyAudioVisualClip(Mp7JrsFactory::CloneObject(tmp->GetKeyAudioVisualClip()));
	if (tmp->IsValidKeyVideoClip())
	{
		// Mp7JrsFactory::DeleteObject(m_KeyVideoClip);
		this->SetKeyVideoClip(Mp7JrsFactory::CloneObject(tmp->GetKeyVideoClip()));
	}
	else
	{
		InvalidateKeyVideoClip();
	}
	if (tmp->IsValidKeyAudioClip())
	{
		// Mp7JrsFactory::DeleteObject(m_KeyAudioClip);
		this->SetKeyAudioClip(Mp7JrsFactory::CloneObject(tmp->GetKeyAudioClip()));
	}
	else
	{
		InvalidateKeyAudioClip();
	}
		// Mp7JrsFactory::DeleteObject(m_KeyFrame);
		this->SetKeyFrame(Mp7JrsFactory::CloneObject(tmp->GetKeyFrame()));
		// Mp7JrsFactory::DeleteObject(m_KeySound);
		this->SetKeySound(Mp7JrsFactory::CloneObject(tmp->GetKeySound()));
}

Mp7JrsNodePtr Mp7JrsSummarySegmentType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsSummarySegmentType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsDSType > Mp7JrsSummarySegmentType::GetBase() const
{
	return m_Base;
}

Mp7JrsIDREFSPtr Mp7JrsSummarySegmentType::GetthemeIDs() const
{
	return m_themeIDs;
}

bool Mp7JrsSummarySegmentType::ExistthemeIDs() const
{
	return m_themeIDs_Exist;
}
void Mp7JrsSummarySegmentType::SetthemeIDs(const Mp7JrsIDREFSPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSummarySegmentType::SetthemeIDs().");
	}
    m_themeIDs = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_themeIDs) m_themeIDs->SetParent(m_myself.getPointer());
	m_themeIDs_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSummarySegmentType::InvalidatethemeIDs()
{
	m_themeIDs_Exist = false;
}
unsigned Mp7JrsSummarySegmentType::Getorder() const
{
	return m_order;
}

bool Mp7JrsSummarySegmentType::Existorder() const
{
	return m_order_Exist;
}
void Mp7JrsSummarySegmentType::Setorder(unsigned item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSummarySegmentType::Setorder().");
	}
    m_order = item;
	m_order_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSummarySegmentType::Invalidateorder()
{
	m_order_Exist = false;
}
Mp7JrsSummarySegmentType_Name_CollectionPtr Mp7JrsSummarySegmentType::GetName() const
{
	return m_Name;
}

Mp7JrsUniqueIDPtr Mp7JrsSummarySegmentType::GetSourceID() const
{
	return m_SourceID;
}

// Element is optional
bool Mp7JrsSummarySegmentType::IsValidSourceID() const
{
	return m_SourceID_Exist;
}

Mp7JrsTemporalSegmentLocatorPtr Mp7JrsSummarySegmentType::GetKeyAudioVisualClip() const
{
	return m_KeyAudioVisualClip;
}

Mp7JrsTemporalSegmentLocatorPtr Mp7JrsSummarySegmentType::GetKeyVideoClip() const
{
	return m_KeyVideoClip;
}

// Element is optional
bool Mp7JrsSummarySegmentType::IsValidKeyVideoClip() const
{
	return m_KeyVideoClip_Exist;
}

Mp7JrsTemporalSegmentLocatorPtr Mp7JrsSummarySegmentType::GetKeyAudioClip() const
{
	return m_KeyAudioClip;
}

// Element is optional
bool Mp7JrsSummarySegmentType::IsValidKeyAudioClip() const
{
	return m_KeyAudioClip_Exist;
}

Mp7JrsSummarySegmentType_KeyFrame_CollectionPtr Mp7JrsSummarySegmentType::GetKeyFrame() const
{
	return m_KeyFrame;
}

Mp7JrsSummarySegmentType_KeySound_CollectionPtr Mp7JrsSummarySegmentType::GetKeySound() const
{
	return m_KeySound;
}

void Mp7JrsSummarySegmentType::SetName(const Mp7JrsSummarySegmentType_Name_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSummarySegmentType::SetName().");
	}
	if (m_Name != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Name);
		m_Name = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Name) m_Name->SetParent(m_myself.getPointer());
		if(m_Name != Mp7JrsSummarySegmentType_Name_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Name->SetContentName(XMLString::transcode("Name"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSummarySegmentType::SetSourceID(const Mp7JrsUniqueIDPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSummarySegmentType::SetSourceID().");
	}
	if (m_SourceID != item || m_SourceID_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_SourceID);
		m_SourceID = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_SourceID) m_SourceID->SetParent(m_myself.getPointer());
		if (m_SourceID && m_SourceID->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:UniqueIDType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_SourceID->UseTypeAttribute = true;
		}
		m_SourceID_Exist = true;
		if(m_SourceID != Mp7JrsUniqueIDPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_SourceID->SetContentName(XMLString::transcode("SourceID"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSummarySegmentType::InvalidateSourceID()
{
	m_SourceID_Exist = false;
}
	// TODO Implement proper cleanup for nested choices (e.g. CompositionShotEditinbTemporalDecompositionType_LocalType)
// implementing setter for choice 
/* element */
void Mp7JrsSummarySegmentType::SetKeyAudioVisualClip(const Mp7JrsTemporalSegmentLocatorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSummarySegmentType::SetKeyAudioVisualClip().");
	}
	if (m_KeyAudioVisualClip != item)
	{
		// Mp7JrsFactory::DeleteObject(m_KeyAudioVisualClip);
		m_KeyAudioVisualClip = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_KeyAudioVisualClip) m_KeyAudioVisualClip->SetParent(m_myself.getPointer());
		if (m_KeyAudioVisualClip && m_KeyAudioVisualClip->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TemporalSegmentLocatorType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_KeyAudioVisualClip->UseTypeAttribute = true;
		}
		if(m_KeyAudioVisualClip != Mp7JrsTemporalSegmentLocatorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_KeyAudioVisualClip->SetContentName(XMLString::transcode("KeyAudioVisualClip"));
		}
	// Mp7JrsFactory::DeleteObject(m_KeyVideoClip);
	m_KeyVideoClip = Mp7JrsTemporalSegmentLocatorPtr();
	// Mp7JrsFactory::DeleteObject(m_KeyAudioClip);
	m_KeyAudioClip = Mp7JrsTemporalSegmentLocatorPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* sequence */
void Mp7JrsSummarySegmentType::SetKeyVideoClip(const Mp7JrsTemporalSegmentLocatorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSummarySegmentType::SetKeyVideoClip().");
	}
	if (m_KeyVideoClip != item || m_KeyVideoClip_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_KeyVideoClip);
		m_KeyVideoClip = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_KeyVideoClip) m_KeyVideoClip->SetParent(m_myself.getPointer());
		if (m_KeyVideoClip && m_KeyVideoClip->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TemporalSegmentLocatorType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_KeyVideoClip->UseTypeAttribute = true;
		}
		m_KeyVideoClip_Exist = true;
		if(m_KeyVideoClip != Mp7JrsTemporalSegmentLocatorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_KeyVideoClip->SetContentName(XMLString::transcode("KeyVideoClip"));
		}
	// Mp7JrsFactory::DeleteObject(m_KeyAudioVisualClip);
	m_KeyAudioVisualClip = Mp7JrsTemporalSegmentLocatorPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSummarySegmentType::InvalidateKeyVideoClip()
{
	m_KeyVideoClip_Exist = false;
}
void Mp7JrsSummarySegmentType::SetKeyAudioClip(const Mp7JrsTemporalSegmentLocatorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSummarySegmentType::SetKeyAudioClip().");
	}
	if (m_KeyAudioClip != item || m_KeyAudioClip_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_KeyAudioClip);
		m_KeyAudioClip = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_KeyAudioClip) m_KeyAudioClip->SetParent(m_myself.getPointer());
		if (m_KeyAudioClip && m_KeyAudioClip->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TemporalSegmentLocatorType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_KeyAudioClip->UseTypeAttribute = true;
		}
		m_KeyAudioClip_Exist = true;
		if(m_KeyAudioClip != Mp7JrsTemporalSegmentLocatorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_KeyAudioClip->SetContentName(XMLString::transcode("KeyAudioClip"));
		}
	// Mp7JrsFactory::DeleteObject(m_KeyAudioVisualClip);
	m_KeyAudioVisualClip = Mp7JrsTemporalSegmentLocatorPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSummarySegmentType::InvalidateKeyAudioClip()
{
	m_KeyAudioClip_Exist = false;
}
void Mp7JrsSummarySegmentType::SetKeyFrame(const Mp7JrsSummarySegmentType_KeyFrame_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSummarySegmentType::SetKeyFrame().");
	}
	if (m_KeyFrame != item)
	{
		// Mp7JrsFactory::DeleteObject(m_KeyFrame);
		m_KeyFrame = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_KeyFrame) m_KeyFrame->SetParent(m_myself.getPointer());
		if(m_KeyFrame != Mp7JrsSummarySegmentType_KeyFrame_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_KeyFrame->SetContentName(XMLString::transcode("KeyFrame"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSummarySegmentType::SetKeySound(const Mp7JrsSummarySegmentType_KeySound_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSummarySegmentType::SetKeySound().");
	}
	if (m_KeySound != item)
	{
		// Mp7JrsFactory::DeleteObject(m_KeySound);
		m_KeySound = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_KeySound) m_KeySound->SetParent(m_myself.getPointer());
		if(m_KeySound != Mp7JrsSummarySegmentType_KeySound_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_KeySound->SetContentName(XMLString::transcode("KeySound"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsSummarySegmentType::Getid() const
{
	return GetBase()->Getid();
}

bool Mp7JrsSummarySegmentType::Existid() const
{
	return GetBase()->Existid();
}
void Mp7JrsSummarySegmentType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSummarySegmentType::Setid().");
	}
	GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSummarySegmentType::Invalidateid()
{
	GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsSummarySegmentType::GettimeBase() const
{
	return GetBase()->GettimeBase();
}

bool Mp7JrsSummarySegmentType::ExisttimeBase() const
{
	return GetBase()->ExisttimeBase();
}
void Mp7JrsSummarySegmentType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSummarySegmentType::SettimeBase().");
	}
	GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSummarySegmentType::InvalidatetimeBase()
{
	GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsSummarySegmentType::GettimeUnit() const
{
	return GetBase()->GettimeUnit();
}

bool Mp7JrsSummarySegmentType::ExisttimeUnit() const
{
	return GetBase()->ExisttimeUnit();
}
void Mp7JrsSummarySegmentType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSummarySegmentType::SettimeUnit().");
	}
	GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSummarySegmentType::InvalidatetimeUnit()
{
	GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsSummarySegmentType::GetmediaTimeBase() const
{
	return GetBase()->GetmediaTimeBase();
}

bool Mp7JrsSummarySegmentType::ExistmediaTimeBase() const
{
	return GetBase()->ExistmediaTimeBase();
}
void Mp7JrsSummarySegmentType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSummarySegmentType::SetmediaTimeBase().");
	}
	GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSummarySegmentType::InvalidatemediaTimeBase()
{
	GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsSummarySegmentType::GetmediaTimeUnit() const
{
	return GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsSummarySegmentType::ExistmediaTimeUnit() const
{
	return GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsSummarySegmentType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSummarySegmentType::SetmediaTimeUnit().");
	}
	GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSummarySegmentType::InvalidatemediaTimeUnit()
{
	GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsSummarySegmentType::GetHeader() const
{
	return GetBase()->GetHeader();
}

void Mp7JrsSummarySegmentType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSummarySegmentType::SetHeader().");
	}
	GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsSummarySegmentType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Name")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Name is item of type TextualType
		// in element collection SummarySegmentType_Name_CollectionType
		
		context->Found = true;
		Mp7JrsSummarySegmentType_Name_CollectionPtr coll = GetName();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateSummarySegmentType_Name_CollectionType;
				SetName(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TextualType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTextualPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("SourceID")) == 0)
	{
		// SourceID is simple element UniqueIDType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetSourceID()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:UniqueIDType")))) != empty)
			{
				// Is type allowed
				Mp7JrsUniqueIDPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetSourceID(p, client);
					if((p = GetSourceID()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("KeyAudioVisualClip")) == 0)
	{
		// KeyAudioVisualClip is simple element TemporalSegmentLocatorType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetKeyAudioVisualClip()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TemporalSegmentLocatorType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTemporalSegmentLocatorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetKeyAudioVisualClip(p, client);
					if((p = GetKeyAudioVisualClip()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("KeyVideoClip")) == 0)
	{
		// KeyVideoClip is simple element TemporalSegmentLocatorType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetKeyVideoClip()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TemporalSegmentLocatorType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTemporalSegmentLocatorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetKeyVideoClip(p, client);
					if((p = GetKeyVideoClip()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("KeyAudioClip")) == 0)
	{
		// KeyAudioClip is simple element TemporalSegmentLocatorType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetKeyAudioClip()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TemporalSegmentLocatorType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTemporalSegmentLocatorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetKeyAudioClip(p, client);
					if((p = GetKeyAudioClip()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("KeyFrame")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:KeyFrame is item of type ImageLocatorType
		// in element collection SummarySegmentType_KeyFrame_CollectionType
		
		context->Found = true;
		Mp7JrsSummarySegmentType_KeyFrame_CollectionPtr coll = GetKeyFrame();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateSummarySegmentType_KeyFrame_CollectionType;
				SetKeyFrame(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ImageLocatorType")))) != empty)
			{
				// Is type allowed
				Mp7JrsImageLocatorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("KeySound")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:KeySound is item of type TemporalSegmentLocatorType
		// in element collection SummarySegmentType_KeySound_CollectionType
		
		context->Found = true;
		Mp7JrsSummarySegmentType_KeySound_CollectionPtr coll = GetKeySound();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateSummarySegmentType_KeySound_CollectionType;
				SetKeySound(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TemporalSegmentLocatorType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTemporalSegmentLocatorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for SummarySegmentType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "SummarySegmentType");
	}		
	
	return result;
}

XMLCh * Mp7JrsSummarySegmentType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsSummarySegmentType::Parse(const XMLCh * const txt)
{
	return false;
}

bool Mp7JrsSummarySegmentType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsSummarySegmentType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsSummarySegmentType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsSummarySegmentType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("SummarySegmentType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_themeIDs_Exist)
	{
	// Collection
	if(m_themeIDs != Mp7JrsIDREFSPtr())
	{
		XMLCh * tmp = m_themeIDs->ToText();
		element->setAttributeNS(X(""), X("themeIDs"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_order_Exist)
	{
	// Value
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_order);
		element->setAttributeNS(X(""), X("order"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:
	if (m_Name != Mp7JrsSummarySegmentType_Name_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Name->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	// Class
	
	if (m_SourceID != Mp7JrsUniqueIDPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_SourceID->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("SourceID"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_SourceID->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_KeyAudioVisualClip != Mp7JrsTemporalSegmentLocatorPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_KeyAudioVisualClip->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("KeyAudioVisualClip"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_KeyAudioVisualClip->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_KeyVideoClip != Mp7JrsTemporalSegmentLocatorPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_KeyVideoClip->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("KeyVideoClip"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_KeyVideoClip->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_KeyAudioClip != Mp7JrsTemporalSegmentLocatorPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_KeyAudioClip->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("KeyAudioClip"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_KeyAudioClip->Serialize(doc, element, &elem);
	}
	if (m_KeyFrame != Mp7JrsSummarySegmentType_KeyFrame_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_KeyFrame->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_KeySound != Mp7JrsSummarySegmentType_KeySound_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_KeySound->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsSummarySegmentType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("themeIDs")))
	{
        // Deserialize collection type
		Mp7JrsIDREFSPtr tmp = CreateIDREFS;
		tmp->Parse(parent->getAttributeNS(X(""), X("themeIDs")));
        this->SetthemeIDs(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("themeIDs")))
	{
     
		// Deserialize collection type
        Mp7JrsIDREFSPtr tmp = CreateIDREFS;
        tmp->Parse(parent->getAttribute(X("themeIDs")));
        this->SetthemeIDs(tmp);
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("order")))
	{
        // deserialize value type
        this->Setorder(Mp7JrsConvert::TextToUnsignedInt(parent->getAttributeNS(X(""), X("order"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("order")))
	{
        // deserialize value type
        this->Setorder(Mp7JrsConvert::TextToUnsignedInt(parent->getAttribute(X("order"))));
        * current = parent;
    }

	// Extensionbase is Mp7JrsDSType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Name"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Name")) == 0))
        {
            // Deserialize factory type
            Mp7JrsSummarySegmentType_Name_CollectionPtr tmp = CreateSummarySegmentType_Name_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetName(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("SourceID"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("SourceID")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateUniqueIDType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetSourceID(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
	do // Deserialize choice just one time!
    {
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("KeyAudioVisualClip"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("KeyAudioVisualClip")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateTemporalSegmentLocatorType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetKeyAudioVisualClip(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("KeyVideoClip"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("KeyVideoClip")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateTemporalSegmentLocatorType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetKeyVideoClip(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("KeyAudioClip"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("KeyAudioClip")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateTemporalSegmentLocatorType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetKeyAudioClip(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }

                // FIXME: sat 2004-04-15: `continue' would be good here; we
                //finished deserializing the sequence but don't know if it
                //succeeded, so we can't.

                // continue;
	} while(false);
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("KeyFrame"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("KeyFrame")) == 0))
        {
            // Deserialize factory type
            Mp7JrsSummarySegmentType_KeyFrame_CollectionPtr tmp = CreateSummarySegmentType_KeyFrame_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetKeyFrame(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("KeySound"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("KeySound")) == 0))
        {
            // Deserialize factory type
            Mp7JrsSummarySegmentType_KeySound_CollectionPtr tmp = CreateSummarySegmentType_KeySound_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetKeySound(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsSummarySegmentType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsSummarySegmentType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Name")) == 0))
  {
    Mp7JrsSummarySegmentType_Name_CollectionPtr tmp = CreateSummarySegmentType_Name_CollectionType;
    this->SetName(tmp);
    child = tmp;
  }
  if (XMLString::compareString(elementname, X("SourceID")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateUniqueIDType;
    }
    this->SetSourceID(child);
  }
  if (XMLString::compareString(elementname, X("KeyAudioVisualClip")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateTemporalSegmentLocatorType;
    }
    this->SetKeyAudioVisualClip(child);
  }
  if (XMLString::compareString(elementname, X("KeyVideoClip")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateTemporalSegmentLocatorType;
    }
    this->SetKeyVideoClip(child);
  }
  if (XMLString::compareString(elementname, X("KeyAudioClip")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateTemporalSegmentLocatorType;
    }
    this->SetKeyAudioClip(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("KeyFrame")) == 0))
  {
    Mp7JrsSummarySegmentType_KeyFrame_CollectionPtr tmp = CreateSummarySegmentType_KeyFrame_CollectionType;
    this->SetKeyFrame(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("KeySound")) == 0))
  {
    Mp7JrsSummarySegmentType_KeySound_CollectionPtr tmp = CreateSummarySegmentType_KeySound_CollectionType;
    this->SetKeySound(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsSummarySegmentType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetName() != Mp7JrsNodePtr())
		result.Insert(GetName());
	if (GetSourceID() != Mp7JrsNodePtr())
		result.Insert(GetSourceID());
	if (GetKeyFrame() != Mp7JrsNodePtr())
		result.Insert(GetKeyFrame());
	if (GetKeySound() != Mp7JrsNodePtr())
		result.Insert(GetKeySound());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	if (GetKeyAudioVisualClip() != Mp7JrsNodePtr())
		result.Insert(GetKeyAudioVisualClip());
	if (GetKeyVideoClip() != Mp7JrsNodePtr())
		result.Insert(GetKeyVideoClip());
	if (GetKeyAudioClip() != Mp7JrsNodePtr())
		result.Insert(GetKeyAudioClip());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// begin extension included
// file Mp7JrsSummarySegmentType_ExtMethodImpl.h

#include "Mp7JrsImageLocatorType.h"
#include "Mp7JrsMediaTimeType.h"
#include "Mp7JrsTextualType.h"
#include "Mp7JrsmediaTimePointType.h"
#include "Mp7JrsTextualBaseType.h"

void IMp7JrsSummarySegmentType::AddKeyframe(Mp7JrsmediaTimePointPtr timepoint, bool copyTimePoint, Mp7JrsClientID client) {
	if (GetKeyFrame()==NULL) SetKeyFrame(CreateSummarySegmentType_KeyFrame_CollectionType, client);
	Mp7JrsImageLocatorPtr locator = CreateImageLocatorType;
	
	Mp7JrsmediaTimePointPtr tp = timepoint;
	if (copyTimePoint) tp = (Mp7JrsmediaTimePointPtr) Mp7JrsFactory::CloneObject(timepoint);

	locator->SetMediaTimePoint(tp, client);
	GetKeyFrame()->addElement(locator);
}

int IMp7JrsSummarySegmentType::GetNrKeyframes() {
	if (GetKeyFrame()==NULL) return 0;
	return GetKeyFrame()->size();
}

///////

void IMp7JrsSummarySegmentType::SetKeyVideoClip(Mp7JrsmediaTimePointPtr beginTime, Mp7JrsmediaTimePointPtr endTime, bool copyTimePoints, Mp7JrsClientID client) {
	if (GetKeyVideoClip()==NULL) SetKeyVideoClip(CreateTemporalSegmentLocatorType, client);
	GetKeyVideoClip()->SetMediaTime(CreateMediaTimeType, client);
	Mp7JrsmediaTimePointPtr beginTp = beginTime;
	Mp7JrsmediaTimePointPtr endTp = endTime;
	if (copyTimePoints) {
		beginTp = (Mp7JrsmediaTimePointPtr) Mp7JrsFactory::CloneObject(beginTime);
		endTp = (Mp7JrsmediaTimePointPtr) Mp7JrsFactory::CloneObject(endTime);
	}
	GetKeyVideoClip()->GetMediaTime()->SetTime(beginTp,endTp,client);
}

void IMp7JrsSummarySegmentType::GetKeyVideoClip(Mp7JrsmediaTimePointPtr& beginTime, Mp7JrsmediaTimePointPtr& endTime) {
	if (GetKeyVideoClip()==NULL) return;
	GetKeyVideoClip()->GetMediaTime()->GetTime(beginTime,endTime);
}


void IMp7JrsSummarySegmentType::AddName(XMLCh* name, Mp7JrsClientID client) {
	Mp7JrsTextualPtr text = CreateTextualType;
	text->SetContent(name, client);
	GetName()->addElement(text, client);
}
// end extension included


