 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsVariationDescriptionType_ExtImplInclude.h



#include "Mp7JrsContentAbstractionType.h"
#include "Mp7JrsVariationDescriptionType_VariationSet_CollectionType.h"
#include "Mp7JrsVariationDescriptionType_Variation_CollectionType.h"
#include "Mp7JrsContentDescriptionType_Affective_CollectionType.h"
#include "Mp7JrsDescriptionMetadataType.h"
#include "Mp7JrsCompleteDescriptionType_Relationships_CollectionType.h"
#include "Mp7JrsCompleteDescriptionType_OrderingKey_CollectionType.h"
#include "Mp7JrsVariationDescriptionType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsGraphType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Relationships
#include "Mp7JrsOrderingKeyType.h" // Element collection urn:mpeg:mpeg7:schema:2004:OrderingKey
#include "Mp7JrsAffectiveType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Affective
#include "Mp7JrsVariationSetType.h" // Element collection urn:mpeg:mpeg7:schema:2004:VariationSet
#include "Mp7JrsVariationType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Variation

#include <assert.h>
IMp7JrsVariationDescriptionType::IMp7JrsVariationDescriptionType()
{

// no includefile for extension defined 
// file Mp7JrsVariationDescriptionType_ExtPropInit.h

}

IMp7JrsVariationDescriptionType::~IMp7JrsVariationDescriptionType()
{
// no includefile for extension defined 
// file Mp7JrsVariationDescriptionType_ExtPropCleanup.h

}

Mp7JrsVariationDescriptionType::Mp7JrsVariationDescriptionType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsVariationDescriptionType::~Mp7JrsVariationDescriptionType()
{
	Cleanup();
}

void Mp7JrsVariationDescriptionType::Init()
{
	// Init base
	m_Base = CreateContentAbstractionType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_VariationSet = Mp7JrsVariationDescriptionType_VariationSet_CollectionPtr(); // Collection
	m_Variation = Mp7JrsVariationDescriptionType_Variation_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsVariationDescriptionType_ExtMyPropInit.h

}

void Mp7JrsVariationDescriptionType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsVariationDescriptionType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_VariationSet);
	// Mp7JrsFactory::DeleteObject(m_Variation);
}

void Mp7JrsVariationDescriptionType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use VariationDescriptionTypePtr, since we
	// might need GetBase(), which isn't defined in IVariationDescriptionType
    const Mp7JrsPtr< Mp7JrsVariationDescriptionType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsContentAbstractionPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsVariationDescriptionType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_VariationSet);
		this->SetVariationSet(Mp7JrsFactory::CloneObject(tmp->GetVariationSet()));
		// Mp7JrsFactory::DeleteObject(m_Variation);
		this->SetVariation(Mp7JrsFactory::CloneObject(tmp->GetVariation()));
}

Mp7JrsNodePtr Mp7JrsVariationDescriptionType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsVariationDescriptionType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsContentAbstractionType > Mp7JrsVariationDescriptionType::GetBase() const
{
	return m_Base;
}

Mp7JrsVariationDescriptionType_VariationSet_CollectionPtr Mp7JrsVariationDescriptionType::GetVariationSet() const
{
	return m_VariationSet;
}

Mp7JrsVariationDescriptionType_Variation_CollectionPtr Mp7JrsVariationDescriptionType::GetVariation() const
{
	return m_Variation;
}

// implementing setter for choice 
/* element */
void Mp7JrsVariationDescriptionType::SetVariationSet(const Mp7JrsVariationDescriptionType_VariationSet_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVariationDescriptionType::SetVariationSet().");
	}
	if (m_VariationSet != item)
	{
		// Mp7JrsFactory::DeleteObject(m_VariationSet);
		m_VariationSet = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_VariationSet) m_VariationSet->SetParent(m_myself.getPointer());
		if(m_VariationSet != Mp7JrsVariationDescriptionType_VariationSet_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_VariationSet->SetContentName(XMLString::transcode("VariationSet"));
		}
	// Mp7JrsFactory::DeleteObject(m_Variation);
	m_Variation = Mp7JrsVariationDescriptionType_Variation_CollectionPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsVariationDescriptionType::SetVariation(const Mp7JrsVariationDescriptionType_Variation_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVariationDescriptionType::SetVariation().");
	}
	if (m_Variation != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Variation);
		m_Variation = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Variation) m_Variation->SetParent(m_myself.getPointer());
		if(m_Variation != Mp7JrsVariationDescriptionType_Variation_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Variation->SetContentName(XMLString::transcode("Variation"));
		}
	// Mp7JrsFactory::DeleteObject(m_VariationSet);
	m_VariationSet = Mp7JrsVariationDescriptionType_VariationSet_CollectionPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsContentDescriptionType_Affective_CollectionPtr Mp7JrsVariationDescriptionType::GetAffective() const
{
	return GetBase()->GetBase()->GetAffective();
}

void Mp7JrsVariationDescriptionType::SetAffective(const Mp7JrsContentDescriptionType_Affective_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVariationDescriptionType::SetAffective().");
	}
	GetBase()->GetBase()->SetAffective(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsDescriptionMetadataPtr Mp7JrsVariationDescriptionType::GetDescriptionMetadata() const
{
	return GetBase()->GetBase()->GetBase()->GetDescriptionMetadata();
}

// Element is optional
bool Mp7JrsVariationDescriptionType::IsValidDescriptionMetadata() const
{
	return GetBase()->GetBase()->GetBase()->IsValidDescriptionMetadata();
}

Mp7JrsCompleteDescriptionType_Relationships_CollectionPtr Mp7JrsVariationDescriptionType::GetRelationships() const
{
	return GetBase()->GetBase()->GetBase()->GetRelationships();
}

Mp7JrsCompleteDescriptionType_OrderingKey_CollectionPtr Mp7JrsVariationDescriptionType::GetOrderingKey() const
{
	return GetBase()->GetBase()->GetBase()->GetOrderingKey();
}

void Mp7JrsVariationDescriptionType::SetDescriptionMetadata(const Mp7JrsDescriptionMetadataPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVariationDescriptionType::SetDescriptionMetadata().");
	}
	GetBase()->GetBase()->GetBase()->SetDescriptionMetadata(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsVariationDescriptionType::InvalidateDescriptionMetadata()
{
	GetBase()->GetBase()->GetBase()->InvalidateDescriptionMetadata();
}
void Mp7JrsVariationDescriptionType::SetRelationships(const Mp7JrsCompleteDescriptionType_Relationships_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVariationDescriptionType::SetRelationships().");
	}
	GetBase()->GetBase()->GetBase()->SetRelationships(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsVariationDescriptionType::SetOrderingKey(const Mp7JrsCompleteDescriptionType_OrderingKey_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVariationDescriptionType::SetOrderingKey().");
	}
	GetBase()->GetBase()->GetBase()->SetOrderingKey(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsVariationDescriptionType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("VariationSet")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:VariationSet is item of type VariationSetType
		// in element collection VariationDescriptionType_VariationSet_CollectionType
		
		context->Found = true;
		Mp7JrsVariationDescriptionType_VariationSet_CollectionPtr coll = GetVariationSet();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateVariationDescriptionType_VariationSet_CollectionType;
				SetVariationSet(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:VariationSetType")))) != empty)
			{
				// Is type allowed
				Mp7JrsVariationSetPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Variation")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Variation is item of type VariationType
		// in element collection VariationDescriptionType_Variation_CollectionType
		
		context->Found = true;
		Mp7JrsVariationDescriptionType_Variation_CollectionPtr coll = GetVariation();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateVariationDescriptionType_Variation_CollectionType;
				SetVariation(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:VariationType")))) != empty)
			{
				// Is type allowed
				Mp7JrsVariationPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for VariationDescriptionType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "VariationDescriptionType");
	}		
	
	return result;
}

XMLCh * Mp7JrsVariationDescriptionType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsVariationDescriptionType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}

bool Mp7JrsVariationDescriptionType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsVariationDescriptionType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsVariationDescriptionType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsVariationDescriptionType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("VariationDescriptionType"));
	// Element serialization:
	if (m_VariationSet != Mp7JrsVariationDescriptionType_VariationSet_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_VariationSet->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_Variation != Mp7JrsVariationDescriptionType_Variation_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Variation->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsVariationDescriptionType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsContentAbstractionType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
	do // Deserialize choice just one time!
    {
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("VariationSet"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("VariationSet")) == 0))
        {
            // Deserialize factory type
            Mp7JrsVariationDescriptionType_VariationSet_CollectionPtr tmp = CreateVariationDescriptionType_VariationSet_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetVariationSet(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Variation"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Variation")) == 0))
        {
            // Deserialize factory type
            Mp7JrsVariationDescriptionType_Variation_CollectionPtr tmp = CreateVariationDescriptionType_Variation_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetVariation(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
// no includefile for extension defined 
// file Mp7JrsVariationDescriptionType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsVariationDescriptionType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("VariationSet")) == 0))
  {
    Mp7JrsVariationDescriptionType_VariationSet_CollectionPtr tmp = CreateVariationDescriptionType_VariationSet_CollectionType;
    this->SetVariationSet(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Variation")) == 0))
  {
    Mp7JrsVariationDescriptionType_Variation_CollectionPtr tmp = CreateVariationDescriptionType_Variation_CollectionType;
    this->SetVariation(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsVariationDescriptionType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetVariationSet() != Mp7JrsNodePtr())
		result.Insert(GetVariationSet());
	if (GetVariation() != Mp7JrsNodePtr())
		result.Insert(GetVariation());
	if (GetAffective() != Mp7JrsNodePtr())
		result.Insert(GetAffective());
	if (GetDescriptionMetadata() != Mp7JrsNodePtr())
		result.Insert(GetDescriptionMetadata());
	if (GetRelationships() != Mp7JrsNodePtr())
		result.Insert(GetRelationships());
	if (GetOrderingKey() != Mp7JrsNodePtr())
		result.Insert(GetOrderingKey());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsVariationDescriptionType_ExtMethodImpl.h


