
// Based on PatternType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



#include <stdio.h>
#include <xercesc/util/regx/RegularExpression.hpp>
#include <xercesc/util/regx/Match.hpp>
#include <assert.h>
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"

// no includefile for extension defined 
// file Mp7JrscurrencyCode_ExtImplInclude.h


#include "Mp7JrscurrencyCode.h"

Mp7JrscurrencyCode::Mp7JrscurrencyCode()
    : m_nsURI(X("urn:mpeg:mpeg7:schema:2004"))
{
        Init();
}

Mp7JrscurrencyCode::~Mp7JrscurrencyCode()
{
        Cleanup();
}

void Mp7JrscurrencyCode::Init()
{       
        m_Content = NULL;

// no includefile for extension defined 
// file Mp7JrscurrencyCode_ExtPropInit.h

}

void Mp7JrscurrencyCode::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrscurrencyCode_ExtPropCleanup.h


        XMLString::release(&m_Content);
}

void Mp7JrscurrencyCode::DeepCopy(const Mp7JrsNodePtr &original)
{
        Mp7JrsPtr< Mp7JrscurrencyCode > tmp = original;
        if (tmp == NULL) return; // EXIT: wrong type passed
        Cleanup();
        SetContent(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::replicate(tmp->GetContent()));
}

Mp7JrsNodePtr Mp7JrscurrencyCode::GetBaseRoot()
{
        return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrscurrencyCode::GetBaseRoot() const
{
        return m_myself.getPointer();
}

XMLCh * Mp7JrscurrencyCode::GetContent() const
{
        return m_Content;
}

void Mp7JrscurrencyCode::SetContent(XMLCh * val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrscurrencyCode::SetContent().");
        }
        if(m_Content != val)
        {
                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&m_Content);
                m_Content = val;
        }
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}


XMLCh * Mp7JrscurrencyCode::ToText() const
{
        XMLCh * buf = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("");

        // For symmetry, create an entry, which must be released
        buf = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::replicate(m_Content);
        return buf;
}


/////////////////////////////////////////////////////////////////

bool Mp7JrscurrencyCode::Parse(const XMLCh * const txt)
{
        Cleanup();
        // mio 07 10 2004
        // FIXME: Regular expression validation is not implemented for this general pattern type.
        // (developers: see PatternType_CPP.template comments for more details.) 
        
        m_Content =  XERCES_CPP_NAMESPACE_QUALIFIER XMLString::replicate(txt);
        return true;
}

bool Mp7JrscurrencyCode::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	// Mp7JrscurrencyCode has no attributes so forward it to Parse()
	return Parse(txt);
}
XMLCh* Mp7JrscurrencyCode::ContentToString() const
{
	// Mp7JrscurrencyCode has no attributes so forward it to ToText()
	return ToText();
}

void Mp7JrscurrencyCode::Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem)
{
        // The element we serialize into
        XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* element;

        XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* localNewElem = NULL;
        if (!newElem) newElem = &localNewElem;

        if (*newElem) {
                element = *newElem;
        } else if(parent == NULL) {
                element = doc->getDocumentElement();
                *newElem = element;
        } else {
                if(this->GetContentName() != (XMLCh*)NULL) {
//  OLD                      XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * elem = doc->createElement(this->GetContentName());
                        DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
                        parent->appendChild(elem);
                        *newElem = elem;
                        element = elem;
                } else
                        assert(false);
        }
        assert(element);

        if(this->UseTypeAttribute && ! element->hasAttribute(X("xsi:type")))
        {
			element->setAttribute(X("xsi:type"), X(Mp7JrsFactory::GetTypeName(Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:currencyCode")))));
        }

// no includefile for extension defined 
// file Mp7JrscurrencyCode_ExtPreImplementCleanup.h


        if(this->GetContentName() != NULL)
        {
                element->appendChild(doc->createTextNode(this->GetContentName())); 
        }
}

bool Mp7JrscurrencyCode::Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent,  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current)
{
// no includefile for extension defined 
// file Mp7JrscurrencyCode_ExtPostDeserialize.h



        if(parent == NULL)
        {
                return false;
        }

        bool found = false;

        XMLCh * buf = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::replicate(Mp7JrsUtil::GetElementText(parent));
        if(buf != NULL)
        {
                found = true;
                * current = parent;
                SetContent(buf);
        }
        return found;
}

// no includefile for extension defined 
// file Mp7JrscurrencyCode_ExtMethodImpl.h








