
// Based on PatternType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2010, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



#include <stdio.h>
#include <xercesc/util/regx/RegularExpression.hpp>
#include <xercesc/util/regx/Match.hpp>
#include <assert.h>
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"

// no includefile for extension defined 
// file Mp7JrsmediaDurationType_ExtImplInclude.h


#include "Mp7JrsmediaDurationType.h"

Mp7JrsmediaDurationType::Mp7JrsmediaDurationType()
    : m_nsURI(X("urn:mpeg:mpeg7:schema:2004"))
{
        Init();
}

Mp7JrsmediaDurationType::~Mp7JrsmediaDurationType()
{
        Cleanup();
}

void Mp7JrsmediaDurationType::Init()
{       
        m_DurationSign = 1;
        m_Days = -1;
        m_Hours = -1;
        m_Minutes = -1;
        m_Seconds = -1;
        m_NumberOfFractions = -1;
        m_Time_Exist = false;
        m_FractionsPerSecond = -1;

// no includefile for extension defined 
// file Mp7JrsmediaDurationType_ExtPropInit.h

}

void Mp7JrsmediaDurationType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsmediaDurationType_ExtPropCleanup.h


}

void Mp7JrsmediaDurationType::DeepCopy(const Mp7JrsNodePtr &original)
{
        Mp7JrsPtr< Mp7JrsmediaDurationType > tmp = original;
        if (tmp == NULL) return; // EXIT: wrong type passed
        Cleanup();
        // FIXME: this will fail for derived classes
  
        this->Parse(tmp->ToText());
        
        this->m_ContentName = NULL;
}

Mp7JrsNodePtr Mp7JrsmediaDurationType::GetBaseRoot()
{
        return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsmediaDurationType::GetBaseRoot() const
{
        return m_myself.getPointer();
}

int Mp7JrsmediaDurationType::GetDurationSign() const
{
        return m_DurationSign;
}

void Mp7JrsmediaDurationType::SetDurationSign(int val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsmediaDurationType::SetDurationSign().");
        }
        m_DurationSign = val;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}
int Mp7JrsmediaDurationType::GetDays() const
{
        return m_Days;
}

void Mp7JrsmediaDurationType::SetDays(int val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsmediaDurationType::SetDays().");
        }
        m_Days = val;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}
int Mp7JrsmediaDurationType::GetHours() const
{
        return m_Hours;
}

void Mp7JrsmediaDurationType::SetHours(int val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsmediaDurationType::SetHours().");
        }
        m_Hours = val;
        m_Time_Exist = true;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}
int Mp7JrsmediaDurationType::GetMinutes() const
{
        return m_Minutes;
}

void Mp7JrsmediaDurationType::SetMinutes(int val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsmediaDurationType::SetMinutes().");
        }
        m_Minutes = val;
        m_Time_Exist = true;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}
int Mp7JrsmediaDurationType::GetSeconds() const
{
        return m_Seconds;
}

void Mp7JrsmediaDurationType::SetSeconds(int val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsmediaDurationType::SetSeconds().");
        }
        m_Seconds = val;
        m_Time_Exist = true;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}
int Mp7JrsmediaDurationType::GetNumberOfFractions() const
{
        return m_NumberOfFractions;
}

void Mp7JrsmediaDurationType::SetNumberOfFractions(int val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsmediaDurationType::SetNumberOfFractions().");
        }
        m_NumberOfFractions = val;
        m_Time_Exist = true;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}
bool Mp7JrsmediaDurationType::GetTime_Exist() const
{
        return m_Time_Exist;
}

void Mp7JrsmediaDurationType::SetTime_Exist(bool val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsmediaDurationType::SetTime_Exist().");
        }
        m_Time_Exist = val;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}
int Mp7JrsmediaDurationType::GetFractionsPerSecond() const
{
        return m_FractionsPerSecond;
}

void Mp7JrsmediaDurationType::SetFractionsPerSecond(int val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsmediaDurationType::SetFractionsPerSecond().");
        }
        m_FractionsPerSecond = val;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}


XMLCh * Mp7JrsmediaDurationType::ToText() const
{
        XMLCh * buf = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("");
        XMLCh * tmp = NULL;
        
        if ( m_Days >= 0 || m_Time_Exist || m_FractionsPerSecond >= 0 ) {

            if(m_DurationSign < 0)
            {
                Mp7JrsUtil::CatString(&buf, X("-"));
            }

        
            Mp7JrsUtil::CatString(&buf, X("P"));

            if(m_Days >= 0)
            {    
                tmp = Mp7JrsConvert::BinToText(m_Days);
                Mp7JrsUtil::CatString(&buf, tmp);
                Mp7JrsUtil::CatString(&buf, X("D"));
                XMLString::release(&tmp);
            }

            if(m_Time_Exist)
            {
                Mp7JrsUtil::CatString(&buf, X("T"));

                if(m_Hours >= 0)
                {
                        tmp = Mp7JrsConvert::BinToText(m_Hours);
                        Mp7JrsUtil::CatString(&buf, tmp);
                        Mp7JrsUtil::CatString(&buf, X("H"));
                        XMLString::release(&tmp);
                }

                if(m_Minutes >= 0)
                {
                        tmp = Mp7JrsConvert::BinToText(m_Minutes);
                        Mp7JrsUtil::CatString(&buf, tmp);
                        Mp7JrsUtil::CatString(&buf, X("M"));
                        XMLString::release(&tmp);
                }

                if(m_Seconds >= 0)
                {
                        tmp = Mp7JrsConvert::BinToText(m_Seconds);
                        Mp7JrsUtil::CatString(&buf, tmp);
                        Mp7JrsUtil::CatString(&buf, X("S"));
                        XMLString::release(&tmp);
                }

                if(m_NumberOfFractions >= 0)
                {
                        tmp = Mp7JrsConvert::BinToText(m_NumberOfFractions);
                        Mp7JrsUtil::CatString(&buf, tmp);
                        Mp7JrsUtil::CatString(&buf, X("N"));
                        XMLString::release(&tmp);
                }
            }

            if(m_FractionsPerSecond >= 0)
            {
                tmp = Mp7JrsConvert::BinToText(m_FractionsPerSecond);
                Mp7JrsUtil::CatString(&buf, tmp);
                Mp7JrsUtil::CatString(&buf, X("F"));
                XMLString::release(&tmp);
            }
        }
        return buf;
}


/////////////////////////////////////////////////////////////////

bool Mp7JrsmediaDurationType::Parse(const XMLCh * const txt)
{
        Match m;
        XMLCh substring [128] = {0};
        RegularExpression r("\\-?P(\\d+D)?(T(\\d+H)?(\\d+M)?(\\d+S)?(\\d+N)?)?(\\d+F)?");
        if(r.matches(txt, &m))
        {
                for(int i = 0; i < m.getNoGroups(); i++)
                {
                        int i1 = m.getStartPos(i);
                        int i2 = Mp7JrsUtil::GetNextEndPos(m, i);
                        if(i1 == i2) return false;
                        switch(i)
                        {
                                case 0:
                                        if((i1 >= 0) && (i2 > (i1 + 1))) // len = 2 -> - sign
                                        {
                                                // \\-?P
                                                m_DurationSign = -1;
                                        }
                                break;

                                case 1:
                                        if(i1 >= 0)
                                        {
                                                // (\\d+D)?
                                                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i2 - 1);
                                                m_Days = Mp7JrsConvert::TextToUnsignedInt(substring);
                                        }
                                break;

                                case 2:
                                        if(i1 >= 0)
                                        {
                                                // T
                                                // Nothing to do
                                        }
                                break;

                                case 3:
                                        if(i1 >= 0)
                                        {
                                                // (\\d+H)?
                                                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i2 - 1);
                                                m_Hours = Mp7JrsConvert::TextToUnsignedInt(substring);
                                                m_Time_Exist = true;
                                        }
                                break;

                                case 4:
                                        if(i1 >= 0)
                                        {
                                                // (\\d+M)?
                                                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i2 - 1);
                                                m_Minutes = Mp7JrsConvert::TextToUnsignedInt(substring);
                                                m_Time_Exist = true;
                                        }
                                break;

                                case 5:
                                        if(i1 >= 0)
                                        {
                                                // (\\d+S)?
                                                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i2 - 1);
                                                m_Seconds = Mp7JrsConvert::TextToUnsignedInt(substring);
                                                m_Time_Exist = true;
                                        }
                                break;

                                case 6:
                                        if(i1 >= 0)
                                        {
                                                // (\\d+N)?
                                                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i2 - 1);
                                                m_NumberOfFractions = Mp7JrsConvert::TextToUnsignedInt(substring);
                                                m_Time_Exist = true;
                                        }
                                break;

                                case 7:
                                        if(i1 >= 0)
                                        {
                                                // (\\d+F)?
                                                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i2 - 1);
                                                m_FractionsPerSecond = Mp7JrsConvert::TextToUnsignedInt(substring);
                                        }
                                break;
        
                                default: return false;
                                break;
                        }
                }
        }
        return true;
}

bool Mp7JrsmediaDurationType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	// Mp7JrsmediaDurationType has no attributes so forward it to Parse()
	return Parse(txt);
}
XMLCh* Mp7JrsmediaDurationType::ContentToString() const
{
	// Mp7JrsmediaDurationType has no attributes so forward it to ToText()
	return ToText();
}

void Mp7JrsmediaDurationType::Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem)
{
        // The element we serialize into
        XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* element;

        XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* localNewElem = NULL;
        if (!newElem) newElem = &localNewElem;

        if (*newElem) {
                element = *newElem;
        } else if(parent == NULL) {
                element = doc->getDocumentElement();
                *newElem = element;
        } else {
                if(this->GetContentName() != (XMLCh*)NULL) {
//  OLD                      XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * elem = doc->createElement(this->GetContentName());
                        DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
                        parent->appendChild(elem);
                        *newElem = elem;
                        element = elem;
                } else
                        assert(false);
        }
        assert(element);

        if(this->UseTypeAttribute && ! element->hasAttribute(X("xsi:type")))
        {
			element->setAttribute(X("xsi:type"), X(Mp7JrsFactory::GetTypeName(Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:mediaDurationType")))));
        }

// no includefile for extension defined 
// file Mp7JrsmediaDurationType_ExtPreImplementCleanup.h


        XMLCh * buf = this->ToText();
        if(buf != NULL)
        {
                element->appendChild(doc->createTextNode(buf));
                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&buf);
        }
}

bool Mp7JrsmediaDurationType::Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent,  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current)
{
// no includefile for extension defined 
// file Mp7JrsmediaDurationType_ExtPostDeserialize.h


        bool found = this->Parse(Mp7JrsUtil::GetElementText(parent));
        if(found)
        {
                * current = parent;
        }
        return found;   
}

// begin extension included
// file Mp7JrsmediaDurationType_ExtMethodImpl.h


int IMp7JrsmediaDurationType::GetTotalNrFractions() {
	unsigned int sum = 0;
	// days and moths missing!!!!!
	sum = 60 * (GetHours() + sum);
	sum = 60 * (GetMinutes() + sum);
	sum = GetFractionsPerSecond() * (GetSeconds() + sum);
	sum += GetNumberOfFractions(); 
	return (int) sum;
}

void IMp7JrsmediaDurationType::SetTotalNrFractions(int fractions, Mp7JrsClientID client) {
	unsigned int remainder = (unsigned int) fractions;
	SetNumberOfFractions(remainder % GetFractionsPerSecond(),client);
	remainder /= GetFractionsPerSecond();
	SetSeconds(remainder % 60,client);
	remainder /= 60;
	SetMinutes(remainder % 60,client);
	remainder /= 60;
	SetHours(remainder,client);
	// days and months ignored!!!!
}

// end extension included








